package chemShiftPredict;
/*
HASH Ha Chemical Shift Prediction  Software Version 1.0
Copyright (C) 2009-2012 Bruce Donald Lab, Duke University

HASH is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free
Software Foundation, either version 3 of the License, or (at your option) any
later version.

HASH is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
details.

You should have received a copy of the GNU Lesser General Public License
along with this library; if not, see:
    <http://www.gnu.org/licenses/>.

There are additional restrictions imposed on the use and distribution of this
open-source code, including: (A) this header must be included in any
modification or extension of the code; (B) you are required to cite our
papers in any publications that use this code. The citation for the various
different modules of our software, together with a complete list of
requirements and restrictions are found in the document license.pdf enclosed
with this distribution.

Contact Info:
    Bruce R. Donald
    Duke University
    Department of Computer Science
    Levine Science Research Center (LSRC)
    Durham, NC 27708-0129
    USA
    email: www.cs.duke.edu/brd/

<signature of Bruce Donald>, 01 May, 2011
Bruce R. Donald, Professor of Computer Science and Biochemistry
*/

import java.util.Vector;

import rdcPanda.H1CS;

public class ChemicalShiftIndex {
	
	public ChemicalShiftIndex(){}
	
	//get reference chemical shift value
	public double getReferenceChemicalShift(String res, String atom){
		if (res.equalsIgnoreCase("ALA") || res.equalsIgnoreCase("A")){
			if(atom.equalsIgnoreCase("CA"))
				return 52.5;
		}
		if (res.equalsIgnoreCase("CYS") || res.equalsIgnoreCase("C")){//temp
			if(atom.equalsIgnoreCase("CA"))
				return 58.3;
		}
		if (res.equalsIgnoreCase("ASP") || res.equalsIgnoreCase("D")){
			if(atom.equalsIgnoreCase("CA"))
				return 54.1;
		}
		if (res.equalsIgnoreCase("GLU") || res.equalsIgnoreCase("E")){
			if(atom.equalsIgnoreCase("CA"))
				return 56.7;
		}
		if (res.equalsIgnoreCase("PHE") || res.equalsIgnoreCase("F")){
			if(atom.equalsIgnoreCase("CA"))
				return 57.9;
		}
		if (res.equalsIgnoreCase("GLY") || res.equalsIgnoreCase("G")){
			if(atom.equalsIgnoreCase("CA"))
				return 45.0;
		}
		if (res.equalsIgnoreCase("HIS") || res.equalsIgnoreCase("H")){
			if(atom.equalsIgnoreCase("CA"))
				return 55.8;
		}
		if (res.equalsIgnoreCase("ILE") || res.equalsIgnoreCase("I")){
			if(atom.equalsIgnoreCase("CA"))
				return 62.6;
		}
		if (res.equalsIgnoreCase("LYS") || res.equalsIgnoreCase("K")){
			if(atom.equalsIgnoreCase("CA"))
				return 56.7;
		}
		if (res.equalsIgnoreCase("LEU") || res.equalsIgnoreCase("L")){
			if(atom.equalsIgnoreCase("CA"))
				return 55.7;
		}
		if (res.equalsIgnoreCase("MET") || res.equalsIgnoreCase("M")){
			if(atom.equalsIgnoreCase("CA"))
				return 56.6;
		}
		if (res.equalsIgnoreCase("ASN") || res.equalsIgnoreCase("N")){
			if(atom.equalsIgnoreCase("CA"))
				return 53.6;
		}
		if (res.equalsIgnoreCase("PRO") || res.equalsIgnoreCase("P")){
			if(atom.equalsIgnoreCase("CA"))
				return 62.9;
		}
		if (res.equalsIgnoreCase("GLN") || res.equalsIgnoreCase("Q")){
			if(atom.equalsIgnoreCase("CA"))
				return 56.2;
		}
		if (res.equalsIgnoreCase("ARG") || res.equalsIgnoreCase("R")){
			if(atom.equalsIgnoreCase("CA"))
				return 56.3;
		}
		if (res.equalsIgnoreCase("SER") || res.equalsIgnoreCase("S")){
			if(atom.equalsIgnoreCase("CA"))
				return 58.3;
		}
		if (res.equalsIgnoreCase("THR") || res.equalsIgnoreCase("T")){
			if(atom.equalsIgnoreCase("CA"))
				return 63.1;
		}
		if (res.equalsIgnoreCase("TRP") || res.equalsIgnoreCase("W")){
			if(atom.equalsIgnoreCase("CA"))
				return 57.8;
		}
		if (res.equalsIgnoreCase("TYR") || res.equalsIgnoreCase("Y")){
			if(atom.equalsIgnoreCase("CA"))
				return 58.6;
		}		
		if (res.equalsIgnoreCase("VAL") || res.equalsIgnoreCase("V")){
			if(atom.equalsIgnoreCase("HA"))
				return 4.0;
			if(atom.equalsIgnoreCase("CA"))
				return 63.0;
			if(atom.equalsIgnoreCase("CB"))
				return 31.7;
			if(atom.equalsIgnoreCase("CO")|| atom.equalsIgnoreCase("C"))
				return 177.1;
		}//if (res.equalsIgnoreCase("VAL"))		
		
		return -9999.9;			
	}
	
	//compute the index from Ca, Cb and CO chemical shifts...
	public int computeChemicalShiftIndexBasedCa(String res, double cs_ca){
		double error=0.7;//ppm
		double refCS=getReferenceChemicalShift(res,"CA");
		if(Math.abs(cs_ca - refCS)<= error)
			return 0;
		else if(cs_ca > (refCS+error ) )
			return 1;
		else if(cs_ca < (refCS - error ) )
			return -1;
		
		return -2;
	}
	
	//add the new resonance to the vector
	public void addNewResonance(Vector <H1CS> vecResonances, H1CS h1cs){
		boolean isIn=false;
		int resNo=h1cs.getResidueNo();
		String atomName=h1cs.getAtomName();
		for(int i=0;i<vecResonances.size();i++){
			H1CS h1csTemp=vecResonances.elementAt(i);
			int resNoTemp=h1csTemp.getResidueNo();
			String atomNameTemp=h1csTemp.getAtomName();
			if(resNo==resNoTemp && atomName.equalsIgnoreCase(atomNameTemp))
				isIn=true;
		}
		if(!isIn)
			vecResonances.add(h1cs);
	}
	
	public Vector <H1CS> extractHelixChemicalShifts(Vector <H1CS> vecAllResonances){
	   
	   Vector <H1CS> vecHelixResonance=new Vector <H1CS>();
		 //find max res no
	   	int maxNo=-999;
	   	for(int i=0;i<vecAllResonances.size();i++){
	   		H1CS h1cs=(H1CS)vecAllResonances.elementAt(i);
	   		int resNo=h1cs.getResidueNo();
	   		if(resNo>maxNo)
	   			maxNo=resNo;    		
	   	}//	for(int i=0;i<vecAllResonances.size();i++)
	   	
	   //	for(int no=1;no<=maxNo;no++){
	   	int startNo=1;
	   	while(startNo<=(maxNo-4) ){
						
	   		int endNo=startNo;
	   		boolean isHelix=false;
	   		
	   		int curNo=startNo;
	   		int counter=0;
	   		boolean isSatisfied=true;
	   		while(curNo<maxNo){
	   			double csCA=-9999.9;
	   		
	   			//get residue name
	    		String resName="";
	    		for(int i=0;i<vecAllResonances.size();i++){
		    		H1CS h1cs=(H1CS)vecAllResonances.elementAt(i);
		    		int resNo=h1cs.getResidueNo();		    		
		    		if(resNo==curNo){
		    			resName=h1cs.getResidueType();
		    			break;
		    		}	    			   		
		    	}
	   			//find cs of CA
				for(int i=0;i<vecAllResonances.size();i++){
		    		H1CS h1cs=(H1CS)vecAllResonances.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==curNo && atom.equalsIgnoreCase("CA")){
		    			csCA=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}//for(int i=0;i<vecAllResonances.size();i++)
				if(csCA<-100){
					startNo++;
					break;
				}
				
				int curCSIndex=computeChemicalShiftIndexBasedCa(resName,csCA);
				if((curCSIndex==1) && (curNo==startNo) )
					counter++;
				if(curCSIndex==-1)
					isSatisfied=false;
				
				//next residue number
				int nextResNo=curNo+1;
				//get next residue name
	    		String resNameNext="";
	    		for(int i=0;i<vecAllResonances.size();i++){
		    		H1CS h1cs=(H1CS)vecAllResonances.elementAt(i);
		    		int resNo=h1cs.getResidueNo();		    		
		    		if(resNo==nextResNo){
		    			resNameNext=h1cs.getResidueType();
		    			break;
		    		}	    			   		
		    	}
	    		//find cs of CA in next residue
	    		double csCANext=-9999.9;
				for(int i=0;i<vecAllResonances.size();i++){
		    		H1CS h1cs=(H1CS)vecAllResonances.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==nextResNo && atom.equalsIgnoreCase("CA")){
		    			csCANext=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}//for(int i=0;i<vecAllResonances.size();i++)
				if(csCANext<-100){
					startNo=nextResNo+1;
					break;
				}
				int nextCSIndex=computeChemicalShiftIndexBasedCa(resNameNext,csCANext);
				if(nextCSIndex==1)
					counter++;
				if(nextCSIndex==-1)
					isSatisfied=false;
				
				//check whether the criterion is satisfied
				if(nextResNo==(startNo+4) ){
					if( (counter>=4) && isSatisfied){
						for(int k=startNo; k<=nextResNo;k++){
							for(int t=0;t<vecAllResonances.size();t++){
					    		H1CS h1cs=(H1CS)vecAllResonances.elementAt(t);
					    		int resNo=h1cs.getResidueNo();		    		
					    		if(resNo==k){
					    			addNewResonance(vecHelixResonance, h1cs);
					    		}	    			   		
					    	}
						}//for(int k=startNo; k<=nextResNo;k++)
						
					}//if( (counter>=4) && isSatisfied)
					
					startNo++;
					break;
					
				}//if(nextResNo==(startNo+4) )
					
									
				//termination for helix fragment
				if( (curCSIndex==1 && nextCSIndex==-1) || (curCSIndex==0 && nextCSIndex==0)  ){
					startNo=nextResNo+1;
					break;
				}

				curNo++;
	   		}//while(curNo<maxNo)
			
		
			
		}//while(startNo<maxNo)		
	

	   return vecHelixResonance;
   }
   
   
}
