package chemShiftPredict;

/*
HASH Ha Chemical Shift Prediction  Software Version 1.0
Copyright (C) 2009-2012 Bruce Donald Lab, Duke University

HASH is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free
Software Foundation, either version 3 of the License, or (at your option) any
later version.

HASH is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
details.

You should have received a copy of the GNU Lesser General Public License
along with this library; if not, see:
    <http://www.gnu.org/licenses/>.

There are additional restrictions imposed on the use and distribution of this
open-source code, including: (A) this header must be included in any
modification or extension of the code; (B) you are required to cite our
papers in any publications that use this code. The citation for the various
different modules of our software, together with a complete list of
requirements and restrictions are found in the document license.pdf enclosed
with this distribution.

Contact Info:
    Bruce R. Donald
    Duke University
    Department of Computer Science
    Levine Science Research Center (LSRC)
    Durham, NC 27708-0129
    USA
    email: www.cs.duke.edu/brd/

<signature of Bruce Donald>, 01 May, 2011
Bruce R. Donald, Professor of Computer Science and Biochemistry
*/


import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.*;



import rdcPanda.Cartesian;
import rdcPanda.H1CS;
import rdcPanda.Pdb;
import rdcPanda.PhiPsi;


//used for testing the TALOS database
public class chemicalShiftDatabase {

	
	public chemicalShiftDatabase(){};
    
	//get random coil chemical shifts
	//use the table from Camshift paper.
	public double getRandomCoilChemShift(String res, String atom,boolean isNCorrection){
		if(res.equalsIgnoreCase("VAL") || res.equalsIgnoreCase("V")  ){
			double corN=0.0;
			if(isNCorrection)
				corN=3.83;
			else 
				corN=0.0;
			if(atom.equalsIgnoreCase("CA"))
				return 62.512466;
			else if(atom.equalsIgnoreCase("HA"))
				return 4.086605;
			else if(atom.equalsIgnoreCase("CB"))
				return 31.7;
			else if(atom.equalsIgnoreCase("C") || atom.equalsIgnoreCase("CO"))
				return 177.1;
			else if(atom.equalsIgnoreCase("HN") || atom.equalsIgnoreCase("H"))
				return 8.03;
			else if(atom.equalsIgnoreCase("N"))
				return corN+119.2;
		}
		
		
		return -99999.9;
	}
	
	//get random coil chemical shifts
	//use the table from Camshift paper.
	public double getRandomCoilChemShift(Vector<randomCoilShift> vecRandomShifts,String res, String atom,String prevResType){	
		if(res.equalsIgnoreCase("A") )
			res="ALA";
		else if(res.equalsIgnoreCase("C"))
			res="CYS";
		else if(res.equalsIgnoreCase("D"))
			res="ASP";
		else if(res.equalsIgnoreCase("E"))
			res="GLU";
		else if(res.equalsIgnoreCase("F"))
			res="PHE";
		else if(res.equalsIgnoreCase("G"))
			res="GLY";
		else if(res.equalsIgnoreCase("H"))
			res="HIS";
		else if(res.equalsIgnoreCase("I"))
			res="ILE";		
		else if(res.equalsIgnoreCase("K"))
			res="LYS";
		else if(res.equalsIgnoreCase("L"))
			res="LEU";
		else if(res.equalsIgnoreCase("M"))
			res="MET";
		else if(res.equalsIgnoreCase("N"))
			res="ASN";
		else if(res.equalsIgnoreCase("P"))
			res="PRO";
		else if(res.equalsIgnoreCase("Q"))
			res="GLN";
		else if(res.equalsIgnoreCase("R"))
			res="ARG";
		else if(res.equalsIgnoreCase("S"))
			res="SER";
		else if(res.equalsIgnoreCase("T"))
			res="THR";
		else if(res.equalsIgnoreCase("V") )
			res="VAL";
		else if(res.equalsIgnoreCase("W"))
			res="TRP";
		else if(res.equalsIgnoreCase("Y"))
			res="TYR";		
		
		double corN=0.0;
		for(int i=0;i<vecRandomShifts.size();i++){
			randomCoilShift rcs =vecRandomShifts.elementAt(i);
			if(!rcs.resType.equalsIgnoreCase(prevResType))
				continue;			
			corN=rcs.nCorrection;			
		}//for(int i=0;i<vecRandomShifts.size();i++)
		
		for(int i=0;i<vecRandomShifts.size();i++){
			randomCoilShift rcs =vecRandomShifts.elementAt(i);
			if(!rcs.resType.equalsIgnoreCase(res))
				continue;			
			if(atom.equalsIgnoreCase("HA")|| atom.equalsIgnoreCase("HA1")||atom.equalsIgnoreCase("HA2")||atom.equalsIgnoreCase("HA3"))
				return rcs.haCS;
			if(atom.equalsIgnoreCase("CA"))
				return rcs.caCS;
			if(atom.equalsIgnoreCase("HN") || atom.equalsIgnoreCase("H"))
				return rcs.hnCS;
			if(atom.equalsIgnoreCase("N"))
				return rcs.nCS+corN;
			if(atom.equalsIgnoreCase("CO") || atom.equalsIgnoreCase("C"))
				return rcs.coCS;
			if(atom.equalsIgnoreCase("CB"))
				return rcs.cbCS;
			
		}//for(int i=0;i<vecRandomShifts.size();i++)
		
		return -99999.9;
	}
	
	//convert an angle to be in between [-pi,pi]
	public static double convertAngleInPI(double angle){
		//float theta2=theta2*(float)Math.PI/180.0f;
		double residual = (angle%(2*Math.PI)); // in range
		if (residual>Math.PI)
			residual=residual-2*Math.PI;
		if (residual<-Math.PI)
			residual=residual+2*Math.PI;		
		return residual;
	}
	
	public static String convertResNameFromThreeLettersToOneLetter(String res){
		String strAAs="";
		if(res.equalsIgnoreCase("ALA"))
			strAAs=" A";
		else if(res.equalsIgnoreCase("CYS"))
			strAAs=" C";
		else if(res.equalsIgnoreCase("ASP"))
			strAAs=" D";
		else if(res.equalsIgnoreCase("GLU"))
			strAAs=" E";
		else if(res.equalsIgnoreCase("PHE"))
			strAAs=" F";
		else if(res.equalsIgnoreCase("GLY"))
			strAAs=" G";
		else if(res.equalsIgnoreCase("HIS"))
			strAAs=" H";
		else if(res.equalsIgnoreCase("ILE"))
			strAAs=" I";
		else if(res.equalsIgnoreCase("LYS"))
			strAAs=" K";
		else if(res.equalsIgnoreCase("LEU"))
			strAAs=" L";
		else if(res.equalsIgnoreCase("MET"))
			strAAs=" M";
		else if(res.equalsIgnoreCase("ASN"))
			strAAs=" N";
		else if(res.equalsIgnoreCase("PRO"))
			strAAs=" P";
		else if(res.equalsIgnoreCase("GLN"))
			strAAs=" Q";
		else if(res.equalsIgnoreCase("ARG"))
			strAAs=" R";
		else if(res.equalsIgnoreCase("SER"))
			strAAs=" S";		
		else if(res.equalsIgnoreCase("THR"))
			strAAs=" T";
		else if(res.equalsIgnoreCase("VAL"))
			strAAs=" V";
		else if(res.equalsIgnoreCase("TRP"))
			strAAs=" W";
		else if(res.equalsIgnoreCase("TYR"))
			strAAs=" Y";
		strAAs=strAAs.trim();
		return strAAs;
	}
	public String convertResNameFromOneLetterToThreeLetters(String aaLetter){
		String resName="";
		if(aaLetter.equalsIgnoreCase("A"))
			resName= "ALA";
		else if(aaLetter.equalsIgnoreCase("C"))
			resName= "CYS";
		else if(aaLetter.equalsIgnoreCase("D"))
			resName= "ASP";
		else if(aaLetter.equalsIgnoreCase("E"))
			resName= "GLU";
		else if(aaLetter.equalsIgnoreCase("F"))
			resName= "PHE";
		else if(aaLetter.equalsIgnoreCase("G"))
			resName= "GLY";
		else if(aaLetter.equalsIgnoreCase("H"))
			resName= "HIS";
		else if(aaLetter.equalsIgnoreCase("I"))
			resName= "ILE";
		else if(aaLetter.equalsIgnoreCase("K"))
			resName= "LYS";
		else if(aaLetter.equalsIgnoreCase("L"))
			resName= "LEU";
		else if(aaLetter.equalsIgnoreCase("M"))
			resName= "MET";
		else if(aaLetter.equalsIgnoreCase("N"))
			resName= "ASN";
		else if(aaLetter.equalsIgnoreCase("P"))
			resName= "PRO";
		else if(aaLetter.equalsIgnoreCase("Q"))
			resName= "GLN";
		else if(aaLetter.equalsIgnoreCase("R"))
			resName= "ARG";
		else if(aaLetter.equalsIgnoreCase("S"))
			resName= "SER";
		else if(aaLetter.equalsIgnoreCase("T"))
			resName= "THR";
		else if(aaLetter.equalsIgnoreCase("V"))
			resName= "VAL";
		else if(aaLetter.equalsIgnoreCase("W"))
			resName= "TRP";
		else if(aaLetter.equalsIgnoreCase("Y"))
			resName= "TYR";	
		
		return resName;
	}
	public void outputOneLetterAAs(Vector<Pdb> vecPdb){
		String strAAs="";
		for(int i=0; i<vecPdb.size();i++){
			Pdb pdb=vecPdb.elementAt(i);
			String res=pdb.getResidue();
			if(res.equalsIgnoreCase("ALA"))
				strAAs=strAAs+" A";
    		else if(res.equalsIgnoreCase("CYS"))
    			strAAs=strAAs+" C";
    		else if(res.equalsIgnoreCase("ASP"))
    			strAAs=strAAs+" D";
    		else if(res.equalsIgnoreCase("GLU"))
    			strAAs=strAAs+" E";
    		else if(res.equalsIgnoreCase("PHE"))
    			strAAs=strAAs+" F";
    		else if(res.equalsIgnoreCase("GLY"))
    			strAAs=strAAs+" G";
    		else if(res.equalsIgnoreCase("HIS"))
    			strAAs=strAAs+" H";
    		else if(res.equalsIgnoreCase("ILE"))
    			strAAs=strAAs+" I";
    		else if(res.equalsIgnoreCase("LYS"))
    			strAAs=strAAs+" K";
    		else if(res.equalsIgnoreCase("LEU"))
    			strAAs=strAAs+" L";
    		else if(res.equalsIgnoreCase("MET"))
    			strAAs=strAAs+" M";
    		else if(res.equalsIgnoreCase("ASN"))
    			strAAs=strAAs+" N";
    		else if(res.equalsIgnoreCase("PRO"))
    			strAAs=strAAs+" P";
    		else if(res.equalsIgnoreCase("GLN"))
    			strAAs=strAAs+" Q";
    		else if(res.equalsIgnoreCase("ARG"))
    			strAAs=strAAs+" R";
    		else if(res.equalsIgnoreCase("SER"))
    			strAAs=strAAs+" S";		
    		else if(res.equalsIgnoreCase("THR"))
    			strAAs=strAAs+" T";
    		else if(res.equalsIgnoreCase("VAL"))
    			strAAs=strAAs+" V";
    		else if(res.equalsIgnoreCase("TRP"))
    			strAAs=strAAs+" W";
    		else if(res.equalsIgnoreCase("TYR"))
    			strAAs=strAAs+" Y";
		}//for(int i=0; i<vecPdb.size();i++)
		strAAs=strAAs.trim();
		System.out.println(strAAs);
	}
	//initialization of conditional probability table
	//the first dimension corresponds to the residue type: 
	// A  C  D  E  F  G  H  I  K  L  M  N  P  Q  R  S  T  V  W  Y
	// 0  1  2  3  4  5  6  7  8  9  10 11 12 13 14 15 16 17 18 19
	//2-6 dimensions correspond to the discretization of Ca Cb CO, HN and N respectively. 
	//7 and 8 dimensions correspond to the delta chemical shifts of hbond and ring current effects.
	public void initializeCPT(){
		float[][][][][][][][] cpTable=new float[20][][][][][][][];
			
		int maxNum_bond=35;
		int maxNum_ring=35;
		
		double resolution=0.5;
		int numSigma=4;
		//double hResol=0.05; //resolution for h chemical shifts
		//double heavyResol=0.5;//resolution for heavy atom chemical shifts
		
		H1CS h1CS=new H1CS();
		String userDir = System.getProperty("user.dir");
		String pathCSDatabase=userDir+"/system/cs_database/shifts/";  //chemical shift input file directory	
		String pathPdbDatabase=userDir+"/system/cs_database/pdb/";  //Pdb input file directory
		
		//read BMRB statistical information:
		String srcBMRB=userDir+"/system/BMRB_CS.txt";    	
    	Vector<H1CS> vecBMRB=h1CS.ReadBMRBSatistics(srcBMRB);
    	Vector<H1CS> vecBMRBNew=new Vector<H1CS>();		
    	vecBMRBNew=h1CS.CSNameConvertFromBMRBNewToPDBNew(vecBMRB);//assume that the default naming scheme of cs is BMRB
    	
    	for(int k=0;k<20;k++){ //all 20 aa types   	
	    	double ca_low=-9999.9,ca_up=-9999.9;//test on leu residue type
	    	double n_low=-9999.9,n_up=-9999.9;
	    	double hn_low=-9999.9,hn_up=-9999.9;double co_low=-9999.9,co_up=-9999.9;
	    	double ha_low=-9999.9,ha_up=-9999.9;double cb_low=-9999.9,cb_up=-9999.9;
	    	
	    	double ca_mean=-9999.9, ca_dv=-9999.9;
			double ha_mean=-9999.9, ha_dv=-9999.9;
			double n_mean=-9999.9, n_dv=-9999.9;
			double hn_mean=-9999.9, hn_dv=-9999.9;
			double cb_mean=-9999.9, cb_dv=-9999.9;
			double co_mean=-9999.9, co_dv=-9999.9;
			
			//get the bounds of ca, cb, co, hn and n chemical shifts.
			for(int i=0;i<vecBMRBNew.size();i++){				
	    		H1CS h1cs=vecBMRBNew.elementAt(i);
	    		String res=h1cs.getResidueType();   
	    		int resIndex=-1;    			
	    		if(res.equalsIgnoreCase("ALA"))
	    			resIndex=0;
	    		else if(res.equalsIgnoreCase("CYS"))
	    			resIndex=1;
	    		else if(res.equalsIgnoreCase("ASP"))
	    			resIndex=2;
	    		else if(res.equalsIgnoreCase("GLU"))
	    			resIndex=3;
	    		else if(res.equalsIgnoreCase("PHE"))
	    			resIndex=4;
	    		else if(res.equalsIgnoreCase("GLY"))
	    			resIndex=5;
	    		else if(res.equalsIgnoreCase("HIS"))
	    			resIndex=6;
	    		else if(res.equalsIgnoreCase("ILE"))
	    			resIndex=7;
	    		else if(res.equalsIgnoreCase("LYS"))
	    			resIndex=8;
	    		else if(res.equalsIgnoreCase("LEU"))
	    			resIndex=9;
	    		else if(res.equalsIgnoreCase("MET"))
	    			resIndex=10;
	    		else if(res.equalsIgnoreCase("ASN"))
	    			resIndex=11;
	    		else if(res.equalsIgnoreCase("PRO"))
	    			resIndex=12;
	    		else if(res.equalsIgnoreCase("GLN"))
	    			resIndex=13;
	    		else if(res.equalsIgnoreCase("ARG"))
	    			resIndex=14;
	    		else if(res.equalsIgnoreCase("SER"))
	    			resIndex=15;    		
	    		else if(res.equalsIgnoreCase("THR"))
	    			resIndex=16;
	    		else if(res.equalsIgnoreCase("VAL"))
	    			resIndex=17;
	    		else if(res.equalsIgnoreCase("TRP"))
	    			resIndex=18;
	    		else if(res.equalsIgnoreCase("TYR"))
	    			resIndex=19;
	    		
	    		
	    		String atom=h1cs.getAtomName();
	    		if(atom.equalsIgnoreCase("CA") && resIndex==k ){
	    			ca_mean=h1cs.getCSAverage();
	    			ca_dv=h1cs.getCSSigma();
	    			ca_low=ca_mean -numSigma*ca_dv;
	    			ca_up=ca_mean +numSigma*ca_dv;    			
	    		}
	    		
	    		if(atom.equalsIgnoreCase("CB") && resIndex==k ){
	    			cb_mean=h1cs.getCSAverage();
	    			cb_dv=h1cs.getCSSigma();
	    			cb_low=cb_mean -numSigma*cb_dv;
	    			cb_up=cb_mean +numSigma*cb_dv;    			
	    		}
	    		if( (atom.equalsIgnoreCase("CO") || atom.equalsIgnoreCase("C") ) && resIndex==k){
	    			co_mean=h1cs.getCSAverage();
	    			co_dv=h1cs.getCSSigma();
	    			co_low=co_mean -numSigma*co_dv;
	    			co_up=co_mean +numSigma*co_dv;	    			
	    		}
	    		if( (atom.equalsIgnoreCase("HN") || atom.equalsIgnoreCase("H") ) && resIndex==k){
	    			hn_mean=h1cs.getCSAverage();
	    			hn_dv=h1cs.getCSSigma();
	    			hn_low=hn_mean -numSigma*hn_dv;
	    			hn_up=hn_mean +numSigma*hn_dv;	    			
	    		}
	    		if(atom.equalsIgnoreCase("N") && resIndex==k){
	    			n_mean=h1cs.getCSAverage();
	    			n_dv=h1cs.getCSSigma();
	    			n_low=n_mean-numSigma*n_dv;    
	    			n_up=n_mean+numSigma*n_dv; 	    			
	    		}
	    		/*if(atom.equalsIgnoreCase("HA")){
	    			ha_mean=h1cs.getCSAverage();
	    			ha_dv=h1cs.getCSSigma();
	    		}*/    		
	    		
				
	    	}//for(int i=0;i<vecBMRBNew.size();i++)
			
			resolution=ca_dv/3;
			int maxNum_ca=(int) ( (ca_up-ca_low)/resolution )+1;
			
			resolution=cb_dv/3;
			int maxNum_cb=(int) ( (cb_up-cb_low)/resolution )+1;
			
			resolution=co_dv/3;
			int maxNum_co=(int) ( (co_up-co_low)/resolution )+1;
			
			resolution=hn_dv/3;
			int maxNum_hn=(int) ( (hn_up-hn_low)/resolution )+1;
			
			resolution=n_dv/3;
			int maxNum_n=(int) ( (n_up-n_low)/resolution )+1;
			
			cpTable[k]=new float[maxNum_ca+1][][][][][][];			
			for(int ind_ca=0;ind_ca<=maxNum_ca;ind_ca++){
				cpTable[k][ind_ca]=new float[maxNum_cb+1][][][][][];
				for(int ind_cb=0;ind_cb<=maxNum_cb;ind_cb++){
					cpTable[k][ind_ca][ind_cb]=new float[maxNum_co+1][][][][];
					for(int ind_co=0;ind_co<=maxNum_co;ind_co++){
						cpTable[k][ind_ca][ind_cb][ind_co]=new float[maxNum_hn+1][][][];
						for(int ind_hn=0;ind_hn<=maxNum_hn;ind_hn++){
							cpTable[k][ind_ca][ind_cb][ind_co][ind_hn]=new float[maxNum_n+1][][];
							for(int ind_n=0;ind_n<=maxNum_n;ind_n++){
								cpTable[k][ind_ca][ind_cb][ind_co][ind_hn][ind_n]=new float[maxNum_bond+1][];
								for(int ind_hbond=0;ind_hbond<=maxNum_bond;ind_hbond++){
									cpTable[k][ind_ca][ind_cb][ind_co][ind_hn][ind_n][ind_hbond]=new float[maxNum_ring+1];
								}
							}//for(int ind_n=0;ind_n<=maxNum_n;ind_n++)
						}//for(int ind_hn=0;ind_hn<=maxNum_hn;ind_hn++)
					}//for(int ind_co=0;ind_co<=maxNum_co;ind_co++)
				}//for(int ind_cb=0;ind_cb<=maxNum_cb;ind_cb++)
			}//for(int ind_ca=0;ind_ca<=maxNum_ca;ind_ca++)			
    	}//for(int k=0;k<20;k++)
	}

	
	public static void main (String[] args){
		ringEffect ring_effect=new ringEffect();
		hBondEffect hbond_effect=new hBondEffect();
		
		
		int counterTemp=0;
		chemicalShiftDatabase csd=new chemicalShiftDatabase();
	    //	csd.initializeCPT();
		
		
		
		H1CS h1CS=new H1CS();
		Pdb pp=new Pdb();
		String userDir = System.getProperty("user.dir");
		
		//two places need to be changed to switch between training and testing 
		//First place:
		
	    ////String pathCSDatabase=userDir+"/system/cs_database/shifts/";  //chemical shift input file directory	
	    /// String pathPdbDatabase=userDir+"/system/cs_database/pdb/";  //Pdb input file directory
	    String pathCSDatabase=userDir+"/system/cs_database/shiftx/shifts_test/";  //chemical shift input file directory	
	    String pathPdbDatabase=userDir+"/system/cs_database/shiftx/pdb_test/";  //Pdb input file directory
		
		String pathRandomCoilShift=userDir+"/system/cs_database/random_coil_shifts.txt"; //random coil shift table
			
		//read random coil chemical shift table:
		randomCoilShift rcs=new randomCoilShift();
		Vector<randomCoilShift> vecRandomShifts=rcs.randomCoilShiftReader(pathRandomCoilShift);
		
		//read BMRB statistical information:
		String srcBMRB=userDir+"/system/BMRB_CS.txt";    	
    	Vector<H1CS> vecBMRB=h1CS.ReadBMRBSatistics(srcBMRB);
    	Vector<H1CS> vecBMRBNew=new Vector<H1CS>();		
    	vecBMRBNew=h1CS.CSNameConvertFromBMRBNewToPDBNew(vecBMRB);//assume that the default naming scheme of cs is BMRB
    	
    	  	
    	
    	//read all chemical shift files
    	int counter=1;
    	File myDir = new File(pathCSDatabase);
    	String[] contents = myDir.list();//select all structures
    	if (contents == null) 
    		System.out.println(myDir.getName() + " is not a directory");
    	for (int t=0; t<contents.length;t++)  {  
    		//int counter_ha=0;
    		//double sum_ha=0.0;
    		
    		String filename = contents[t]; 
    		String fileNameSaved=filename.substring(0,filename.lastIndexOf("."));
    		
	    	String strCSFile="";
	    	strCSFile=pathCSDatabase+filename;	    
	    	filename=filename.substring(0,filename.lastIndexOf("."))+".pdb";
	    	String strPdbFile=pathPdbDatabase+filename;
	    	
	    	//two places need to be changed to switch between training and testing 
			//Second place:
	// Vector<H1CS> vecCSTemp=h1CS.h1CSReader_TALOS(strCSFile);///for training
	   	Vector<H1CS> vecCSTemp=h1CS.h1CSReader_BMRB(strCSFile); //for testing
	 
	 
	 // Vector<H1CS> vecCSTemp=h1CS.h1CSReader_CYANA(strCSFile);///for training
	    		    	
	    	Vector<Pdb> vecPdb = pp.readPdb(strPdbFile);
	    	
	    	////csd.outputOneLetterAAs(vecPdb);
	   // Vector<Pdb> vecTempDebug=pp.OutputBackbone(vecPdb);
	    
	  	//pp.print(vecTempDebug);
	    	//temp
	 /*  	for(int x=0;x<vecPdb.size();x++){
	    		Pdb pdbTemp=vecPdb.elementAt(x);
	    		int noTemp=pdbTemp.getResidueNo();
	    		pdbTemp.setResidueNo(noTemp-45);
	    	}
	    	pp.print(vecPdb);
	   */ 	
	    	
	    	boolean isBMRBNEWNameScheme=false;;// whether using the BMRB_NEW naming scheme for atom names.
	    	for(int x=0;x<vecPdb.size();x++){
	    		Pdb pdb=vecPdb.elementAt(x);
	    		String res=pdb.getResidue();
	    		Vector<Cartesian> vecAtom =pdb.getAtomVec();
	    		for(int y=0;y<vecAtom.size();y++){
	    			Cartesian cc=vecAtom.elementAt(y);
	    			String atom=cc.getAtom();
	    			if( !( res.equalsIgnoreCase("ALA")||res.equalsIgnoreCase("GLY")|| res.equalsIgnoreCase("ILE")
	    					||res.equalsIgnoreCase("PRO")||res.equalsIgnoreCase("THR")||res.equalsIgnoreCase("VAL")) ){
	    				if(atom.equalsIgnoreCase("HB3"))
	    					isBMRBNEWNameScheme=true;
	    				break;
	    			}	    			
	    		}//for(int y=0;y<vecAtom.size();y++)
	    		if(isBMRBNEWNameScheme)
	    			break;
	    	}//for(int x=0;x<vecPdb.size();x++)
	    	if(isBMRBNEWNameScheme){
	    		Vector<Pdb>  vecPdbTemp=pp.nameConvert2PDBNew(vecPdb);
	    		vecPdb=new Vector<Pdb>();
	    		vecPdb.addAll(vecPdbTemp);
	    	}
	    	
	    	//update residue numbers for some PDB (with negative residue numbers)
	    	for(int g=0;g<vecPdb.size();g++){
	    		Pdb pdbTemp=vecPdb.elementAt(g);
	    		int resNoTemp=pdbTemp.getResidueNo();
	    		if(resNoTemp<0){
	    			///pdbTemp.setResidueNo(-resNoTemp);
	    			vecPdb.remove(g);
	    			g--;
	    		}	    			
	    	}
	    	
	    	Vector<PhiPsi> vecPhiPsi=pp.PhiPsiTotalPdb(vecPdb);
	    	
	    	//only extract helix chemical shifts
	    	/*ChemicalShiftIndex csi=new ChemicalShiftIndex();
	    	Vector<H1CS> vecCSHelix=csi.extractHelixChemicalShifts(vecCSTemp);
	    	vecCSTemp=new Vector<H1CS>();
	    	vecCSTemp.addAll(vecCSHelix);*/	    	
	    	   	
	    	//find max res no and min res no	    		    	
	    	int minNoFromCS=9999999;
	    	int maxNoFromCS=-99999;
	    	for(int i=0;i<vecCSTemp.size();i++){
	    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
	    		int resNo=h1cs.getResidueNo();
	    		if(resNo>maxNoFromCS)
	    			maxNoFromCS=resNo;  
	    		if(resNo<minNoFromCS)
	    			minNoFromCS=resNo;
	    	}
	    	int minNoFromPdb=9999999;
	    	int maxNoFromPdb=-99999;
	    	for(int i=0;i<vecPdb.size();i++){
	    		Pdb pdb=(Pdb)vecPdb.elementAt(i);
	    		int resNo=pdb.getResidueNo();
	    		if(resNo>maxNoFromPdb)
	    			maxNoFromPdb=resNo;  
	    		if(resNo<minNoFromPdb)
	    			minNoFromPdb=resNo;
	    	}
	    	int maxNo=Math.min(maxNoFromCS, maxNoFromPdb);
	    	int minNo=Math.max(minNoFromCS, minNoFromPdb);
	    	
	    	//compute the chemical shift delta from hbond effect:
	    	double[] haCSDelta=new double[maxNo+1];
	    	double[] hnCSDelta=new double[maxNo+1];
	    	for(int i=0;i<=maxNo;i++){
	    		haCSDelta[i]=0.0;
	    		hnCSDelta[i]=0.0;
	    	}
	    	//hbond_effect.computeAllDeltaFromHBondShiftx(vecPdb, haCSDelta, hnCSDelta);
	    	
	    	for(int no=minNo+4;no<maxNo-2;no++){	    		
	    		
	    		//we skip the first residue
	    		if (no==1)
	    			continue;
	    		int ind= Collections.binarySearch(vecPdb, new Pdb(no), new Pdb.PdbComparator());
	     	    if(ind<0)
	     	    	continue;
	     	    Pdb curPdb=vecPdb.elementAt(ind);
	     	    String curRes=curPdb.getResidue();
	     	    
	     	   int ind2= Collections.binarySearch(vecPhiPsi, new PhiPsi(no), new PhiPsi.PPComparator());
	     	   if(ind2<0)
	     	    	continue;
	     	   
	     	   PhiPsi phipsi=vecPhiPsi.elementAt(ind2);
	     	 
	     	   int indPrev= Collections.binarySearch(vecPhiPsi, new PhiPsi(no-1), new PhiPsi.PPComparator());
	     	   double phiPrev=0.0;
	    	   double psiPrev=0.0;
	    	   if( indPrev>-1){
	    		   PhiPsi phipsiPre=vecPhiPsi.elementAt(indPrev);
	    		   phiPrev=convertAngleInPI( phipsiPre.getPhi() );
	    		   psiPrev=convertAngleInPI( phipsiPre.getPsi() );
	    	   }
	    	   
	    	   int indNext= Collections.binarySearch(vecPhiPsi, new PhiPsi(no+1), new PhiPsi.PPComparator());
	    	   double phiNext=0.0;
	    	   double psiNext=0.0;
	    	   if( indNext>-1){
	    		   PhiPsi phipsiNext=vecPhiPsi.elementAt(indNext);
	    		   phiNext=convertAngleInPI( phipsiNext.getPhi() );
	    		   psiNext=convertAngleInPI( phipsiNext.getPsi() );
	    	   }
	     	   
	     	   //check the residue name of the next residue
	     	   String prevResType="";
	     	   ind=Collections.binarySearch(vecPdb, new Pdb(no-1), new Pdb.PdbComparator());
	     	   if(ind>-1){	     	    	
		     	   Pdb prevPdb=vecPdb.elementAt(ind);
		     	   prevResType=prevPdb.getResidue();
	     	   }
	     	   
	     	   String prePreResType="";
	     	   ind=Collections.binarySearch(vecPdb, new Pdb(no-2), new Pdb.PdbComparator());
	     	   if(ind>-1){	     	    	
		     	   Pdb prevPrePdb=vecPdb.elementAt(ind);
		     	   prePreResType=prevPrePdb.getResidue();
	     	   }
	     	   
	     	  String preThreeResType="";
	     	   ind=Collections.binarySearch(vecPdb, new Pdb(no-3), new Pdb.PdbComparator());
	     	   if(ind>-1){	     	    	
		     	   Pdb preThreePdb=vecPdb.elementAt(ind);
		     	  preThreeResType=preThreePdb.getResidue();
	     	   }
	     	   
	     	   String preFourResType="";
	     	   ind=Collections.binarySearch(vecPdb, new Pdb(no-4), new Pdb.PdbComparator());
	     	   if(ind>-1){	     	    	
		     	   Pdb preFourPdb=vecPdb.elementAt(ind);
		     	   preFourResType=preFourPdb.getResidue();
	     	   } 
	     	   
	     	   String nextResType="";
	     	   ind=Collections.binarySearch(vecPdb, new Pdb(no+1), new Pdb.PdbComparator());
	     	   if(ind>-1){	     	    	
		     	   Pdb nextPdb=vecPdb.elementAt(ind);
		     	   nextResType=nextPdb.getResidue();
	     	   }
	     	   
	     	   String nextTwoResType="";
	     	   ind=Collections.binarySearch(vecPdb, new Pdb(no+2), new Pdb.PdbComparator());
	     	   if(ind>-1){	     	    	
		     	   Pdb nextTwoPdb=vecPdb.elementAt(ind);
		     	  nextTwoResType=nextTwoPdb.getResidue();
	     	   }
	     	   
	     	  String nextThreeResType="";
	     	  ind=Collections.binarySearch(vecPdb, new Pdb(no+3), new Pdb.PdbComparator());
	     	  if(ind>-1){	     	    	
		     	 Pdb nextThreePdb=vecPdb.elementAt(ind);
		     	nextThreeResType=nextThreePdb.getResidue();
	     	  }
	     	  
	     	   /////////////////////////////////
	     	   //get the bounds
	     	  double ca_low=-9999.9,ca_up=-9999.9;//test on leu residue type
	      	  double n_low=-9999.9,n_up=-9999.9;
	      	  double hn_low=-9999.9,hn_up=-9999.9;double co_low=-9999.9,co_up=-9999.9;
	      	  double ha_low=-9999.9,ha_up=-9999.9;double cb_low=-9999.9,cb_up=-9999.9;
	      	
	      	  double ca_mean=-9999.9, ca_dv=-9999.9;
	  		  double ha_mean=-9999.9, ha_dv=-9999.9;
	  		  double n_mean=-9999.9, n_dv=-9999.9;
	  		  double hn_mean=-9999.9, hn_dv=-9999.9;
	  		  double cb_mean=-9999.9, cb_dv=-9999.9;
	  		  double co_mean=-9999.9, co_dv=-9999.9;
	      	  for(int i=0;i<vecBMRBNew.size();i++){
	      		  H1CS h1cs=vecBMRBNew.elementAt(i);
	      		  String res=h1cs.getResidueType();
	      		  if(!res.equalsIgnoreCase(curRes))
	      			  continue;
	      		
	      		  String atom=h1cs.getAtomName();
	      		  if(atom.equalsIgnoreCase("CA")){
	      			  ca_mean=h1cs.getCSAverage();
	      			  ca_dv=h1cs.getCSSigma();
	      		  }
	      		  if(atom.equalsIgnoreCase("N")){
	      			n_mean=h1cs.getCSAverage();
	      			n_dv=h1cs.getCSSigma();
	      		  }
	      		  if(atom.equalsIgnoreCase("HA")){
	      			ha_mean=h1cs.getCSAverage();
	      			ha_dv=h1cs.getCSSigma();
	      		  }
	      		  if(res.equalsIgnoreCase("GLY")){
	      			if(atom.equalsIgnoreCase("HA1")){
		      			ha_mean=h1cs.getCSAverage();
		      			ha_dv=h1cs.getCSSigma();
		      		  }
	      		  }
	      		  if(atom.equalsIgnoreCase("HN") || atom.equalsIgnoreCase("H")){
	      			hn_mean=h1cs.getCSAverage();
	      			hn_dv=h1cs.getCSSigma();
	      		  }
	      		  if(atom.equalsIgnoreCase("CB")){
	      			cb_mean=h1cs.getCSAverage();
	      			cb_dv=h1cs.getCSSigma();
	      		  }
	      		  if(atom.equalsIgnoreCase("CO") || atom.equalsIgnoreCase("C")){
	      			co_mean=h1cs.getCSAverage();
	      			co_dv=h1cs.getCSSigma();
	      		  }
	      	  }//for(int i=0;i<vecBMRBNew.size();i++)
	          ca_low=ca_mean -5*ca_dv;ca_up=ca_mean +5*ca_dv;
	          n_low=n_mean-5*n_dv;    n_up=n_mean+5*n_dv;        
	          ha_low=ha_mean -5*ha_dv;ha_up=ha_mean +5*ha_dv;
	      	  hn_low=hn_mean -5*hn_dv;hn_up=hn_mean +5*hn_dv;
	      	  cb_low=cb_mean -5*cb_dv;cb_up=cb_mean +5*cb_dv;
	      	  co_low=co_mean -5*co_dv;co_up=co_mean +5*co_dv;
	      	  //////////////////////////////////////////
	      	  
	     	   ////////////////////////////////
	     	    //compute the chemical shift delta from hbond and ring.
	     	   float deltaHBond_ha=(float)hbond_effect.computeDeltaFromHBondForHaOld(no, vecPdb);
	     	   float deltaHBond_hn=(float)hbond_effect.computeDeltaFromHBondForHnOld(no, vecPdb);
	     	   
	     	   float deltaRing=(float)ring_effect.computeDeltaChemShiftFromRingEffectForHa(no, vecPdb)*1000;
	     	
	     	   float deltaRingCa=(float)ring_effect.computeDeltaChemShiftFromRingEffectForCa(no, vecPdb)*1000;
	     	  
	   	    		
	    		double csN=-9999.0;
	    		double csCA=-9999.0;
	    		double csHN=-9999.0;
	    		double csHA=-9999.0;
	    		double csCB=-9999.0;
	    		double csCO=-9999.0;
	    		
	    		//find cs of N
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && atom.equalsIgnoreCase("N")){
		    			csN=h1cs.getH1CS();
		    			break;
		    		}	    		
		    	}	    		
	    		
	    		//find cs of CA
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && atom.equalsIgnoreCase("CA")){
		    			csCA=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	    	
	    		//find cs of HN
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && (atom.equalsIgnoreCase("HN")|| atom.equalsIgnoreCase("H"))  ){
		    			csHN=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	    		
	    		//find cs of HA
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();		    		
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && atom.equalsIgnoreCase("HA")){
		    			csHA=h1cs.getH1CS();
		    			break;
		    		}	
		    	}//for(int i=0;i<vecCSTemp.size();i++)
	    		//for GLY, we compute the mean of two HA chemical shifts
	    		if(curRes.equalsIgnoreCase("GLY") || curRes.equalsIgnoreCase("G")){
	    			double sumHaCS=0.0;	  
	    			int counterHaCS=0;
	    			boolean isFoundHaForGly=false;
	    			for(int i=0;i<vecCSTemp.size();i++){
			    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
			    		int resNo=h1cs.getResidueNo();		    		
			    		String atom=h1cs.getAtomName();
			    		if(atom.length()>=2)
			    			atom=atom.substring(0,2);
			    		if(resNo==no && atom.equalsIgnoreCase("HA")){
			    			sumHaCS=sumHaCS+h1cs.getH1CS();
			    			counterHaCS++;
			    			isFoundHaForGly=true;
			    		}	
			    	}//for(int i=0;i<vecCSTemp.size();i++)
	    			if(isFoundHaForGly){
	    				csHA=sumHaCS/counterHaCS;
	    			}
	    			
	    		}//if(curRes.equalsIgnoreCase("GLY") || curRes.equalsIgnoreCase("G"))
	    		
	    		//find cs of CB
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && atom.equalsIgnoreCase("CB")){
		    			csCB=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	    		
	    		//find cs of CO
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && (atom.equalsIgnoreCase("CO") ||atom.equalsIgnoreCase("C"))){
		    			csCO=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	    		

	    		
	    		double csRandCoilHa=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"HA",prevResType);
	    		double csRandCoilCa=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"CA",prevResType);
	    		double csRandCoilCb=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"CB",prevResType);
	    		double csRandCoilCo=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"C",prevResType);
	    		double csRandCoilHn=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"HN",prevResType);
	    		double csRandCoilN=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"N",prevResType);
	    		
	    		csHA=csHA-csRandCoilHa;
	    		csCA=csCA-csRandCoilCa;
	    		csCB=csCB-csRandCoilCb;
	    		csCO=csCO-csRandCoilCo;
	    		csHN=csHN-csRandCoilHn;
	    		csN=csN-csRandCoilN;
	    		
	    		
	    		//////////////////////////
	    		//get the chemical shifts of previous bb atoms
	    		double csPreN=-9999.0;
	    		double csPreCA=-9999.0;
	    		double csPreHN=-9999.0;	    		
	    		double csPreCB=-9999.0;
	    		double csPreCO=-9999.0;
	    		//find cs of N for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no-1 && atom.equalsIgnoreCase("N")){
		    			csPreN=h1cs.getH1CS();
		    			break;
		    		}	    		
		    	}	   		
	    		
	    		//find cs of CA for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no-1 && atom.equalsIgnoreCase("CA")){
		    			csPreCA=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}	    	
	    		//find cs of HN for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no-1 && (atom.equalsIgnoreCase("HN")|| atom.equalsIgnoreCase("H"))  ){
		    			csPreHN=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	      		//find cs of CB for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no-1 && atom.equalsIgnoreCase("CB")){
		    			csPreCB=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}	    		
	    		//find cs of CO for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no-1 && (atom.equalsIgnoreCase("CO") ||atom.equalsIgnoreCase("C"))){
		    			csPreCO=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	    		csRandCoilHa=csd.getRandomCoilChemShift(vecRandomShifts,prevResType,"HA",prePreResType);
	    		csRandCoilCa=csd.getRandomCoilChemShift(vecRandomShifts,prevResType,"CA",prePreResType);
	    		csRandCoilCb=csd.getRandomCoilChemShift(vecRandomShifts,prevResType,"CB",prePreResType);
	    		csRandCoilCo=csd.getRandomCoilChemShift(vecRandomShifts,prevResType,"C",prePreResType);
	    		csRandCoilHn=csd.getRandomCoilChemShift(vecRandomShifts,prevResType,"HN",prePreResType);
	    		csRandCoilN=csd.getRandomCoilChemShift(vecRandomShifts,prevResType,"N",prePreResType);	    		
	    		
	    		csPreCA=csPreCA-csRandCoilCa;
	    		csPreCB=csPreCB-csRandCoilCb;
	    		csPreCO=csPreCO-csRandCoilCo;
	    		csPreHN=csPreHN-csRandCoilHn;
	    		csPreN=csPreN-csRandCoilN;
	    		///////////////////////////////////
	    		
	    		//////////////////////////
	    		//get the chemical shifts of next bb atoms
	    		double csNextN=-9999.0;
	    		double csNextCA=-9999.0;
	    		double csNextHN=-9999.0;	    		
	    		double csNextCB=-9999.0;
	    		double csNextCO=-9999.0;
	    		
	    		//find cs of N for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no+1 && atom.equalsIgnoreCase("N")){
		    			csNextN=h1cs.getH1CS();
		    			break;
		    		}	    	
		    	}	   		
	    		
	    		//find cs of CA for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no+1 && atom.equalsIgnoreCase("CA")){
		    			csNextCA=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}	    	
	    		//find cs of HN for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no+1 && (atom.equalsIgnoreCase("HN")|| atom.equalsIgnoreCase("H"))  ){
		    			csNextHN=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	      		//find cs of CB for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no+1 && atom.equalsIgnoreCase("CB")){
		    			csNextCB=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}	    		
	    		//find cs of CO for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no+1 && (atom.equalsIgnoreCase("CO") ||atom.equalsIgnoreCase("C"))){
		    			csNextCO=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	    		
	    		csRandCoilCa=csd.getRandomCoilChemShift(vecRandomShifts,nextResType,"CA",curRes);
	    		csRandCoilCb=csd.getRandomCoilChemShift(vecRandomShifts,nextResType,"CB",curRes);
	    		csRandCoilCo=csd.getRandomCoilChemShift(vecRandomShifts,nextResType,"C",curRes);
	    		csRandCoilHn=csd.getRandomCoilChemShift(vecRandomShifts,nextResType,"HN",curRes);
	    		csRandCoilN=csd.getRandomCoilChemShift(vecRandomShifts,nextResType,"N",curRes);	    		
	    		
	    		csNextCA=csNextCA-csRandCoilCa;
	    		csNextCB=csNextCB-csRandCoilCb;
	    		csNextCO=csNextCO-csRandCoilCo;
	    		csNextHN=csNextHN-csRandCoilHn;
	    		csNextN=csNextN-csRandCoilN;
	    		///////////////////////////////////
	    		
	    		//////////////////////////
	    		//get the chemical shifts of bb atoms in i-2 residue
	    		double csPreTwoN=-9999.0;
	    		double csPreTwoCA=-9999.0;
	    		double csPreTwoHN=-9999.0;	    		
	    		double csPreTwoCB=-9999.0;
	    		double csPreTwoCO=-9999.0;
	    		//find cs of N for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no-2 && atom.equalsIgnoreCase("N")){
		    			csPreTwoN=h1cs.getH1CS();
		    			break;
		    		}	    		
		    	}	   		
	    		
	    		//find cs of CA for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no-2 && atom.equalsIgnoreCase("CA")){
		    			csPreTwoCA=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}	    	
	    		//find cs of HN for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no-2 && (atom.equalsIgnoreCase("HN")|| atom.equalsIgnoreCase("H"))  ){
		    			csPreTwoHN=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	      		//find cs of CB for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no-2 && atom.equalsIgnoreCase("CB")){
		    			csPreTwoCB=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}	    		
	    		//find cs of CO for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no-2 && (atom.equalsIgnoreCase("CO") ||atom.equalsIgnoreCase("C"))){
		    			csPreTwoCO=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	    		csRandCoilHa=csd.getRandomCoilChemShift(vecRandomShifts,prePreResType,"HA",preThreeResType);
	    		csRandCoilCa=csd.getRandomCoilChemShift(vecRandomShifts,prePreResType,"CA",preThreeResType);
	    		csRandCoilCb=csd.getRandomCoilChemShift(vecRandomShifts,prePreResType,"CB",preThreeResType);
	    		csRandCoilCo=csd.getRandomCoilChemShift(vecRandomShifts,prePreResType,"C",preThreeResType);
	    		csRandCoilHn=csd.getRandomCoilChemShift(vecRandomShifts,prePreResType,"HN",preThreeResType);
	    		csRandCoilN=csd.getRandomCoilChemShift(vecRandomShifts,prePreResType,"N",preThreeResType);	    		
	    		
	    		csPreTwoCA=csPreTwoCA-csRandCoilCa;
	    		csPreTwoCB=csPreTwoCB-csRandCoilCb;
	    		csPreTwoCO=csPreTwoCO-csRandCoilCo;
	    		csPreTwoHN=csPreTwoHN-csRandCoilHn;
	    		csPreTwoN=csPreTwoN-csRandCoilN;
	    		///////////////////////////////////
	    		
	    		//////////////////////////
	    		//get the chemical shifts of bb atoms in residue i+2
	    		double csNextTwoN=-9999.0;
	    		double csNextTwoCA=-9999.0;
	    		double csNextTwoHN=-9999.0;	    		
	    		double csNextTwoCB=-9999.0;
	    		double csNextTwoCO=-9999.0;
	    		
	    		//find cs of N for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no+2 && atom.equalsIgnoreCase("N")){
		    			csNextTwoN=h1cs.getH1CS();
		    			break;
		    		}	    	
		    	}	   		
	    		
	    		//find cs of CA for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no+2 && atom.equalsIgnoreCase("CA")){
		    			csNextTwoCA=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}	    	
	    		//find cs of HN for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no+2 && (atom.equalsIgnoreCase("HN")|| atom.equalsIgnoreCase("H"))  ){
		    			csNextTwoHN=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	      		//find cs of CB for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no+2 && atom.equalsIgnoreCase("CB")){
		    			csNextTwoCB=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}	    		
	    		//find cs of CO for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no+2 && (atom.equalsIgnoreCase("CO") ||atom.equalsIgnoreCase("C"))){
		    			csNextTwoCO=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	    		
	    		csRandCoilCa=csd.getRandomCoilChemShift(vecRandomShifts,nextTwoResType,"CA",nextResType);
	    		csRandCoilCb=csd.getRandomCoilChemShift(vecRandomShifts,nextTwoResType,"CB",nextResType);
	    		csRandCoilCo=csd.getRandomCoilChemShift(vecRandomShifts,nextTwoResType,"C",nextResType);
	    		csRandCoilHn=csd.getRandomCoilChemShift(vecRandomShifts,nextTwoResType,"HN",nextResType);
	    		csRandCoilN=csd.getRandomCoilChemShift(vecRandomShifts,nextTwoResType,"N",nextResType);	    		
	    		
	    		csNextTwoCA=csNextTwoCA-csRandCoilCa;
	    		csNextTwoCB=csNextTwoCB-csRandCoilCb;
	    		csNextTwoCO=csNextTwoCO-csRandCoilCo;
	    		csNextTwoHN=csNextTwoHN-csRandCoilHn;
	    		csNextTwoN=csNextTwoN-csRandCoilN;
	    		///////////////////////////////////
	    		
	    		//////////////////////////
	    		//get the chemical shifts of bb atoms in i-3 residue
	    		double csPreThreeN=-9999.0;
	    		double csPreThreeCA=-9999.0;
	    		double csPreThreeHN=-9999.0;	    		
	    		double csPreThreeCB=-9999.0;
	    		double csPreThreeCO=-9999.0;
	    		//find cs of N for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no-3 && atom.equalsIgnoreCase("N")){
		    			csPreThreeN=h1cs.getH1CS();
		    			break;
		    		}	    		
		    	}	   		
	    		
	    		//find cs of CA for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no-3 && atom.equalsIgnoreCase("CA")){
		    			csPreThreeCA=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}	    	
	    		//find cs of HN for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no-3 && (atom.equalsIgnoreCase("HN")|| atom.equalsIgnoreCase("H"))  ){
		    			csPreThreeHN=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	      		//find cs of CB for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no-3 && atom.equalsIgnoreCase("CB")){
		    			csPreThreeCB=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}	    		
	    		//find cs of CO for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no-3 && (atom.equalsIgnoreCase("CO") ||atom.equalsIgnoreCase("C"))){
		    			csPreThreeCO=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	    		
	    		csRandCoilCa=csd.getRandomCoilChemShift(vecRandomShifts,preThreeResType,"CA",preFourResType);
	    		csRandCoilCb=csd.getRandomCoilChemShift(vecRandomShifts,preThreeResType,"CB",preFourResType);
	    		csRandCoilCo=csd.getRandomCoilChemShift(vecRandomShifts,preThreeResType,"C",preFourResType);
	    		csRandCoilHn=csd.getRandomCoilChemShift(vecRandomShifts,preThreeResType,"HN",preFourResType);
	    		csRandCoilN=csd.getRandomCoilChemShift(vecRandomShifts,preThreeResType,"N",preFourResType);	    		
	    		
	    		csPreThreeCA=csPreThreeCA-csRandCoilCa;
	    		csPreThreeCB=csPreThreeCB-csRandCoilCb;
	    		csPreThreeCO=csPreThreeCO-csRandCoilCo;
	    		csPreThreeHN=csPreThreeHN-csRandCoilHn;
	    		csPreThreeN=csPreThreeN-csRandCoilN;
	    		///////////////////////////////////
	    		
	    		//////////////////////////
	    		//get the chemical shifts of bb atoms in residue i+2
	    		double csNextThreeN=-9999.0;
	    		double csNextThreeCA=-9999.0;
	    		double csNextThreeHN=-9999.0;	    		
	    		double csNextThreeCB=-9999.0;
	    		double csNextThreeCO=-9999.0;
	    		
	    		//find cs of N for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no+3 && atom.equalsIgnoreCase("N")){
		    			csNextThreeN=h1cs.getH1CS();
		    			break;
		    		}	    	
		    	}	   		
	    		
	    		//find cs of CA for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no+3 && atom.equalsIgnoreCase("CA")){
		    			csNextThreeCA=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}	    	
	    		//find cs of HN for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no+3 && (atom.equalsIgnoreCase("HN")|| atom.equalsIgnoreCase("H"))  ){
		    			csNextThreeHN=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	      		//find cs of CB for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no+3 && atom.equalsIgnoreCase("CB")){
		    			csNextThreeCB=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}	    		
	    		//find cs of CO for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no+3 && (atom.equalsIgnoreCase("CO") ||atom.equalsIgnoreCase("C"))){
		    			csNextThreeCO=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	    		
	    		csRandCoilCa=csd.getRandomCoilChemShift(vecRandomShifts,nextThreeResType,"CA",nextTwoResType);
	    		csRandCoilCb=csd.getRandomCoilChemShift(vecRandomShifts,nextThreeResType,"CB",nextTwoResType);
	    		csRandCoilCo=csd.getRandomCoilChemShift(vecRandomShifts,nextThreeResType,"C",nextTwoResType);
	    		csRandCoilHn=csd.getRandomCoilChemShift(vecRandomShifts,nextThreeResType,"HN",nextTwoResType);
	    		csRandCoilN=csd.getRandomCoilChemShift(vecRandomShifts,nextThreeResType,"N",nextTwoResType);	    		
	    		
	    		csNextThreeCA=csNextThreeCA-csRandCoilCa;
	    		csNextThreeCB=csNextThreeCB-csRandCoilCb;
	    		csNextThreeCO=csNextThreeCO-csRandCoilCo;
	    		csNextThreeHN=csNextThreeHN-csRandCoilHn;
	    		csNextThreeN=csNextThreeN-csRandCoilN;
	    		///////////////////////////////////
	    		
	    		
	    		double phi=convertAngleInPI( phipsi.getPhi());//1.5*Math.cos(3*(phipsi.getPhi() + 2.45*Math.PI/180 ))-3.0*Math.cos(phipsi.getPhi()-2.7*Math.PI/180 );
	    		// 
	    		double psi=convertAngleInPI( phipsi.getPsi() );//-0.4*Math.cos(3* ( phipsi.getPsi() +4.8*Math.PI/180 ) )-1.5*Math.cos(phipsi.getPsi() -0.84*Math.PI/180 )+0.7;
	    		
	    		
	    		/*System.out.println(counter+ " "+ (float)(csHA-csRandCoilHa)+ " "+(float)(csCA-csRandCoilCa)+" "+(float)(csCB-csRandCoilCb)+" " +(float)(csCO-csRandCoilCo)
	    				+" "+(float)(csHN-csRandCoilHn)+" "+(float)(csN-csRandCoilN)+ " "+(float)deltaHBond_ha+" "+(float)deltaHBond_hn+" "+ (float)deltaRing+" "+ (float)phi
	    				+" "+(float)psi+ " "+ (float)phiPrev+" "+(float)psiPrev +" "+ (float)phiNext+" "+(float)psiNext);
	    		*/
	    		
	    		//System.out.println(counter+ " "+ (float)(csHA)+ " "+(float)(csCA)+" "+(float)(csCB)+" " +(float)(csCO)
	    		//		+" "+(float)(csHN)+" "+(float)(csN)+ " "+(float)deltaHBond_ha+" "+(float)deltaHBond_hn+" "+ (float)deltaRing+" "+deltaRingCa+" "+ (float)phi
	    		//		+" "+(float)psi+ " "+ (float)phiPrev+" "+(float)psiPrev +" "+ (float)phiNext+" "+(float)psiNext+" "+curRes+" "+ no+" "+fileNameSaved);
	    		
	    		System.out.println(counter+ " "+ (float)(csHA)+ " "+(float)(csCA)+" "+(float)(csCB)+" " +(float)(csCO)+" "+(float)(csHN)+" "+(float)(csN)
	    				+" "+(float)(csPreCA)+" "+(float)(csPreCB)+" " +(float)(csPreCO)+" "+(float)(csPreHN)+" "+(float)(csPreN)
	    				+" "+(float)(csNextCA)+" "+(float)(csNextCB)+" " +(float)(csNextCO)+" "+(float)(csNextHN)+" "+(float)(csNextN)
	    				+" "+(float)(csPreTwoCA)+" "+(float)(csPreTwoCB)+" " +(float)(csPreTwoCO)+" "+(float)(csPreTwoHN)+" "+(float)(csPreTwoN)
	    				+" "+(float)(csNextTwoCA)+" "+(float)(csNextTwoCB)+" " +(float)(csNextTwoCO)+" "+(float)(csNextTwoHN)+" "+(float)(csNextTwoN)
	    				//+" "+(float)(csPreThreeCA)+" "+(float)(csPreThreeCB)+" " +(float)(csPreThreeCO)+" "+(float)(csPreThreeHN)+" "+(float)(csPreThreeN)
	    				//+" "+(float)(csNextThreeCA)+" "+(float)(csNextThreeCB)+" " +(float)(csNextThreeCO)+" "+(float)(csNextThreeHN)+" "+(float)(csNextThreeN)	    				
	    				+ " "+(float)deltaHBond_ha+" "+(float)deltaHBond_hn+" "+ (float)deltaRing+" "+deltaRingCa+" "+ (float)phi
	    				+" "+(float)psi+ " "+ (float)phiPrev+" "+(float)psiPrev +" "+ (float)phiNext+" "+(float)psiNext+" "+curRes+" "+ no+" "+fileNameSaved);
	    					    		
	    		counter++;
	    		
	    		//counter_ha++;counter_ha++;
	    		//sum_ha=sum_ha+csHA+csHN;
	    	}//for(int i=1;i<=maxNo;i++)   	
	    	
	    	//sum_ha=sum_ha/counter_ha;
	    	//System.out.println("Average chemical shift of Ha:"+sum_ha);
    	}//for (int t=0; t<contents.length;t++) 
    	
    	//System.out.println("Number of residues with missing or incomplete resonance assignments:"+counterTemp);
    
	}//end of main function
	
	
	public static void main00 (String[] args){
		H1CS h1CS=new H1CS();
		String userDir = System.getProperty("user.dir");
		String pathCSDatabase=userDir+"/system/cs_database/shifts/";  //input file directory	
		
		//read BMRB statistical information:
		String srcBMRB=userDir+"/system/BMRB_CS.txt";    	
    	Vector<H1CS> vecBMRB=h1CS.ReadBMRBSatistics(srcBMRB);
    	Vector<H1CS> vecBMRBNew=new Vector<H1CS>();		
    	vecBMRBNew=h1CS.CSNameConvertFromBMRBNewToPDBNew(vecBMRB);//assume that the default naming scheme of cs is BMRB
    	
    	
    	double ca_low=55.65-3*2.13,ca_up=55.65+3*2.13;//test on leu residue type
    	double n_low=121.87-3* 3.94,n_up=121.87+3* 3.94;
    	double hn_low=8.22-3*0.65,hn_up=8.22+3*0.65;
    	double ha_low=4.32-3*0.47,ha_up=4.32+3*0.47;
    	
    	double csGap_h=0.06;
    	double csGap_heavy=0.6;
    	
    	int maxCa=(int) (Math.abs(ca_up-ca_low)/csGap_heavy)+1;
    	int maxN=(int) (Math.abs(n_up-n_low)/csGap_heavy)+1;
    	int maxHn=(int) (Math.abs(hn_up-hn_low)/csGap_h)+1;
    	int maxHa=(int) (Math.abs(ha_up-ha_low)/csGap_h)+1;
    	
    	double[][][][] prob=new double[maxN][maxCa][maxHn][maxHa];
    	for(int i=0;i<maxN;i++)
    		for(int j=0;j<maxCa;j++)
    			for(int k=0;k<maxHn;k++)
    				for(int t=0;t<maxHa;t++)
    					prob[i][j][k][t]=0.0;//initialization
    	
    	int sum=0;
    	//read all shift files
    	File myDir = new File(pathCSDatabase);
    	String[] contents = myDir.list();//select all structures
    	if (contents == null) 
    		System.out.println(myDir.getName() + " is not a directory");
    	for (int t=0; t<contents.length;t++)  	{    
    		String filename = contents[t]; 
	    			
	    	String strCSFile="";
	    	strCSFile=pathCSDatabase+filename;	    
	    	
	    	Vector vecCSTemp=h1CS.h1CSReader_TALOS(strCSFile);
	    	
	    	//find max res no
	    	int maxNo=-999;
	    	for(int i=0;i<vecCSTemp.size();i++){
	    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
	    		int resNo=h1cs.getResidueNo();
	    		if(resNo>maxNo)
	    			maxNo=resNo;    		
	    	}
	    	
	    	for(int no=1;no<=maxNo;no++){
	    		
	    		//get residue name
	    		String resName="";
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();		    		
		    		if(resNo==no){
		    			resName=h1cs.getResidueType();
		    			break;
		    		}	    			   		
		    	}
	    		//if(!resName.equalsIgnoreCase("L"))
	    			//continue;
	    		
	    		double csN=-9999.0;
	    		double csCA=-9999.0;
	    		double csHN=-9999.0;
	    		double csHA=-9999.0;
	    		//find cs of N
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && atom.equalsIgnoreCase("N")){
		    			csN=h1cs.getH1CS();
		    			break;
		    		}	    		
		    	}
	    		if(csN<n_low || csN>n_up)
	    			continue;
	    		
	    		//find cs of CA
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && atom.equalsIgnoreCase("CA")){
		    			csCA=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	    		if(csCA<ca_low || csCA>ca_up)
	    			continue;
	    		
	    		//find cs of HN
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && atom.equalsIgnoreCase("HN")){
		    			csHN=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	    		if(csHN<hn_low || csHN>hn_up)
	    			continue;
	    		
	    		//find cs of HA
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && atom.equalsIgnoreCase("HA")){
		    			csHA=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	    		if(csHA<ha_low || csHA>ha_up)
	    			continue;
	    		
	    		if (  (csN<-100) || (csCA<-100) || ( csHN<-100) || (csHA<-100) )
	    			continue;
	    		int index_N= (int)((csN-n_low)/csGap_heavy);
	    		int index_CA= (int)((csCA-ca_low)/csGap_heavy);
	    		int index_HN= (int)((csHN-hn_low)/csGap_h);
	    		int index_HA= (int)((csHA-ha_low)/csGap_h);
	    		
	    		prob[index_N][index_CA][index_HN][index_HA]=prob[index_N][index_CA][index_HN][index_HA]+1;
	    		sum++;
	    	}//for(int i=1;i<=maxNo;i++)   		
    	}//for (int t=0; t<contents.length;t++) 
    	
    	int sumT=1;
    	for(int i=0;i<maxN;i++)
    		for(int j=0;j<maxCa;j++)
    			for(int k=0;k<maxHn;k++)
    				for(int t=0;t<maxHa;t++)
    					prob[i][j][k][t]=prob[i][j][k][t]/sumT;//initialization
    	
    	
    	//test:
    	double csN=128.400;
		double csCA=53.176;
		double csHN=8.460;
		double csHA= 5.290;
    	int index_N= (int)((csN-n_low)/csGap_heavy);
		int index_CA= (int)((csCA-ca_low)/csGap_heavy);
		int index_HN= (int)((csHN-hn_low)/csGap_h);
		int index_HA= (int)((csHA-ha_low)/csGap_h);
    	
		System.out.println("the total num is: "+ sum);
		
		double test=0.0;
		for(int i=0;i<maxN;i++)
			for(int j=0;j<maxCa;j++)
				for(int k=0;k<maxHn;k++)
				test=test+prob[i][j][k][index_HA];
		
		System.out.println("the probability of CSHA is: "+ prob[index_N][index_CA][index_HN][index_HA]);
	}//end of main function
	
	/** 
	 * Read the Shiftx training or testing data.
	 *  @param pathCSDatabase path of cs directory
	 *  @param pathPdbDatabase path of PDB directory
	 *  @param isTraining whether it is training data, TRUE for training data, FALSE for testing data
	 *  @return a vector of residueCS
	 */
	public Vector <Vector<residueCS>> readTalosData(String pathCSDatabase,String pathPdbDatabase,boolean isTraining){		
		Vector<Vector<residueCS>> vecData=new  Vector<Vector<residueCS>>();
		
		ringEffect ring_effect=new ringEffect();
		hBondEffect hbond_effect=new hBondEffect();		
		H1CS h1CS=new H1CS();
		Pdb pp=new Pdb();
		chemicalShiftDatabase csd=new chemicalShiftDatabase();
		
		String userDir = System.getProperty("user.dir");
		
		//read random coil chemical shift table:
	    String pathRandomCoilShift=userDir+"/system/cs_database/random_coil_shifts.txt"; //random coil shift table
		randomCoilShift rcs=new randomCoilShift();
		Vector<randomCoilShift> vecRandomShifts=rcs.randomCoilShiftReader(pathRandomCoilShift);
		
		//read all chemical shift files
    	int counter=1;
    	File myDir = new File(pathCSDatabase);
    	String[] contents = myDir.list();//select all structures
    	if (contents == null) 
    		System.out.println(myDir.getName() + " is not a directory");
    	for (int t=0; t<contents.length;t++)  {  
    	
    		String filename = contents[t]; 
    		String fileNameSaved=filename.substring(0,filename.lastIndexOf("."));
    		
	    	String strCSFile="";
	    	strCSFile=pathCSDatabase+filename;	    
	    	filename=filename.substring(0,filename.lastIndexOf("."))+".pdb";
	    	String strPdbFile=pathPdbDatabase+filename;	    	
	    	
	    	//read the training chemical shift data
	    	Vector<H1CS> vecCSTemp= new Vector<H1CS>();
	    	
	    	if(isTraining)
	    		vecCSTemp=h1CS.h1CSReader_TALOS(strCSFile); ///for training	 
	    	else
	    		vecCSTemp=h1CS.h1CSReader_BMRB(strCSFile); 	 //read the testing data
	    	    	
	    	Vector<Pdb> vecPdb = pp.readPdb(strPdbFile);
	    	
	    	boolean isBMRBNEWNameScheme=false;;// whether using the BMRB_NEW naming scheme for atom names.
	    	for(int x=0;x<vecPdb.size();x++){
	    		Pdb pdb=vecPdb.elementAt(x);
	    		String res=pdb.getResidue();
	    		Vector<Cartesian> vecAtom =pdb.getAtomVec();
	    		for(int y=0;y<vecAtom.size();y++){
	    			Cartesian cc=vecAtom.elementAt(y);
	    			String atom=cc.getAtom();
	    			if( !( res.equalsIgnoreCase("ALA")||res.equalsIgnoreCase("GLY")|| res.equalsIgnoreCase("ILE")
	    					||res.equalsIgnoreCase("PRO")||res.equalsIgnoreCase("THR")||res.equalsIgnoreCase("VAL")) ){
	    				if(atom.equalsIgnoreCase("HB3"))
	    					isBMRBNEWNameScheme=true;
	    				break;
	    			}	    			
	    		}//for(int y=0;y<vecAtom.size();y++)
	    		if(isBMRBNEWNameScheme)
	    			break;
	    	}//for(int x=0;x<vecPdb.size();x++)
	    	if(isBMRBNEWNameScheme){
	    		Vector<Pdb>  vecPdbTemp=pp.nameConvert2PDBNew(vecPdb);
	    		vecPdb=new Vector<Pdb>();
	    		vecPdb.addAll(vecPdbTemp);
	    	}
	    	
	    	//update residue numbers for some PDB (with negative residue numbers)
	    	for(int g=0;g<vecPdb.size();g++){
	    		Pdb pdbTemp=vecPdb.elementAt(g);
	    		int resNoTemp=pdbTemp.getResidueNo();
	    		if(resNoTemp<0){
	    			///pdbTemp.setResidueNo(-resNoTemp);
	    			vecPdb.remove(g);
	    			g--;
	    		}	    			
	    	}
	    	
	    	//compute the phi/psi angles
	    	Vector<PhiPsi> vecPhiPsi=pp.PhiPsiTotalPdb(vecPdb);
	    	
	    	   
	    	   	
	    	//find max res no and min res no	    		    	
	    	int minNoFromCS=9999999;
	    	int maxNoFromCS=-99999;
	    	for(int i=0;i<vecCSTemp.size();i++){
	    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
	    		int resNo=h1cs.getResidueNo();
	    		if(resNo>maxNoFromCS)
	    			maxNoFromCS=resNo;  
	    		if(resNo<minNoFromCS)
	    			minNoFromCS=resNo;
	    	}
	    	int minNoFromPdb=9999999;
	    	int maxNoFromPdb=-99999;
	    	for(int i=0;i<vecPdb.size();i++){
	    		Pdb pdb=(Pdb)vecPdb.elementAt(i);
	    		int resNo=pdb.getResidueNo();
	    		if(resNo>maxNoFromPdb)
	    			maxNoFromPdb=resNo;  
	    		if(resNo<minNoFromPdb)
	    			minNoFromPdb=resNo;
	    	}
	    	int maxNo=Math.min(maxNoFromCS, maxNoFromPdb);
	    	int minNo=Math.max(minNoFromCS, minNoFromPdb);
	    	
	    	//for computing the chemical shift delta from hbond effect:
	    	double[] haCSDelta=new double[maxNo+1];
	    	double[] hnCSDelta=new double[maxNo+1];
	    	for(int i=0;i<=maxNo;i++){
	    		haCSDelta[i]=0.0;
	    		hnCSDelta[i]=0.0;
	    	}
	    	Vector<residueCS> vecOneProteinData=new Vector<residueCS>();
	    	
	    	//we skip the first residue and the last residue
	    	for(int no=minNo+1;no<=maxNo-1;no++){     	
	    		
	    		int ind= Collections.binarySearch(vecPdb, new Pdb(no), new Pdb.PdbComparator());
	     	    if(ind<0)
	     	    	continue;
	     	    Pdb curPdb=vecPdb.elementAt(ind);
	     	    String curRes=curPdb.getResidue();
	     	    
	     	    int ind2= Collections.binarySearch(vecPhiPsi, new PhiPsi(no), new PhiPsi.PPComparator());
	     	    if(ind2<0)
	     	    	continue;
	     	   
	     	    PhiPsi phipsi=vecPhiPsi.elementAt(ind2);
	     	 
	     	    int indPrev= Collections.binarySearch(vecPhiPsi, new PhiPsi(no-1), new PhiPsi.PPComparator());
	     	    double phiPrev=0.0;
	     	    double psiPrev=0.0;
	     	    if( indPrev>-1){
	    		   PhiPsi phipsiPre=vecPhiPsi.elementAt(indPrev);
	    		   phiPrev=chemicalShiftDatabase.convertAngleInPI( phipsiPre.getPhi() );
	    		   psiPrev=chemicalShiftDatabase.convertAngleInPI( phipsiPre.getPsi() );
	     	    }
	    	   
	     	    int indNext= Collections.binarySearch(vecPhiPsi, new PhiPsi(no+1), new PhiPsi.PPComparator());
	     	    double phiNext=0.0;
	     	    double psiNext=0.0;
	     	    if( indNext>-1){
	    		   PhiPsi phipsiNext=vecPhiPsi.elementAt(indNext);
	    		   phiNext=chemicalShiftDatabase.convertAngleInPI( phipsiNext.getPhi() );
	    		   psiNext=chemicalShiftDatabase.convertAngleInPI( phipsiNext.getPsi() );
	     	    }
	     	   
	     	    //check the residue name of the neighboring residues
	     	    String prevResType="";
	     	    ind=Collections.binarySearch(vecPdb, new Pdb(no-1), new Pdb.PdbComparator());
	     	    if(ind>-1){	     	    	
		     	   Pdb prevPdb=vecPdb.elementAt(ind);
		     	   prevResType=prevPdb.getResidue();
	     	    }   
	      	  
	     	   ////////////////////////////////
	     	   //compute the chemical shift delta from hbond and ring.
	     	   float deltaHBond_ha=(float)hbond_effect.computeDeltaFromHBondForHaOld(no, vecPdb);
	     	   float deltaHBond_hn=(float)hbond_effect.computeDeltaFromHBondForHnOld(no, vecPdb);	     	   
	     	   float deltaRing=(float)ring_effect.computeDeltaChemShiftFromRingEffectForHa(no, vecPdb)*1000;	     	
	     	   float deltaRingCa=(float)ring_effect.computeDeltaChemShiftFromRingEffectForCa(no, vecPdb)*1000;	    	 		
	    		
	     	   //find the bb chemical shift of current residue
	    		double csN=-9999.0;
	    		double csCA=-9999.0;
	    		double csHN=-9999.0;
	    		double csHA=-9999.0;
	    		double csCB=-9999.0;
	    		double csCO=-9999.0;
	    		
	    		//find cs of N
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && atom.equalsIgnoreCase("N")){
		    			csN=h1cs.getH1CS();
		    			break;
		    		}
	    		}  		
	    		
	    		//find cs of CA
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && atom.equalsIgnoreCase("CA")){
		    			csCA=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	    	
	    		//find cs of HN
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && (atom.equalsIgnoreCase("HN")|| atom.equalsIgnoreCase("H"))  ){
		    			csHN=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	    		
	    		//find cs of HA
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();		    		
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && atom.equalsIgnoreCase("HA")){
		    			csHA=h1cs.getH1CS();
		    			break;
		    		}	
		    	}//for(int i=0;i<vecCSTemp.size();i++)
	    		
	    		//for GLY, we compute the mean of two HA chemical shifts
	    		if(curRes.equalsIgnoreCase("GLY") || curRes.equalsIgnoreCase("G")){
	    			double sumHaCS=0.0;	  
	    			int counterHaCS=0;
	    			boolean isFoundHaForGly=false;	    			
	    			for(int i=0;i<vecCSTemp.size();i++){
			    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
			    		int resNo=h1cs.getResidueNo();		    		
			    		String atom=h1cs.getAtomName();
			    		if(atom.length()>=2)
			    			atom=atom.substring(0,2);
			    		if(resNo==no && atom.equalsIgnoreCase("HA")){
			    			sumHaCS=sumHaCS+h1cs.getH1CS();
			    			counterHaCS++;
			    			isFoundHaForGly=true;
			    		}	
			    	}//for(int i=0;i<vecCSTemp.size();i++)
	    			
	    			if(isFoundHaForGly && counterHaCS==2){
	    				csHA=sumHaCS/counterHaCS;
	    			}
	    			else
	    				csHA=-9999.0;	    			
	    		}//if(curRes.equalsIgnoreCase("GLY") || curRes.equalsIgnoreCase("G"))
	    		
	    		//find cs of CB
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && atom.equalsIgnoreCase("CB")){
		    			csCB=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	    		
	    		//find cs of CO
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && (atom.equalsIgnoreCase("CO") ||atom.equalsIgnoreCase("C"))){
		    			csCO=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}    		
	  
	    		double csRandCoilHa=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"HA",prevResType);
	    		double csRandCoilCa=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"CA",prevResType);
	    		double csRandCoilCb=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"CB",prevResType);
	    		double csRandCoilCo=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"C",prevResType);
	    		double csRandCoilHn=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"HN",prevResType);
	    		double csRandCoilN=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"N",prevResType);
	    		
	    		csHA=csHA-csRandCoilHa;
	    		csCA=csCA-csRandCoilCa;
	    		csCB=csCB-csRandCoilCb;
	    		csCO=csCO-csRandCoilCo;
	    		csHN=csHN-csRandCoilHn;
	    		csN=csN-csRandCoilN;      		
	    		   		
	    		double phi=chemicalShiftDatabase.convertAngleInPI( phipsi.getPhi());//1.5*Math.cos(3*(phipsi.getPhi() + 2.45*Math.PI/180 ))-3.0*Math.cos(phipsi.getPhi()-2.7*Math.PI/180 );
	    		double psi=chemicalShiftDatabase.convertAngleInPI( phipsi.getPsi() );//-0.4*Math.cos(3* ( phipsi.getPsi() +4.8*Math.PI/180 ) )-1.5*Math.cos(phipsi.getPsi() -0.84*Math.PI/180 )+0.7;
	    		
	    		residueCS resCS=new residueCS(no,curRes,fileNameSaved,(float)csHA,(float)csCA,(float)csCB,(float)csCO,(float)csHN,(float)csN,
	    				(float)deltaHBond_ha,(float)deltaHBond_hn,(float)deltaRing,(float)deltaRingCa,
	    				(float)phi,(float)psi,(float)phiPrev,(float)psiPrev,(float)phiNext,(float)psiNext);
	    		
	    		vecOneProteinData.add(resCS);
	    		///////////////////	    		
	    		//String str_csHA=csToString((float)(csHA));String str_csCA=csToString((float)(csCA));String str_csCB=csToString((float)(csCB));
	    		//String str_csCO=csToString((float)(csCO));String str_csHN=csToString((float)(csHN));String str_csN=csToString((float)(csN));	    						    		
	    		counter++;	    		
	    	}//for(int i=1;i<=maxNo;i++)
	    	
	    	Collections.sort(vecOneProteinData, new residueCS.residueCSComparator());//sort the data points based on residue number
	    	vecData.add(vecOneProteinData);	    	
    	}//for (int t=0; t<contents.length;t++) 
    	
    	return vecData;
		
	}
}
