package chemShiftPredict;
/*
HASH Ha Chemical Shift Prediction  Software Version 1.0
Copyright (C) 2009-2012 Bruce Donald Lab, Duke University

HASH is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free
Software Foundation, either version 3 of the License, or (at your option) any
later version.

HASH is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
details.

You should have received a copy of the GNU Lesser General Public License
along with this library; if not, see:
    <http://www.gnu.org/licenses/>.

There are additional restrictions imposed on the use and distribution of this
open-source code, including: (A) this header must be included in any
modification or extension of the code; (B) you are required to cite our
papers in any publications that use this code. The citation for the various
different modules of our software, together with a complete list of
requirements and restrictions are found in the document license.pdf enclosed
with this distribution.

Contact Info:
    Bruce R. Donald
    Duke University
    Department of Computer Science
    Levine Science Research Center (LSRC)
    Durham, NC 27708-0129
    USA
    email: www.cs.duke.edu/brd/

<signature of Bruce Donald>, 01 May, 2011
Bruce R. Donald, Professor of Computer Science and Biochemistry
*/

import java.util.Collections;
import java.util.Comparator;
import java.util.Vector;

import rdcPanda.Cartesian;
import rdcPanda.H1CS;
import rdcPanda.Pdb;
import rdcPanda.Peak;
import rdcPanda.RotaPattern;
import rdcPanda.Pdb.PdbComparator;

public class hBondEffect {
	public int donorResNo=-1;//residue number of donor
	public int acceptorResNo=-1;//residue number of acceptor
	public  Cartesian donorCC=null;//donor atom coordinates.
	public Cartesian acceptorCC=null;//acceptor atom coordinates.
	public String donorAtomName="";
	public String acceptorAtomName="";
	public double distance=0;//distance of hbond.
	
	public hBondEffect(int donorNo,int acceptNo,Cartesian donor_CC, Cartesian acceptor_CC,String atomDonor, String atomAccept,double dist){
		donorResNo=donorNo;
		acceptorResNo=acceptNo;
		donorCC=donor_CC;
		acceptorCC=acceptor_CC;
		donorAtomName=atomDonor;
		acceptorAtomName=atomAccept;
		distance=dist;
	};
	
	public hBondEffect(){};
	
	  /**
     * The Class scoreComparator.
     */
    public static class distComparator implements Comparator<Object>{
    	
	    /* (non-Javadoc)
	     * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
	     */
	    public int compare(Object o1, Object o2){
	    	hBondEffect n1 = (hBondEffect)o1;
	    	hBondEffect n2 = (hBondEffect)o2;
    	    double d1 = n1.distance;
    	    double d2 = n2.distance;
    	    if (d1 <d2)
    		return -1;
    	    else if (d1 >d2)
    		return 1;
    	    else return 0;
    	}
        }
    
	//tyep="GAMMA1","GAMMA2" and "DIST";parameters are from CAMSHIFT paper.
	public double computeHbondEnergy(String res,String type,double x){
		double p1=0.0,p2=0.0,p3=0.0,p4=0.0,p5=0.0,r=0.0,s=0.0;
		if(type.equalsIgnoreCase("GAMMA1")){
			p1=15.0486;
			p2=-1.79469;
			p3=-1.79469;
			p4=1.49398;
			p5=1.49398;
			r=15.5;
			s=2.5;
		}
		else if(type.equalsIgnoreCase("GAMMA2")){
			p1=12.1514;
			p2=-1.12258;
			p3=-1.12258;
			p4=1.30953;
			p5=1.30953;
			r=15.5;
			s=3.0;
		}
		else if(type.equalsIgnoreCase("DIST")){
			p1=-0.403531;
			p2=13.9148;
			p3=-0.421725;
			p4=-3.08016;
			p5=0.987251;
			r=3.0;
			s=-16.5;
		}
		else
			System.out.println("Unrecognized type...");
		
		double energy=p1*(Math.pow( ( (p2/x) + p4) , r) - Math.pow( ( (p3/x)+p5) , s)   );
		return energy;
	}
	public String getAtomNameBaseforO(String res, String atomO){
		String atomBase="";
		if(atomO.equalsIgnoreCase("O")){
			atomBase="C";
			return atomBase;
		}
		else if( (res.equalsIgnoreCase("ASP") || res.equalsIgnoreCase("ASN") )&& atomO.substring(0,2).equalsIgnoreCase("OD"))
			atomBase="CG";
		else if( (res.equalsIgnoreCase("GLU") || res.equalsIgnoreCase("GLN") ) && atomO.substring(0,2).equalsIgnoreCase("OE"))
			atomBase="CD";
		else if( (res.equalsIgnoreCase("SER") || res.equalsIgnoreCase("THR") ) && atomO.substring(0,2).equalsIgnoreCase("OG"))
			atomBase="CB";
		else if( (res.equalsIgnoreCase("TYR")) && atomO.substring(0,2).equalsIgnoreCase("OH"))
			atomBase="CZ";
		return atomBase;
	}
	
	  /**
    compute the  h-bond energy for a ha atom, using the energy format (as in CAMSHIFT method)
	 
     * @param resNo_ha current residue ID of the Ha atom
     * @param deltaShift ha chemical shift delta.
     * @return boolean true if h-boneded, false otherwise.
     */
	public double[] computeHBondEnergyForHa(int resNo_ha,Vector<Pdb> vecPdb){
		Pdb pp=new Pdb();		
		double [] coordA = {0.0, 0.0, 0.0};
    	double [] coordB = {0.0, 0.0, 0.0};	
    	double distCutOff=3.0;
    	double[] energy=new double[3];
    	energy[0]=0.0; 
    	energy[1]=0.0;
    	energy[2]=0.0;
		int Index= Collections.binarySearch(vecPdb, new Pdb(resNo_ha), new Pdb.PdbComparator());
 	    if(Index<0)
 	    {
 	    	//System.out.println("We couldnot find the residue number...");
 	    	return energy;
 	    	//System.exit(0); 	    	
 	    }
 	    Pdb curPdb=vecPdb.elementAt(Index);
 	    int curNo=curPdb.getResidueNo();
 	    String curRes=curPdb.getResidue();
 	    Vector<Cartesian> atomVec=curPdb.getAtomVec();
 	    for (int j=0; j<atomVec.size(); j++)
	   	{ 
 	    	Cartesian cc = (Cartesian)atomVec.elementAt(j);
	   	    String atom = cc.getAtom();
	   	    if (atom.equals("HA"))
	   	    	coordA = cc.getXYZ();	//for translation	    
	   	    
	   	} //for (int j=0; j<atomVec.size(); j++)	
 	    double [] coordABase = {0.0, 0.0, 0.0};
 	    for (int j=0; j<atomVec.size(); j++)
	    { 
	    	Cartesian cc = (Cartesian)atomVec.elementAt(j);
	   	    String atom = cc.getAtom();
	   	    if (atom.equals("CA"))
	   	    	coordABase = cc.getXYZ();	//for translation	    
	   	    
	    } //for (int j=0; j<atomVec.size(); j++)
 	    if(curRes.equalsIgnoreCase("GLY")){
  	    	double [] coordA1 = {0.0, 0.0, 0.0};
  	    	double [] coordA2 = {0.0, 0.0, 0.0};
  	    	for (int j=0; j<atomVec.size(); j++)
  		   	{ 
  	 	    	Cartesian cc = (Cartesian)atomVec.elementAt(j);
  		   	    String atom = cc.getAtom();
  		   	    if (atom.equals("HA1"))
  		   	    	coordA1 = cc.getXYZ();	
  		   	    if (atom.equals("HA2"))
		   	    	coordA2 = cc.getXYZ();	 		   	    
  		   	} //for (int j=0; j<atomVec.size(); j++)
  	    	coordA[0]=(coordA1[0]+coordA2[0])/2;
  	    	coordA[1]=(coordA1[1]+coordA2[1])/2;
  	    	coordA[2]=(coordA1[2]+coordA2[2])/2;
  	     }//if(curRes.equalsIgnoreCase("GLY")) 
 	 
 	     boolean isHbonded=false;
 	     double minDist=99999.9;
	     double gamma1_min=9999.9;
	     double gamma2_min=9999.9;
    	 for (int j=0;j<vecPdb.size();j++){
    		Pdb pdbB = (Pdb)vecPdb.elementAt(j);
    		String resB=pdbB.getResidue();
    		Vector<Cartesian> atomVecB=pdbB.getAtomVec();
    		int resNoB=pdbB.getResidueNo();
    		if(curNo==resNoB)
    			continue;
    		for(int k=0;k<atomVecB.size();k++){
    			Cartesian ccB=atomVecB.elementAt(k);
    			String atomB=ccB.getAtom();
    			if(!atomB.substring(0,1).equalsIgnoreCase("O"))
    				continue;
    			coordB=ccB.getXYZ();
    			String atomNamebase=getAtomNameBaseforO(resB,atomB);
    			double [] coordB_base = {0.0, 0.0, 0.0};	
    			for(int t=0;t<atomVecB.size();t++){
    				Cartesian ccBBase=atomVecB.elementAt(t);
        			String atomBBase=ccB.getAtom();
        			if(atomBBase.equalsIgnoreCase(atomNamebase)){
        				coordB_base=ccBBase.getXYZ();
        				break;
        			}
    			}//for(int t=0;t<atomVecB.size();t++)		
    			
    			double distance= Math.sqrt((coordA[0] - coordB[0]) * (coordA[0] - coordB[0]) 
						+ (coordA[1] - coordB[1]) * (coordA[1] - coordB[1]) 
						+ (coordA[2] - coordB[2]) * (coordA[2] - coordB[2]));
    			double[] hbondVec=new double[3];
    			double[] donatorToHVec=new double[3];
    			double[] oToBaseVec=new double[3];
    			hbondVec=pp.internuclearVec(coordA,coordB);
    			donatorToHVec=pp.internuclearVec(coordA,coordABase);
    			oToBaseVec=pp.internuclearVec(coordB,coordB_base);
    			double gamma1=pp.interAngle(donatorToHVec, hbondVec)*(180/Math.PI);
    			hbondVec=pp.internuclearVec(coordB,coordA);
    			double gamma2=pp.interAngle(hbondVec,oToBaseVec)*(180/Math.PI);
    			if( (distance<=distCutOff) && (gamma1>=90.0)  && (gamma2>=90.0) ){   
    				isHbonded=true;
    				if(distance<minDist){
	    				minDist=distance;
	    				gamma1_min=gamma1;
	    				gamma2_min=gamma2;
	    			}
	    			
    			}//if(distance<=distCutOff)
    		}//for(int k=0;k<atomVecB.size();k++)
    		
    	}//for (int j=0;j<vecPdb.size();j++)    	
    	
    	if(isHbonded){
    		energy[0]=computeHbondEnergy("VAL","GAMMA1",gamma1_min);
			energy[1]=computeHbondEnergy("VAL","GAMMA2",gamma2_min);
			energy[2]=computeHbondEnergy("VAL","DIST",minDist);
    	}
    	return energy;    	
	}
	
	  /**
    compute the  h-bond energy for a Hn atom, using the energy format (as in CAMSHIFT method)
	 
     * @param resNo_ha current residue ID of the Ha atom
     * @param deltaShift ha chemical shift delta.
     * @return boolean true if h-boneded, false otherwise.
     */
	public double[] computeHBondEnergyForHn(int resNo_ha,Vector<Pdb> vecPdb){
		Pdb pp=new Pdb();		
		double [] coordA = {0.0, 0.0, 0.0};
    	double [] coordB = {0.0, 0.0, 0.0};	
    	double distCutOff=3.0;
    	double[] energy=new double[3];
    	energy[0]=0.0; 
    	energy[1]=0.0;
    	energy[2]=0.0;
		int Index= Collections.binarySearch(vecPdb, new Pdb(resNo_ha), new Pdb.PdbComparator());
 	    if(Index<0)
 	    {
 	    	//System.out.println("We couldnot find the residue number...");
 	    	return energy;
 	    	//System.exit(0); 	    	
 	    }
 	    Pdb curPdb=vecPdb.elementAt(Index);
 	    int curNo=curPdb.getResidueNo();
 	    String curRes=curPdb.getResidue();
 	    if(curRes.equalsIgnoreCase("PRO"))
 	    	return energy;
 	    
 	    Vector<Cartesian> atomVec=curPdb.getAtomVec();
 	    for (int j=0; j<atomVec.size(); j++)
	   	{ 
 	    	Cartesian cc = (Cartesian)atomVec.elementAt(j);
	   	    String atom = cc.getAtom();
	   	    if (atom.equalsIgnoreCase("HN") || atom.equalsIgnoreCase("H"))
	   	    	coordA = cc.getXYZ();	//for translation	    
	   	    
	   	} //for (int j=0; j<atomVec.size(); j++)	
 	    double [] coordABase = {0.0, 0.0, 0.0};
 	    for (int j=0; j<atomVec.size(); j++)
	    { 
	    	Cartesian cc = (Cartesian)atomVec.elementAt(j);
	   	    String atom = cc.getAtom();
	   	    if (atom.equals("N"))
	   	    	coordABase = cc.getXYZ();		   	    
	    } //for (int j=0; j<atomVec.size(); j++)	   
 	 
 	     boolean isHbonded=false;
 	     double minDist=99999.9;
	     double gamma1_min=9999.9;
	     double gamma2_min=9999.9;
    	 for (int j=0;j<vecPdb.size();j++){
    		Pdb pdbB = (Pdb)vecPdb.elementAt(j);
    		String resB=pdbB.getResidue();
    		Vector<Cartesian> atomVecB=pdbB.getAtomVec();
    		int resNoB=pdbB.getResidueNo();
    		if(curNo==resNoB)
    			continue;
    		for(int k=0;k<atomVecB.size();k++){
    			Cartesian ccB=atomVecB.elementAt(k);
    			String atomB=ccB.getAtom();
    			if(!atomB.substring(0,1).equalsIgnoreCase("O"))
    				continue;
    			coordB=ccB.getXYZ();
    			String atomNamebase=getAtomNameBaseforO(resB,atomB);
    			double [] coordB_base = {0.0, 0.0, 0.0};	
    			for(int t=0;t<atomVecB.size();t++){
    				Cartesian ccBBase=atomVecB.elementAt(t);
        			String atomBBase=ccB.getAtom();
        			if(atomBBase.equalsIgnoreCase(atomNamebase)){
        				coordB_base=ccBBase.getXYZ();
        				break;
        			}
    			}//for(int t=0;t<atomVecB.size();t++)		
    			
    			double distance= Math.sqrt((coordA[0] - coordB[0]) * (coordA[0] - coordB[0]) 
						+ (coordA[1] - coordB[1]) * (coordA[1] - coordB[1]) 
						+ (coordA[2] - coordB[2]) * (coordA[2] - coordB[2]));
    			double[] hbondVec=new double[3];
    			double[] donatorToHVec=new double[3];
    			double[] oToBaseVec=new double[3];
    			hbondVec=pp.internuclearVec(coordA,coordB);
    			donatorToHVec=pp.internuclearVec(coordA,coordABase);
    			oToBaseVec=pp.internuclearVec(coordB,coordB_base);
    			double gamma1=pp.interAngle(donatorToHVec, hbondVec)*(180/Math.PI);
    			hbondVec=pp.internuclearVec(coordB,coordA);
    			double gamma2=pp.interAngle(hbondVec,oToBaseVec)*(180/Math.PI);
    			if( (distance<=distCutOff) && (gamma1>=90.0)  && (gamma2>=90.0) ){   
    				isHbonded=true;
    				if(distance<minDist){
	    				minDist=distance;
	    				gamma1_min=gamma1;
	    				gamma2_min=gamma2;
	    			}
	    			
    			}//if(distance<=distCutOff)
    		}//for(int k=0;k<atomVecB.size();k++)
    		
    	}//for (int j=0;j<vecPdb.size();j++)    	
    	
    	if(isHbonded){
    		energy[0]=computeHbondEnergy("VAL","GAMMA1",gamma1_min);
			energy[1]=computeHbondEnergy("VAL","GAMMA2",gamma2_min);
			energy[2]=computeHbondEnergy("VAL","DIST",minDist);
    	}
    	return energy;    	
	}
	  /**
    compute the  h-bond energy for O atom, using the energy format (as in CAMSHIFT method)
	 
     * @param resNo_ha current residue ID of the Ha atom
     * @param deltaShift ha chemical shift delta.
     * @return boolean true if h-boneded, false otherwise.
     */
	public double[] computeHBondEnergyForO(int resNo_ha,Vector<Pdb> vecPdb){
		Pdb pp=new Pdb();		
		double [] coordA = {0.0, 0.0, 0.0};
    	double [] coordB = {0.0, 0.0, 0.0};	
    	double distCutOff=3.0;
    	double[] energy=new double[3];
    	energy[0]=0.0; 
    	energy[1]=0.0;
    	energy[2]=0.0;
		int Index= Collections.binarySearch(vecPdb, new Pdb(resNo_ha), new Pdb.PdbComparator());
 	    if(Index<0)
 	    {
 	    	//System.out.println("We couldnot find the residue number...");
 	    	return energy;
 	    	//System.exit(0); 	    	
 	    }
 	    Pdb curPdb=vecPdb.elementAt(Index);
 	    int curNo=curPdb.getResidueNo();
 	    String curRes=curPdb.getResidue();
 	     	    
 	    Vector<Cartesian> atomVec=curPdb.getAtomVec();
 	    for (int j=0; j<atomVec.size(); j++)
	   	{ 
 	    	Cartesian cc = (Cartesian)atomVec.elementAt(j);
	   	    String atom = cc.getAtom();
	   	    if (atom.equalsIgnoreCase("O"))
	   	    	coordA = cc.getXYZ();	//for translation	    
	   	    
	   	} //for (int j=0; j<atomVec.size(); j++)	
 	    double [] coordABase = {0.0, 0.0, 0.0};
 	    for (int j=0; j<atomVec.size(); j++)
	    { 
	    	Cartesian cc = (Cartesian)atomVec.elementAt(j);
	   	    String atom = cc.getAtom();
	   	    if (atom.equalsIgnoreCase("C") || atom.equalsIgnoreCase("CO"))
	   	    	coordABase = cc.getXYZ();		   	    
	    } //for (int j=0; j<atomVec.size(); j++)	   
 	 
 	     boolean isHbonded=false;
 	     double minDist=99999.9;
	     double gamma1_min=9999.9;
	     double gamma2_min=9999.9;
    	 for (int j=0;j<vecPdb.size();j++){
    		Pdb pdbB = (Pdb)vecPdb.elementAt(j);
    		String resB=pdbB.getResidue();
    		Vector<Cartesian> atomVecB=pdbB.getAtomVec();
    		int resNoB=pdbB.getResidueNo();
    		if(curNo==resNoB)
    			continue;
    		for(int k=0;k<atomVecB.size();k++){
    			Cartesian ccB=atomVecB.elementAt(k);
    			String atomB=ccB.getAtom();
    			if(!atomB.substring(0,1).equalsIgnoreCase("H"))
    				continue;
    			coordB=ccB.getXYZ();
    			Peak pk=new Peak();
    			String atomNamebase=pk.GetHeavyAtomFromProton(resB,atomB);
    			double [] coordB_base = {0.0, 0.0, 0.0};	
    			for(int t=0;t<atomVecB.size();t++){
    				Cartesian ccBBase=atomVecB.elementAt(t);
        			String atomBBase=ccB.getAtom();
        			if(atomBBase.equalsIgnoreCase(atomNamebase)){
        				coordB_base=ccBBase.getXYZ();
        				break;
        			}
    			}//for(int t=0;t<atomVecB.size();t++)		
    			
    			double distance= Math.sqrt((coordA[0] - coordB[0]) * (coordA[0] - coordB[0]) 
						+ (coordA[1] - coordB[1]) * (coordA[1] - coordB[1]) 
						+ (coordA[2] - coordB[2]) * (coordA[2] - coordB[2]));
    			double[] hbondVec=new double[3];
    			double[] donatorToHVec=new double[3];
    			double[] oToBaseVec=new double[3];
    			hbondVec=pp.internuclearVec(coordA,coordB);
    			donatorToHVec=pp.internuclearVec(coordA,coordABase);
    			oToBaseVec=pp.internuclearVec(coordB,coordB_base);
    			double gamma1=pp.interAngle(donatorToHVec, hbondVec)*(180/Math.PI);
    			hbondVec=pp.internuclearVec(coordB,coordA);
    			double gamma2=pp.interAngle(hbondVec,oToBaseVec)*(180/Math.PI);
    			if( (distance<=distCutOff) && (gamma1>=90.0)  && (gamma2>=90.0) ){   
    				isHbonded=true;
    				if(distance<minDist){
	    				minDist=distance;
	    				gamma1_min=gamma1;
	    				gamma2_min=gamma2;
	    			}
	    			
    			}//if(distance<=distCutOff)
    		}//for(int k=0;k<atomVecB.size();k++)
    		
    	}//for (int j=0;j<vecPdb.size();j++)    	
    	
    	if(isHbonded){
    		energy[0]=computeHbondEnergy("VAL","GAMMA1",gamma1_min);
			energy[1]=computeHbondEnergy("VAL","GAMMA2",gamma2_min);
			energy[2]=computeHbondEnergy("VAL","DIST",minDist);
    	}
    	return energy;    	
	}
	 /**
    compute  the chemical shift delta for a residue (using Camshift method)
     * @param resNo_ha current residue ID of the Ha atom
     * @param deltaShift ha chemical shift delta.
     * @return boolean true if h-boneded, false otherwise.
     */
	
	public double computeDeltaFromHBondForHa(int resNo_ha,Vector<Pdb> vecPdb){
		//
		double[] delta=new double[3];
		double sum_delta=0.0;
		
		//compute the hbond effects from the current residue i:
		double c_proton_dist=-0.001389;
		double c_proton_gamma1=0.0001071;
		double c_proton_gamma2=-5.55E-07;
		
		double c_oxygen_dist=1.14E-06;
		double c_oxygen_gamma1=0.0003084;
		double c_oxygen_gamma2=-2.808E-05;
		
		double[] energy=computeHBondEnergyForHa(resNo_ha,vecPdb);
		delta[0]=energy[0]*c_proton_gamma1;
		delta[1]=energy[1]*c_proton_gamma2;
		delta[2]=energy[2]*c_proton_dist;
		sum_delta=sum_delta+delta[0]+delta[1]+delta[2];
		
		/*energy=computeHBondEnergyForHn(resNo_ha,vecPdb);
		delta[0]=energy[0]*c_proton_gamma1;
		delta[1]=energy[1]*c_proton_gamma2;
		delta[2]=energy[2]*c_proton_dist;
		sum_delta=sum_delta+delta[0]+delta[1]+delta[2];
		*/
		energy=computeHBondEnergyForO(resNo_ha,vecPdb);
		delta[0]=energy[0]*c_oxygen_gamma1;
		delta[1]=energy[1]*c_oxygen_gamma2;
		delta[2]=energy[2]*c_oxygen_dist;
		sum_delta=sum_delta+delta[0]+delta[1]+delta[2];
		
		//compute the hbond effects from the previous residue i-1:
		c_oxygen_dist=-2.82E-08;
		c_oxygen_gamma1=-0.0005796;
		c_oxygen_gamma2=6.866E-05;
		
		energy=computeHBondEnergyForO(resNo_ha-1,vecPdb);
		delta[0]=energy[0]*c_oxygen_gamma1;
		delta[1]=energy[1]*c_oxygen_gamma2;
		delta[2]=energy[2]*c_oxygen_dist;
		sum_delta=sum_delta+delta[0]+delta[1]+delta[2];
		
		//compute the hbond effects from the next residue i+1:
		c_proton_dist=-0.0017769;
		c_proton_gamma1=0.0008555;
		c_proton_gamma2=-8.147E-05;
		
		energy=computeHBondEnergyForHa(resNo_ha+1,vecPdb);
		delta[0]=energy[0]*c_proton_gamma1;
		delta[1]=energy[1]*c_proton_gamma2;
		delta[2]=energy[2]*c_proton_dist;
		sum_delta=sum_delta+delta[0]+delta[1]+delta[2];
		
	/*	energy=computeHBondEnergyForHn(resNo_ha+1,vecPdb);
		delta[0]=energy[0]*c_proton_gamma1;
		delta[1]=energy[1]*c_proton_gamma2;
		delta[2]=energy[2]*c_proton_dist;
		sum_delta=sum_delta+delta[0]+delta[1]+delta[2];
		*/
		return sum_delta;
	}
	
	  /**
    create a list of h-bonds  using the SHIFTX method.
	//1. compute the strongest h-bond for the Ha atom

     * @param resNo_ha current residue ID of the Ha atom
     * @param deltaShift ha chemical shift delta.
     * @return boolean true if h-boneded, false otherwise.
     */
	
	public Vector<hBondEffect> createHBondListShiftx(Vector<Pdb> vecPdb){
		Pdb pp=new Pdb();
		Vector<hBondEffect> vechBondList=new Vector<hBondEffect>();
		for(int t=0;t<vecPdb.size();t++){
			Pdb pdb=vecPdb.elementAt(t);
			int resNo=pdb.getResidueNo();
			String curRes=pdb.getResidue();
	 	    Vector<Cartesian> atomVec=pdb.getAtomVec();
	 	    
	 	    //compute the hbond list for Ha atom
	 	    double [] coordA = {0.0, 0.0, 0.0};
	    	double [] coordB = {0.0, 0.0, 0.0};	
	    	double distCutOff=2.77;
	    	
	 	    for (int j=0; j<atomVec.size(); j++)
		   	{ 
	 	    	Cartesian cc = (Cartesian)atomVec.elementAt(j);
		   	    String atom = cc.getAtom();
		   	    if (atom.equalsIgnoreCase("HA"))
		   	    	coordA = cc.getXYZ();			   	    
		   	} //for (int j=0; j<atomVec.size(); j++)	
	 	   
	 	    if(curRes.equalsIgnoreCase("GLY")){
	  	    	double [] coordA1 = {0.0, 0.0, 0.0};
	  	    	double [] coordA2 = {0.0, 0.0, 0.0};
	  	    	for (int j=0; j<atomVec.size(); j++)
	  		   	{ 
	  	 	    	Cartesian cc = (Cartesian)atomVec.elementAt(j);
	  		   	    String atom = cc.getAtom();
	  		   	    if (atom.equalsIgnoreCase("HA1"))
	  		   	    	coordA1 = cc.getXYZ();	
	  		   	    if (atom.equalsIgnoreCase("HA2"))
			   	    	coordA2 = cc.getXYZ();	 		   	    
	  		   	} //for (int j=0; j<atomVec.size(); j++)
	  	    	coordA[0]=(coordA1[0]+coordA2[0])/2;
	  	    	coordA[1]=(coordA1[1]+coordA2[1])/2;
	  	    	coordA[2]=(coordA1[2]+coordA2[2])/2;
	  	    }//if(curRes.equalsIgnoreCase("GLY"))
	 	    Cartesian ccDonor=new Cartesian("HA",coordA);
	 	    
	 	    for (int j=0;j<vecPdb.size();j++){
	    		Pdb pdbB = (Pdb)vecPdb.elementAt(j);
	    		String resB=pdbB.getResidue();
	    		Vector<Cartesian> atomVecB=pdbB.getAtomVec();
	    		int resNoB=pdbB.getResidueNo();
	    		if(resNo==resNoB)
	    			continue;
	    		for(int k=0;k<atomVecB.size();k++){
	    			Cartesian ccB=atomVecB.elementAt(k);
	    			String atomB=ccB.getAtom();
	    			if(!atomB.substring(0,1).equalsIgnoreCase("O"))
	    				continue;
	    			coordB=ccB.getXYZ();    			
	    			
	    			double dist= Math.sqrt((coordA[0] - coordB[0]) * (coordA[0] - coordB[0]) 
							+ (coordA[1] - coordB[1]) * (coordA[1] - coordB[1]) 
							+ (coordA[2] - coordB[2]) * (coordA[2] - coordB[2]));
	    			
	    			if( (dist<=distCutOff)){// && (gamma1>=90.0)  && (gamma2>=90.0) ){   				
	    				vechBondList.add(new hBondEffect(resNo,resNoB,ccDonor,ccB,"HA","O",dist));
	    			}//if(distance<=distCutOff)
	    		}//for(int k=0;k<atomVecB.size();k++)	    		
	    	}//for (int j=0;j<vecPdb.size();j++)    	
	 	    
	 	    //compute the hbond list for HN atom
	 	    if(curRes.equalsIgnoreCase("PRO"))
	 	    	continue;
	 	    distCutOff=3.5;
	 	    for (int j=0; j<atomVec.size(); j++)
		   	{ 
	 	    	Cartesian cc = (Cartesian)atomVec.elementAt(j);
		   	    String atom = cc.getAtom();
		   	    if (atom.equalsIgnoreCase("HN") ||atom.equalsIgnoreCase("H") )
		   	    	coordA = cc.getXYZ();			   	    
		   	} //for (int j=0; j<atomVec.size(); j++)
	 	    
	 	    double [] coordABase = {0.0, 0.0, 0.0};
		    for (int j=0; j<atomVec.size(); j++)
		    { 
		    	Cartesian cc = (Cartesian)atomVec.elementAt(j);
		   	    String atom = cc.getAtom();
		   	    if (atom.equals("N"))
		   	    	coordABase = cc.getXYZ();			   	    
		    } //for (int j=0; j<atomVec.size(); j++)
		    
	 	    Cartesian ccDonorHN=new Cartesian("HN",coordA);
	 	    
	 	    for (int j=0;j<vecPdb.size();j++){
	    		Pdb pdbB = (Pdb)vecPdb.elementAt(j);
	    		String resB=pdbB.getResidue();
	    		Vector<Cartesian> atomVecB=pdbB.getAtomVec();
	    		int resNoB=pdbB.getResidueNo();
	    		if(resNo==resNoB)
	    			continue;
	    		for(int k=0;k<atomVecB.size();k++){
	    			Cartesian ccB=atomVecB.elementAt(k);
	    			String atomB=ccB.getAtom();
	    			if(!atomB.substring(0,1).equalsIgnoreCase("O"))
	    				continue;
	    			coordB=ccB.getXYZ();    			
	    			String atomNamebase=getAtomNameBaseforO(resB,atomB);
	    			double [] coordB_base = {0.0, 0.0, 0.0};	
	    			for(int h=0;h<atomVecB.size();h++){
	    				Cartesian ccBBase=atomVecB.elementAt(h);
	        			String atomBBase=ccB.getAtom();
	        			if(atomBBase.equalsIgnoreCase(atomNamebase)){
	        				coordB_base=ccBBase.getXYZ();
	        				break;
	        			}
	    			}//for(int t=0;t<atomVecB.size();t++)	
	    			
	    			double dist= Math.sqrt((coordA[0] - coordB[0]) * (coordA[0] - coordB[0]) 
							+ (coordA[1] - coordB[1]) * (coordA[1] - coordB[1]) 
							+ (coordA[2] - coordB[2]) * (coordA[2] - coordB[2]));
	    			
	    			double[] hbondVec=new double[3];
	    			double[] donatorToHVec=new double[3];
	    			double[] oToBaseVec=new double[3];
	    			hbondVec=pp.internuclearVec(coordA,coordB);
	    			donatorToHVec=pp.internuclearVec(coordA,coordABase);
	    			oToBaseVec=pp.internuclearVec(coordB,coordB_base);
	    			double gamma1=pp.interAngle(donatorToHVec, hbondVec)*(180/Math.PI);
	    			hbondVec=pp.internuclearVec(coordB,coordA);
	    			double gamma2=pp.interAngle(hbondVec,oToBaseVec)*(180/Math.PI);			
	    			
	    			if( (dist<=distCutOff) && (gamma1>=90.0)  && (gamma2>=90.0) ){   				
	    				vechBondList.add(new hBondEffect(resNo,resNoB,ccDonorHN,ccB,"HN","O",dist));
	    			}//if(distance<=distCutOff)
	    		}//for(int k=0;k<atomVecB.size();k++)	    		
	    	}//for (int j=0;j<vecPdb.size();j++)    	
	 	    
	 	    
		}//for(int t=0;t<vecPdb.size();t++)
		
		return vechBondList;
	}
	  /**
    compute the O atom in the h-bond for a ha atom (SHIFTX method)
	//1. compute the strongest h-bond for the Ha atom
	//2. check whether this h-bond is also the strongest for the O atom (need to check and implement...)
	//3. Others: use distance bound 2.77A. Donor and receptor must be in different residues.
     * 
     * @param vecPdb pdb of the whole protein.
     * @param 
     * @return haCSDelta,hnCSDelta for storing the delta chemical shifts for ha and hn in the pdb.
     */
	public void computeAllDeltaFromHBondShiftx(Vector<Pdb> vecPdb, double[] haCSDelta,double[] hnCSDelta){
		//initialization		
    /*	int maxNo=-999;//find max res no
    	for(int i=0;i<vecPdb.size();i++){
    		Pdb pdb=(Pdb)vecPdb.elementAt(i);
    		int resNo=pdb.getResidueNo();
    		if(resNo>maxNo)
    			maxNo=resNo;    		
    	}
    	haCSDelta=new double[maxNo+1];
    	hnCSDelta=new double[maxNo+1];
    	for(int i=0;i<=maxNo;i++){
    		haCSDelta[i]=0.0;
    		hnCSDelta[i]=0.0;
    	}*/
    	
    	//create the hbond candidate list:
		Vector<hBondEffect> vecHBond=createHBondListShiftx(vecPdb);
		if(vecHBond.size()<1)
			return;
		Collections.sort(vecHBond,new distComparator() );
		
		
		while(vecHBond.size()>0){
			hBondEffect hbond=vecHBond.elementAt(0);
			double r=hbond.distance;
			int resNo_donor=hbond.donorResNo;
			int resNo_acceptor=hbond.acceptorResNo;
			String atom_donor=hbond.donorAtomName;
			String atom_acceptor=hbond.acceptorAtomName;
			
			if(atom_donor.equalsIgnoreCase("HA")){
				/*	if( minDist > 2.60756 )
	   		  	{
	   			  	r = 2.60756;
	   		  	}
	   		    if( minDist < 2.26808 )
	   		    {
	   			   r = 2.26808;      
	   		    }*/

	    		double shift_delta = 0.147521/r - (1.65458E-05)/Math.pow(r, 1.5) - 0.000134668/(r*r) + 
				 0.0598561/Math.pow(r, 2.5) + 15.6855/(r*r*r) - 0.673905; //from SHIFTX source code.
	    		haCSDelta[resNo_donor]=shift_delta;
			}
			else
			{
				double shift_delta =9.7464/(r * r * r) - 0.9887;
				hnCSDelta[resNo_donor]=shift_delta;
			}
			
		    //remove other hbond candidates involving the donor and acceptor 
			for(int j=0;j<vecHBond.size();j++){
				hBondEffect hbondTemp=vecHBond.elementAt(j);
				int resNo_donorTemp=hbondTemp.donorResNo;
				int resNo_acceptTemp=hbondTemp.acceptorResNo;
				String atom_donorTemp=hbondTemp.donorAtomName;
				String atom_acceptTemp=hbondTemp.acceptorAtomName;
				
				boolean isInvoling=false;
				if(resNo_donorTemp==resNo_donor && atom_donorTemp.equalsIgnoreCase(atom_donor))
					isInvoling=true;
				if(resNo_acceptTemp==resNo_acceptor && atom_acceptTemp.equalsIgnoreCase(atom_acceptor))
					isInvoling=true;
				if(isInvoling){
					vecHBond.remove(j);
					j--;
				}
				
			}//for(int j=0;j<vecHBond.size();j++){
			
		}//while(curInd<vecHBond.size())
	}
	
   /**
    compute the O atom in the h-bond for a ha atom (SHIFTX method, but only compute the strongest one)
	//1. compute the strongest h-bond for the Ha atom
	//2. check whether this h-bond is also the strongest for the O atom (need to check and implement...)
	//3. Others: use distance bound 2.77A. Donor and receptor must be in different residues.
     * 
     * @param resNo_ha current residue ID of the Ha atom
     * @param deltaShift ha chemical shift delta.
     * @return boolean true if h-boneded, false otherwise.
     */
	
	public double computeDeltaFromHBondForHaOld(int resNo_ha,Vector<Pdb> vecPdb){
		double [] coordA = {0.0, 0.0, 0.0};
    	double [] coordB = {0.0, 0.0, 0.0};	
    	double distCutOff=2.77;
    	Pdb pp=new Pdb();
    	
		int Index= Collections.binarySearch(vecPdb, new Pdb(resNo_ha), new Pdb.PdbComparator());
 	    if(Index<0)
 	    {
 	    	System.out.println("We couldnot find the residue number...");
 	    	System.exit(0); 	    	
 	    }
 	    Pdb curPdb=vecPdb.elementAt(Index);
 	    int curNo=curPdb.getResidueNo();
 	    String curRes=curPdb.getResidue();
 	    Vector<Cartesian> atomVec=curPdb.getAtomVec();
 	    for (int j=0; j<atomVec.size(); j++)
	   	{ 
 	    	Cartesian cc = (Cartesian)atomVec.elementAt(j);
	   	    String atom = cc.getAtom();
	   	    if (atom.equals("HA"))
	   	    	coordA = cc.getXYZ();	//for translation	    
	   	    
	   	} //for (int j=0; j<atomVec.size(); j++)	
 	   double [] coordABase = {0.0, 0.0, 0.0};
	    for (int j=0; j<atomVec.size(); j++)
	    { 
	    	Cartesian cc = (Cartesian)atomVec.elementAt(j);
	   	    String atom = cc.getAtom();
	   	    if (atom.equals("CA"))
	   	    	coordABase = cc.getXYZ();	//for translation	    
	   	    
	    } //for (int j=0; j<atomVec.size(); j++)
 	   if(curRes.equalsIgnoreCase("GLY")){
  	    	double [] coordA1 = {0.0, 0.0, 0.0};
  	    	double [] coordA2 = {0.0, 0.0, 0.0};
  	    	for (int j=0; j<atomVec.size(); j++)
  		   	{ 
  	 	    	Cartesian cc = (Cartesian)atomVec.elementAt(j);
  		   	    String atom = cc.getAtom();
  		   	    if (atom.equals("HA1"))
  		   	    	coordA1 = cc.getXYZ();	
  		   	    if (atom.equals("HA2"))
		   	    	coordA2 = cc.getXYZ();	 		   	    
  		   	} //for (int j=0; j<atomVec.size(); j++)
  	    	coordA[0]=(coordA1[0]+coordA2[0])/2;
  	    	coordA[1]=(coordA1[1]+coordA2[1])/2;
  	    	coordA[2]=(coordA1[2]+coordA2[2])/2;
  	    }//if(curRes.equalsIgnoreCase("GLY"))
 	   
 	   
 	    double minDist=99999.9;
 	    int minResNo=0;
 	    Cartesian minCC=null;
 	    boolean isHbonded=false;
    	for (int j=0;j<vecPdb.size();j++){
    		Pdb pdbB = (Pdb)vecPdb.elementAt(j);
    		String resB=pdbB.getResidue();
    		Vector<Cartesian> atomVecB=pdbB.getAtomVec();
    		int resNoB=pdbB.getResidueNo();
    		if(curNo==resNoB)
    			continue;
    		for(int k=0;k<atomVecB.size();k++){
    			Cartesian ccB=atomVecB.elementAt(k);
    			String atomB=ccB.getAtom();
    			if(!atomB.substring(0,1).equalsIgnoreCase("O"))
    				continue;
    			coordB=ccB.getXYZ();
    			String atomNamebase=getAtomNameBaseforO(resB,atomB);
    			double [] coordB_base = {0.0, 0.0, 0.0};	
    			for(int t=0;t<atomVecB.size();t++){
    				Cartesian ccBBase=atomVecB.elementAt(t);
        			String atomBBase=ccB.getAtom();
        			if(atomBBase.equalsIgnoreCase(atomNamebase)){
        				coordB_base=ccBBase.getXYZ();
        				break;
        			}
    			}//for(int t=0;t<atomVecB.size();t++)	
    			
    			double distance= Math.sqrt((coordA[0] - coordB[0]) * (coordA[0] - coordB[0]) 
						+ (coordA[1] - coordB[1]) * (coordA[1] - coordB[1]) 
						+ (coordA[2] - coordB[2]) * (coordA[2] - coordB[2]));
    			double[] hbondVec=new double[3];
    			double[] donatorToHVec=new double[3];
    			double[] oToBaseVec=new double[3];
    			hbondVec=pp.internuclearVec(coordA,coordB);
    			donatorToHVec=pp.internuclearVec(coordA,coordABase);
    			oToBaseVec=pp.internuclearVec(coordB,coordB_base);
    			double gamma1=pp.interAngle(donatorToHVec, hbondVec)*(180/Math.PI);
    			hbondVec=pp.internuclearVec(coordB,coordA);
    			double gamma2=pp.interAngle(hbondVec,oToBaseVec)*(180/Math.PI);
    			
    			if( (distance<=distCutOff) ){// && (gamma1>=90.0)  && (gamma2>=90.0) ){   				
	    			isHbonded=true;
	    			if(distance<minDist){
	    				minDist=distance;
	    				minResNo=resNoB;
	    				minCC=ccB;
	    			}
    			}//if(distance<=distCutOff)
    		}//for(int k=0;k<atomVecB.size();k++)
    		
    	}//for (int j=0;j<vecPdb.size();j++)    	
    	
    	boolean isStrongest=true;//checkIsStrongestForPartner(minResNo,minCC,vecPdb,resNo_ha,minDist);
    	
    	if(isHbonded && isStrongest){
    		double r=minDist;
    		if( minDist > 2.60756 )
   		  	{
   			  	r = 2.60756;
   		  	}
   		    if( minDist < 2.26808 )
   		    {
   			   r = 2.26808;      
   		    }

    		double shift_delta = 0.147521/r - (1.65458E-05)/Math.pow(r, 1.5) - 0.000134668/(r*r) + 
			 0.0598561/Math.pow(r, 2.5) + 15.6855/(r*r*r) - 0.673905; //from SHIFTX source code.
    		
    		
    		
    		
    		//deltaShift[0]=shift_delta;
    		return shift_delta;
    	}
    	else {
    		//deltaShift[0]=-99999.9;
    		return 0.0;
    	}	
	}
	  /**
    compute the O atom in the h-bond for a ha atom (SHIFTX method, but only compute the strongest one)
	//1. compute the strongest h-bond for the Ha atom
	//2. check whether this h-bond is also the strongest for the O atom (need to check and implement...)
	//3. Others: use distance bound 2.77A. Donor and receptor must be in different residues.
     * 
     * @param resNo_ha current residue ID of the Ha atom
     * @param deltaShift ha chemical shift delta.
     * @return boolean true if h-boneded, false otherwise.
     */
	
	public double computeDeltaFromHBondForHnOld(int resNo_hn,Vector<Pdb> vecPdb){
		double [] coordA = {0.0, 0.0, 0.0};
    	double [] coordB = {0.0, 0.0, 0.0};	
    	double distCutOff=3.5;
    	Pdb pp=new Pdb();
    	
		int Index= Collections.binarySearch(vecPdb, new Pdb(resNo_hn), new Pdb.PdbComparator());
 	    if(Index<0)
 	    {
 	    	System.out.println("We couldnot find the residue number...");
 	    	System.exit(0); 	    	
 	    }
 	    Pdb curPdb=vecPdb.elementAt(Index);
 	    int curNo=curPdb.getResidueNo();
 	    String curRes=curPdb.getResidue();
 	    Vector<Cartesian> atomVec=curPdb.getAtomVec();
 	    for (int j=0; j<atomVec.size(); j++)
	   	{ 
 	    	Cartesian cc = (Cartesian)atomVec.elementAt(j);
	   	    String atom = cc.getAtom();
	   	    if (atom.equalsIgnoreCase("HN") || atom.equalsIgnoreCase("H"))
	   	    	coordA = cc.getXYZ();	//for translation	    
	   	    
	   	} //for (int j=0; j<atomVec.size(); j++)	
 	   double [] coordABase = {0.0, 0.0, 0.0};
	    for (int j=0; j<atomVec.size(); j++)
	    { 
	    	Cartesian cc = (Cartesian)atomVec.elementAt(j);
	   	    String atom = cc.getAtom();
	   	    if (atom.equals("N"))
	   	    	coordABase = cc.getXYZ();	//for translation	    
	   	    
	    } //for (int j=0; j<atomVec.size(); j++)
 	 
 	   
 	    double minDist=99999.9;
 	    int minResNo=0;
 	    Cartesian minCC=null;
 	    boolean isHbonded=false;
    	for (int j=0;j<vecPdb.size();j++){
    		Pdb pdbB = (Pdb)vecPdb.elementAt(j);
    		String resB=pdbB.getResidue();
    		Vector<Cartesian> atomVecB=pdbB.getAtomVec();
    		int resNoB=pdbB.getResidueNo();
    		if(curNo==resNoB)
    			continue;
    		for(int k=0;k<atomVecB.size();k++){
    			Cartesian ccB=atomVecB.elementAt(k);
    			String atomB=ccB.getAtom();
    			if(!atomB.substring(0,1).equalsIgnoreCase("O"))
    				continue;
    			coordB=ccB.getXYZ();
    			String atomNamebase=getAtomNameBaseforO(resB,atomB);
    			double [] coordB_base = {0.0, 0.0, 0.0};	
    			for(int t=0;t<atomVecB.size();t++){
    				Cartesian ccBBase=atomVecB.elementAt(t);
        			String atomBBase=ccB.getAtom();
        			if(atomBBase.equalsIgnoreCase(atomNamebase)){
        				coordB_base=ccBBase.getXYZ();
        				break;
        			}
    			}//for(int t=0;t<atomVecB.size();t++)	
    			
    			double distance= Math.sqrt((coordA[0] - coordB[0]) * (coordA[0] - coordB[0]) 
						+ (coordA[1] - coordB[1]) * (coordA[1] - coordB[1]) 
						+ (coordA[2] - coordB[2]) * (coordA[2] - coordB[2]));
    			double[] hbondVec=new double[3];
    			double[] donatorToHVec=new double[3];
    			double[] oToBaseVec=new double[3];
    			hbondVec=pp.internuclearVec(coordA,coordB);
    			donatorToHVec=pp.internuclearVec(coordA,coordABase);
    			oToBaseVec=pp.internuclearVec(coordB,coordB_base);
    			double gamma1=pp.interAngle(donatorToHVec, hbondVec)*(180/Math.PI);
    			hbondVec=pp.internuclearVec(coordB,coordA);
    			double gamma2=pp.interAngle(hbondVec,oToBaseVec)*(180/Math.PI);
    			
    			if( (distance<=distCutOff) && (gamma1>=90.0)  && (gamma2>=90.0) ){// && (gamma1>=90.0)  && (gamma2>=90.0) ){   				
	    			isHbonded=true;
	    			if(distance<minDist){
	    				minDist=distance;
	    				minResNo=resNoB;
	    				minCC=ccB;
	    			}
    			}//if(distance<=distCutOff)
    		}//for(int k=0;k<atomVecB.size();k++)
    		
    	}//for (int j=0;j<vecPdb.size();j++)    	
    	
    	boolean isStrongest=true;//checkIsStrongestForPartner(minResNo,minCC,vecPdb,resNo_ha,minDist);
    	
    	if(isHbonded && isStrongest){
    		double r=minDist;
    	/*	if( minDist > 2.60756 )
   		  	{
   			  	r = 2.60756;
   		  	}
   		    if( minDist < 2.26808 )
   		    {
   			   r = 2.26808;      
   		    }*/
    		double shift_delta =9.7464/(r * r * r) - 0.9887;
    		
    		
    		
    		//deltaShift[0]=shift_delta;
    		return shift_delta;
    	}
    	else {
    		//deltaShift[0]=-99999.9;
    		return 0.0;
    	}	
	}
	
	 /**
    	check whether the h-bond is the strongest one for partner (i.e., O).	
     * 
     * @param resNo_partner residue number of the partner O
     * @param ccPartner coordinates of the partner O
     * @param vecPdb pdb file of the whole protein.
     * @param resNo_Ha residue number of the orginal Ha atom
     * @param distToHa distance between the Ha atom and the partner O.
     * @return boolean true if it is the strongest one, false otherwise.
     */
	public boolean checkIsStrongestForPartner(int resNo_partner,Cartesian ccPartner, Vector<Pdb> vecPdb,int resNo_Ha,double distToHa ){
		double [] coordA = {0.0, 0.0, 0.0};
    	double [] coordB = {0.0, 0.0, 0.0};	
    	coordA=ccPartner.getXYZ();
    	for (int j=0;j<vecPdb.size();j++){
    		Pdb pdbB = (Pdb)vecPdb.elementAt(j);
    		Vector<Cartesian> atomVecB=pdbB.getAtomVec();
    		int resNoB=pdbB.getResidueNo();
    		if(resNo_partner==resNoB)
    			continue;
    		for(int k=0;k<atomVecB.size();k++){
    			Cartesian ccB=atomVecB.elementAt(k);
    			String atomB=ccB.getAtom();
    			if( !(atomB.equalsIgnoreCase("HA") || atomB.equalsIgnoreCase("HN") || atomB.equalsIgnoreCase("H") ))
    				continue;
    			coordB=ccB.getXYZ();
    			
    			if(atomB.equalsIgnoreCase("HA") && resNo_Ha==resNoB)
    				continue;
    			double distance= Math.sqrt((coordA[0] - coordB[0]) * (coordA[0] - coordB[0]) 
						+ (coordA[1] - coordB[1]) * (coordA[1] - coordB[1]) 
						+ (coordA[2] - coordB[2]) * (coordA[2] - coordB[2]));
    			if(distance<=distToHa)
    				return false;    			
    		}//for(int k=0;k<atomVecB.size();k++)
    	}//for (int j=0;j<vecPdb.size();j++)
    	return true;
	}
	
}
