package chemShiftPredict;
/*
HASH Ha Chemical Shift Prediction  Software Version 1.0
Copyright (C) 2009-2012 Bruce Donald Lab, Duke University

HASH is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free
Software Foundation, either version 3 of the License, or (at your option) any
later version.

HASH is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
details.

You should have received a copy of the GNU Lesser General Public License
along with this library; if not, see:
    <http://www.gnu.org/licenses/>.

There are additional restrictions imposed on the use and distribution of this
open-source code, including: (A) this header must be included in any
modification or extension of the code; (B) you are required to cite our
papers in any publications that use this code. The citation for the various
different modules of our software, together with a complete list of
requirements and restrictions are found in the document license.pdf enclosed
with this distribution.

Contact Info:
    Bruce R. Donald
    Duke University
    Department of Computer Science
    Levine Science Research Center (LSRC)
    Durham, NC 27708-0129
    USA
    email: www.cs.duke.edu/brd/

<signature of Bruce Donald>, 01 May, 2011
Bruce R. Donald, Professor of Computer Science and Biochemistry
*/

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.Collections;
import java.util.StringTokenizer;
import java.util.Vector;

import Jampack.JampackException;

import rdcPanda.Cartesian;
import rdcPanda.H1CS;
import rdcPanda.Pdb;
import rdcPanda.PhiPsi;

//The main class for Ha chemical shift prediction
public class haPredictMain {
	
	
	
	public static void main (String[] args) throws IOException, InterruptedException{
		long startTime = System.currentTimeMillis();
		
		shiftx2Database shiftx=new shiftx2Database();		
		haPredictMain haPredMain=new haPredictMain();
		chemicalShiftDatabase csd=new chemicalShiftDatabase();
		
		String userDir = System.getProperty("user.dir");
		
		/////////////////////////////////
		//read the training data from Shiftx database
		String pathCSDatabase=userDir+"/system/cs_database/shiftx/shifts/";  //chemical shift input file directory	
		String pathPdbDatabase=userDir+"/system/cs_database/shiftx/pdb/";    //Pdb input file directory
        ///Vector<Vector<residueCS>> vecTrainDataShiftx=shiftx.readShiftxData(pathCSDatabase, pathPdbDatabase,true);
			
		String trainSaveFileName=userDir+"/system/cs_database/shiftx/shiftx_train_data.txt";
	    //haPredMain.writeResdiueCSDataToFile(vecTrainDataShiftx,trainSaveFileName);
		
        Vector<Vector<residueCS>> vecTrainDataShiftx=haPredMain.readResdiueCSDataFromFile(trainSaveFileName);	
	    
	    //write as training data in R.
	    String trainRFileName=userDir+"/system/cs_database/r_files/r_train_data.txt";
	    // haPredMain.writeResdiueCSDataToFileForR(vecTrainDataShiftx,trainRFileName);
		////////////////////////////////
		
	    //////////////////////////////////
	    //read the training data from TALOS
		String pathCSDatabaseTalos=userDir+"/system/cs_database/talos/shifts/";  //chemical shift input file directory	
		String pathPdbDatabaseTalos=userDir+"/system/cs_database/talos/pdb/";    //Pdb input file directory
	    ////Vector<Vector<residueCS>> vecTrainDataTalos=csd.readTalosData(pathCSDatabaseTalos, pathPdbDatabaseTalos,true);
	    
		String trainSaveFileNameTalos=userDir+"/system/cs_database/talos/talos_train_data.txt";
	    // haPredMain.writeResdiueCSDataToFile(vecTrainDataTalos,trainSaveFileNameTalos);
		
        Vector<Vector<residueCS>> vecTrainDataTalos=haPredMain.readResdiueCSDataFromFile(trainSaveFileNameTalos);		    
	    //////////////////////////////////    
	    
	    
		///////////////////////////
		//read the refDB training data
		refDB refdb=new refDB();
		String pathRefCSDatabase=userDir+"/system/cs_database/refdb/shift_refdb/";  //chemical shift input file directory	
		System.out.println("Reading refDB training data...");
	    ///  Vector<Vector<residueCS>> vecTrainDataRefDB=refdb.readRefDBTrainingData(pathRefCSDatabase);
		
		String savedRefCSDatabase=userDir+"/system/cs_database/refdb/refdb_data.txt"; 
		//haPredMain.writeResdiueCSDataToFile(vecTrainDataRefDB,savedRefCSDatabase);
		
	    Vector<Vector<residueCS>> vecTrainDataRefDB=haPredMain.readResdiueCSDataFromFile(savedRefCSDatabase);
		///////////////////////////		
			
		//read the testing data from Shiftx database
		pathCSDatabase=userDir+"/system/cs_database/shiftx/shifts_test/";   //chemical shift input file directory	
		pathPdbDatabase=userDir+"/system/cs_database/shiftx/pdb_test/";     //Pdb input file directory
		Vector<Vector<residueCS>> vecTestDataShiftx=shiftx.readShiftxData(pathCSDatabase, pathPdbDatabase,false);
		
		String testSaveFileName=userDir+"/system/cs_database/shiftx/shiftx_test_data.txt";
		//haPredMain.writeResdiueCSDataToFile(vecTestDataShiftx,testSaveFileName);
		
	    ///Vector<Vector<residueCS>> vecTestDataShiftx=haPredMain.readResdiueCSDataFromFile(testSaveFileName);
		
		
		//write the testing data to R data table
		String testRFileName=userDir+"/system/cs_database/r_files/r_test_data_two_neighbors.txt";		
		haPredMain.writeResdiueCSDataToFileForRTwoNeighbors(vecTestDataShiftx,testRFileName);
		
		String testOneNeighborRFileName=userDir+"/system/cs_database/r_files/r_test_data_one_neighbor.txt";	
		haPredMain.writeResdiueCSDataToFileForROneNeighbor(vecTestDataShiftx,testOneNeighborRFileName);
		
		String testNoNeighborRFileName=userDir+"/system/cs_database/r_files/r_test_data_no_neighbor.txt";	
		haPredMain.writeResdiueCSDataToFileForRNoNeighbor(vecTestDataShiftx,testNoNeighborRFileName);
		
		
		int resWindow=6;//8
		float hErr=0.06f;//0.08f;
		float heavyErr=0.6f;//0.7f;
		boolean isCaUsed=false;
		boolean isCbUsed=false;
		boolean isCoUsed=false;
		boolean isHnUsed=false;
		boolean isNUsed=true;
		
		System.out.println("Residue window = "+ resWindow +", hErr="+hErr+", heavyErr="+heavyErr);
	
		
		Vector<Vector<residueCS>> vecTrainData=new Vector<Vector<residueCS>>();
	    vecTrainData.addAll(vecTrainDataShiftx);
	    vecTrainData.addAll(vecTrainDataTalos);		
	    vecTrainData.addAll(vecTrainDataRefDB);
		
		//haPredMain.computeBestMatchedHaCS(vecTestDataShiftx,vecTrainDataShiftx,resWindow,hErr,heavyErr,isCaUsed,isCbUsed,isCoUsed,isHnUsed,isNUsed);
		haPredMain.computeBestMatchedHaCSHDistance(vecTestDataShiftx,vecTrainData,resWindow,hErr,heavyErr);
		//haPredMain.computeBestMatchedHaCSHDistanceAverage(vecTestDataShiftx,vecTrainData,resWindow,hErr,heavyErr);
		
		//////////////////////////////////////
		//execute the R package to perform the prediction based on the structural information
	    String strRBin="E:/workspace_java_duke/ProteinNMR/system/cs_database/r_files/R-2.15.2/bin/Rscript";
		///String rScriptsFile="r_scripts_two_neighbors.R";
		String rScriptsFile="r_scripts_two_neighbors_no_structure.R";
		///String rScriptsFile="r_scripts_two_neighbors_bb.R";
		
		String rOutFile="RunRResults.txt";		
		haPredMain.runRScripts(strRBin,rScriptsFile,rOutFile);
		
		///rScriptsFile="r_scripts_one_neighbor.R";
	    rScriptsFile="r_scripts_one_neighbor_no_structure.R";
	    /// rScriptsFile="r_scripts_one_neighbor_bb.R";
	   
		haPredMain.runRScripts(strRBin,rScriptsFile,rOutFile);
		
		///rScriptsFile="r_scripts_no_neighbor.R";
		rScriptsFile="r_scripts_no_neighbor_no_structure.R";
	   /// rScriptsFile="r_scripts_no_neighbor_bb.R";
		haPredMain.runRScripts(strRBin,rScriptsFile,rOutFile); //temp
	
		///////////////////////////////////////////////////////
		//read the predicted Ha chemical shifts from R, using both chemical shift and structural information
    	String rResultFileName=userDir+"/system/cs_database/r_files/r_predicted_ha.txt";
		
		Vector<Vector<residueCS>> vecRPredictedHaCS=haPredMain.readHaChemicalShiftsFromRFile(rResultFileName);		
		System.out.println("Updating the ha chemical shifts using R predicted results...");
		haPredMain.updateHaChemicalShiftFromRResults(vecTestDataShiftx,vecRPredictedHaCS);
		
	
		String rResultOneNeighborFileName=userDir+"/system/cs_database/r_files/r_predicted_ha_one_neighbor.txt";	
		
		Vector<Vector<residueCS>> vecRPredictedHaCSOneNeighbor=haPredMain.readHaChemicalShiftsFromRFile(rResultOneNeighborFileName);		
		System.out.println("Updating the ha chemical shifts using R predicted results, using only structural information...");
	    haPredMain.updateHaChemicalShiftFromRResults(vecTestDataShiftx,vecRPredictedHaCSOneNeighbor);
		
		String rResultNoNeighborFileName=userDir+"/system/cs_database/r_files/r_predicted_ha_no_neighbor.txt";		
		Vector<Vector<residueCS>> vecRPredictedHaCSNoNeighbor=haPredMain.readHaChemicalShiftsFromRFile(rResultNoNeighborFileName);		
		System.out.println("Updating the ha chemical shifts using R predicted results, using only structural information...");
		haPredMain.updateHaChemicalShiftFromRResults(vecTestDataShiftx,vecRPredictedHaCSNoNeighbor);
		
	/*	String rResultOnlyStructureFileName=userDir+"/system/cs_database/r_files/r_predicted_ha_only_structure.txt";		
		Vector<Vector<residueCS>> vecRPredictedHaCSOnlyStr=haPredMain.readHaChemicalShiftsFromRFile(rResultOnlyStructureFileName);		
		System.out.println("Updating the ha chemical shifts using R predicted results, using only structural information...");
		haPredMain.updateHaChemicalShiftFromRResults(vecTestDataShiftx,vecRPredictedHaCSOnlyStr);
		*/
		
		
		/////////////////////////////////////
		
		haPredMain.haPredictionSummary(vecTestDataShiftx);
		
		haPredMain.haPredictionPrintOut(vecTestDataShiftx);
		
		
		long endTime = System.currentTimeMillis() ;
	 	double totalTime = (double) ((endTime - startTime) / 60000.0); //in minutes
	 	System.out.println("Time= "+ totalTime +" minutes");
	}
	
	/**The main function for prediction, called by the JSP page. TO be changed
	 * @param csTrainDataFile: location of training database
	 * @param strRScriptBin: R script bin file path.
	 * @param isOutputAllShifts: whether output all shifts or only Ha shifts
	 * @return
	 */
	public Vector<H1CS> runHaPredictionMain (String csTrainDataFile, String csTestDataFile,String pdbTestDataFile, String strRScriptBin, boolean isOutputAllShifts ) 
			throws IOException, InterruptedException{
		long startTime = System.currentTimeMillis();
		
		shiftx2Database shiftx=new shiftx2Database();		
		haPredictMain haPredMain=new haPredictMain();
		chemicalShiftDatabase csd=new chemicalShiftDatabase();
		
		//String userDir = System.getProperty("user.dir");
		
		/////////////////////////////////
		//read the training data from Shiftx database
		String pathCSDatabase=csTrainDataFile+"/shiftx/shifts/";  //chemical shift input file directory	
		String pathPdbDatabase=csTrainDataFile+"/shiftx/pdb/";    //Pdb input file directory
		//Vector<Vector<residueCS>> vecTrainDataShiftx=shiftx.readShiftxData(pathCSDatabase, pathPdbDatabase,true);
			
		String trainSaveFileName=csTrainDataFile+"/shiftx/shiftx_train_data.txt";
	    //haPredMain.writeResdiueCSDataToFile(vecTrainDataShiftx,trainSaveFileName);
		
		Vector<Vector<residueCS>> vecTrainDataShiftx=haPredMain.readResdiueCSDataFromFile(trainSaveFileName);	
	    
	    //write as training data in R.
	    //String trainRFileName=csTrainDataFile+"/r_files/r_train_data.txt";
	    // haPredMain.writeResdiueCSDataToFileForR(vecTrainDataShiftx,trainRFileName);
		////////////////////////////////
		
	    //////////////////////////////////
	    //read the training data from TALOS
		String pathCSDatabaseTalos=csTrainDataFile+"/talos/shifts/";  //chemical shift input file directory	
		String pathPdbDatabaseTalos=csTrainDataFile+"/talos/pdb/";    //Pdb input file directory
		//Vector<Vector<residueCS>> vecTrainDataTalos=csd.readTalosData(pathCSDatabaseTalos, pathPdbDatabaseTalos,true);
	    
		String trainSaveFileNameTalos=csTrainDataFile+"/talos/talos_train_data.txt";
	   // haPredMain.writeResdiueCSDataToFile(vecTrainDataTalos,trainSaveFileNameTalos);
		
	    Vector<Vector<residueCS>> vecTrainDataTalos=haPredMain.readResdiueCSDataFromFile(trainSaveFileNameTalos);		    
	    //////////////////////////////////
	    
	    
	    
		///////////////////////////
		//read the refDB training data
		refDB refdb=new refDB();
		//String pathRefCSDatabase=userDir+"/system/cs_database/refdb/shift_refdb/";  //chemical shift input file directory	
		System.out.println("Reading refDB training data...");
		//Vector<Vector<residueCS>> vecTrainDataRefDB=refdb.readRefDBTrainingData(pathRefCSDatabase);
		
		String savedRefCSDatabase=csTrainDataFile+"/refdb/refdb_data.txt"; 
		//haPredMain.writeResdiueCSDataToFile(vecTrainDataRefDB,savedRefCSDatabase);
		
		Vector<Vector<residueCS>> vecTrainDataRefDB=haPredMain.readResdiueCSDataFromFile(savedRefCSDatabase);
		///////////////////////////
		
			
		///////////////////////////////////////////////////////
		//read the testing data:
		//////////////////////////////////////////////////////
		/////////////////////////////////////////////////////////
		//read the testing data from Shiftx database
		/*pathCSDatabase=csTrainDataFile+"/shiftx/shifts_test/";   //chemical shift input file directory	
		pathPdbDatabase=csTrainDataFile+"/shiftx/pdb_test/";     //Pdb input file directory
		//Vector<Vector<residueCS>> vecTestDataShiftx=shiftx.readShiftxData(pathCSDatabase, pathPdbDatabase,false);
		
		String testSaveFileName=csTrainDataFile+"/shiftx/shiftx_test_data.txt";
		//haPredMain.writeResdiueCSDataToFile(vecTestDataShiftx,testSaveFileName);
		 
		 Vector<Vector<residueCS>> vecTestDataShiftx=haPredMain.readResdiueCSDataFromFile(testSaveFileName);
		 * */
		
		////////////////////////////////////////////////
		pathCSDatabase=csTestDataFile;//+"/shift/";
		pathPdbDatabase=csTrainDataFile;//+"/shiftx/pdb_test/";     //Pdb input file directory
		Vector<Vector<residueCS>> vecTestDataShiftx=haPredMain.readTestData(pathCSDatabase, pathPdbDatabase,csTrainDataFile);
		
		/////////////////////////////////////////////////
		
		
		
		//write the testing data to R data table
		String testRFileName=csTrainDataFile+"/r_files/r_test_data_two_neighbors.txt";		
		haPredMain.writeResdiueCSDataToFileForRTwoNeighbors(vecTestDataShiftx,testRFileName);
		
		String testOneNeighborRFileName=csTrainDataFile+"/r_files/r_test_data_one_neighbor.txt";	
		haPredMain.writeResdiueCSDataToFileForROneNeighbor(vecTestDataShiftx,testOneNeighborRFileName);
		
		String testNoNeighborRFileName=csTrainDataFile+"/r_files/r_test_data_no_neighbor.txt";	
		haPredMain.writeResdiueCSDataToFileForRNoNeighbor(vecTestDataShiftx,testNoNeighborRFileName);
		
		
		int resWindow=8;
		float hErr=0.06f;//0.08f;
		float heavyErr=0.6f;//0.7f;
	
		System.out.println("Residue window = "+ resWindow +", hErr="+hErr+", heavyErr="+heavyErr);
	
		
		Vector<Vector<residueCS>> vecTrainData=new Vector<Vector<residueCS>>();
		vecTrainData.addAll(vecTrainDataShiftx);
		vecTrainData.addAll(vecTrainDataTalos);		
		vecTrainData.addAll(vecTrainDataRefDB);
		
		//haPredMain.computeBestMatchedHaCS(vecTestDataShiftx,vecTrainDataShiftx,resWindow,hErr,heavyErr,isCaUsed,isCbUsed,isCoUsed,isHnUsed,isNUsed);
		haPredMain.computeBestMatchedHaCSHDistance(vecTestDataShiftx,vecTrainData,resWindow,hErr,heavyErr);
		//haPredMain.computeBestMatchedHaCSHDistanceAverage(vecTestDataShiftx,vecTrainData,resWindow,hErr,heavyErr);
		
		//////////////////////////////////////
		//execute the R package to perform the prediction based on the structural information
		//String strRBin="H:/workspace/ProteinNMR/system/cs_database/r_files/R-2.11.1/bin/Rscript";
		String strRBin=strRScriptBin;
		String rScriptsFile="r_scripts_two_neighbors.R";
		String rOutFile="RunRResults.txt";		
		String srCurcRPath=csTrainDataFile+"/r_files/";
		haPredMain.runRScripts(strRBin,rScriptsFile,rOutFile,srCurcRPath);
		
		rScriptsFile="r_scripts_one_neighbor.R";
		haPredMain.runRScripts(strRBin,rScriptsFile,rOutFile,srCurcRPath);
		
		rScriptsFile="r_scripts_no_neighbor.R";
		haPredMain.runRScripts(strRBin,rScriptsFile,rOutFile,srCurcRPath);
		
		///////////////////////////////////////////////////////
		//read the predicted Ha chemical shifts from R, using both chemical shift and structural information
    	String rResultFileName=csTrainDataFile+"/r_files/r_predicted_ha.txt";
		
		Vector<Vector<residueCS>> vecRPredictedHaCS=haPredMain.readHaChemicalShiftsFromRFile(rResultFileName);		
		System.out.println("Updating the ha chemical shifts using R predicted results...");
		haPredMain.updateHaChemicalShiftFromRResults(vecTestDataShiftx,vecRPredictedHaCS);
		
	
		String rResultOneNeighborFileName=csTrainDataFile+"/r_files/r_predicted_ha_one_neighbor.txt";		
		Vector<Vector<residueCS>> vecRPredictedHaCSOneNeighbor=haPredMain.readHaChemicalShiftsFromRFile(rResultOneNeighborFileName);		
		System.out.println("Updating the ha chemical shifts using R predicted results, using only structural information...");
		haPredMain.updateHaChemicalShiftFromRResults(vecTestDataShiftx,vecRPredictedHaCSOneNeighbor);
		
		String rResultNoNeighborFileName=csTrainDataFile+"/r_files/r_predicted_ha_no_neighbor.txt";		
		Vector<Vector<residueCS>> vecRPredictedHaCSNoNeighbor=haPredMain.readHaChemicalShiftsFromRFile(rResultNoNeighborFileName);		
		System.out.println("Updating the ha chemical shifts using R predicted results, using only structural information...");
		haPredMain.updateHaChemicalShiftFromRResults(vecTestDataShiftx,vecRPredictedHaCSNoNeighbor);
		
	/*	String rResultOnlyStructureFileName=userDir+"/system/cs_database/r_files/r_predicted_ha_only_structure.txt";		
		Vector<Vector<residueCS>> vecRPredictedHaCSOnlyStr=haPredMain.readHaChemicalShiftsFromRFile(rResultOnlyStructureFileName);		
		System.out.println("Updating the ha chemical shifts using R predicted results, using only structural information...");
		haPredMain.updateHaChemicalShiftFromRResults(vecTestDataShiftx,vecRPredictedHaCSOnlyStr);
		*/
		
		Vector<H1CS> vecH1CS=haPredMain.convertToH1CSFormat(vecTestDataShiftx,isOutputAllShifts);
		/////////////////////////////////////
		
		haPredMain.haPredictionSummary(vecTestDataShiftx);
		
		long endTime = System.currentTimeMillis() ;
	 	double totalTime = (double) ((endTime - startTime) / 60000.0); //in minutes
	 	System.out.println("Time= "+ totalTime +" minutes");
	 	
	 	return vecH1CS;
	}
	
	//called by runHaPredictionMain().
	private static Vector<H1CS> convertToH1CSFormat(Vector<Vector<residueCS>> vecTestDataShiftx, boolean isAllShifts){
		
		Vector<H1CS> vecH1CS=new Vector<H1CS>();
		for(int i=0;i<vecTestDataShiftx.size();i++){
			Vector<residueCS> vecOneData=vecTestDataShiftx.elementAt(i);
			for(int j=0;j<vecOneData.size();j++ ){
				residueCS resCS=vecOneData.elementAt(j);
				float haPredicted=resCS.getPredictedHaCS();
				float caCS=resCS.getCaCS();
				float cbCS=resCS.getCbCS();
				float coCS=resCS.getCoCS();				
			    float hnCS=resCS.getHnCS();
				float nCS=resCS.getNCS();
				
				int resNo=resCS.getResidueNo();
				String resName=resCS.getResidueName();
				if ( (haPredicted<100.0) && (haPredicted >-100.0) ){
					vecH1CS.add(new H1CS(resNo, resName, "HA", haPredicted));
				}
					
				if(!isAllShifts)
					continue;
				
				if ( (caCS<100.0) && (caCS >-100.0) ){
					vecH1CS.add(new H1CS(resNo, resName, "CA", caCS));
				}
				if ( (cbCS<100.0) && (cbCS >-100.0) ){
					vecH1CS.add(new H1CS(resNo, resName, "CB", cbCS));
				}
				if ( (coCS<100.0) && (coCS >-100.0) ){
					vecH1CS.add(new H1CS(resNo, resName, "C", coCS));
				}
				if ( (hnCS<100.0) && (hnCS >-100.0) ){
					vecH1CS.add(new H1CS(resNo, resName, "H", hnCS));
				}
				if ( (nCS<100.0) && (nCS >-100.0) ){
					vecH1CS.add(new H1CS(resNo, resName, "N", nCS));
				}		
				
			}//for(int j=0;j<vecOneData.size();j++ )
		}//for(int i=0;i<vecTestDataShiftx.size();i++)
		
		Collections.sort(vecH1CS, new H1CS.resNoComparator());
		return vecH1CS;
	}
	
	
	//update the remaining Ha chemical shifts based on R-results
	public void updateHaChemicalShiftFromRResults(Vector<Vector<residueCS>> vecTestDataShiftx, Vector<Vector<residueCS>> vecRPredictedHaCS){
		for(int i=0;i<vecTestDataShiftx.size();i++){
			Vector<residueCS> vecOneData=vecTestDataShiftx.elementAt(i);
			for(int j=0;j<vecOneData.size();j++ ){
				residueCS resCS=vecOneData.elementAt(j);
				float haPredicted=resCS.getPredictedHaCS();
				if(haPredicted>-100.0 && haPredicted < 100.0)
					continue;
				String proteinName=resCS.getProteinName();
				int no=resCS.getResidueNo();
				boolean isFound=false;
				
				for(int k=0;k<vecRPredictedHaCS.size();k++){
					Vector<residueCS> vecOneDataFromR=vecRPredictedHaCS.elementAt(k);
					if(vecOneDataFromR.size()>0){
						residueCS resCSRTemp=vecOneDataFromR.elementAt(0);
						String proteinNameR=resCSRTemp.getProteinName();
						if(!proteinNameR.equalsIgnoreCase(proteinName))
							continue;						
					}//if(vecOneDataFromR.size()>0)
					
					for(int h=0;h<vecOneDataFromR.size();h++){
						residueCS resCSR=vecOneDataFromR.elementAt(h);
						float haPredictedFromR=resCSR.getPredictedHaCS();
						int noR=resCSR.getResidueNo();
						if(haPredictedFromR>100.0 || haPredictedFromR <-100.0)
							continue;
						if(noR!=no)
							continue;
						
						resCS.setPredictedHaCS(haPredictedFromR);
						isFound=true;
						break;
					}//for(int h=0;h<vecOneDataFromR.size();h++)
					
					if(isFound)
						break;
				}//for(int k=0;k<vecRPredictedHaCS.size();k++)
				
				
			}//for(int j=0;j<vecOneData.size();j++ )
		}//for(int i=0;i<vecTestDataShiftx.size();i++)
		
	}
	
	/** 
	 * Read the jsp testing data. Similar to Shift2Database with minor changes
	 *  @param pathCSDatabase path of cs directory
	 *  @param pathPdbDatabase path of PDB directory
	 *  @param strTrainDatabase: location of random coil shifts
	 *  @return a vector of residueCS
	 */
	public Vector <Vector<residueCS>> readTestData(String pathCSDatabase,String pathPdbDatabase, String strTrainDatabase){		
		Vector<Vector<residueCS>> vecData=new  Vector<Vector<residueCS>>();
		
		shiftx2Database shiftx2=new shiftx2Database();
		
		ringEffect ring_effect=new ringEffect();
		hBondEffect hbond_effect=new hBondEffect();		
		H1CS h1CS=new H1CS();
		Pdb pp=new Pdb();
		chemicalShiftDatabase csd=new chemicalShiftDatabase();
		
		String userDir = System.getProperty("user.dir");
		
		//read random coil chemical shift table:
	    String pathRandomCoilShift=strTrainDatabase+"/random_coil_shifts.txt"; //random coil shift table
		randomCoilShift rcs=new randomCoilShift();
		Vector<randomCoilShift> vecRandomShifts=rcs.randomCoilShiftReader(pathRandomCoilShift);
		
		//read all chemical shift files
    	int counter=1;
    	File myDir = new File(pathCSDatabase);
    	String[] contents = myDir.list();//select all structures
    	if (contents == null) 
    		System.out.println(myDir.getName() + " is not a directory");
    	for (int t=0; t<contents.length;t++)  {  
    	
    		String filename = contents[t]; 
    		String fileNameSaved=filename.substring(0,filename.indexOf("_"));
    		
	    	String strCSFile="";
	    	strCSFile=pathCSDatabase+filename;	 	    	
	    	
	    	String strFoundPdbFile=shiftx2.findMatchedPdbFile(pathPdbDatabase,fileNameSaved);
	    	String strPdbFile=pathPdbDatabase+strFoundPdbFile;
	    		    		    	
	    	//read the training chemical shift data
	    	Vector<H1CS> vecCSTemp= new Vector<H1CS>();
	    	
	    	
	    	vecCSTemp=h1CS.h1CSReader_BMRB(strCSFile); 	 
	    	
	    	    	
	    	//read the training pdb data	    	
	    	Vector<Pdb> vecPdbOld = pp.readPdb(strPdbFile);
	    	Vector<Pdb> vecPdb=pp.nameConvert(vecPdbOld); 
	    	    	
	    	//align the residue number
	    	shiftx2.alignResidueNum(vecPdb,vecCSTemp);
	    	
	    	//update PDB residue names
	    	shiftx2.updatePdbResidueName(vecPdb,vecCSTemp);
	    	
	    	boolean isBMRBNEWNameScheme=false;;// whether using the BMRB_NEW naming scheme for atom names.
	    	for(int x=0;x<vecPdb.size();x++){
	    		Pdb pdb=vecPdb.elementAt(x);
	    		String res=pdb.getResidue();
	    		Vector<Cartesian> vecAtom =pdb.getAtomVec();
	    		for(int y=0;y<vecAtom.size();y++){
	    			Cartesian cc=vecAtom.elementAt(y);
	    			String atom=cc.getAtom();
	    			if( !( res.equalsIgnoreCase("ALA")||res.equalsIgnoreCase("GLY")|| res.equalsIgnoreCase("ILE")
	    					||res.equalsIgnoreCase("PRO")||res.equalsIgnoreCase("THR")||res.equalsIgnoreCase("VAL")) ){
	    				if(atom.equalsIgnoreCase("HB3"))
	    					isBMRBNEWNameScheme=true;
	    				break;
	    			}	    			
	    		}//for(int y=0;y<vecAtom.size();y++)
	    		if(isBMRBNEWNameScheme)
	    			break;
	    	}//for(int x=0;x<vecPdb.size();x++)
	    	if(isBMRBNEWNameScheme){
	    		Vector<Pdb>  vecPdbTemp=pp.nameConvert2PDBNew(vecPdb);
	    		vecPdb=new Vector<Pdb>();
	    		vecPdb.addAll(vecPdbTemp);
	    	}	    	
	    	
	    	//compute the phi/psi angles
	    	Vector<PhiPsi> vecPhiPsi=pp.PhiPsiTotalPdb(vecPdb);	    
	    	   	
	    	//find max res no and min res no	    		    	
	    	int minNoFromCS=9999999;
	    	int maxNoFromCS=-99999;
	    	for(int i=0;i<vecCSTemp.size();i++){
	    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
	    		int resNo=h1cs.getResidueNo();
	    		if(resNo>maxNoFromCS)
	    			maxNoFromCS=resNo;  
	    		if(resNo<minNoFromCS)
	    			minNoFromCS=resNo;
	    	}
	    	int minNoFromPdb=9999999;
	    	int maxNoFromPdb=-99999;
	    	for(int i=0;i<vecPdb.size();i++){
	    		Pdb pdb=(Pdb)vecPdb.elementAt(i);
	    		int resNo=pdb.getResidueNo();
	    		if(resNo>maxNoFromPdb)
	    			maxNoFromPdb=resNo;  
	    		if(resNo<minNoFromPdb)
	    			minNoFromPdb=resNo;
	    	}
	    	int maxNo=Math.min(maxNoFromCS, maxNoFromPdb);
	    	int minNo=Math.max(minNoFromCS, minNoFromPdb);
	    	
	    	//for computing the chemical shift delta from hbond effect:
	    	double[] haCSDelta=new double[maxNo+1];
	    	double[] hnCSDelta=new double[maxNo+1];
	    	for(int i=0;i<=maxNo;i++){
	    		haCSDelta[i]=0.0;
	    		hnCSDelta[i]=0.0;
	    	}
	    	Vector<residueCS> vecOneProteinData=new Vector<residueCS>();
	    	
	    	//we skip the first residue and the last residue
	    	for(int no=minNo+1;no<=maxNo-1;no++){     	
	    		
	    		int ind= Collections.binarySearch(vecPdb, new Pdb(no), new Pdb.PdbComparator());
	     	    if(ind<0)
	     	    	continue;
	     	    Pdb curPdb=vecPdb.elementAt(ind);
	     	    String curRes=curPdb.getResidue();
	     	    
	     	    int ind2= Collections.binarySearch(vecPhiPsi, new PhiPsi(no), new PhiPsi.PPComparator());
	     	    if(ind2<0)
	     	    	continue;
	     	   
	     	    PhiPsi phipsi=vecPhiPsi.elementAt(ind2);
	     	 
	     	    int indPrev= Collections.binarySearch(vecPhiPsi, new PhiPsi(no-1), new PhiPsi.PPComparator());
	     	    double phiPrev=0.0;
	     	    double psiPrev=0.0;
	     	    if( indPrev>-1){
	    		   PhiPsi phipsiPre=vecPhiPsi.elementAt(indPrev);
	    		   phiPrev=chemicalShiftDatabase.convertAngleInPI( phipsiPre.getPhi() );
	    		   psiPrev=chemicalShiftDatabase.convertAngleInPI( phipsiPre.getPsi() );
	     	    }
	    	   
	     	    int indNext= Collections.binarySearch(vecPhiPsi, new PhiPsi(no+1), new PhiPsi.PPComparator());
	     	    double phiNext=0.0;
	     	    double psiNext=0.0;
	     	    if( indNext>-1){
	    		   PhiPsi phipsiNext=vecPhiPsi.elementAt(indNext);
	    		   phiNext=chemicalShiftDatabase.convertAngleInPI( phipsiNext.getPhi() );
	    		   psiNext=chemicalShiftDatabase.convertAngleInPI( phipsiNext.getPsi() );
	     	    }
	     	   
	     	    //check the residue name of the neighboring residues
	     	    String prevResType="";
	     	    ind=Collections.binarySearch(vecPdb, new Pdb(no-1), new Pdb.PdbComparator());
	     	    if(ind>-1){	     	    	
		     	   Pdb prevPdb=vecPdb.elementAt(ind);
		     	   prevResType=prevPdb.getResidue();
	     	    }   
	      	  
	     	   ////////////////////////////////
	     	   //compute the chemical shift delta from hbond and ring.
	     	   float deltaHBond_ha=(float)hbond_effect.computeDeltaFromHBondForHaOld(no, vecPdb);
	     	   float deltaHBond_hn=(float)hbond_effect.computeDeltaFromHBondForHnOld(no, vecPdb);	     	   
	     	   float deltaRing=(float)ring_effect.computeDeltaChemShiftFromRingEffectForHa(no, vecPdb)*1000;	     	
	     	   float deltaRingCa=(float)ring_effect.computeDeltaChemShiftFromRingEffectForCa(no, vecPdb)*1000;	    	 		
	    		
	     	   //find the bb chemical shift of current residue
	    		double csN=-9999.0;
	    		double csCA=-9999.0;
	    		double csHN=-9999.0;
	    		double csHA=-9999.0;
	    		double csCB=-9999.0;
	    		double csCO=-9999.0;
	    		
	    		//find cs of N
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && atom.equalsIgnoreCase("N")){
		    			csN=h1cs.getH1CS();
		    			break;
		    		}
	    		}  		
	    		
	    		//find cs of CA
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && atom.equalsIgnoreCase("CA")){
		    			csCA=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	    	
	    		//find cs of HN
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && (atom.equalsIgnoreCase("HN")|| atom.equalsIgnoreCase("H"))  ){
		    			csHN=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	    		
	    		//find cs of HA
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();		    		
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && atom.equalsIgnoreCase("HA")){
		    			csHA=h1cs.getH1CS();
		    			break;
		    		}	
		    	}//for(int i=0;i<vecCSTemp.size();i++)
	    		
	    		//for GLY, we compute the mean of two HA chemical shifts
	    		if(curRes.equalsIgnoreCase("GLY") || curRes.equalsIgnoreCase("G")){
	    			double sumHaCS=0.0;	  
	    			int counterHaCS=0;
	    			boolean isFoundHaForGly=false;	    			
	    			for(int i=0;i<vecCSTemp.size();i++){
			    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
			    		int resNo=h1cs.getResidueNo();		    		
			    		String atom=h1cs.getAtomName();
			    		if(atom.length()>=2)
			    			atom=atom.substring(0,2);
			    		if(resNo==no && atom.equalsIgnoreCase("HA")){
			    			sumHaCS=sumHaCS+h1cs.getH1CS();
			    			counterHaCS++;
			    			isFoundHaForGly=true;
			    		}	
			    	}//for(int i=0;i<vecCSTemp.size();i++)
	    			
	    			if(isFoundHaForGly && counterHaCS==2){
	    				csHA=sumHaCS/counterHaCS;
	    			}
	    			else
	    				csHA=-9999.0;	    			
	    		}//if(curRes.equalsIgnoreCase("GLY") || curRes.equalsIgnoreCase("G"))
	    		
	    		//find cs of CB
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && atom.equalsIgnoreCase("CB")){
		    			csCB=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	    		
	    		//find cs of CO
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && (atom.equalsIgnoreCase("CO") ||atom.equalsIgnoreCase("C"))){
		    			csCO=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}    		
	  
	    		double csRandCoilHa=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"HA",prevResType);
	    		double csRandCoilCa=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"CA",prevResType);
	    		double csRandCoilCb=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"CB",prevResType);
	    		double csRandCoilCo=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"C",prevResType);
	    		double csRandCoilHn=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"HN",prevResType);
	    		double csRandCoilN=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"N",prevResType);
	    		
	    		csHA=csHA-csRandCoilHa;
	    		csCA=csCA-csRandCoilCa;
	    		csCB=csCB-csRandCoilCb;
	    		csCO=csCO-csRandCoilCo;
	    		csHN=csHN-csRandCoilHn;
	    		csN=csN-csRandCoilN;      		
	    		   		
	    		double phi=chemicalShiftDatabase.convertAngleInPI( phipsi.getPhi());//1.5*Math.cos(3*(phipsi.getPhi() + 2.45*Math.PI/180 ))-3.0*Math.cos(phipsi.getPhi()-2.7*Math.PI/180 );
	    		double psi=chemicalShiftDatabase.convertAngleInPI( phipsi.getPsi() );//-0.4*Math.cos(3* ( phipsi.getPsi() +4.8*Math.PI/180 ) )-1.5*Math.cos(phipsi.getPsi() -0.84*Math.PI/180 )+0.7;
	    		
	    		residueCS resCS=new residueCS(no,curRes,fileNameSaved,(float)csHA,(float)csCA,(float)csCB,(float)csCO,(float)csHN,(float)csN,
	    				(float)deltaHBond_ha,(float)deltaHBond_hn,(float)deltaRing,(float)deltaRingCa,
	    				(float)phi,(float)psi,(float)phiPrev,(float)psiPrev,(float)phiNext,(float)psiNext);
	    		
	    		vecOneProteinData.add(resCS);
	    		///////////////////	    		
	    		//String str_csHA=csToString((float)(csHA));String str_csCA=csToString((float)(csCA));String str_csCB=csToString((float)(csCB));
	    		//String str_csCO=csToString((float)(csCO));String str_csHN=csToString((float)(csHN));String str_csN=csToString((float)(csN));	    						    		
	    		counter++;	    		
	    	}//for(int i=1;i<=maxNo;i++)
	    	
	    	Collections.sort(vecOneProteinData, new residueCS.residueCSComparator());//sort the data points based on residue number
	    	vecData.add(vecOneProteinData);	    	
    	}//for (int t=0; t<contents.length;t++) 
    	
    	return vecData;
	}
	
	
	
	public Vector<Vector<residueCS>>  readHaChemicalShiftsFromRFile(String fileName){
		Vector<Vector<residueCS>> vecData=new  Vector<Vector<residueCS>>();
		String ss="";
		StringTokenizer st = new StringTokenizer("");
		//int csID=0;
		float csHA=-99999.9f;
		float csHAPredicted=-99999.9f;
		float csCA=-99999.9f;
		float csCB=-99999.9f;
		float csCO=-99999.9f;;
		float csHN=-99999.9f;
		float csN=-99999.9f;
		String curRes="";
		int no=0;
		String proteinName="";
		String preProteinName="";
		Vector<residueCS> vecOneData=new Vector<residueCS>();
		int counter=0;
		try
    	{
    		BufferedReader in = new BufferedReader(new FileReader(fileName));	
    		ss = in.readLine();
    		ss=ss.trim();
    		
    		stop:
    			while(true) 
    			{   	
    				if(ss.equalsIgnoreCase(""))
     				{
     					if ((ss = in.readLine()) == null )
         				{
     						vecData.add(vecOneData);
        					vecOneData=new Vector<residueCS>(); 
         					break stop;
         				}
     					continue;
     				}
    				 
    				st = new StringTokenizer(ss);
    				if (st.hasMoreTokens())
    					proteinName=st.nextToken();//protein name
    				
    				if (st.hasMoreTokens())
    					no =  Integer.parseInt(st.nextToken());
    				
    				
    				if (st.hasMoreTokens())
    					csHAPredicted  = (float) new Double(st.nextToken()).doubleValue();    				
    				   								    				
    				    	
    				counter++;   				
    				
    				residueCS resCS=new residueCS(no,curRes,proteinName,(float)csHA,(float)csCA,(float)csCB,(float)csCO,(float)csHN,(float)csN);
    				resCS.setPredictedHaCS(csHAPredicted);
    				
    				if( (!preProteinName.equalsIgnoreCase(proteinName)) && (!preProteinName.equalsIgnoreCase("") ) ){
    					vecData.add(vecOneData);
    					vecOneData=new Vector<residueCS>(); 
    				}    				
    				
    				vecOneData.add(resCS);
    				
    				preProteinName=proteinName;
    				
    				if ((ss = in.readLine()) == null){
    					vecData.add(vecOneData);
    					vecOneData=new Vector<residueCS>(); 
    					break stop;
    				}
    				else if(ss.equalsIgnoreCase(""))
    				{
    					if ((ss = in.readLine()) == null )
        				{
    						vecData.add(vecOneData);
        					vecOneData=new Vector<residueCS>(); 
        					break stop;
        				}
    				}
    			}
    	}catch (FileNotFoundException e) {
    		System.out.println("File not found: " + fileName);
    	}catch (IOException e) 
    	{
    		System.out.println("IOException: the stack trace is:");
    		e.printStackTrace();
    	}
    	System.out.println("Total number of residue chemical shifts that have been read: "+ counter);
		return vecData;
	}
		
	public Vector<Vector<residueCS>>  readResdiueCSDataFromFile(String fileName){
		Vector<Vector<residueCS>> vecData=new  Vector<Vector<residueCS>>();
		String ss="";
		StringTokenizer st = new StringTokenizer("");
		//int csID=0;
		float csHA=0.0f;
		float csCA=0.0f;
		float csCB=0.0f;
		float csCO=0.0f;
		float csHN=0.0f;
		float csN=0.0f;
		float deltaHBond_ha=0.0f;
		float deltaHBond_hn=0.0f;
		float deltaRing=0.0f;
		float deltaRingCa=0.0f;
		float phi=0.0f;
		float psi=0.0f;
		float phiPrev=0.0f;
		float psiPrev=0.0f;
		float phiNext=0.0f;
		float psiNext=0.0f;
		String curRes="";
		int no=0;
		String proteinName="";
		String preProteinName="";
		Vector<residueCS> vecOneData=new Vector<residueCS>();
		int counter=0;
		try
    	{
    		BufferedReader in = new BufferedReader(new FileReader(fileName));	
    		ss = in.readLine();
    		ss=ss.trim();
    		
    		stop:
    			while(true) 
    			{   	
    				if(ss.equalsIgnoreCase(""))
     				{
     					if ((ss = in.readLine()) == null )
         				{
     						vecData.add(vecOneData);
        					vecOneData=new Vector<residueCS>(); 
         					break stop;
         				}
     					continue;
     				}
    				 
    				st = new StringTokenizer(ss);
    				if (st.hasMoreTokens())
    					st.nextToken();//csID =  Integer.parseInt(st.nextToken());    					
    				if (st.hasMoreTokens())
    					csHA  = (float) new Double(st.nextToken()).doubleValue();    				
    				if (st.hasMoreTokens())
    					csCA  = (float) new Double(st.nextToken()).doubleValue();    				
    				if (st.hasMoreTokens())
    					csCB  = (float) new Double(st.nextToken()).doubleValue();    				
    				if (st.hasMoreTokens())
    					csCO  = (float) new Double(st.nextToken()).doubleValue();    				
    				if (st.hasMoreTokens())
    					csHN  = (float) new Double(st.nextToken()).doubleValue();
    				if (st.hasMoreTokens())
    					csN  = (float) new Double(st.nextToken()).doubleValue();
    				
    				if (st.hasMoreTokens())
    					deltaHBond_ha  = (float) new Double(st.nextToken()).doubleValue();
    				if (st.hasMoreTokens())
    					deltaHBond_hn  = (float) new Double(st.nextToken()).doubleValue();
    				if (st.hasMoreTokens())
    					deltaRing  = (float) new Double(st.nextToken()).doubleValue();
    				if (st.hasMoreTokens())
    					deltaRingCa  = (float) new Double(st.nextToken()).doubleValue();
    				if (st.hasMoreTokens())
    					phi  = (float) new Double(st.nextToken()).doubleValue();
    				if (st.hasMoreTokens())
    					psi  = (float) new Double(st.nextToken()).doubleValue();
    				if (st.hasMoreTokens())
    					phiPrev  = (float) new Double(st.nextToken()).doubleValue();
    				if (st.hasMoreTokens())
    					psiPrev  = (float) new Double(st.nextToken()).doubleValue();    				
    				if (st.hasMoreTokens())
    					phiNext  = (float) new Double(st.nextToken()).doubleValue();
    				if (st.hasMoreTokens())
    					psiNext  = (float) new Double(st.nextToken()).doubleValue();   				
    				
    				if (st.hasMoreTokens())
    					curRes = st.nextToken();    				
    				if (st.hasMoreTokens())
    					no =  Integer.parseInt(st.nextToken());
    				if (st.hasMoreTokens())
    					proteinName = st.nextToken();
    				    	
    				counter++;   				
    				
    				residueCS resCS=new residueCS(no,curRes,proteinName,(float)csHA,(float)csCA,(float)csCB,(float)csCO,(float)csHN,(float)csN,
    	    				(float)deltaHBond_ha,(float)deltaHBond_hn,(float)deltaRing,(float)deltaRingCa,
    	    				(float)phi,(float)psi,(float)phiPrev,(float)psiPrev,(float)phiNext,(float)psiNext);
    				
    				if( (!preProteinName.equalsIgnoreCase(proteinName)) && (!preProteinName.equalsIgnoreCase("") ) ){
    					vecData.add(vecOneData);
    					vecOneData=new Vector<residueCS>(); 
    				}    				
    				
    				vecOneData.add(resCS);
    				
    				preProteinName=proteinName;
    				
    				if ((ss = in.readLine()) == null){
    					vecData.add(vecOneData);
    					vecOneData=new Vector<residueCS>(); 
    					break stop;
    				}
    				else if(ss.equalsIgnoreCase(""))
    				{
    					if ((ss = in.readLine()) == null )
        				{
    						vecData.add(vecOneData);
        					vecOneData=new Vector<residueCS>(); 
        					break stop;
        				}
    				}
    			}
    	}catch (FileNotFoundException e) {
    		System.out.println("File not found: " + fileName);
    	}catch (IOException e) 
    	{
    		System.out.println("IOException: the stack trace is:");
    		e.printStackTrace();
    	}
    	System.out.println("Total number of residue chemical shifts that have been read: "+ counter);
		return vecData;
	}
		
	/**
	 * write the residue chemical shift data to a file.
	 * @throws IOException 
	 */	
	public void writeResdiueCSDataToFileForRAll(Vector<Vector<residueCS>> vecResidueCSData, String fileName) throws IOException{
		PrintWriter out = new PrintWriter(new BufferedWriter(new FileWriter(fileName)));
		int counter=1;
		for(int i=0;i<vecResidueCSData.size();i++){
			Vector<residueCS> vecOneData=vecResidueCSData.elementAt(i);
			for(int j=0;j<vecOneData.size();j++){
				residueCS resCS=vecOneData.elementAt(j);
				float csHA=resCS.getHaCS();
				
				if(csHA>100.0 || csHA <-100.0)//ignore the missing ha
					continue;
				
				float csCA=resCS.getCaCS();
				float csCB=resCS.getCbCS();
				float csCO=resCS.getCoCS();
				float csHN=resCS.getHnCS();
				float csN=resCS.getNCS();
				float deltaHBond_ha=resCS.getHBondEffectHa();
				float deltaHBond_hn=resCS.getHBondEffectHn();
				float deltaRing=resCS.getRingEffectHa();
				float deltaRingCa=resCS.getRingEffectCa();
				float phi=resCS.getPhi();
				float psi=resCS.getPsi();
				float phiPrev=resCS.getPrePhi();
				float psiPrev=resCS.getPrePsi();
				float phiNext=resCS.getNextPhi();
				float psiNext=resCS.getNextPsi();
				String curRes=resCS.getResidueName();
				int no=resCS.getResidueNo();
				String proteinName=resCS.getProteinName();
				
				//get the chemical shifts of the previous residue
				float csPrevHA=-99999.9f;
				float csPrevCA=-99999.9f;
				float csPrevCB=-99999.9f;
				float csPrevCO=-99999.9f;
				float csPrevHN=-99999.9f;
				float csPrevN=-99999.9f;
				int ind= Collections.binarySearch(vecOneData, new residueCS(no-1), new residueCS.residueCSComparator());
	     	    if(ind>0){
	     	    	residueCS preResCS=vecOneData.elementAt(ind);
	     	    	csPrevHA=preResCS.getHaCS();
					csPrevCA=preResCS.getCaCS();
					csPrevCB=preResCS.getCbCS();
					csPrevCO=preResCS.getCoCS();
					csPrevHN=preResCS.getHnCS();
					csPrevN=preResCS.getNCS();	     	    	
	     	    }//if(ind>0)
				
				//get the chemical shifts of the next residue	     	  
	     	    float csNextHA=-99999.9f;
				float csNextCA=-99999.9f;
				float csNextCB=-99999.9f;
				float csNextCO=-99999.9f;
				float csNextHN=-99999.9f;
				float csNextN=-99999.9f;
				ind= Collections.binarySearch(vecOneData, new residueCS(no+1), new residueCS.residueCSComparator());
	     	    if(ind>0){
	     	    	residueCS nextResCS=vecOneData.elementAt(ind);
	     	    	csNextHA=nextResCS.getHaCS();
	     	    	csNextCA=nextResCS.getCaCS();
	     	    	csNextCB=nextResCS.getCbCS();
	     	    	csNextCO=nextResCS.getCoCS();
	     	    	csNextHN=nextResCS.getHnCS();
	     	    	csNextN=nextResCS.getNCS();	     	    	
	     	    }//if(ind>0)
	     	    
	     	    //get the chemical shifts of the second previous residue (i-2)
				float csPrevTwoCA=-99999.9f;
				float csPrevTwoCB=-99999.9f;
				float csPrevTwoCO=-99999.9f;
				float csPrevTwoHN=-99999.9f;
				float csPrevTwoN=-99999.9f;
				ind= Collections.binarySearch(vecOneData, new residueCS(no-2), new residueCS.residueCSComparator());
	     	    if(ind>0){
	     	    	residueCS preResTwoCS=vecOneData.elementAt(ind);	     	    	
					csPrevTwoCA=preResTwoCS.getCaCS();
					csPrevTwoCB=preResTwoCS.getCbCS();
					csPrevTwoCO=preResTwoCS.getCoCS();
					csPrevTwoHN=preResTwoCS.getHnCS();
					csPrevTwoN=preResTwoCS.getNCS();	     	    	
	     	    }//if(ind>0)
	     	    
	     		//get the chemical shifts of the second next residue	     	  
				float csNextTwoCA=-99999.9f;
				float csNextTwoCB=-99999.9f;
				float csNextTwoCO=-99999.9f;
				float csNextTwoHN=-99999.9f;
				float csNextTwoN=-99999.9f;
				ind= Collections.binarySearch(vecOneData, new residueCS(no+2), new residueCS.residueCSComparator());
	     	    if(ind>0){
	     	    	residueCS nextTwoResCS=vecOneData.elementAt(ind);	     	    	
	     	    	csNextTwoCA=nextTwoResCS.getCaCS();
	     	    	csNextTwoCB=nextTwoResCS.getCbCS();
	     	    	csNextTwoCO=nextTwoResCS.getCoCS();
	     	    	csNextTwoHN=nextTwoResCS.getHnCS();
	     	    	csNextTwoN=nextTwoResCS.getNCS();	     	    	
	     	    }//if(ind>0)
	     	    
	     	    //////////////     	  
	     		///////////////////	    		
	    		String str_csHA=csToString((float)(csHA));String str_csCA=csToString((float)(csCA));String str_csCB=csToString((float)(csCB));
	    		String str_csCO=csToString((float)(csCO));String str_csHN=csToString((float)(csHN));String str_csN=csToString((float)(csN));
	    		String str_csPreCA=csToString((float)(csPrevCA));String str_csPreCB=csToString((float)(csPrevCB));String str_csPreCO=csToString((float)(csPrevCO));
	    		String str_csPreHN=csToString((float)(csPrevHN));String str_csPreN=csToString((float)(csPrevN));
	    		String str_csNextCA=csToString((float)(csNextCA));String str_csNextCB=csToString((float)(csNextCB));String str_csNextCO=csToString((float)(csNextCO));
	    		String str_csNextHN=csToString((float)(csNextHN));String str_csNextN=csToString((float)(csNextN));
	    		String str_csPreTwoCA=csToString((float)(csPrevTwoCA));String str_csPreTwoCB=csToString((float)(csPrevTwoCB));String str_csPreTwoCO=csToString((float)(csPrevTwoCO));
	    		String str_csPreTwoHN=csToString((float)(csPrevTwoHN));String str_csPreTwoN=csToString((float)(csPrevTwoN));
	    		String str_csNextTwoCA=csToString((float)(csNextTwoCA));String str_csNextTwoCB=csToString((float)(csNextTwoCB));String str_csNextTwoCO=csToString((float)(csNextTwoCO));
	    		String str_csNextTwoHN=csToString((float)(csNextTwoHN));String str_csNextTwoN=csToString((float)(csNextTwoN));
	    		
	    		out.println(counter+ " "+ str_csHA+ " "+str_csCA+" "+str_csCB+" " +str_csCO+" "+str_csHN+" "+str_csN
	    				+" "+str_csPreCA+" "+str_csPreCB+" " +str_csPreCO+" "+str_csPreHN+" "+str_csPreN
	    				+" "+str_csNextCA+" "+str_csNextCB+" " +str_csNextCO+" "+str_csNextHN+" "+str_csNextN
	    				+" "+str_csPreTwoCA+" "+str_csPreTwoCB+" " +str_csPreTwoCO+" "+str_csPreTwoHN+" "+str_csPreTwoN
	    				+" "+str_csNextTwoCA+" "+str_csNextTwoCB+" " +str_csNextTwoCO+" "+str_csNextTwoHN+" "+str_csNextTwoN
	    				//+" "+(float)(csPreThreeCA)+" "+(float)(csPreThreeCB)+" " +(float)(csPreThreeCO)+" "+(float)(csPreThreeHN)+" "+(float)(csPreThreeN)
	    				//+" "+(float)(csNextThreeCA)+" "+(float)(csNextThreeCB)+" " +(float)(csNextThreeCO)+" "+(float)(csNextThreeHN)+" "+(float)(csNextThreeN)	    				
	    				+ " "+(float)deltaHBond_ha+" "+(float)deltaHBond_hn+" "+ (float)deltaRing+" "+deltaRingCa+" "+ (float)phi
	    				+" "+(float)psi+ " "+ (float)phiPrev+" "+(float)psiPrev +" "+ (float)phiNext+" "+(float)psiNext+" "+curRes+" "+ no+" "+proteinName);
	    		    		
				counter++;
			}//for(int j=0;j<vecOneData.size();j++)
		}//for(int i=0;i<vecResidueCSData.size();i++)
		out.close();
	}
	
	/**
	 * write the residue chemical shift data to a file.
	 * @throws IOException 
	 */	
	public void writeResdiueCSDataToFileForRTwoNeighbors(Vector<Vector<residueCS>> vecResidueCSData, String fileName) throws IOException{
		PrintWriter out = new PrintWriter(new BufferedWriter(new FileWriter(fileName)));
		int counter=1;
		for(int i=0;i<vecResidueCSData.size();i++){
			Vector<residueCS> vecOneData=vecResidueCSData.elementAt(i);
			for(int j=0;j<vecOneData.size();j++){
				residueCS resCS=vecOneData.elementAt(j);
				float csHA=resCS.getHaCS();
				
				if(csHA>100.0 || csHA <-100.0)//ignore the missing ha
					continue;
				
				float csCA=resCS.getCaCS();
				float csCB=resCS.getCbCS();
				float csCO=resCS.getCoCS();
				float csHN=resCS.getHnCS();
				float csN=resCS.getNCS();
				float deltaHBond_ha=resCS.getHBondEffectHa();
				float deltaHBond_hn=resCS.getHBondEffectHn();
				float deltaRing=resCS.getRingEffectHa();
				float deltaRingCa=resCS.getRingEffectCa();
				float phi=resCS.getPhi();
				float psi=resCS.getPsi();
				float phiPrev=resCS.getPrePhi();
				float psiPrev=resCS.getPrePsi();
				float phiNext=resCS.getNextPhi();
				float psiNext=resCS.getNextPsi();
				String curRes=resCS.getResidueName();
				int no=resCS.getResidueNo();
				String proteinName=resCS.getProteinName();
				
				//get the chemical shifts of the previous residue
				float csPrevHA=-99999.9f;
				float csPrevCA=-99999.9f;
				float csPrevCB=-99999.9f;
				float csPrevCO=-99999.9f;
				float csPrevHN=-99999.9f;
				float csPrevN=-99999.9f;
				int ind= Collections.binarySearch(vecOneData, new residueCS(no-1), new residueCS.residueCSComparator());
	     	    if(ind>0){
	     	    	residueCS preResCS=vecOneData.elementAt(ind);
	     	    	csPrevHA=preResCS.getHaCS();
					csPrevCA=preResCS.getCaCS();
					csPrevCB=preResCS.getCbCS();
					csPrevCO=preResCS.getCoCS();
					csPrevHN=preResCS.getHnCS();
					csPrevN=preResCS.getNCS();	     	    	
	     	    }//if(ind>0)
				
				//get the chemical shifts of the next residue
	     	    float csNextHA=-99999.9f;
				float csNextCA=-99999.9f;
				float csNextCB=-99999.9f;
				float csNextCO=-99999.9f;
				float csNextHN=-99999.9f;
				float csNextN=-99999.9f;
				ind= Collections.binarySearch(vecOneData, new residueCS(no+1), new residueCS.residueCSComparator());
	     	    if(ind>0){
	     	    	residueCS nextResCS=vecOneData.elementAt(ind);
	     	    	csNextHA=nextResCS.getHaCS();
	     	    	csNextCA=nextResCS.getCaCS();
	     	    	csNextCB=nextResCS.getCbCS();
	     	    	csNextCO=nextResCS.getCoCS();
	     	    	csNextHN=nextResCS.getHnCS();
	     	    	csNextN=nextResCS.getNCS();	     	    	
	     	    }//if(ind>0)
	     	    
	     	    //get the chemical shifts of the second previous residue (i-2)
				float csPrevTwoHA=-99999.9f;
				float csPrevTwoCA=-99999.9f;
				float csPrevTwoCB=-99999.9f;
				float csPrevTwoCO=-99999.9f;
				float csPrevTwoHN=-99999.9f;
				float csPrevTwoN=-99999.9f;
				ind= Collections.binarySearch(vecOneData, new residueCS(no-2), new residueCS.residueCSComparator());
	     	    if(ind>0){
	     	    	residueCS preResTwoCS=vecOneData.elementAt(ind);
	     	    	csPrevTwoHA=preResTwoCS.getHaCS();
					csPrevTwoCA=preResTwoCS.getCaCS();
					csPrevTwoCB=preResTwoCS.getCbCS();
					csPrevTwoCO=preResTwoCS.getCoCS();
					csPrevTwoHN=preResTwoCS.getHnCS();
					csPrevTwoN=preResTwoCS.getNCS();	     	    	
	     	    }//if(ind>0)
	     	    
	     		//get the chemical shifts of the second next residue
	     	    float csNextTwoHA=-99999.9f;
				float csNextTwoCA=-99999.9f;
				float csNextTwoCB=-99999.9f;
				float csNextTwoCO=-99999.9f;
				float csNextTwoHN=-99999.9f;
				float csNextTwoN=-99999.9f;
				ind= Collections.binarySearch(vecOneData, new residueCS(no+2), new residueCS.residueCSComparator());
	     	    if(ind>0){
	     	    	residueCS nextTwoResCS=vecOneData.elementAt(ind);
	     	    	csNextTwoHA=nextTwoResCS.getHaCS();
	     	    	csNextTwoCA=nextTwoResCS.getCaCS();
	     	    	csNextTwoCB=nextTwoResCS.getCbCS();
	     	    	csNextTwoCO=nextTwoResCS.getCoCS();
	     	    	csNextTwoHN=nextTwoResCS.getHnCS();
	     	    	csNextTwoN=nextTwoResCS.getNCS();	     	    	
	     	    }//if(ind>0)
	     	    
	     	    //////////////
	     	   if(csCA>100.0 || csCA <-100.0)//ignore the missing chemical shifts
					continue;
	     	   if(csCB>100.0 || csCB <-100.0)//ignore the missing chemical shifts
					continue;
	     	   if(csHN>100.0 || csHN <-100.0)//ignore the missing chemical shifts
					continue;
	     	    	     	   
	     	   if(csPrevCA>100.0 || csPrevCA <-100.0)//ignore the missing chemical shifts
					continue;
	     	   if(csPrevCB>100.0 || csPrevCB <-100.0)//ignore the missing chemical shifts
					continue;
	     	   if(csPrevHN>100.0 || csPrevHN <-100.0)//ignore the missing chemical shifts
					continue;
	     	   
	     	   if(csNextCA>100.0 || csNextCA <-100.0)//ignore the missing chemical shifts
					continue;
	     	   if(csNextCB>100.0 || csNextCB <-100.0)//ignore the missing chemical shifts
					continue;
	     	   if(csNextHN>100.0 || csNextHN <-100.0)//ignore the missing chemical shifts
					continue;
	     	   
	     	  if(csPrevTwoCA>100.0 || csPrevTwoCA <-100.0)//ignore the missing chemical shifts
					continue;
	     	   if(csPrevTwoCB>100.0 || csPrevTwoCB <-100.0)//ignore the missing chemical shifts
					continue;
	     	   if(csPrevTwoHN>100.0 || csPrevTwoHN <-100.0)//ignore the missing chemical shifts
					continue;
	     	   
	     	   if(csNextTwoCA>100.0 || csNextTwoCA <-100.0)//ignore the missing chemical shifts
					continue;
	     	   if(csNextTwoCB>100.0 || csNextTwoCB <-100.0)//ignore the missing chemical shifts
					continue;
	     	   if(csNextTwoHN>100.0 || csNextTwoHN <-100.0)//ignore the missing chemical shifts
					continue;
	     	   
	     	   
	     		///////////////////	    		
	    		String str_csHA=csToString((float)(csHA));String str_csCA=csToString((float)(csCA));String str_csCB=csToString((float)(csCB));
	    		String str_csCO=csToString((float)(csCO));String str_csHN=csToString((float)(csHN));String str_csN=csToString((float)(csN));
	    		String str_csPreCA=csToString((float)(csPrevCA));String str_csPreCB=csToString((float)(csPrevCB));String str_csPreCO=csToString((float)(csPrevCO));
	    		String str_csPreHN=csToString((float)(csPrevHN));String str_csPreN=csToString((float)(csPrevN));
	    		String str_csNextCA=csToString((float)(csNextCA));String str_csNextCB=csToString((float)(csNextCB));String str_csNextCO=csToString((float)(csNextCO));
	    		String str_csNextHN=csToString((float)(csNextHN));String str_csNextN=csToString((float)(csNextN));
	    		String str_csPreTwoCA=csToString((float)(csPrevTwoCA));String str_csPreTwoCB=csToString((float)(csPrevTwoCB));String str_csPreTwoCO=csToString((float)(csPrevTwoCO));
	    		String str_csPreTwoHN=csToString((float)(csPrevTwoHN));String str_csPreTwoN=csToString((float)(csPrevTwoN));
	    		String str_csNextTwoCA=csToString((float)(csNextTwoCA));String str_csNextTwoCB=csToString((float)(csNextTwoCB));String str_csNextTwoCO=csToString((float)(csNextTwoCO));
	    		String str_csNextTwoHN=csToString((float)(csNextTwoHN));String str_csNextTwoN=csToString((float)(csNextTwoN));
	    		
	    		out.println(counter+ " "+ str_csHA+ " "+str_csCA+" "+str_csCB+" " +str_csCO+" "+str_csHN+" "+str_csN
	    				+" "+str_csPreCA+" "+str_csPreCB+" " +str_csPreCO+" "+str_csPreHN+" "+str_csPreN
	    				+" "+str_csNextCA+" "+str_csNextCB+" " +str_csNextCO+" "+str_csNextHN+" "+str_csNextN
	    				+" "+str_csPreTwoCA+" "+str_csPreTwoCB+" " +str_csPreTwoCO+" "+str_csPreTwoHN+" "+str_csPreTwoN
	    				+" "+str_csNextTwoCA+" "+str_csNextTwoCB+" " +str_csNextTwoCO+" "+str_csNextTwoHN+" "+str_csNextTwoN
	    				//+" "+(float)(csPreThreeCA)+" "+(float)(csPreThreeCB)+" " +(float)(csPreThreeCO)+" "+(float)(csPreThreeHN)+" "+(float)(csPreThreeN)
	    				//+" "+(float)(csNextThreeCA)+" "+(float)(csNextThreeCB)+" " +(float)(csNextThreeCO)+" "+(float)(csNextThreeHN)+" "+(float)(csNextThreeN)	    				
	    				+ " "+(float)deltaHBond_ha+" "+(float)deltaHBond_hn+" "+ (float)deltaRing+" "+deltaRingCa+" "+ (float)phi
	    				+" "+(float)psi+ " "+ (float)phiPrev+" "+(float)psiPrev +" "+ (float)phiNext+" "+(float)psiNext+" "+curRes+" "+ no+" "+proteinName);
	    		    		
				counter++;
			}//for(int j=0;j<vecOneData.size();j++)
		}//for(int i=0;i<vecResidueCSData.size();i++)
		out.close();
	}
	
	/**
	 * write the residue chemical shift data to a file.
	 * @throws IOException 
	 */	
	public void writeResdiueCSDataToFileForROneNeighbor(Vector<Vector<residueCS>> vecResidueCSData, String fileName) throws IOException{
		PrintWriter out = new PrintWriter(new BufferedWriter(new FileWriter(fileName)));
		int counter=1;
		for(int i=0;i<vecResidueCSData.size();i++){
			Vector<residueCS> vecOneData=vecResidueCSData.elementAt(i);
			for(int j=0;j<vecOneData.size();j++){
				residueCS resCS=vecOneData.elementAt(j);
				float csHA=resCS.getHaCS();
				
				if(csHA>100.0 || csHA <-100.0)//ignore the missing ha
					continue;
				
				float csCA=resCS.getCaCS();
				float csCB=resCS.getCbCS();
				float csCO=resCS.getCoCS();
				float csHN=resCS.getHnCS();
				float csN=resCS.getNCS();
				float deltaHBond_ha=resCS.getHBondEffectHa();
				float deltaHBond_hn=resCS.getHBondEffectHn();
				float deltaRing=resCS.getRingEffectHa();
				float deltaRingCa=resCS.getRingEffectCa();
				float phi=resCS.getPhi();
				float psi=resCS.getPsi();
				float phiPrev=resCS.getPrePhi();
				float psiPrev=resCS.getPrePsi();
				float phiNext=resCS.getNextPhi();
				float psiNext=resCS.getNextPsi();
				String curRes=resCS.getResidueName();
				int no=resCS.getResidueNo();
				String proteinName=resCS.getProteinName();
				
				//get the chemical shifts of the previous residue
				float csPrevHA=-99999.9f;
				float csPrevCA=-99999.9f;
				float csPrevCB=-99999.9f;
				float csPrevCO=-99999.9f;
				float csPrevHN=-99999.9f;
				float csPrevN=-99999.9f;
				int ind= Collections.binarySearch(vecOneData, new residueCS(no-1), new residueCS.residueCSComparator());
	     	    if(ind>0){
	     	    	residueCS preResCS=vecOneData.elementAt(ind);
	     	    	csPrevHA=preResCS.getHaCS();
					csPrevCA=preResCS.getCaCS();
					csPrevCB=preResCS.getCbCS();
					csPrevCO=preResCS.getCoCS();
					csPrevHN=preResCS.getHnCS();
					csPrevN=preResCS.getNCS();	     	    	
	     	    }//if(ind>0)
				
				//get the chemical shifts of the next residue
	     	    float csNextHA=-99999.9f;
				float csNextCA=-99999.9f;
				float csNextCB=-99999.9f;
				float csNextCO=-99999.9f;
				float csNextHN=-99999.9f;
				float csNextN=-99999.9f;
				ind= Collections.binarySearch(vecOneData, new residueCS(no+1), new residueCS.residueCSComparator());
	     	    if(ind>0){
	     	    	residueCS nextResCS=vecOneData.elementAt(ind);
	     	    	csNextHA=nextResCS.getHaCS();
	     	    	csNextCA=nextResCS.getCaCS();
	     	    	csNextCB=nextResCS.getCbCS();
	     	    	csNextCO=nextResCS.getCoCS();
	     	    	csNextHN=nextResCS.getHnCS();
	     	    	csNextN=nextResCS.getNCS();	     	    	
	     	    }//if(ind>0)
	     	    
	     	    //get the chemical shifts of the second previous residue (i-2)
				float csPrevTwoHA=-99999.9f;
				float csPrevTwoCA=-99999.9f;
				float csPrevTwoCB=-99999.9f;
				float csPrevTwoCO=-99999.9f;
				float csPrevTwoHN=-99999.9f;
				float csPrevTwoN=-99999.9f;
				ind= Collections.binarySearch(vecOneData, new residueCS(no-2), new residueCS.residueCSComparator());
	     	    if(ind>0){
	     	    	residueCS preResTwoCS=vecOneData.elementAt(ind);
	     	    	csPrevTwoHA=preResTwoCS.getHaCS();
					csPrevTwoCA=preResTwoCS.getCaCS();
					csPrevTwoCB=preResTwoCS.getCbCS();
					csPrevTwoCO=preResTwoCS.getCoCS();
					csPrevTwoHN=preResTwoCS.getHnCS();
					csPrevTwoN=preResTwoCS.getNCS();	     	    	
	     	    }//if(ind>0)
	     	    
	     		//get the chemical shifts of the second next residue
	     	    float csNextTwoHA=-99999.9f;
				float csNextTwoCA=-99999.9f;
				float csNextTwoCB=-99999.9f;
				float csNextTwoCO=-99999.9f;
				float csNextTwoHN=-99999.9f;
				float csNextTwoN=-99999.9f;
				ind= Collections.binarySearch(vecOneData, new residueCS(no+2), new residueCS.residueCSComparator());
	     	    if(ind>0){
	     	    	residueCS nextTwoResCS=vecOneData.elementAt(ind);
	     	    	csNextTwoHA=nextTwoResCS.getHaCS();
	     	    	csNextTwoCA=nextTwoResCS.getCaCS();
	     	    	csNextTwoCB=nextTwoResCS.getCbCS();
	     	    	csNextTwoCO=nextTwoResCS.getCoCS();
	     	    	csNextTwoHN=nextTwoResCS.getHnCS();
	     	    	csNextTwoN=nextTwoResCS.getNCS();	     	    	
	     	    }//if(ind>0)
	     	    
	     	    //////////////
	     	   if(csCA>100.0 || csCA <-100.0)//ignore the missing chemical shifts
					continue;
	     	   if(csCB>100.0 || csCB <-100.0)//ignore the missing chemical shifts
					continue;
	     	   if(csHN>100.0 || csHN <-100.0)//ignore the missing chemical shifts
					continue;
	     	    	     	   
	     	   if(csPrevCA>100.0 || csPrevCA <-100.0)//ignore the missing chemical shifts
					continue;
	     	   if(csPrevCB>100.0 || csPrevCB <-100.0)//ignore the missing chemical shifts
					continue;
	     	   if(csPrevHN>100.0 || csPrevHN <-100.0)//ignore the missing chemical shifts
					continue;
	     	   
	     	   if(csNextCA>100.0 || csNextCA <-100.0)//ignore the missing chemical shifts
					continue;
	     	   if(csNextCB>100.0 || csNextCB <-100.0)//ignore the missing chemical shifts
					continue;
	     	   if(csNextHN>100.0 || csNextHN <-100.0)//ignore the missing chemical shifts
					continue;
	     	   
	     	  	     	   
	     	   
	     		///////////////////	    		
	    		String str_csHA=csToString((float)(csHA));String str_csCA=csToString((float)(csCA));String str_csCB=csToString((float)(csCB));
	    		String str_csCO=csToString((float)(csCO));String str_csHN=csToString((float)(csHN));String str_csN=csToString((float)(csN));
	    		String str_csPreCA=csToString((float)(csPrevCA));String str_csPreCB=csToString((float)(csPrevCB));String str_csPreCO=csToString((float)(csPrevCO));
	    		String str_csPreHN=csToString((float)(csPrevHN));String str_csPreN=csToString((float)(csPrevN));
	    		String str_csNextCA=csToString((float)(csNextCA));String str_csNextCB=csToString((float)(csNextCB));String str_csNextCO=csToString((float)(csNextCO));
	    		String str_csNextHN=csToString((float)(csNextHN));String str_csNextN=csToString((float)(csNextN));
	    		String str_csPreTwoCA=csToString((float)(csPrevTwoCA));String str_csPreTwoCB=csToString((float)(csPrevTwoCB));String str_csPreTwoCO=csToString((float)(csPrevTwoCO));
	    		String str_csPreTwoHN=csToString((float)(csPrevTwoHN));String str_csPreTwoN=csToString((float)(csPrevTwoN));
	    		String str_csNextTwoCA=csToString((float)(csNextTwoCA));String str_csNextTwoCB=csToString((float)(csNextTwoCB));String str_csNextTwoCO=csToString((float)(csNextTwoCO));
	    		String str_csNextTwoHN=csToString((float)(csNextTwoHN));String str_csNextTwoN=csToString((float)(csNextTwoN));
	    		
	    		out.println(counter+ " "+ str_csHA+ " "+str_csCA+" "+str_csCB+" " +str_csCO+" "+str_csHN+" "+str_csN
	    				+" "+str_csPreCA+" "+str_csPreCB+" " +str_csPreCO+" "+str_csPreHN+" "+str_csPreN
	    				+" "+str_csNextCA+" "+str_csNextCB+" " +str_csNextCO+" "+str_csNextHN+" "+str_csNextN
	    				+" "+str_csPreTwoCA+" "+str_csPreTwoCB+" " +str_csPreTwoCO+" "+str_csPreTwoHN+" "+str_csPreTwoN
	    				+" "+str_csNextTwoCA+" "+str_csNextTwoCB+" " +str_csNextTwoCO+" "+str_csNextTwoHN+" "+str_csNextTwoN
	    				//+" "+(float)(csPreThreeCA)+" "+(float)(csPreThreeCB)+" " +(float)(csPreThreeCO)+" "+(float)(csPreThreeHN)+" "+(float)(csPreThreeN)
	    				//+" "+(float)(csNextThreeCA)+" "+(float)(csNextThreeCB)+" " +(float)(csNextThreeCO)+" "+(float)(csNextThreeHN)+" "+(float)(csNextThreeN)	    				
	    				+ " "+(float)deltaHBond_ha+" "+(float)deltaHBond_hn+" "+ (float)deltaRing+" "+deltaRingCa+" "+ (float)phi
	    				+" "+(float)psi+ " "+ (float)phiPrev+" "+(float)psiPrev +" "+ (float)phiNext+" "+(float)psiNext+" "+curRes+" "+ no+" "+proteinName);
	    		    		
				counter++;
			}//for(int j=0;j<vecOneData.size();j++)
		}//for(int i=0;i<vecResidueCSData.size();i++)
		out.close();
	}
	
	
	/**
	 * write the residue chemical shift data to a file.
	 * @throws IOException 
	 */	
	public void writeResdiueCSDataToFileForRNoNeighbor(Vector<Vector<residueCS>> vecResidueCSData, String fileName) throws IOException{
		PrintWriter out = new PrintWriter(new BufferedWriter(new FileWriter(fileName)));
		int counter=1;
		for(int i=0;i<vecResidueCSData.size();i++){
			Vector<residueCS> vecOneData=vecResidueCSData.elementAt(i);
			for(int j=0;j<vecOneData.size();j++){
				residueCS resCS=vecOneData.elementAt(j);
				float csHA=resCS.getHaCS();
				
				if(csHA>100.0 || csHA <-100.0)//ignore the missing ha
					continue;
				
				float csCA=resCS.getCaCS();
				float csCB=resCS.getCbCS();
				float csCO=resCS.getCoCS();
				float csHN=resCS.getHnCS();
				float csN=resCS.getNCS();
				float deltaHBond_ha=resCS.getHBondEffectHa();
				float deltaHBond_hn=resCS.getHBondEffectHn();
				float deltaRing=resCS.getRingEffectHa();
				float deltaRingCa=resCS.getRingEffectCa();
				float phi=resCS.getPhi();
				float psi=resCS.getPsi();
				float phiPrev=resCS.getPrePhi();
				float psiPrev=resCS.getPrePsi();
				float phiNext=resCS.getNextPhi();
				float psiNext=resCS.getNextPsi();
				String curRes=resCS.getResidueName();
				int no=resCS.getResidueNo();
				String proteinName=resCS.getProteinName();
				
				//get the chemical shifts of the previous residue
				float csPrevHA=-99999.9f;
				float csPrevCA=-99999.9f;
				float csPrevCB=-99999.9f;
				float csPrevCO=-99999.9f;
				float csPrevHN=-99999.9f;
				float csPrevN=-99999.9f;
				int ind= Collections.binarySearch(vecOneData, new residueCS(no-1), new residueCS.residueCSComparator());
	     	    if(ind>0){
	     	    	residueCS preResCS=vecOneData.elementAt(ind);
	     	    	csPrevHA=preResCS.getHaCS();
					csPrevCA=preResCS.getCaCS();
					csPrevCB=preResCS.getCbCS();
					csPrevCO=preResCS.getCoCS();
					csPrevHN=preResCS.getHnCS();
					csPrevN=preResCS.getNCS();	     	    	
	     	    }//if(ind>0)
				
				//get the chemical shifts of the next residue
	     	    float csNextHA=-99999.9f;
				float csNextCA=-99999.9f;
				float csNextCB=-99999.9f;
				float csNextCO=-99999.9f;
				float csNextHN=-99999.9f;
				float csNextN=-99999.9f;
				ind= Collections.binarySearch(vecOneData, new residueCS(no+1), new residueCS.residueCSComparator());
	     	    if(ind>0){
	     	    	residueCS nextResCS=vecOneData.elementAt(ind);
	     	    	csNextHA=nextResCS.getHaCS();
	     	    	csNextCA=nextResCS.getCaCS();
	     	    	csNextCB=nextResCS.getCbCS();
	     	    	csNextCO=nextResCS.getCoCS();
	     	    	csNextHN=nextResCS.getHnCS();
	     	    	csNextN=nextResCS.getNCS();	     	    	
	     	    }//if(ind>0)
	     	    
	     	    //get the chemical shifts of the second previous residue (i-2)
				float csPrevTwoHA=-99999.9f;
				float csPrevTwoCA=-99999.9f;
				float csPrevTwoCB=-99999.9f;
				float csPrevTwoCO=-99999.9f;
				float csPrevTwoHN=-99999.9f;
				float csPrevTwoN=-99999.9f;
				ind= Collections.binarySearch(vecOneData, new residueCS(no-2), new residueCS.residueCSComparator());
	     	    if(ind>0){
	     	    	residueCS preResTwoCS=vecOneData.elementAt(ind);
	     	    	csPrevTwoHA=preResTwoCS.getHaCS();
					csPrevTwoCA=preResTwoCS.getCaCS();
					csPrevTwoCB=preResTwoCS.getCbCS();
					csPrevTwoCO=preResTwoCS.getCoCS();
					csPrevTwoHN=preResTwoCS.getHnCS();
					csPrevTwoN=preResTwoCS.getNCS();	     	    	
	     	    }//if(ind>0)
	     	    
	     		//get the chemical shifts of the second next residue
	     	    float csNextTwoHA=-99999.9f;
				float csNextTwoCA=-99999.9f;
				float csNextTwoCB=-99999.9f;
				float csNextTwoCO=-99999.9f;
				float csNextTwoHN=-99999.9f;
				float csNextTwoN=-99999.9f;
				ind= Collections.binarySearch(vecOneData, new residueCS(no+2), new residueCS.residueCSComparator());
	     	    if(ind>0){
	     	    	residueCS nextTwoResCS=vecOneData.elementAt(ind);
	     	    	csNextTwoHA=nextTwoResCS.getHaCS();
	     	    	csNextTwoCA=nextTwoResCS.getCaCS();
	     	    	csNextTwoCB=nextTwoResCS.getCbCS();
	     	    	csNextTwoCO=nextTwoResCS.getCoCS();
	     	    	csNextTwoHN=nextTwoResCS.getHnCS();
	     	    	csNextTwoN=nextTwoResCS.getNCS();	     	    	
	     	    }//if(ind>0)
	     	    
	     	    //////////////
	     	   if(csCA>100.0 || csCA <-100.0)//ignore the missing chemical shifts
					continue;
	     	   if(csCB>100.0 || csCB <-100.0)//ignore the missing chemical shifts
					continue;
	     	   if(csHN>100.0 || csHN <-100.0)//ignore the missing chemical shifts
					continue;	     	   
	     	  	     	   
	     	   ///////////////////	    		
	    		String str_csHA=csToString((float)(csHA));String str_csCA=csToString((float)(csCA));String str_csCB=csToString((float)(csCB));
	    		String str_csCO=csToString((float)(csCO));String str_csHN=csToString((float)(csHN));String str_csN=csToString((float)(csN));
	    		String str_csPreCA=csToString((float)(csPrevCA));String str_csPreCB=csToString((float)(csPrevCB));String str_csPreCO=csToString((float)(csPrevCO));
	    		String str_csPreHN=csToString((float)(csPrevHN));String str_csPreN=csToString((float)(csPrevN));
	    		String str_csNextCA=csToString((float)(csNextCA));String str_csNextCB=csToString((float)(csNextCB));String str_csNextCO=csToString((float)(csNextCO));
	    		String str_csNextHN=csToString((float)(csNextHN));String str_csNextN=csToString((float)(csNextN));
	    		String str_csPreTwoCA=csToString((float)(csPrevTwoCA));String str_csPreTwoCB=csToString((float)(csPrevTwoCB));String str_csPreTwoCO=csToString((float)(csPrevTwoCO));
	    		String str_csPreTwoHN=csToString((float)(csPrevTwoHN));String str_csPreTwoN=csToString((float)(csPrevTwoN));
	    		String str_csNextTwoCA=csToString((float)(csNextTwoCA));String str_csNextTwoCB=csToString((float)(csNextTwoCB));String str_csNextTwoCO=csToString((float)(csNextTwoCO));
	    		String str_csNextTwoHN=csToString((float)(csNextTwoHN));String str_csNextTwoN=csToString((float)(csNextTwoN));
	    		
	    		out.println(counter+ " "+ str_csHA+ " "+str_csCA+" "+str_csCB+" " +str_csCO+" "+str_csHN+" "+str_csN
	    				+" "+str_csPreCA+" "+str_csPreCB+" " +str_csPreCO+" "+str_csPreHN+" "+str_csPreN
	    				+" "+str_csNextCA+" "+str_csNextCB+" " +str_csNextCO+" "+str_csNextHN+" "+str_csNextN
	    				+" "+str_csPreTwoCA+" "+str_csPreTwoCB+" " +str_csPreTwoCO+" "+str_csPreTwoHN+" "+str_csPreTwoN
	    				+" "+str_csNextTwoCA+" "+str_csNextTwoCB+" " +str_csNextTwoCO+" "+str_csNextTwoHN+" "+str_csNextTwoN
	    				//+" "+(float)(csPreThreeCA)+" "+(float)(csPreThreeCB)+" " +(float)(csPreThreeCO)+" "+(float)(csPreThreeHN)+" "+(float)(csPreThreeN)
	    				//+" "+(float)(csNextThreeCA)+" "+(float)(csNextThreeCB)+" " +(float)(csNextThreeCO)+" "+(float)(csNextThreeHN)+" "+(float)(csNextThreeN)	    				
	    				+ " "+(float)deltaHBond_ha+" "+(float)deltaHBond_hn+" "+ (float)deltaRing+" "+deltaRingCa+" "+ (float)phi
	    				+" "+(float)psi+ " "+ (float)phiPrev+" "+(float)psiPrev +" "+ (float)phiNext+" "+(float)psiNext+" "+curRes+" "+ no+" "+proteinName);
	    		    		
				counter++;
			}//for(int j=0;j<vecOneData.size();j++)
		}//for(int i=0;i<vecResidueCSData.size();i++)
		out.close();
	}
	
	
	private static String csToString(float cs){
		String str_cs="";
		if(cs>-100 && cs< 100)
			str_cs=Float.toString( (float)(cs));
		else
			str_cs="0.0";//str_cs="NA";
		
		//str_cs=Float.toString( (float)(cs));
		
		return str_cs;
	}
	

	
	
	/**
	 * write the residue chemical shift data to a file.
	 * @throws IOException 
	 */
	public void writeResdiueCSDataToFile(Vector<Vector<residueCS>> vecResidueCSData, String fileName) throws IOException{
		PrintWriter out = new PrintWriter(new BufferedWriter(new FileWriter(fileName)));
		int counter=1;
		for(int i=0;i<vecResidueCSData.size();i++){
			Vector<residueCS> vecOneData=vecResidueCSData.elementAt(i);
			for(int j=0;j<vecOneData.size();j++){
				residueCS resCS=vecOneData.elementAt(j);
				float csHA=resCS.getHaCS();
				float csCA=resCS.getCaCS();
				float csCB=resCS.getCbCS();
				float csCO=resCS.getCoCS();
				float csHN=resCS.getHnCS();
				float csN=resCS.getNCS();
				float deltaHBond_ha=resCS.getHBondEffectHa();
				float deltaHBond_hn=resCS.getHBondEffectHn();
				float deltaRing=resCS.getRingEffectHa();
				float deltaRingCa=resCS.getRingEffectCa();
				float phi=resCS.getPhi();
				float psi=resCS.getPsi();
				float phiPrev=resCS.getPrePhi();
				float psiPrev=resCS.getPrePsi();
				float phiNext=resCS.getNextPhi();
				float psiNext=resCS.getNextPsi();
				String curRes=resCS.getResidueName();
				int no=resCS.getResidueNo();
				String proteinName=resCS.getProteinName();
				
				out.println(counter+ " "+ (float)(csHA)+ " "+(float)(csCA)+" "+(float)(csCB)+" " +(float)(csCO)
	    				+" "+(float)(csHN)+" "+(float)(csN)+ " "+(float)deltaHBond_ha+" "+(float)deltaHBond_hn+" "+ (float)deltaRing+" "+deltaRingCa+" "+ (float)phi
	    				+" "+(float)psi+ " "+ (float)phiPrev+" "+(float)psiPrev +" "+ (float)phiNext+" "+(float)psiNext+" "+curRes+" "+ no+" "+proteinName);				
				counter++;
			}//for(int j=0;j<vecOneData.size();j++)
		}//for(int i=0;i<vecResidueCSData.size();i++)
		out.close();
	}
	
	public static double getPearsonCorrelation(double[] scores1,double[] scores2){
	        double result = 0;
	        double sum_sq_x = 0;
	        double sum_sq_y = 0;
	        double sum_coproduct = 0;
	        double mean_x = scores1[0];
	        double mean_y = scores2[0];
	        for(int i=2;i<scores1.length+1;i+=1){
	            double sweep =Double.valueOf(i-1)/i;
	            double delta_x = scores1[i-1]-mean_x;
	            double delta_y = scores2[i-1]-mean_y;
	            sum_sq_x += delta_x * delta_x * sweep;
	            sum_sq_y += delta_y * delta_y * sweep;
	            sum_coproduct += delta_x * delta_y * sweep;
	            mean_x += delta_x / i;
	            mean_y += delta_y / i;
	        }
	        double pop_sd_x = (double) Math.sqrt(sum_sq_x/scores1.length);
	        double pop_sd_y = (double) Math.sqrt(sum_sq_y/scores1.length);
	        double cov_x_y = sum_coproduct / scores1.length;
	        result = cov_x_y / (pop_sd_x*pop_sd_y);
	        return result;
	    }
	 
	public static double computePearsonCorrelation(Vector<Vector<residueCS>> vecTestData){
		
		int counter=0;
	       
		for(int i=0;i<vecTestData.size();i++){
			Vector<residueCS> vecOneTestData=vecTestData.elementAt(i);			
			
			for(int j=0;j<vecOneTestData.size();j++){
				residueCS curTestResCS=vecOneTestData.elementAt(j);
				float haCS=curTestResCS.getHaCS();
				float haCSPredicted=curTestResCS.getPredictedHaCS();
				if(haCS>100.0 || haCS < -100.0)
					continue;
				
				
				if(haCSPredicted<-100.0 ||  haCSPredicted>100.0)
					continue;
				
				counter++;
			}//for(int j=0;j<vecOneTestData.size();j++)

		}//for(int i=0;i<vecTestData.size();i++)
		
		double[] haOrigin=new double[counter];
		double[] haPredict=new double[counter];
		counter=0;
		for(int i=0;i<vecTestData.size();i++){
			Vector<residueCS> vecOneTestData=vecTestData.elementAt(i);			
			
			for(int j=0;j<vecOneTestData.size();j++){
				residueCS curTestResCS=vecOneTestData.elementAt(j);
				float haCS=curTestResCS.getHaCS();
				float haCSPredicted=curTestResCS.getPredictedHaCS();
				if(haCS>100.0 || haCS < -100.0)
					continue;
				
				
				if(haCSPredicted<-100.0 ||  haCSPredicted>100.0)
					continue;
				
				haOrigin[counter]=haCS;
				haPredict[counter]=haCSPredicted;	
				counter++;
			}//for(int j=0;j<vecOneTestData.size();j++)

		}//for(int i=0;i<vecTestData.size();i++)
		
		double r=getPearsonCorrelation(haOrigin,haPredict);
		
		return r;
    }
	
	
	 
	//
	/**summarize the prediction result.
	 * 
	 */
	public void haPredictionSummary(Vector<Vector<residueCS>> vecTestData){
		System.out.println("Here is the summary of the prediction: ");
		int counter=0;
		int numPredicted=0;
		float sum=0.0f;
		for(int i=0;i<vecTestData.size();i++){
			Vector<residueCS> vecOneTestData=vecTestData.elementAt(i);			
			
			for(int j=0;j<vecOneTestData.size();j++){
				residueCS curTestResCS=vecOneTestData.elementAt(j);
				float haCS=curTestResCS.getHaCS();
				float haCSPredicted=curTestResCS.getPredictedHaCS();
				if(haCS>100.0 || haCS < -100.0)
					continue;
				
				counter++;
				if(haCSPredicted>-100.0 && haCSPredicted<100.0){
					numPredicted++;
					sum=sum+(haCS-haCSPredicted)*(haCS-haCSPredicted);
					
					//System.out.println("Tested Ha: "+ haCS);
				}			
				else
					System.out.println("Not matched: "+ curTestResCS.getProteinName()+" - "+ curTestResCS.getResidueNo());
			}//for(int j=0;j<vecOneTestData.size();j++)
		}//for(int i=0;i<vecTestData.size();i++)
		
		
		
		float rmsd=(float)Math.sqrt(sum/numPredicted);
		
		System.out.println("The total number of testing data points for Ha chemical shifts is: "+counter);
		System.out.println("The total number of predicted testing data points is: "+numPredicted);
		System.out.println("Percentage of predicted testing data points is: "+(float)((float)numPredicted/(float)counter));
		System.out.println("The rmsd is: "+rmsd);
		
		double r=computePearsonCorrelation(vecTestData);
		System.out.println("The correlation between experimental and predicted shifts is: "+r);
	}
	
	/**print out the prediction result.
	 * 
	 */
	public void haPredictionPrintOut(Vector<Vector<residueCS>> vecTestData){
		System.out.println("Here is the list of the prediction: (predicted reference) format");
		int counter=0;
		int numPredicted=0;
		float sum=0.0f;
		for(int i=0;i<vecTestData.size();i++){
			Vector<residueCS> vecOneTestData=vecTestData.elementAt(i);			
			
			for(int j=0;j<vecOneTestData.size();j++){
				residueCS curTestResCS=vecOneTestData.elementAt(j);
				float haCS=curTestResCS.getHaCS();
				float haCSPredicted=curTestResCS.getPredictedHaCS();
				if(haCS>100.0 || haCS < -100.0)
					continue;
				
				counter++;
				if(haCSPredicted>-100.0 && haCSPredicted<100.0){
					numPredicted++;
					sum=sum+(haCS-haCSPredicted)*(haCS-haCSPredicted);
					
				    System.out.println(haCSPredicted+"  "+ haCS);
					
					//System.out.println("Tested Ha: "+ haCS);
				}			
				//else
					//System.out.println("Not matched: "+ curTestResCS.getProteinName()+" - "+ curTestResCS.getResidueNo());
			}//for(int j=0;j<vecOneTestData.size();j++)
		}//for(int i=0;i<vecTestData.size();i++)		
		
		
	}

	/**Find the best matched Ha chemical shifts using a residue window.
	 * 
	 * @param vecTrainData: training data
	 * @param vecTestData: testing data
	 * @param resWindow: residue window 
	 * @param hErr,heavyErr: error windows for protons and heavy atoms, such as carbon and nitrogen atoms
	 * @param isCaUsed: whether the ca chemical shift is used to find the best match
	 */
	public void computeBestMatchedHaCS(Vector<Vector<residueCS>> vecTestData,Vector<Vector<residueCS>> vecTrainData,int resWindow,
			float hErr, float heavyErr,	boolean isCaUsed,boolean isCbUsed,boolean isCoUsed,boolean isHnUsed,boolean isNUsed){
		
		for(int i=0;i<vecTestData.size();i++){
			Vector<residueCS> vecOneTestData=vecTestData.elementAt(i);			
			
			for(int j=0;j<vecOneTestData.size();j++){
				residueCS curTestResCS=vecOneTestData.elementAt(j);
				float minRmsd=99999.9f;
				residueCS matchedTrainResCS=null;
				
				for(int h=0;h<vecTrainData.size();h++){
					Vector<residueCS> vecOneTrainData=vecTrainData.elementAt(h);
					
					for(int k=0;k<vecOneTrainData.size();k++){
						residueCS curTrainResCS=vecOneTrainData.elementAt(k);
						float rmsd=computOneToOneMatchScore(curTestResCS,curTrainResCS,vecOneTestData,vecOneTrainData,resWindow,
								hErr, heavyErr,	isCaUsed, isCbUsed,isCoUsed,isHnUsed,isNUsed);
						
						if(rmsd<minRmsd){
							minRmsd=rmsd;
							matchedTrainResCS=curTrainResCS;
						}
					}//for(int k=0;k<vecOneTrainData.size();k++)					
				}//for(int h=0;h<vecTrainData.size();h++)
												
				if(minRmsd<100.0){
					float matchedHaCS=matchedTrainResCS.getHaCS();
					curTestResCS.setPredictedHaCS(matchedHaCS);
				}
				
			}//for(int j=0;j<vecOneTestData.size();j++)			
		}//for(int i=0;i<vecTestData.size();i++)
	}
	
	/**Find the best matched Ha chemical shifts using a residue window.
	 * 
	 * @param vecTrainData: training data
	 * @param vecTestData: testing data
	 * @param resWindow: residue window 
	 * @param hErr,heavyErr: error windows for protons and heavy atoms, such as carbon and nitrogen atoms
	 * @param isCaUsed: whether the ca chemical shift is used to find the best match
	 */
	public void computeBestMatchedHaCSHDistance(Vector<Vector<residueCS>> vecTestData,Vector<Vector<residueCS>> vecTrainData,int resWindow,
			float hErr, float heavyErr){
		
		float[] rmsd=new float[1];		
	
		for(int i=0;i<vecTestData.size();i++){
			Vector<residueCS> vecOneTestData=vecTestData.elementAt(i);			
		
			
			for(int j=0;j<vecOneTestData.size();j++){
				residueCS curTestResCS=vecOneTestData.elementAt(j);
					
				
				float haTest=curTestResCS.getHaCS();
				if(haTest>100.0 || haTest<-100.0)
					continue;
				String resNameTest=curTestResCS.getResidueName();//double check later...
				
				float minRmsd=99999.9f;
				int maxMatchNum=0;
				residueCS matchedTrainResCS=null;
				
				for(int h=0;h<vecTrainData.size();h++){
					Vector<residueCS> vecOneTrainData=vecTrainData.elementAt(h);
					
					for(int k=0;k<vecOneTrainData.size();k++){
						residueCS curTrainResCS=vecOneTrainData.elementAt(k);
					
						
					//if(curTrainResCS.getProteinName().equalsIgnoreCase("bmr4032") && curTrainResCS.getResidueNo()==4)
						//System.out.println("debugging...training...");
							
						
						
						float haTrain=curTrainResCS.getHaCS();
						if(haTrain>100.0 || haTrain<-100.0)
							continue;
						String resName=curTrainResCS.getResidueName();
						
						
						if(!resNameTest.equalsIgnoreCase(resName))
							continue;
					
						
						
													
						//int numMatch=computOneToOneMatchScoreHDistance(curTestResCS,curTrainResCS,vecOneTestData,vecOneTrainData,resWindow,
							///	hErr, heavyErr,rmsd);
						//int numMatch=computOneToOneMatchScoreHDistanceVariedResWindows(curTestResCS,curTrainResCS,vecOneTestData,vecOneTrainData,resWindow,
								//hErr, heavyErr,rmsd);					
						int numMatch=computOneToOneMatchScoreTwoPerResidueVariedResWindows(curTestResCS,curTrainResCS,vecOneTestData,vecOneTrainData,resWindow,
								hErr, heavyErr,rmsd);					
						
						
					
						if(numMatch<=0)
							continue;
						if(numMatch>maxMatchNum){
							maxMatchNum=numMatch;
							minRmsd=rmsd[0];
							matchedTrainResCS=curTrainResCS;
						}//if(numMatch>maxMatchNum)
						
						if(numMatch==maxMatchNum){
							if(rmsd[0]<minRmsd){
								minRmsd=rmsd[0];
								matchedTrainResCS=curTrainResCS;
							}
						}//if(numMatch==maxMatchNum)
						
					}//for(int k=0;k<vecOneTrainData.size();k++)					
				}//for(int h=0;h<vecTrainData.size();h++)
					
				
		//		if(curTestResCS.getProteinName().equalsIgnoreCase("A001") && curTestResCS.getResidueNo()==65){
			//		System.out.println("debugging...the predicted ha for 65 residue is: "+ (float)debugSum/(float)(debugCounter));
		//		}
				
				
				
				int numMatchCutOff=2;
				if(curTestResCS.getResidueName().equalsIgnoreCase("PRO")||curTestResCS.getResidueName().equalsIgnoreCase("GLY"))
					numMatchCutOff=2;
				
				if(maxMatchNum>=numMatchCutOff){
					float matchedHaCS=matchedTrainResCS.getHaCS();
					curTestResCS.setPredictedHaCS(matchedHaCS);
					
					System.out.println("Match: "+ curTestResCS.getProteinName()+" - "+ curTestResCS.getResidueNo() +
							"(testing) vs. "+ matchedTrainResCS.getProteinName()+" - "+ matchedTrainResCS.getResidueNo()+ "(training)."+
							"original ha: "+curTestResCS.getHaCS() +", predicted ha: " +matchedTrainResCS.getHaCS()+
							"  difference= "+(curTestResCS.getHaCS() -matchedTrainResCS.getHaCS()));
					if(Math.abs(curTestResCS.getHaCS()-matchedTrainResCS.getHaCS())>0.08)
						System.out.println("Warning...");
					if(!curTestResCS.getResidueName().equalsIgnoreCase(matchedTrainResCS.getResidueName())){
						System.out.println("Here we found a pair of match with different residue name: ");
						System.out.println("Testing: " + curTestResCS.getProteinName()+" - "+ curTestResCS.getResidueNo());
						System.out.println("Training: " + matchedTrainResCS.getProteinName()+" - "+ matchedTrainResCS.getResidueNo());
					}
				}//if(maxMatchNum>=2)
				
			}//for(int j=0;j<vecOneTestData.size();j++)			
		}//for(int i=0;i<vecTestData.size();i++)
	}
	
	/**Find the best matched Ha chemical shifts using a residue window (Using the average value of ensemble of matched chemical shifts).
	 * 
	 * @param vecTrainData: training data
	 * @param vecTestData: testing data
	 * @param resWindow: residue window 
	 * @param hErr,heavyErr: error windows for protons and heavy atoms, such as carbon and nitrogen atoms
	 * @param isCaUsed: whether the ca chemical shift is used to find the best match
	 */
	public void computeBestMatchedHaCSHDistanceAverage(Vector<Vector<residueCS>> vecTestData,Vector<Vector<residueCS>> vecTrainData,int resWindow,
			float hErr, float heavyErr){
		
		float[] rmsd=new float[1];
		
		
		
		for(int i=0;i<vecTestData.size();i++){
			Vector<residueCS> vecOneTestData=vecTestData.elementAt(i);			
		
			
			for(int j=0;j<vecOneTestData.size();j++){
				residueCS curTestResCS=vecOneTestData.elementAt(j);
					
				float csSum=0.0f;///////////////
				int csCounter=0;
				
				float haTest=curTestResCS.getHaCS();
				if(haTest>100.0 || haTest<-100.0)
					continue;
				String resNameTest=curTestResCS.getResidueName();//double check later...
				
				float minRmsd=99999.9f;
				int maxMatchNum=0;
				residueCS matchedTrainResCS=null;
				
				for(int h=0;h<vecTrainData.size();h++){
					Vector<residueCS> vecOneTrainData=vecTrainData.elementAt(h);
					
					for(int k=0;k<vecOneTrainData.size();k++){
						residueCS curTrainResCS=vecOneTrainData.elementAt(k);
					
						
					//if(curTrainResCS.getProteinName().equalsIgnoreCase("bmr4032") && curTrainResCS.getResidueNo()==4)
						//System.out.println("debugging...training...");
							
						
						
						float haTrain=curTrainResCS.getHaCS();
						if(haTrain>100.0 || haTrain<-100.0)
							continue;
						String resName=curTrainResCS.getResidueName();
						if(!resNameTest.equalsIgnoreCase(resName))
							continue;
					
						
						
						if(curTestResCS.getProteinName().equalsIgnoreCase("A001") && curTestResCS.getResidueNo()==119
								&& curTrainResCS.getProteinName().equalsIgnoreCase("R072") && curTrainResCS.getResidueNo()==119)
							System.out.println("debugging...");
							
						//int numMatch=computOneToOneMatchScoreHDistance(curTestResCS,curTrainResCS,vecOneTestData,vecOneTrainData,resWindow,
								//hErr, heavyErr,rmsd);
						////int numMatch=computOneToOneMatchScoreHDistanceVariedResWindows(curTestResCS,curTrainResCS,vecOneTestData,vecOneTrainData,resWindow,
							////	hErr, heavyErr,rmsd);					
						int numMatch=computOneToOneMatchScoreTwoPerResidueVariedResWindows(curTestResCS,curTrainResCS,vecOneTestData,vecOneTrainData,resWindow,
								hErr, heavyErr,rmsd);					
						
						
						if(numMatch>=2*resWindow){
							csCounter++;
							csSum=csSum+curTrainResCS.getHaCS();							
						}
						
						
						
					}//for(int k=0;k<vecOneTrainData.size();k++)					
				}//for(int h=0;h<vecTrainData.size();h++)
					
				if(csCounter>0){
					curTestResCS.setPredictedHaCS((float)csSum/(float)csCounter);
					System.out.println("Match: "+ curTestResCS.getProteinName()+" - "+ curTestResCS.getResidueNo()+ ". Original Ha chemical shift: "+
							curTestResCS.getHaCS()+",  Predicted Ha chemical shift: "+curTestResCS.getPredictedHaCS());
					if(Math.abs(curTestResCS.getHaCS()-curTestResCS.getPredictedHaCS())>0.08)
						System.out.println("Warning...");
				}
				
				
				
			}//for(int j=0;j<vecOneTestData.size();j++)			
		}//for(int i=0;i<vecTestData.size();i++)
	}
	
	
	//called by findBestMatchedHaCS
	/**
	 * compute the match score between a testing data point and a training data point.
	 */
	public float computOneToOneMatchScore(residueCS curTestResCS,residueCS curTrainResCS,Vector<residueCS> vecOneTestData,Vector<residueCS> vecOneTrainData,int resWindow,
			float hErr, float heavyErr,	boolean isCaUsed,boolean isCbUsed,boolean isCoUsed,boolean isHnUsed,boolean isNUsed){		
		float rmsd=99999.9f;
		float sum=0.0f;
		int counter=0;
		int curResTestNo=curTestResCS.getResidueNo();
		int curResTrainNo=curTrainResCS.getResidueNo();
		int minResNo=vecOneTestData.elementAt(0).getResidueNo();
		int maxResNo=vecOneTestData.elementAt(vecOneTestData.size()-1).getResidueNo();
		
		int halfResWin1=(int)(resWindow/2);   //first half of the residue window
		int halfResWin2=resWindow-halfResWin1;//second half of the residue window
		
		if( (curResTestNo-halfResWin1)<minResNo){
			halfResWin1=curResTestNo-minResNo;
			halfResWin2=resWindow-halfResWin1;
		}
		
		if( (curResTestNo+halfResWin2)>maxResNo){
			halfResWin2=maxResNo-curResTestNo;
			halfResWin1=resWindow-halfResWin2;
		}
		
		//debugging
	/*	if( (curResTestNo-halfResWin1)<minResNo){
			return rmsd;
		}
		
		if( (curResTestNo+halfResWin2)>maxResNo){
			return rmsd;
		}*/
		///////////
		
		
		for(int i=-halfResWin1;i<=halfResWin2;i++){
			int noTest=curResTestNo+i;			
			int indTest= Collections.binarySearch(vecOneTestData, new residueCS(noTest), new residueCS.residueCSComparator());
     	    if(indTest<0){
     	    	//System.out.println("Testing Residue not found...");
     	    	return rmsd;
     	    	//System.exit(0);//changed later...
     	    }
     	    residueCS tempResCSTest=vecOneTestData.elementAt(indTest);     	    
     		float csCaTest=tempResCSTest.getCaCS();
    		float csCbTest=tempResCSTest.getCbCS();
    		float csCoTest=tempResCSTest.getCoCS();
    		float csHnTest=tempResCSTest.getHnCS();
    		float csNTest=tempResCSTest.getNCS();    				
    		
     	    int noTrain=curResTrainNo+i;
     	    int indTrain= Collections.binarySearch(vecOneTrainData, new residueCS(noTrain), new residueCS.residueCSComparator());
    	    if(indTrain<0){
    	    	//System.out.println("Training Residue not found...");
    	    	return rmsd;
    	    	//System.exit(0);//changed later...
    	    }
     	    residueCS tempResCSTrain=vecOneTrainData.elementAt(indTrain);
     	    float csCaTrain=tempResCSTrain.getCaCS();
     	    float csCbTrain=tempResCSTrain.getCbCS();
     	    float csCoTrain=tempResCSTrain.getCoCS();
     	    float csHnTrain=tempResCSTrain.getHnCS();
     	    float csNTrain=tempResCSTrain.getNCS();
     	    
     	    if(isCaUsed){
     	    	if(csCaTest>100.0 || csCaTest<-100.0 ) //the chemical shift must be valid
     	    		return rmsd;
     	    	if(csCaTrain>100.0 || csCaTrain<-100.0 ) //the chemical shift must be valid
     	    		return rmsd;
     	    	if(Math.abs(csCaTest-csCaTrain)>heavyErr )
     	    		return rmsd;
     	    	sum=sum+(csCaTest-csCaTrain)*(csCaTest-csCaTrain);
     	    	counter++;
     	    }
     	    if(isCbUsed){
     	    	if(csCbTest>100.0 || csCbTest<-100.0 ) //the chemical shift must be valid
     	    		return rmsd;
     	    	if(csCbTrain>100.0 || csCbTrain<-100.0 ) //the chemical shift must be valid
     	    		return rmsd;
     	    	if(Math.abs(csCbTest-csCbTrain)>heavyErr )
     	    		return rmsd;
     	    	sum=sum+(csCbTest-csCbTrain)*(csCbTest-csCbTrain);
     	    	counter++;
     	    }
     	    if(isCoUsed){
     	    	if(csCoTest>100.0 || csCoTest<-100.0 ) //the chemical shift must be valid
     	    		return rmsd;
     	    	if(csCoTrain>100.0 || csCoTrain<-100.0 ) //the chemical shift must be valid
     	    		return rmsd;
     	    	if(Math.abs(csCoTest-csCoTrain)>heavyErr )
     	    		return rmsd;
     	    	sum=sum+(csCoTest-csCoTrain)*(csCoTest-csCoTrain);
     	    	counter++;
     	    }
     	   if(isNUsed){
     		  if(csNTest>100.0 || csNTest<-100.0 ) //the chemical shift must be valid
   	    		return rmsd;
   	    	  if(csNTrain>100.0 || csNTrain<-100.0 ) //the chemical shift must be valid
   	    		return rmsd;
     		  if(Math.abs(csNTest-csNTrain)>heavyErr )
   	    		return rmsd;
     		  sum=sum+(csNTest-csNTrain)*(csNTest-csNTrain);
  	    	  counter++;
     	   }
     	   if(isHnUsed){
     		  if(csHnTest>100.0 || csHnTest<-100.0 ) //the chemical shift must be valid
     	    		return rmsd;
     	    	  if(csHnTrain>100.0 || csHnTrain<-100.0 ) //the chemical shift must be valid
     	    		return rmsd;
     		  if(Math.abs(csHnTest-csHnTrain)>hErr )
     	    		return rmsd;
     		  sum=sum+(csHnTest-csHnTrain)*(csHnTest-csHnTrain)*100;//for each hn chemical shift, multiply by 10, in order to scale it with other heavy atoms
 	    	  counter++;
     	   }
		}//for(int i=-halfResWin1;i<=halfResWin2;i++)
		
		rmsd=(float)Math.sqrt(sum/counter);
		return rmsd;		
	}////end of function findBestMatchedHaCSForOneResidue()
	
	
	/**
	 * compute the match score between a testing data point and a training data point, using a 
	 * Hausdorff-based distance measure.
	 */
	public int computOneToOneMatchScoreHDistance(residueCS curTestResCS,residueCS curTrainResCS,Vector<residueCS> vecOneTestData,Vector<residueCS> vecOneTrainData,int resWindow,
			float hErr, float heavyErr,float[] RMSD){	
		int numMatched=0;
		float rmsd=99999.9f;
		float sum=0.0f;
		int counter=0;
		int curResTestNo=curTestResCS.getResidueNo();
		int curResTrainNo=curTrainResCS.getResidueNo();
		int minResNo=vecOneTestData.elementAt(0).getResidueNo();
		int maxResNo=vecOneTestData.elementAt(vecOneTestData.size()-1).getResidueNo();	
			
		int halfResWin1=(int)(resWindow/2);   //first half of the residue window
		int halfResWin2=resWindow-halfResWin1;//second half of the residue window
		
		if( (curResTestNo-halfResWin1)<minResNo){
			halfResWin1=curResTestNo-minResNo;
			halfResWin2=resWindow-halfResWin1;
		}
		
		if( (curResTestNo+halfResWin2)>maxResNo){
			halfResWin2=maxResNo-curResTestNo;
			halfResWin1=resWindow-halfResWin2;
		}
		
		//debugging
	/*	if( (curResTestNo-halfResWin1)<minResNo){
			return rmsd;
		}
		
		if( (curResTestNo+halfResWin2)>maxResNo){
			return rmsd;
		}*/
		///////////
		
		boolean isCaMatched=true;
		boolean isCbMatched=true;
		boolean isCoMatched=true;
		boolean isHnMatched=true;
		boolean isNMatched=true;
		
		for(int i=-halfResWin1;i<=halfResWin2;i++){
			int noTest=curResTestNo+i;			
			int indTest= Collections.binarySearch(vecOneTestData, new residueCS(noTest), new residueCS.residueCSComparator());
     	    if(indTest<0){
     	    	//System.out.println("Testing Residue not found...");
     	    	return 0;
     	    	//System.exit(0);//changed later...
     	    }
     	    residueCS tempResCSTest=vecOneTestData.elementAt(indTest);     	    
     		float csCaTest=tempResCSTest.getCaCS();
    		float csCbTest=tempResCSTest.getCbCS();
    		float csCoTest=tempResCSTest.getCoCS();
    		float csHnTest=tempResCSTest.getHnCS();
    		float csNTest=tempResCSTest.getNCS();    				
    		
     	    int noTrain=curResTrainNo+i;
     	    int indTrain= Collections.binarySearch(vecOneTrainData, new residueCS(noTrain), new residueCS.residueCSComparator());
    	    if(indTrain<0){
    	    	//System.out.println("Training Residue not found...");
    	    	return 0;
    	    	//System.exit(0);//changed later...
    	    }
     	    residueCS tempResCSTrain=vecOneTrainData.elementAt(indTrain);
     	    float csCaTrain=tempResCSTrain.getCaCS();
     	    float csCbTrain=tempResCSTrain.getCbCS();
     	    float csCoTrain=tempResCSTrain.getCoCS();
     	    float csHnTrain=tempResCSTrain.getHnCS();
     	    float csNTrain=tempResCSTrain.getNCS();
     	    
     	    //check whether ca chemical shift is matched. 	    		
 	    	if( (csCaTest<100.0) &&  (csCaTest>-100.0) && (csCaTrain<100.0) && (csCaTrain>-100.0)
 	    			&& ( Math.abs(csCaTest-csCaTrain)<=heavyErr) ){ 	    	
	 	    	sum=sum+(csCaTest-csCaTrain)*(csCaTest-csCaTrain);
	 	    	counter++;
 	    	}
 	    	else
 	    		isCaMatched=false; 	    	
 	    	
 	    	if(csCbTest<100.0 && csCbTest>-100.0 && csCbTrain<100.0 && csCbTrain>-100.0 && Math.abs(csCbTest-csCbTrain)<=heavyErr){
	 	    	sum=sum+(csCbTest-csCbTrain)*(csCbTest-csCbTrain);
	 	    	counter++;
 	    	}
 	    	else
 	    		isCbMatched=false;
     	   
 	      		
 	    	if(csCoTest<100.0 && csCoTest>-100.0 && csCoTrain<100.0 && csCoTrain>-100.0 && Math.abs(csCoTest-csCoTrain)<=heavyErr ){
 	    		sum=sum+(csCoTest-csCoTrain)*(csCoTest-csCoTrain);
 	    		counter++;
 	    	}
 	    	else
 	    		isCoMatched=false;
 	    	 	    	
 	    	if(csNTest<100.0 && csNTest>-100.0 && csNTrain<100.0 && csNTrain>-100.0 && Math.abs(csNTest-csNTrain)<=heavyErr){
	 	    	sum=sum+(csNTest-csNTrain)*(csNTest-csNTrain);
	 	    	counter++;
 	    	}
 	    	else
 	    		isNMatched=false;
 	         		
     		if(csHnTest<100.0 && csHnTest>-100.0 && csHnTrain<100.0 && csHnTrain>-100.0 && Math.abs(csHnTest-csHnTrain)<=hErr){
	     		sum=sum+(csHnTest-csHnTrain)*(csHnTest-csHnTrain)*100;//for each hn chemical shift, multiply by 10, in order to scale it with other heavy atoms
	 	    	counter++;
     		}
     		else
     			isHnMatched=false;
     	   
     		if(tempResCSTest.getResidueName().equalsIgnoreCase("PRO") && tempResCSTrain.getResidueName().equalsIgnoreCase("PRO"))
     			isHnMatched=true;
     		if(tempResCSTest.getResidueName().equalsIgnoreCase("GLY") && tempResCSTrain.getResidueName().equalsIgnoreCase("GLY"))
     			isCbMatched=true;
     			
		}//for(int i=-halfResWin1;i<=halfResWin2;i++)
		
		if(isCaMatched)
			numMatched++;
		if(isCbMatched)
			numMatched++;
		if(isCoMatched)
			numMatched++;
		if(isHnMatched){
			numMatched++;
			numMatched++;
		}
		if(isNMatched)
			numMatched++;
			
		rmsd=(float)Math.sqrt(sum/counter);
		RMSD[0]= rmsd;		
		return numMatched;
	}////end of function findBestMatchedHaCSForOneResidue()
	
	/**
	 * compute the match score between a testing data point and a training data point, using a 
	 * Hausdorff-based distance measure, and varied residue windows.
	 */
	public int computOneToOneMatchScoreHDistanceVariedResWindows(residueCS curTestResCS,residueCS curTrainResCS,Vector<residueCS> vecOneTestData,Vector<residueCS> vecOneTrainData,int resWindow,
			float hErr, float heavyErr,float[] RMSD){	
		
		
		
		int curResTestNo=curTestResCS.getResidueNo();
		int curResTrainNo=curTrainResCS.getResidueNo();
		int minResNo=vecOneTestData.elementAt(0).getResidueNo();
		int maxResNo=vecOneTestData.elementAt(vecOneTestData.size()-1).getResidueNo();	
				
		int maxNumMatch=0;
		float minRmsd=99999.9f;	
		
		for(int noOffset=0;noOffset<=resWindow;noOffset++){
			int halfResWin1=noOffset;   //first half of the residue window
			int halfResWin2=resWindow-halfResWin1;//second half of the residue window
			
			if( (curResTestNo-halfResWin1)<minResNo) 
				continue;
			
			if( (curResTestNo+halfResWin2)>maxResNo)
				continue;	
			
			boolean isCaMatched=true;
			boolean isCbMatched=true;
			boolean isCoMatched=true;
			boolean isHnMatched=true;
			boolean isNMatched=true;
	
			float rmsd=99999.9f;
			int counter=0;
			float sum=0.0f;
			int numMatched=0;
			
			for(int i=-halfResWin1;i<=halfResWin2;i++){
				int noTest=curResTestNo+i;			
				int indTest= Collections.binarySearch(vecOneTestData, new residueCS(noTest), new residueCS.residueCSComparator());
	     	    if(indTest<0){
	     	    	//System.out.println("Testing Residue not found...");
	     	    	return 0;
	     	    	//System.exit(0);//changed later...
	     	    }
	     	    residueCS tempResCSTest=vecOneTestData.elementAt(indTest);     	    
	     		float csCaTest=tempResCSTest.getCaCS();
	    		float csCbTest=tempResCSTest.getCbCS();
	    		float csCoTest=tempResCSTest.getCoCS();
	    		float csHnTest=tempResCSTest.getHnCS();
	    		float csNTest=tempResCSTest.getNCS();    				
    		
	     	    int noTrain=curResTrainNo+i;
	     	    int indTrain= Collections.binarySearch(vecOneTrainData, new residueCS(noTrain), new residueCS.residueCSComparator());
	    	    if(indTrain<0){
	    	    	//System.out.println("Training Residue not found...");
	    	    	return 0;
	    	    	//System.exit(0);//changed later...
	    	    }
	     	    residueCS tempResCSTrain=vecOneTrainData.elementAt(indTrain);
	     	    
	     	    
	     	    
	     	    float csCaTrain=tempResCSTrain.getCaCS();
	     	    float csCbTrain=tempResCSTrain.getCbCS();
	     	    float csCoTrain=tempResCSTrain.getCoCS();
	     	    float csHnTrain=tempResCSTrain.getHnCS();
	     	    float csNTrain=tempResCSTrain.getNCS();
     	    
	     	    //check whether ca chemical shift is matched. 	    		
	 	    	if( (csCaTest<100.0) &&  (csCaTest>-100.0) && (csCaTrain<100.0) && (csCaTrain>-100.0)
	 	    			&& ( Math.abs(csCaTest-csCaTrain)<=heavyErr) ){ 	    	
		 	    	sum=sum+(csCaTest-csCaTrain)*(csCaTest-csCaTrain);
		 	    	counter++;
	 	    	}
	 	    	else
	 	    		isCaMatched=false; 	    	
	 	    	
	 	    	if(csCbTest<100.0 && csCbTest>-100.0 && csCbTrain<100.0 && csCbTrain>-100.0 && Math.abs(csCbTest-csCbTrain)<=heavyErr){
		 	    	sum=sum+(csCbTest-csCbTrain)*(csCbTest-csCbTrain);
		 	    	counter++;
	 	    	}
	 	    	else
	 	    		isCbMatched=false;
	     	   
	 	      		
	 	    	if(csCoTest<100.0 && csCoTest>-100.0 && csCoTrain<100.0 && csCoTrain>-100.0 && Math.abs(csCoTest-csCoTrain)<=heavyErr ){
	 	    		sum=sum+(csCoTest-csCoTrain)*(csCoTest-csCoTrain);
	 	    		counter++;
	 	    	}
	 	    	else
	 	    		isCoMatched=false;
	 	    	 	    	
	 	    	if(csNTest<100.0 && csNTest>-100.0 && csNTrain<100.0 && csNTrain>-100.0 && Math.abs(csNTest-csNTrain)<=heavyErr){
		 	    	sum=sum+(csNTest-csNTrain)*(csNTest-csNTrain);
		 	    	counter++;
	 	    	}
	 	    	else
	 	    		isNMatched=false;
	 	         		
	     		if(csHnTest<100.0 && csHnTest>-100.0 && csHnTrain<100.0 && csHnTrain>-100.0 && Math.abs(csHnTest-csHnTrain)<=hErr){
		     		sum=sum+(csHnTest-csHnTrain)*(csHnTest-csHnTrain)*100;//for each hn chemical shift, multiply by 10, in order to scale it with other heavy atoms
		 	    	counter++;
	     		}
	     		else
	     			isHnMatched=false;
	     	   
	     		if(tempResCSTest.getResidueName().equalsIgnoreCase("PRO") && tempResCSTrain.getResidueName().equalsIgnoreCase("PRO"))
	     			isHnMatched=true;
	     		if(tempResCSTest.getResidueName().equalsIgnoreCase("GLY") && tempResCSTrain.getResidueName().equalsIgnoreCase("GLY"))
	     			isCbMatched=true;
     			
			}//for(int i=-halfResWin1;i<=halfResWin2;i++)
			
			if(isCaMatched)
				numMatched++;
			if(isCbMatched)
				numMatched++;
			if(isCoMatched)
				numMatched++;
			if(isHnMatched){
				numMatched++;
				numMatched++;
			}
			if(isNMatched)
				numMatched++;
			
			rmsd=(float)Math.sqrt(sum/counter);
			
			if(numMatched>maxNumMatch){
				maxNumMatch=numMatched;
				minRmsd=rmsd;
			}//if(numMatched>maxNumMatch)
			
			if(numMatched==maxNumMatch){
				if(rmsd<minRmsd)
					minRmsd=rmsd;				
			}
		}//	for(int noOffset=0;noOffset<=resWindow;noOffset++)		
		
		RMSD[0]= minRmsd;		
		return maxNumMatch;
	}////end of function findBestMatchedHaCSForOneResidue()
	
	/**
	 * compute the match score between a testing data point and a training data point, using a 
	 * new measure (in which each residue must have two matched chemical shifts), and varied residue windows.
	 */
	public int computOneToOneMatchScoreTwoPerResidueVariedResWindows(residueCS curTestResCS,residueCS curTrainResCS,Vector<residueCS> vecOneTestData,Vector<residueCS> vecOneTrainData,int resWindow,
			float hErr, float heavyErr,float[] RMSD){	
		
		int curResTestNo=curTestResCS.getResidueNo();
		int curResTrainNo=curTrainResCS.getResidueNo();
		int minResNo=vecOneTestData.elementAt(0).getResidueNo();
		int maxResNo=vecOneTestData.elementAt(vecOneTestData.size()-1).getResidueNo();	
				
		int maxNumMatch=0;
		float minRmsd=99999.9f;
				
		for(int noOffset=0;noOffset<=resWindow;noOffset++){
			int halfResWin1=noOffset;   //first half of the residue window
			int halfResWin2=resWindow-halfResWin1;//second half of the residue window
			
			if( (curResTestNo-halfResWin1)<minResNo) 
				continue;
			
			if( (curResTestNo+halfResWin2)>maxResNo)
				continue;	
			
			/*boolean isCaMatched=true;
			boolean isCbMatched=true;
			boolean isCoMatched=true;
			boolean isHnMatched=true;
			boolean isNMatched=true;*/
	
			boolean isMatched=true;
			float rmsd=99999.9f;
			float sum=0.0f;
			int counter=0;
			
			for(int i=-halfResWin1;i<=halfResWin2;i++){
				int noTest=curResTestNo+i;			
				int indTest= Collections.binarySearch(vecOneTestData, new residueCS(noTest), new residueCS.residueCSComparator());
	     	    if(indTest<0){
	     	    	//System.out.println("Testing Residue not found...");
	     	    	return 0;
	     	    	//System.exit(0);//changed later...
	     	    }
	     	    residueCS tempResCSTest=vecOneTestData.elementAt(indTest);     	    
	     		float csCaTest=tempResCSTest.getCaCS();
	    		float csCbTest=tempResCSTest.getCbCS();
	    		float csCoTest=tempResCSTest.getCoCS();
	    		float csHnTest=tempResCSTest.getHnCS();
	    		float csNTest=tempResCSTest.getNCS();    				
    		
	     	    int noTrain=curResTrainNo+i;
	     	    int indTrain= Collections.binarySearch(vecOneTrainData, new residueCS(noTrain), new residueCS.residueCSComparator());
	    	    if(indTrain<0){
	    	    	//System.out.println("Training Residue not found...");
	    	    	return 0;
	    	    	//System.exit(0);//changed later...
	    	    }
	     	    residueCS tempResCSTrain=vecOneTrainData.elementAt(indTrain);
	     	    
	     	   if(!tempResCSTest.getResidueName().equalsIgnoreCase(tempResCSTrain.getResidueName())){
	     	    	isMatched=false;	     		 
	     			break;
	     	    }
	     	    
	     	    float csCaTrain=tempResCSTrain.getCaCS();
	     	    float csCbTrain=tempResCSTrain.getCbCS();
	     	    float csCoTrain=tempResCSTrain.getCoCS();
	     	    float csHnTrain=tempResCSTrain.getHnCS();
	     	    float csNTrain=tempResCSTrain.getNCS();
     	    
	     	    int numMatchPerResidue=0; 	    
	     	    
	     	    //check whether ca chemical shift is matched. 	    		
	 	    	if( (csCaTest<100.0) &&  (csCaTest>-100.0) && (csCaTrain<100.0) && (csCaTrain>-100.0)
	 	    			&& ( Math.abs(csCaTest-csCaTrain)<=heavyErr) ){ 	    	
		 	    	sum=sum+(csCaTest-csCaTrain)*(csCaTest-csCaTrain);
		 	    	counter++;
		 	    	numMatchPerResidue++;
	 	    	}
	 	    	 	    	
	 	    	
	 	    	if(csCbTest<100.0 && csCbTest>-100.0 && csCbTrain<100.0 && csCbTrain>-100.0 && Math.abs(csCbTest-csCbTrain)<=heavyErr){
		 	    	sum=sum+(csCbTest-csCbTrain)*(csCbTest-csCbTrain);
		 	    	counter++;
		 	    	numMatchPerResidue++;
	 	    	}
	 	    	
	     	   
	 	      		
	 	    	if(csCoTest<100.0 && csCoTest>-100.0 && csCoTrain<100.0 && csCoTrain>-100.0 && Math.abs(csCoTest-csCoTrain)<=heavyErr ){
	 	    		sum=sum+(csCoTest-csCoTrain)*(csCoTest-csCoTrain);
	 	    		counter++;
	 	    		numMatchPerResidue++;
	 	    	}
	 	    	
	 	    	 	    	
	 	    	if(csNTest<100.0 && csNTest>-100.0 && csNTrain<100.0 && csNTrain>-100.0 && Math.abs(csNTest-csNTrain)<=heavyErr){
		 	    	sum=sum+(csNTest-csNTrain)*(csNTest-csNTrain);
		 	    	counter++;
		 	    	numMatchPerResidue++;
	 	    	}
	 	    	
	 	         		
	     		if(csHnTest<100.0 && csHnTest>-100.0 && csHnTrain<100.0 && csHnTrain>-100.0 && Math.abs(csHnTest-csHnTrain)<=hErr){
		     		sum=sum+(csHnTest-csHnTrain)*(csHnTest-csHnTrain)*100;//for each hn chemical shift, multiply by 10, in order to scale it with other heavy atoms
		 	    	counter++;
		 	    	numMatchPerResidue++;
		 	    	numMatchPerResidue++;
	     		}
	     		
	     	   
	     		if(tempResCSTest.getResidueName().equalsIgnoreCase("PRO") && tempResCSTrain.getResidueName().equalsIgnoreCase("PRO"))
	     			numMatchPerResidue++;
	     		if(tempResCSTest.getResidueName().equalsIgnoreCase("GLY") && tempResCSTrain.getResidueName().equalsIgnoreCase("GLY"))
	     			numMatchPerResidue++;
     			
	     		if(numMatchPerResidue<2){
	     			isMatched=false;	     		 
	     			break;
	     		}//if(numMatchPerResidue<2)
			}//for(int i=-halfResWin1;i<=halfResWin2;i++)
			
			if(!isMatched)
				continue;
						
			rmsd=(float)Math.sqrt(sum/counter);
			
			if(counter>maxNumMatch){
				maxNumMatch=counter;
				minRmsd=rmsd;
			}//if(numMatched>maxNumMatch)
			
			if(counter==maxNumMatch){
				if(rmsd<minRmsd)
					minRmsd=rmsd;				
			}
		}//	for(int noOffset=0;noOffset<=resWindow;noOffset++)		
		
		RMSD[0]= minRmsd;		
		return maxNumMatch;
	}////end of function findBestMatchedHaCSForOneResidue()
	
	 /**
 	 * Run R scripts
 	 * 
 	 * @param src location of the input file
 	 * @param strOut location of the output file
 	 * @param strInput  input file name
 	 * 
 	 * @return void
 	 * 
 	 * @throws JampackException the jampack exception
 	 * @throws InterruptedException the interrupted exception
 	 */       
    public void runRScripts(String strRBin, String rScriptsFile,String outLogFile) throws InterruptedException
    {	     
    	System.out.println("Running R scripts...");
    	long startTime = System.currentTimeMillis();
    	
    	String userDir = System.getProperty("user.dir");////
    	String srcRPath=userDir+"/system/cs_database/r_files/";
    	
        System.setProperty("user.dir",	srcRPath);
    	
    	String userDirNew = System.getProperty("user.dir");////
    	System.out.println("Current working directory is:"+userDirNew);//for debugging p   	
    	
    	String strRun=strRBin+" "+rScriptsFile;//" r_scripts.R";
    	Runtime runtime = Runtime.getRuntime();
    	try
        {    		
    		//execute the xplor scripts:
    		File  direc =new File (userDirNew);
    		Process r = runtime.exec(strRun, null, direc );
    		InputStream rOut = r.getInputStream();
    		InputStreamReader isr = new InputStreamReader(rOut);
    	    BufferedReader br = new BufferedReader(isr);
    	    String line;
    	    String outFileName =userDirNew+File.separator+outLogFile;//"RunRResults.txt";
    	    try
	    	{
	    		PrintWriter out = new PrintWriter(new BufferedWriter(new FileWriter(outFileName)));
	    	    while ((line = br.readLine()) != null)
	    	    {
	    	    	 out.println(line);///to file
	    	         System.out.println(line);//for debugging...
	    	    }
	    	    out.close();
	    	
	        }catch (FileNotFoundException e)
			{
				System.out.println("File not found: " + outFileName);
			}catch (IOException e)
			{
			   System.out.println("IOException: the stack trace is:");
			   e.printStackTrace();
			}
		
    	    r.waitFor();
        }
        catch(IOException e)
        {
           System.err.println("Error on exec() method");
           e.printStackTrace();  
        }
       
        System.setProperty("user.dir",	userDir);
        
        long endTime = System.currentTimeMillis();
    	double totalTime = (double) ((endTime - startTime) / 60000.0); //in minutes
    	System.out.println("Finish running R scripts.");
    	System.out.println("Time for running R:  "+ totalTime +" minutes");
    }
    
	 /**
	 * Run R scripts
	 * 
	 * @param src location of the input file
	 * @param strOut location of the output file
	 * @param strInput  input file name
	 * 
	 * @return void
	 * 
	 * @throws JampackException the jampack exception
	 * @throws InterruptedException the interrupted exception
	 */       
   public void runRScripts(String strRBin, String rScriptsFile,String outLogFile,String strCurDirection) throws InterruptedException
   {	     
   	System.out.println("Running R scripts...");
   	long startTime = System.currentTimeMillis();
   	
   	String userDir = System.getProperty("user.dir");////
   	String srcRPath=strCurDirection;//strCurDirection+"/r_files/";
   	
    System.setProperty("user.dir",	srcRPath);
   	
   	String userDirNew = System.getProperty("user.dir");////
   	System.out.println("Current working directory is:"+userDirNew);//for debugging p   	
   	
   	String strRun=strRBin+" "+rScriptsFile;//" r_scripts.R";
   	Runtime runtime = Runtime.getRuntime();
   	try
       {    		
   		//execute the xplor scripts:
   		File  direc =new File (userDirNew);
   		Process r = runtime.exec(strRun, null, direc );
   		InputStream rOut = r.getInputStream();
   		InputStreamReader isr = new InputStreamReader(rOut);
   	    BufferedReader br = new BufferedReader(isr);
   	    String line;
   	    String outFileName =userDirNew+File.separator+outLogFile;//"RunRResults.txt";
   	    try
	    	{
	    		PrintWriter out = new PrintWriter(new BufferedWriter(new FileWriter(outFileName)));
	    	    while ((line = br.readLine()) != null)
	    	    {
	    	    	 out.println(line);///to file
	    	         System.out.println(line);//for debugging...
	    	    }
	    	    out.close();
	    	
	        }catch (FileNotFoundException e)
			{
				System.out.println("File not found: " + outFileName);
			}catch (IOException e)
			{
			   System.out.println("IOException: the stack trace is:");
			   e.printStackTrace();
			}
		
   	    r.waitFor();
       }
       catch(IOException e)
       {
          System.err.println("Error on exec() method");
          e.printStackTrace();  
       }
      
       System.setProperty("user.dir",	userDir);
       
       long endTime = System.currentTimeMillis();
   	double totalTime = (double) ((endTime - startTime) / 60000.0); //in minutes
   	System.out.println("Finish running R scripts.");
   	System.out.println("Time for running R:  "+ totalTime +" minutes");
   }
    
   public String ConvertIpAddressAndTimeStringForSavedFileName(String strOrigin){
	   
	   String strFileName="";
	   for(int i=0;i<strOrigin.length();i++){
		   char s=strOrigin.charAt(i);
		   if(s==':')
			   strFileName=strFileName+".";
		   else
			   strFileName=strFileName+s;
	   }
	   return strFileName;
   }
     
}
