package chemShiftPredict;
/*
HASH Ha Chemical Shift Prediction  Software Version 1.0
Copyright (C) 2009-2012 Bruce Donald Lab, Duke University

HASH is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free
Software Foundation, either version 3 of the License, or (at your option) any
later version.

HASH is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
details.

You should have received a copy of the GNU Lesser General Public License
along with this library; if not, see:
    <http://www.gnu.org/licenses/>.

There are additional restrictions imposed on the use and distribution of this
open-source code, including: (A) this header must be included in any
modification or extension of the code; (B) you are required to cite our
papers in any publications that use this code. The citation for the various
different modules of our software, together with a complete list of
requirements and restrictions are found in the document license.pdf enclosed
with this distribution.

Contact Info:
    Bruce R. Donald
    Duke University
    Department of Computer Science
    Levine Science Research Center (LSRC)
    Durham, NC 27708-0129
    USA
    email: www.cs.duke.edu/brd/

<signature of Bruce Donald>, 01 May, 2011
Bruce R. Donald, Professor of Computer Science and Biochemistry
*/

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.StringTokenizer;
import java.util.Vector;

import rdcPanda.Pdb;


public class randomCoilShift {
	
	String resType="";
	public double haCS=-9999.9;//for storing ha chemical shift
	public double caCS=-9999.9;
	public double hnCS=-9999.9;
	public double  nCS=-9999.9;
	public double cbCS=-9999.9;
	public double coCS=-9999.9;
	public double nCorrection=-9999.9;
	
	public randomCoilShift(){};
	public randomCoilShift(String res, double ha, double ca, double hn, double n, double co, double cb, double nCor){
		resType=res;
		haCS=ha;
		caCS=ca;
		hnCS=hn;
		nCS=n;
		coCS=co;
		cbCS=cb;
		nCorrection=nCor;	
	};
	  /**
     * Read the random coil chemical shifts for all amino acids.
    
     * @param filename the name of the file
         * 
     * @return a vector of random coil chemical shifts
     */
    public Vector<randomCoilShift> randomCoilShiftReader(String filename)
    {    	
    	StringTokenizer st = new StringTokenizer("");         	
    	String ss ="";   	
    	String strFirst="";    	
    	Vector<randomCoilShift> vecRandomShifts=new Vector<randomCoilShift>();
    	try
    	{
    		BufferedReader in = new BufferedReader(new FileReader(filename));	
    		ss = in.readLine();    		
    		stop:
    			while(true) 
    			{   			    
    				st = new StringTokenizer(ss);   				
    				if (st.hasMoreTokens()){
    					strFirst=st.nextToken();
    					if(strFirst.substring(0, 1).equalsIgnoreCase("#")){
    						if ((ss = in.readLine()) == null )
    							break stop;
    						continue;
    					} //if(strFirst.substring(0, 1).equalsIgnoreCase("#"))      					
    				}//if (st.hasMoreTokens())
    				
    				strFirst=strFirst.trim();   				
    			
    				double csHa=-9999.9;
    				double csCa=-9999.9;
    				double csHn=-9999.9;
    				double csN=-9999.9;
    				double csCb=-9999.9;
    				double csCo=-9999.9;
    				double csNCorrection=-9999.9;
    				
    				if (st.hasMoreTokens())    					
    					csHa= new Double(st.nextToken()).doubleValue();
    				if (st.hasMoreTokens())    					
    					csCa= new Double(st.nextToken()).doubleValue();
    				if (st.hasMoreTokens())    					
    					csHn= new Double(st.nextToken()).doubleValue();
    				if (st.hasMoreTokens())   
    					csN= new Double(st.nextToken()).doubleValue();
    				if (st.hasMoreTokens())   
    					csCo= new Double(st.nextToken()).doubleValue();
    				if (st.hasMoreTokens())   
    					csCb= new Double(st.nextToken()).doubleValue();
    				if (st.hasMoreTokens())   
    					csNCorrection= new Double(st.nextToken()).doubleValue();   				
    				vecRandomShifts.add(new randomCoilShift(strFirst,csHa,csCa,csHn,csN,csCo,csCb,csNCorrection) );
    				
    				if ((ss = in.readLine()) == null )    				
    					break stop;    					
    			}
    	}catch (FileNotFoundException e) {
    		System.out.println("File not found: " + filename);
    	}catch (IOException e) 
    	{
    		System.out.println("IOException: the stack trace is:");
    		e.printStackTrace();
    	}
    	return vecRandomShifts;
    }	
}
