package chemShiftPredict;
/*
HASH Ha Chemical Shift Prediction  Software Version 1.0
Copyright (C) 2009-2012 Bruce Donald Lab, Duke University

HASH is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free
Software Foundation, either version 3 of the License, or (at your option) any
later version.

HASH is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
details.

You should have received a copy of the GNU Lesser General Public License
along with this library; if not, see:
    <http://www.gnu.org/licenses/>.

There are additional restrictions imposed on the use and distribution of this
open-source code, including: (A) this header must be included in any
modification or extension of the code; (B) you are required to cite our
papers in any publications that use this code. The citation for the various
different modules of our software, together with a complete list of
requirements and restrictions are found in the document license.pdf enclosed
with this distribution.

Contact Info:
    Bruce R. Donald
    Duke University
    Department of Computer Science
    Levine Science Research Center (LSRC)
    Durham, NC 27708-0129
    USA
    email: www.cs.duke.edu/brd/

<signature of Bruce Donald>, 01 May, 2011
Bruce R. Donald, Professor of Computer Science and Biochemistry
*/

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Collections;
import java.util.StringTokenizer;
import java.util.Vector;

import rdcPanda.H1CS;
import rdcPanda.Pdb;

//a class for processing data from refdb database
public class refDB {
	
	
	/** 
	 * Read the refdb chemical shift training data.
	 *  @param pathCSDatabase path of cs directory
	 
	 *  @param isTraining whether it is training data, TRUE for training data, FALSE for testing data
	 *  @return a vector of residueCS
	 */
	public Vector <Vector<residueCS>> readRefDBTrainingData(String pathCSDatabase){		
		Vector<Vector<residueCS>> vecData=new  Vector<Vector<residueCS>>();
		refDB ref_db=new refDB();
		String userDir = System.getProperty("user.dir");
		
		//read random coil chemical shift table:
	    String pathRandomCoilShift=userDir+"/system/cs_database/random_coil_shifts.txt"; //random coil shift table
		randomCoilShift rcs=new randomCoilShift();
		Vector<randomCoilShift> vecRandomShifts=rcs.randomCoilShiftReader(pathRandomCoilShift);
		
		//read all chemical shift files    	
    	File myDir = new File(pathCSDatabase);
    	String[] contents = myDir.list();//select all structures
    	if (contents == null) 
    		System.out.println(myDir.getName() + " is not a directory");
    	int counter=0;
    	for (int t=0; t<contents.length;t++)  {  
    		String filename = contents[t]; 
    		String fileNameSaved=filename.substring(0,filename.indexOf("."));
    		
	    	String strCSFile="";
	    	strCSFile=pathCSDatabase+filename;	 	    	
	    	
	    	Vector<residueCS> vecOneProteinData=new Vector<residueCS>();	    	
	    
	    	//read the training chemical shift data
	    	Vector<H1CS> vecCSTemp= new Vector<H1CS>();
	    	vecCSTemp=ref_db.refdbReader(strCSFile); 	
	    	vecOneProteinData=convertH1CSToResdueCS(vecCSTemp,fileNameSaved,vecRandomShifts);
	    	
	    	Collections.sort(vecOneProteinData, new residueCS.residueCSComparator());//sort the data points based on residue number
	    	if(vecOneProteinData.size()>0)
	    		vecData.add(vecOneProteinData);	 
	    	counter+=vecOneProteinData.size();
    	}//for (int t=0; t<contents.length;t++)
    	
    	System.out.println("Total number of chemical shift residues read from refDB: "+counter);
    	return vecData;
	}
	
	
	  /**
     * convert H1CS to residueCS.
    
     * @param vecH1CS: vector of H1CS.
         * 
     * @return a vector of random coil chemical shifts
     */
    public Vector<residueCS> convertH1CSToResdueCS(Vector<H1CS> vecH1CS,String proteinName,Vector<randomCoilShift> vecRandomShifts)
    {    
    	chemicalShiftDatabase csd=new chemicalShiftDatabase();
    	Vector<residueCS> vecOneProteinData=new Vector<residueCS>();
    	Collections.sort(vecH1CS, new H1CS.resNoComparator());//sort the H1CSes based on residue number
    	
    	//find max res no and min res no	    		    	
    	int minNoFromCS=9999999;
    	int maxNoFromCS=-99999;
    	for(int i=0;i<vecH1CS.size();i++){
    		H1CS h1cs=(H1CS)vecH1CS.elementAt(i);
    		int resNo=h1cs.getResidueNo();
    		if(resNo>maxNoFromCS)
    			maxNoFromCS=resNo;  
    		if(resNo<minNoFromCS)
    			minNoFromCS=resNo;
    	}
    	
    	for(int no=minNoFromCS+1;no<=maxNoFromCS;no++){ 
    		//find the bb chemical shift of current residue
    		double csN=-9999.0;
    		double csCA=-9999.0;
    		double csHN=-9999.0;
    		double csHA=-9999.0;
    		double csCB=-9999.0;
    		double csCO=-9999.0;
    		String curRes="";
    		
    		 //check the residue name of the neighboring residues
     	    String prevResType="";
     	    int ind=Collections.binarySearch(vecH1CS, new H1CS(no-1), new H1CS.resNoComparator());
     	    if(ind>-1){	     	    	
	     	   H1CS h1csTemp=vecH1CS.elementAt(ind);
	     	   prevResType=h1csTemp.getResidueType();
     	    }   
     	    else 
     	    	continue;
     	    
    		for(int i=0;i<vecH1CS.size();i++){
        		H1CS h1cs=vecH1CS.elementAt(i);
        		
        		int resNo=h1cs.getResidueNo();
        		
        		if(resNo!=no)
        			continue;
        		curRes=h1cs.getResidueType();
        		String atom=h1cs.getAtomName();
        		double cs=h1cs.getH1CS();
        		
        		if(atom.equalsIgnoreCase("HA"))
        			csHA=cs;
        		else if(atom.equalsIgnoreCase("CA"))
        			csCA=cs;
        		else if(atom.equalsIgnoreCase("CB"))
        			csCB=cs;
        		else if(atom.equalsIgnoreCase("C") || atom.equalsIgnoreCase("CO"))
        			csCO=cs;
        		else if(atom.equalsIgnoreCase("H") || atom.equalsIgnoreCase("HN"))
        			csHN=cs;
        		else if(atom.equalsIgnoreCase("N"))
        			csN=cs;
        		
    		}//for(int i=0;i<vecH1CS.size();i++)
    		
    		ind=Collections.binarySearch(vecH1CS, new H1CS(no), new H1CS.resNoComparator());
    		if(ind>-1){	     	    	
   	     	   H1CS h1csTemp=vecH1CS.elementAt(ind);
   	     	   curRes=h1csTemp.getResidueType();
        	}   
        	else 
        	    continue;
    		  
    		//for GLY, we compute the mean of two HA chemical shifts
    		if(curRes.equalsIgnoreCase("GLY") || curRes.equalsIgnoreCase("G")){
    			double sumHaCS=0.0;	  
    			int counterHaCS=0;
    			boolean isFoundHaForGly=false;	    			
    			for(int i=0;i<vecH1CS.size();i++){
		    		H1CS h1cs=(H1CS)vecH1CS.elementAt(i);
		    		int resNo=h1cs.getResidueNo();		    		
		    		String atom=h1cs.getAtomName();
		    		if(atom.length()>=2)
		    			atom=atom.substring(0,2);
		    		if(resNo==no && atom.equalsIgnoreCase("HA")){
		    			sumHaCS=sumHaCS+h1cs.getH1CS();
		    			counterHaCS++;
		    			isFoundHaForGly=true;
		    		}	
		    	}//for(int i=0;i<vecCSTemp.size();i++)
    			
    			if(isFoundHaForGly && counterHaCS==2){
    				csHA=sumHaCS/counterHaCS;
    			}
    			else
    				csHA=-9999.0;	    			
    		}//if(curRes.equalsIgnoreCase("GLY") || curRes.equalsIgnoreCase("G"))
    		
    		double csRandCoilHa=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"HA",prevResType);
    		double csRandCoilCa=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"CA",prevResType);
    		double csRandCoilCb=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"CB",prevResType);
    		double csRandCoilCo=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"C",prevResType);
    		double csRandCoilHn=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"HN",prevResType);
    		double csRandCoilN=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"N",prevResType);
    		
    		csHA=csHA-csRandCoilHa;
    		csCA=csCA-csRandCoilCa;
    		csCB=csCB-csRandCoilCb;
    		csCO=csCO-csRandCoilCo;
    		csHN=csHN-csRandCoilHn;
    		csN=csN-csRandCoilN;     
    		
    		residueCS resCS=new residueCS(no,curRes,proteinName,(float)csHA,(float)csCA,(float)csCB,(float)csCO,(float)csHN,(float)csN);
    		vecOneProteinData.add(resCS);
    		
    	}//for(int no=minNoFromCS;no<=maxNoFromCS;no++)
    	Collections.sort(vecOneProteinData, new residueCS.residueCSComparator());//sort the data points based on residue number
    	return vecOneProteinData;   	
    }
	  /**
     * Read the chemical shifts from refdb.
    
     * @param filename the name of the file
         * 
     * @return a vector of random coil chemical shifts
     */
    public Vector<H1CS> refdbReader(String filename)
    {    
    	Vector<H1CS> inputs = new Vector<H1CS>();
    	String atomName="";
     	String resName="";
     	int resNo=0;
     	double cs=-99999.9;     	
     	chemicalShiftDatabase csd=new chemicalShiftDatabase();        	
    	String ss ="";   	
    	String strFirst="";    	    	
    	try
    	{
    		BufferedReader in = new BufferedReader(new FileReader(filename));	
    		ss = in.readLine();    		
    		stop:
    			while(true) 
    			{     		
    				if (ss == null )
						break stop;
    				ss=ss.trim();
    				
    				
    				strFirst=ss.substring(0,1);   				
					if(strFirst.equalsIgnoreCase("#")){
						if ((ss = in.readLine()) == null )
							break stop;
						continue;
					} //if(strFirst.equalsIgnoreCase("#"))    					
    				
    				String str=ss;
    				String subStr=str.substring(0,str.indexOf(",") );
    				subStr=subStr.trim();
    				resNo= Integer.parseInt(subStr);     				
    				
    				str=str.substring(str.indexOf(",")+1, str.length());
    				subStr=str.substring(0,str.indexOf(",") );
    				subStr=subStr.trim();
    				resName=csd.convertResNameFromOneLetterToThreeLetters(subStr);    				
    				
    				str=str.substring(str.indexOf(",")+1, str.length());
    				subStr=str.substring(0,str.indexOf(",") );
    				subStr=subStr.trim();
    				atomName=subStr;
    				
    				str=str.substring(str.indexOf(",")+1, str.length());
    				subStr=str;
    				subStr=subStr.trim();
    				if(!isNumeric(subStr)){
    					if ((ss = in.readLine()) == null )
							break stop;
						continue;
    				}//if(!isNumeric(subStr))
    					
    				cs= new Double(subStr).doubleValue();
    				
    				inputs.add(new H1CS(resNo, resName, atomName, cs));
    				
    				if ((ss = in.readLine()) == null )    				
    					break stop;    	
    				else if(ss.equalsIgnoreCase(""))
     				{
     					if ((ss = in.readLine()) == null )
         				{
         					break stop;
         				}
     				}
    			}
    	}catch (FileNotFoundException e) {
    		System.out.println("File not found: " + filename);
    	}catch (IOException e) 
    	{
    		System.out.println("IOException: the stack trace is:");
    		e.printStackTrace();
    	}
    	return inputs;
    }	
    
    public static boolean isNumeric(String str)  
    {  
      try  
      {  
        double d = Double.parseDouble(str);  
      }  
      catch(NumberFormatException nfe)  
      {  
        return false;  
      }  
      return true;  
    }  
}
