package chemShiftPredict;
/*
HASH Ha Chemical Shift Prediction  Software Version 1.0
Copyright (C) 2009-2012 Bruce Donald Lab, Duke University

HASH is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free
Software Foundation, either version 3 of the License, or (at your option) any
later version.

HASH is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
details.

You should have received a copy of the GNU Lesser General Public License
along with this library; if not, see:
    <http://www.gnu.org/licenses/>.

There are additional restrictions imposed on the use and distribution of this
open-source code, including: (A) this header must be included in any
modification or extension of the code; (B) you are required to cite our
papers in any publications that use this code. The citation for the various
different modules of our software, together with a complete list of
requirements and restrictions are found in the document license.pdf enclosed
with this distribution.

Contact Info:
    Bruce R. Donald
    Duke University
    Department of Computer Science
    Levine Science Research Center (LSRC)
    Durham, NC 27708-0129
    USA
    email: www.cs.duke.edu/brd/

<signature of Bruce Donald>, 01 May, 2011
Bruce R. Donald, Professor of Computer Science and Biochemistry
*/

import java.util.Comparator;

import rdcPanda.Pdb;



//for storing the chemical shifts and related features for each residue
public class residueCS {

	/** The residue no. */
    private int residueNo=0;    
    
    private String residueName="";//residue name
    private String proteinName="";//protein name   
    
    private residueCS preResidueCS=null;//previous residueCS
    private residueCS nextResidueCS=null;//next residueCS
    
    private float csHa=-99999.9f;//ha chemical shift
    private float csCa=-99999.9f;//ca chemical shift
    private float csCb=-99999.9f;//cb chemical shift
    private float csCo=-99999.9f;//co chemical shift
    private float csHn=-99999.9f;//hn chemical shift
    private float csN=-99999.9f;//n chemical shift
    //private float csHb=-99999.9f;//ha chemical shift, not used for now
    
    private float effectHBondHa=-99999.9f;//h-bond effect on ha
    private float effectHBondHn=-99999.9f;//h-bond effect on hn
    private float effectRingHa=-99999.9f;//ring effect
    private float effectRingCa=-99999.9f;//ring effect on ca
    
    private float phi=-99999.9f;//phi angle, in radian unit
    private float psi=-99999.9f;//psi angle, in radian unit
    private float phiPre=-99999.9f;//previous phi angle
    private float psiPre=-99999.9f;//previous psi angle
    private float phiNext=-99999.9f;//next phi angle
    private float psiNext=-99999.9f;//next phi angle
    
    private float csHaPredicted=-99999.9f;//predicted ha chemical shift
    
    public void setResidueNo(int no){
    	residueNo=no;
    }
    public void setPreResidueCS(residueCS preResCS){
    	preResidueCS=preResCS;
    }
    public void setNextResidueCS(residueCS nextResCS){
    	nextResidueCS=nextResCS;
    }
    public void setHaCS(float cs){
    	csHa=cs;
    }
    public void setCaCS(float cs){
    	csCa=cs;
    }
    public void setCbCS(float cs){
    	csCb=cs;
    }
    public void setCoCS(float cs){
    	csCo=cs;
    }
    public void setHnCS(float cs){
    	csHn=cs;
    }
    public void setNCS(float cs){
    	csN=cs;
    }
    public void setResidueName(String name){
    	residueName=name;
    }
    public void setProteinName(String name){
    	proteinName=name;
    }
    public void setPredictedHaCS(float cs){
    	csHaPredicted=cs;
    }
    
    public void setHBondEffectHa(float effect){effectHBondHa=effect;}
    public void setHBondEffectHn(float effect){effectHBondHn=effect;}
    public void setRingEffectHa(float effect){effectRingHa=effect;}
    public void setRingEffectCa(float effect){effectRingCa=effect;}    
    public void setPhi(float angle){phi=angle;}
    public void setPsi(float angle){psi=angle;}
    public void setPrePhi(float angle){phiPre=angle;}
    public void setPrePsi(float angle){psiPre=angle;}
    public void setNextPhi(float angle){phiNext=angle;}
    public void setNextPsi(float angle){psiNext=angle;}
    
    public int getResidueNo(){return residueNo;}
    public residueCS getPreResidueCS(){return preResidueCS;}
    public residueCS getNextResidueCS(){return nextResidueCS;}
    public float getHaCS(){return csHa;}
    public float getCaCS(){return csCa;}
    public float getCbCS(){return csCb;}
    public float getCoCS(){return csCo;}
    public float getHnCS(){return csHn;}
    public float getNCS(){return csN;}
    public String getResidueName(){return residueName;}
    public String getProteinName(){return proteinName;}
    
    public float getHBondEffectHa(){return effectHBondHa;}
    public float getHBondEffectHn(){return effectHBondHn;}
    public float getRingEffectHa(){return effectRingHa;}
    public float getRingEffectCa(){return effectRingCa;}
    public float getPhi(){return phi;}
    public float getPsi(){return psi;}
    public float getPrePhi(){return phiPre;}
    public float getPrePsi(){return psiPre;}
    public float getNextPhi(){return phiNext;}
    public float getNextPsi(){return psiNext;}
    public float getPredictedHaCS(){return csHaPredicted;}
    
    public residueCS(){};
    public residueCS(int no){residueNo=no; };
    
    public residueCS(int No,String res, String protein, float cs_ha, float cs_ca, float cs_cb, float cs_co, float cs_hn, float cs_n,
    		float eff_hbond_ha,float eff_hbond_hn, float eff_ring_ha,float eff_ring_ca,
    		float phi_angle,float psi_angle,float pre_phi_angle,float pre_psi_angle,float next_phi_angle,float next_psi_angle){
    	residueNo=No; 
    	residueName=res;
    	proteinName=protein;
    	csHa=cs_ha;
    	csCa=cs_ca;
    	csCb=cs_cb;
    	csCo=cs_co;
    	csHn=cs_hn;
    	csN=cs_n;
    	effectHBondHa=eff_hbond_ha;
    	effectHBondHn=eff_hbond_hn;
    	effectRingHa=eff_ring_ha;
    	effectRingCa=eff_ring_ca;
    	phi=phi_angle;
    	psi=psi_angle;
    	phiPre=pre_phi_angle;
    	psiPre=pre_psi_angle;
    	phiNext=next_phi_angle;
    	psiNext=next_psi_angle;
    }
    
    //only chemical shift
    public residueCS(int No,String res, String protein, float cs_ha, float cs_ca, float cs_cb, float cs_co, float cs_hn, float cs_n){
    	residueNo=No; 
    	residueName=res;
    	proteinName=protein;
    	csHa=cs_ha;
    	csCa=cs_ca;
    	csCb=cs_cb;
    	csCo=cs_co;
    	csHn=cs_hn;
    	csN=cs_n;    
    }
    
    //A comparator for sorting the residueCS object based on residue number
    /**
     * The Class PdbComparator.
     */
    public static class residueCSComparator implements Comparator<Object>{
	
	/* (non-Javadoc)
	 * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
	 */
		public int compare(Object o1, Object o2){
			residueCS n1 = (residueCS)o1;
			residueCS n2 = (residueCS)o2;
		    int d1 = n1.getResidueNo();
		    int d2 = n2.getResidueNo();
		    if (d1 < d2)
			return -1;
		    else if (d1 > d2)
			return 1;
		    else return 0;
		}
    } 
}
