package chemShiftPredict;
/*
HASH Ha Chemical Shift Prediction  Software Version 1.0
Copyright (C) 2009-2012 Bruce Donald Lab, Duke University

HASH is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free
Software Foundation, either version 3 of the License, or (at your option) any
later version.

HASH is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
details.

You should have received a copy of the GNU Lesser General Public License
along with this library; if not, see:
    <http://www.gnu.org/licenses/>.

There are additional restrictions imposed on the use and distribution of this
open-source code, including: (A) this header must be included in any
modification or extension of the code; (B) you are required to cite our
papers in any publications that use this code. The citation for the various
different modules of our software, together with a complete list of
requirements and restrictions are found in the document license.pdf enclosed
with this distribution.

Contact Info:
    Bruce R. Donald
    Duke University
    Department of Computer Science
    Levine Science Research Center (LSRC)
    Durham, NC 27708-0129
    USA
    email: www.cs.duke.edu/brd/

<signature of Bruce Donald>, 01 May, 2011
Bruce R. Donald, Professor of Computer Science and Biochemistry
*/

import java.util.Collections;
import java.util.Vector;

import rdcPanda.Cartesian;
import rdcPanda.Pdb;

public class ringEffect {
	double c_phe=0.0;
	double c_tyr=0.0;
	double c_trp6=0.0;
	double c_trp5=0.0;
	double c_his=0.0;
	
	public ringEffect(){
		c_phe=0.0197106;
		c_tyr=0.0187376;
		c_trp5=0.0112872;
		c_trp6=0.0202472;
		c_his=0.011913;
	}
	
	/*	public ringEffect(){
		c_phe=1.05;
		c_tyr=0.92;
		c_trp5=0.90;
		c_trp6=1.04;
		c_his=0.43;
	}*/
	 /**
	   * Compute the cross product (a vector) of two vectors.
	   * 
	   * @param v0, v1        Vectors to compute cross product between [x,y,z].
	   * @param crossProduct  Cross product of specified vectors [x,y,z].
	   */
	  public double[] computeCrossProduct (double[] v0, double[] v1)
	  {
	    double crossProduct[] = new double[3];
	    
	    crossProduct[0] = v0[1] * v1[2] - v0[2] * v1[1];
	    crossProduct[1] = v0[2] * v1[0] - v0[0] * v1[2];
	    crossProduct[2] = v0[0] * v1[1] - v0[1] * v1[0];

	    return crossProduct;
	  }
	  /**
	     * calculate the cos of the angle between two vectors v1 and v2
	     * The returned angle is the [0,Pi] region.
	     * 
	     * @param v1 the v1
	     * @param v2 the v2
	     * 
	     * @return the double
	     */
	    public double cosAngle(double[] v1, double[] v2){
			double v1Len = Math.sqrt(v1[0]*v1[0] + v1[1]*v1[1] + v1[2]*v1[2]);
			double v2Len = Math.sqrt(v2[0]*v2[0] + v2[1]*v2[1] + v2[2]*v2[2]);
			double c = (v1[0]*v2[0]+v1[1]*v2[1]+v1[2]*v2[2]) / (v1Len * v2Len);
			return c;//Math.acos(c);  //acos [0, PI] so should be OK
	    }
	    
	/**
    compute the chemical shift detal from the ring current effect for an Ha atom
	
       @param vecPdb pdb of the whole protein.
     * @param resNo_ha current residue ID of the Ha atom
     * @param deltaShift for storing ha chemical shift delta.
     * @return boolean true if h-boneded, false otherwise.
     */
	public double computeDeltaChemShiftFromRingEffectForHa(int resNo_ha,Vector<Pdb> vecPdb){
		Pdb pp=new Pdb();
		double [] coordA = {0.0, 0.0, 0.0};
		int Index= Collections.binarySearch(vecPdb, new Pdb(resNo_ha), new Pdb.PdbComparator());
 	    if(Index<0)
 	    {
 	    	System.out.println("We couldnot find the residue number...");
 	    	System.exit(0);   	
 	    }
 	    Pdb curPdb=vecPdb.elementAt(Index);
 	    int curNo=curPdb.getResidueNo();
 	    String curRes=curPdb.getResidue();
 	    Vector<Cartesian> atomVec=curPdb.getAtomVec();
 	    for (int j=0; j<atomVec.size(); j++)
	   	{ 
 	    	Cartesian cc = (Cartesian)atomVec.elementAt(j);
	   	    String atom = cc.getAtom();
	   	    if(atom.length()>=2)
	   	    	atom=atom.substring(0,2);
	   	    if (atom.equalsIgnoreCase("HA") )
	   	    	coordA = cc.getXYZ();	    
	   	} //for (int j=0; j<atomVec.size(); j++)	
 	    if(curRes.equalsIgnoreCase("GLY")){
 	    	double [] coordA1 = {0.0, 0.0, 0.0};
 	    	double [] coordA2 = {0.0, 0.0, 0.0};
 	    	for (int j=0; j<atomVec.size(); j++)
 		   	{ 
 	 	    	Cartesian cc = (Cartesian)atomVec.elementAt(j);
 		   	    String atom = cc.getAtom();
 		   	    if (atom.equals("HA1"))
 		   	    	coordA1 = cc.getXYZ();	
 		   	    if (atom.equals("HA2"))
		   	    	coordA2 = cc.getXYZ();	 		   	    
 		   	} //for (int j=0; j<atomVec.size(); j++)
 	    	coordA[0]=(coordA1[0]+coordA2[0])/2;
 	    	coordA[1]=(coordA1[1]+coordA2[1])/2;
 	    	coordA[2]=(coordA1[2]+coordA2[2])/2;
 	    }//if(curRes.equalsIgnoreCase("GLY"))
 	   
 	    double sumDelta=0.0;
		for(int i=0;i<vecPdb.size();i++){
			double[][] ringCoords=new double[6][3];
			Pdb pdb=vecPdb.elementAt(i);
			String res=pdb.getResidue();		
			double [] ringCenter=new double[3];
			double r=0.0;
			double[] ringVector1=new double[3];
			double[] ringVector2=new double[3];
			double[] ringNormVec=new double[3];
			double[] dirVec=new double[3];
			double cosTheta=0.0;
			double constant=0.0;
		    boolean isFoundRing=false;
			
			if(res.equalsIgnoreCase("PHE") || res.equalsIgnoreCase("TYR") ){
				Vector<Cartesian> vecAtomRing=pdb.getAtomVec();
				for(int k=0;k<vecAtomRing.size();k++){
	    			Cartesian ccRing=vecAtomRing.elementAt(k);
	    			String atomRing=ccRing.getAtom();
	    			if(atomRing.equalsIgnoreCase("CG"))
	    				ringCoords[0]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CD2"))
	    				ringCoords[1]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CE2"))
	    				ringCoords[2]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CZ"))
	    				ringCoords[3]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CE1"))
	    				ringCoords[4]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CD1")){
	    				isFoundRing=true;	    			
	    				ringCoords[5]=ccRing.getXYZ();
	    			}
				}//for(int k=0;k<vecAtomRing.size();k++)
				if(!isFoundRing)
					continue;
				
				ringCenter[0]=(ringCoords[0][0]+ringCoords[1][0]+ringCoords[2][0]+ringCoords[3][0]+ringCoords[4][0]+
					ringCoords[5][0])/6;
				ringCenter[1]=(ringCoords[0][1]+ringCoords[1][1]+ringCoords[2][1]+ringCoords[3][1]+ringCoords[4][1]+
						ringCoords[5][1])/6;
				ringCenter[2]=(ringCoords[0][2]+ringCoords[1][2]+ringCoords[2][2]+ringCoords[3][2]+ringCoords[4][2]+
						ringCoords[5][2])/6;
				r= Math.sqrt((coordA[0] - ringCenter[0]) * (coordA[0] - ringCenter[0]) 
						+ (coordA[1] - ringCenter[1]) * (coordA[1] - ringCenter[1]) 
						+ (coordA[2] - ringCenter[2]) * (coordA[2] - ringCenter[2]));
				ringVector1=pp.internuclearVec(ringCoords[1],ringCoords[0]);
				ringVector2=pp.internuclearVec(ringCoords[5],ringCoords[0]);
				ringNormVec=computeCrossProduct(ringVector1,ringVector2);
				dirVec=pp.internuclearVec(ringCenter, coordA);
				cosTheta=cosAngle(ringNormVec,dirVec);
				
				if(res.equalsIgnoreCase("PHE"))
					constant=c_phe;
				else
					constant=c_tyr;
			
				
				sumDelta=sumDelta+constant*(1-3*cosTheta*cosTheta)/(r*r*r);
				
			}//if(res.equalsIgnoreCase("PHE") || res.equalsIgnoreCase("TYR") )
			
			if(res.equalsIgnoreCase("HIS") ){
				Vector<Cartesian> vecAtomRing=pdb.getAtomVec();
				for(int k=0;k<vecAtomRing.size();k++){
	    			Cartesian ccRing=vecAtomRing.elementAt(k);
	    			String atomRing=ccRing.getAtom();
	    			if(atomRing.equalsIgnoreCase("CG"))
	    				ringCoords[0]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("ND1"))
	    				ringCoords[1]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CE1"))
	    				ringCoords[2]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("NE2"))
	    				ringCoords[3]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CD2")){
	    				ringCoords[4]=ccRing.getXYZ();
	    				isFoundRing=true;	
	    			}
				}//for(int k=0;k<vecAtomRing.size();k++)
				if(!isFoundRing)
					continue;
				ringCenter[0]=(ringCoords[0][0]+ringCoords[1][0]+ringCoords[2][0]+ringCoords[3][0]+ringCoords[4][0])/5;
				ringCenter[1]=(ringCoords[0][1]+ringCoords[1][1]+ringCoords[2][1]+ringCoords[3][1]+ringCoords[4][1])/5;
				ringCenter[2]=(ringCoords[0][2]+ringCoords[1][2]+ringCoords[2][2]+ringCoords[3][2]+ringCoords[4][2])/5;
				r= Math.sqrt((coordA[0] - ringCenter[0]) * (coordA[0] - ringCenter[0]) 
						+ (coordA[1] - ringCenter[1]) * (coordA[1] - ringCenter[1]) 
						+ (coordA[2] - ringCenter[2]) * (coordA[2] - ringCenter[2]));
				ringVector1=pp.internuclearVec(ringCoords[1],ringCoords[0]);
				ringVector2=pp.internuclearVec(ringCoords[4],ringCoords[0]);
				ringNormVec=computeCrossProduct(ringVector1,ringVector2);
				dirVec=pp.internuclearVec(ringCenter, coordA);
				cosTheta=cosAngle(ringNormVec,dirVec);
				constant=c_his;
			
				
				sumDelta=sumDelta+constant*(1-3*cosTheta*cosTheta)/(r*r*r);
			}//if(res.equalsIgnoreCase("HIS") )
			
			if(res.equalsIgnoreCase("TRP") ){
				
				//first ring with 6 atoms
				Vector<Cartesian> vecAtomRing=pdb.getAtomVec();
				for(int k=0;k<vecAtomRing.size();k++){
	    			Cartesian ccRing=vecAtomRing.elementAt(k);
	    			String atomRing=ccRing.getAtom();
	    			if(atomRing.equalsIgnoreCase("CD2"))
	    				ringCoords[0]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CE3"))
	    				ringCoords[1]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CZ3"))
	    				ringCoords[2]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CH2"))
	    				ringCoords[3]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CZ2"))
	    				ringCoords[4]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CE2")){
	    				ringCoords[5]=ccRing.getXYZ();	    	
	    				isFoundRing=true;	
	    			}
				}//for(int k=0;k<vecAtomRing.size();k++)
				if(!isFoundRing)
					continue;
				ringCenter[0]=(ringCoords[0][0]+ringCoords[1][0]+ringCoords[2][0]+ringCoords[3][0]+ringCoords[4][0]+
						ringCoords[5][0])/6;
				ringCenter[1]=(ringCoords[0][1]+ringCoords[1][1]+ringCoords[2][1]+ringCoords[3][1]+ringCoords[4][1]+
						ringCoords[5][1])/6;
				ringCenter[2]=(ringCoords[0][2]+ringCoords[1][2]+ringCoords[2][2]+ringCoords[3][2]+ringCoords[4][2]+
						ringCoords[5][2])/6;
				r= Math.sqrt((coordA[0] - ringCenter[0]) * (coordA[0] - ringCenter[0]) 
						+ (coordA[1] - ringCenter[1]) * (coordA[1] - ringCenter[1]) 
						+ (coordA[2] - ringCenter[2]) * (coordA[2] - ringCenter[2]));	
				ringVector1=pp.internuclearVec(ringCoords[1],ringCoords[0]);
				ringVector2=pp.internuclearVec(ringCoords[5],ringCoords[0]);
				ringNormVec=computeCrossProduct(ringVector1,ringVector2);
				dirVec=pp.internuclearVec(ringCenter, coordA);
				cosTheta=cosAngle(ringNormVec,dirVec);
				constant=c_trp6;
				
		
				sumDelta=sumDelta+constant*(1-3*cosTheta*cosTheta)/(r*r*r);
				
				//ring with 5 atoms
				for(int k=0;k<vecAtomRing.size();k++){
	    			Cartesian ccRing=vecAtomRing.elementAt(k);
	    			String atomRing=ccRing.getAtom();
	    			if(atomRing.equalsIgnoreCase("CG"))
	    				ringCoords[0]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CD2"))
	    				ringCoords[1]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CE2"))
	    				ringCoords[2]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("NE1"))
	    				ringCoords[3]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CD1")){
	    				ringCoords[4]=ccRing.getXYZ();
	    				isFoundRing=true;	
	    			}
				}//for(int k=0;k<vecAtomRing.size();k++)
				if(!isFoundRing)
					continue;
				ringCenter[0]=(ringCoords[0][0]+ringCoords[1][0]+ringCoords[2][0]+ringCoords[3][0]+ringCoords[4][0])/5;
				ringCenter[1]=(ringCoords[0][1]+ringCoords[1][1]+ringCoords[2][1]+ringCoords[3][1]+ringCoords[4][1])/5;
				ringCenter[2]=(ringCoords[0][2]+ringCoords[1][2]+ringCoords[2][2]+ringCoords[3][2]+ringCoords[4][2])/5;
				r= Math.sqrt((coordA[0] - ringCenter[0]) * (coordA[0] - ringCenter[0]) 
						+ (coordA[1] - ringCenter[1]) * (coordA[1] - ringCenter[1]) 
						+ (coordA[2] - ringCenter[2]) * (coordA[2] - ringCenter[2]));
				ringVector1=pp.internuclearVec(ringCoords[1],ringCoords[0]);
				ringVector2=pp.internuclearVec(ringCoords[4],ringCoords[0]);
				ringNormVec=computeCrossProduct(ringVector1,ringVector2);
				dirVec=pp.internuclearVec(ringCenter, coordA);
				cosTheta=cosAngle(ringNormVec,dirVec);
				constant=c_trp5;
			
				sumDelta=sumDelta+constant*(1-3*cosTheta*cosTheta)/(r*r*r);
			}//if(res.equalsIgnoreCase("TRP") )
			
		}//for(int i=0;i<vecPdb.size();i++)
		
		return sumDelta;		
	}
	
	/**
    compute the chemical shift detal from the ring current effect for an Ha atom
	
       @param vecPdb pdb of the whole protein.
     * @param resNo_ha current residue ID of the Ha atom
     * @param deltaShift for storing ha chemical shift delta.
     * @return boolean true if h-boneded, false otherwise.
     */
	public double computeDeltaChemShiftFromRingEffectForCa(int resNo_ha,Vector<Pdb> vecPdb){
		Pdb pp=new Pdb();
		double [] coordA = {0.0, 0.0, 0.0};
		int Index= Collections.binarySearch(vecPdb, new Pdb(resNo_ha), new Pdb.PdbComparator());
 	    if(Index<0)
 	    {
 	    	System.out.println("We couldnot find the residue number...");
 	    	System.exit(0);   	
 	    }
 	    Pdb curPdb=vecPdb.elementAt(Index);
 	    int curNo=curPdb.getResidueNo();
 	    Vector<Cartesian> atomVec=curPdb.getAtomVec();
 	    for (int j=0; j<atomVec.size(); j++)
	   	{ 
 	    	Cartesian cc = (Cartesian)atomVec.elementAt(j);
	   	    String atom = cc.getAtom();
	   	    if(atom.length()>=2)
	   	    	atom=atom.substring(0,2);
	   	    if (atom.equalsIgnoreCase("CA") )
	   	    	coordA = cc.getXYZ();	    
	   	} //for (int j=0; j<atomVec.size(); j++)	
 	    
 	    double sumDelta=0.0;
		for(int i=0;i<vecPdb.size();i++){
			double[][] ringCoords=new double[6][3];
			Pdb pdb=vecPdb.elementAt(i);
			String res=pdb.getResidue();		
			double [] ringCenter=new double[3];
			double r=0.0;
			double[] ringVector1=new double[3];
			double[] ringVector2=new double[3];
			double[] ringNormVec=new double[3];
			double[] dirVec=new double[3];
			double cosTheta=0.0;
			double constant=0.0;
		    boolean isFoundRing=false;
			
			if(res.equalsIgnoreCase("PHE") || res.equalsIgnoreCase("TYR") ){
				Vector<Cartesian> vecAtomRing=pdb.getAtomVec();
				for(int k=0;k<vecAtomRing.size();k++){
	    			Cartesian ccRing=vecAtomRing.elementAt(k);
	    			String atomRing=ccRing.getAtom();
	    			if(atomRing.equalsIgnoreCase("CG"))
	    				ringCoords[0]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CD2"))
	    				ringCoords[1]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CE2"))
	    				ringCoords[2]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CZ"))
	    				ringCoords[3]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CE1"))
	    				ringCoords[4]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CD1")){
	    				isFoundRing=true;	    			
	    				ringCoords[5]=ccRing.getXYZ();
	    			}
				}//for(int k=0;k<vecAtomRing.size();k++)
				if(!isFoundRing)
					continue;
				
				ringCenter[0]=(ringCoords[0][0]+ringCoords[1][0]+ringCoords[2][0]+ringCoords[3][0]+ringCoords[4][0]+
					ringCoords[5][0])/6;
				ringCenter[1]=(ringCoords[0][1]+ringCoords[1][1]+ringCoords[2][1]+ringCoords[3][1]+ringCoords[4][1]+
						ringCoords[5][1])/6;
				ringCenter[2]=(ringCoords[0][2]+ringCoords[1][2]+ringCoords[2][2]+ringCoords[3][2]+ringCoords[4][2]+
						ringCoords[5][2])/6;
				r= Math.sqrt((coordA[0] - ringCenter[0]) * (coordA[0] - ringCenter[0]) 
						+ (coordA[1] - ringCenter[1]) * (coordA[1] - ringCenter[1]) 
						+ (coordA[2] - ringCenter[2]) * (coordA[2] - ringCenter[2]));
				ringVector1=pp.internuclearVec(ringCoords[1],ringCoords[0]);
				ringVector2=pp.internuclearVec(ringCoords[5],ringCoords[0]);
				ringNormVec=computeCrossProduct(ringVector1,ringVector2);
				dirVec=pp.internuclearVec(ringCenter, coordA);
				cosTheta=cosAngle(ringNormVec,dirVec);
				
				if(res.equalsIgnoreCase("PHE"))
					constant=c_phe;
				else
					constant=c_tyr;
			
				
				sumDelta=sumDelta+constant*(1-3*cosTheta*cosTheta)/(r*r*r);
				
			}//if(res.equalsIgnoreCase("PHE") || res.equalsIgnoreCase("TYR") )
			
			if(res.equalsIgnoreCase("HIS") ){
				Vector<Cartesian> vecAtomRing=pdb.getAtomVec();
				for(int k=0;k<vecAtomRing.size();k++){
	    			Cartesian ccRing=vecAtomRing.elementAt(k);
	    			String atomRing=ccRing.getAtom();
	    			if(atomRing.equalsIgnoreCase("CG"))
	    				ringCoords[0]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("ND1"))
	    				ringCoords[1]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CE1"))
	    				ringCoords[2]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("NE2"))
	    				ringCoords[3]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CD2")){
	    				ringCoords[4]=ccRing.getXYZ();
	    				isFoundRing=true;	
	    			}
				}//for(int k=0;k<vecAtomRing.size();k++)
				if(!isFoundRing)
					continue;
				ringCenter[0]=(ringCoords[0][0]+ringCoords[1][0]+ringCoords[2][0]+ringCoords[3][0]+ringCoords[4][0])/5;
				ringCenter[1]=(ringCoords[0][1]+ringCoords[1][1]+ringCoords[2][1]+ringCoords[3][1]+ringCoords[4][1])/5;
				ringCenter[2]=(ringCoords[0][2]+ringCoords[1][2]+ringCoords[2][2]+ringCoords[3][2]+ringCoords[4][2])/5;
				r= Math.sqrt((coordA[0] - ringCenter[0]) * (coordA[0] - ringCenter[0]) 
						+ (coordA[1] - ringCenter[1]) * (coordA[1] - ringCenter[1]) 
						+ (coordA[2] - ringCenter[2]) * (coordA[2] - ringCenter[2]));
				ringVector1=pp.internuclearVec(ringCoords[1],ringCoords[0]);
				ringVector2=pp.internuclearVec(ringCoords[4],ringCoords[0]);
				ringNormVec=computeCrossProduct(ringVector1,ringVector2);
				dirVec=pp.internuclearVec(ringCenter, coordA);
				cosTheta=cosAngle(ringNormVec,dirVec);
				constant=c_his;
			
				
				sumDelta=sumDelta+constant*(1-3*cosTheta*cosTheta)/(r*r*r);
			}//if(res.equalsIgnoreCase("HIS") )
			
			if(res.equalsIgnoreCase("TRP") ){
				
				//first ring with 6 atoms
				Vector<Cartesian> vecAtomRing=pdb.getAtomVec();
				for(int k=0;k<vecAtomRing.size();k++){
	    			Cartesian ccRing=vecAtomRing.elementAt(k);
	    			String atomRing=ccRing.getAtom();
	    			if(atomRing.equalsIgnoreCase("CD2"))
	    				ringCoords[0]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CE3"))
	    				ringCoords[1]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CZ3"))
	    				ringCoords[2]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CH2"))
	    				ringCoords[3]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CZ2"))
	    				ringCoords[4]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CE2")){
	    				ringCoords[5]=ccRing.getXYZ();	    	
	    				isFoundRing=true;	
	    			}
				}//for(int k=0;k<vecAtomRing.size();k++)
				if(!isFoundRing)
					continue;
				ringCenter[0]=(ringCoords[0][0]+ringCoords[1][0]+ringCoords[2][0]+ringCoords[3][0]+ringCoords[4][0]+
						ringCoords[5][0])/6;
				ringCenter[1]=(ringCoords[0][1]+ringCoords[1][1]+ringCoords[2][1]+ringCoords[3][1]+ringCoords[4][1]+
						ringCoords[5][1])/6;
				ringCenter[2]=(ringCoords[0][2]+ringCoords[1][2]+ringCoords[2][2]+ringCoords[3][2]+ringCoords[4][2]+
						ringCoords[5][2])/6;
				r= Math.sqrt((coordA[0] - ringCenter[0]) * (coordA[0] - ringCenter[0]) 
						+ (coordA[1] - ringCenter[1]) * (coordA[1] - ringCenter[1]) 
						+ (coordA[2] - ringCenter[2]) * (coordA[2] - ringCenter[2]));	
				ringVector1=pp.internuclearVec(ringCoords[1],ringCoords[0]);
				ringVector2=pp.internuclearVec(ringCoords[5],ringCoords[0]);
				ringNormVec=computeCrossProduct(ringVector1,ringVector2);
				dirVec=pp.internuclearVec(ringCenter, coordA);
				cosTheta=cosAngle(ringNormVec,dirVec);
				constant=c_trp6;
				
		
				sumDelta=sumDelta+constant*(1-3*cosTheta*cosTheta)/(r*r*r);
				
				//ring with 5 atoms
				for(int k=0;k<vecAtomRing.size();k++){
	    			Cartesian ccRing=vecAtomRing.elementAt(k);
	    			String atomRing=ccRing.getAtom();
	    			if(atomRing.equalsIgnoreCase("CG"))
	    				ringCoords[0]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CD2"))
	    				ringCoords[1]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CE2"))
	    				ringCoords[2]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("NE1"))
	    				ringCoords[3]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CD1")){
	    				ringCoords[4]=ccRing.getXYZ();
	    				isFoundRing=true;	
	    			}
				}//for(int k=0;k<vecAtomRing.size();k++)
				if(!isFoundRing)
					continue;
				ringCenter[0]=(ringCoords[0][0]+ringCoords[1][0]+ringCoords[2][0]+ringCoords[3][0]+ringCoords[4][0])/5;
				ringCenter[1]=(ringCoords[0][1]+ringCoords[1][1]+ringCoords[2][1]+ringCoords[3][1]+ringCoords[4][1])/5;
				ringCenter[2]=(ringCoords[0][2]+ringCoords[1][2]+ringCoords[2][2]+ringCoords[3][2]+ringCoords[4][2])/5;
				r= Math.sqrt((coordA[0] - ringCenter[0]) * (coordA[0] - ringCenter[0]) 
						+ (coordA[1] - ringCenter[1]) * (coordA[1] - ringCenter[1]) 
						+ (coordA[2] - ringCenter[2]) * (coordA[2] - ringCenter[2]));
				ringVector1=pp.internuclearVec(ringCoords[1],ringCoords[0]);
				ringVector2=pp.internuclearVec(ringCoords[4],ringCoords[0]);
				ringNormVec=computeCrossProduct(ringVector1,ringVector2);
				dirVec=pp.internuclearVec(ringCenter, coordA);
				cosTheta=cosAngle(ringNormVec,dirVec);
				constant=c_trp5;
			
				sumDelta=sumDelta+constant*(1-3*cosTheta*cosTheta)/(r*r*r);
			}//if(res.equalsIgnoreCase("TRP") )
			
		}//for(int i=0;i<vecPdb.size();i++)
		
		return sumDelta;		
	}
	/**
    compute the chemical shift detal from the ring current effect for an Ha atom
	
       @param vecPdb pdb of the whole protein.
     * @param resNo_ha current residue ID of the Ha atom
     * @param deltaShift for storing ha chemical shift delta.
     * @param tag 0: all, 1:Phe, 2: Tyr, 3:Trp6, 4:Trp5, 5:His
     * @return boolean true if h-boneded, false otherwise.
     */
	public double computeDeltaChemShiftFromRingEffectForHaSeparate(int resNo_ha,Vector<Pdb> vecPdb,int tag){
		double sumPhe=0.0,sumTyr=0.0,sumTrp6=0.0, sumTrp5=0.0, sumHis=0.0;
		Pdb pp=new Pdb();
		double [] coordA = {0.0, 0.0, 0.0};
		int Index= Collections.binarySearch(vecPdb, new Pdb(resNo_ha), new Pdb.PdbComparator());
 	    if(Index<0)
 	    {
 	    	System.out.println("We couldnot find the residue number...");
 	    	System.exit(0);   	
 	    }
 	    Pdb curPdb=vecPdb.elementAt(Index);
 	    int curNo=curPdb.getResidueNo();
 	    Vector<Cartesian> atomVec=curPdb.getAtomVec();
 	    for (int j=0; j<atomVec.size(); j++)
	   	{ 
 	    	Cartesian cc = (Cartesian)atomVec.elementAt(j);
	   	    String atom = cc.getAtom();
	   	    if (atom.equals("CA"))
	   	    	coordA = cc.getXYZ();	    
	   	} //for (int j=0; j<atomVec.size(); j++)	
 	    
 	    double sumDelta=0.0;
		for(int i=0;i<vecPdb.size();i++){
			double[][] ringCoords=new double[6][3];
			Pdb pdb=vecPdb.elementAt(i);
			String res=pdb.getResidue();		
			double [] ringCenter=new double[3];
			double r=0.0;
			double[] ringVector1=new double[3];
			double[] ringVector2=new double[3];
			double[] ringNormVec=new double[3];
			double[] dirVec=new double[3];
			double cosTheta=0.0;
			double constant=1.0;
		    boolean isFoundRing=false;
			
			if(res.equalsIgnoreCase("PHE") ){
				Vector<Cartesian> vecAtomRing=pdb.getAtomVec();
				for(int k=0;k<vecAtomRing.size();k++){
	    			Cartesian ccRing=vecAtomRing.elementAt(k);
	    			String atomRing=ccRing.getAtom();
	    			if(atomRing.equalsIgnoreCase("CG"))
	    				ringCoords[0]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CD2"))
	    				ringCoords[1]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CE2"))
	    				ringCoords[2]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CZ"))
	    				ringCoords[3]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CE1"))
	    				ringCoords[4]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CD1")){
	    				isFoundRing=true;	    			
	    				ringCoords[5]=ccRing.getXYZ();
	    			}
				}//for(int k=0;k<vecAtomRing.size();k++)
				if(!isFoundRing)
					continue;
				
				ringCenter[0]=(ringCoords[0][0]+ringCoords[1][0]+ringCoords[2][0]+ringCoords[3][0]+ringCoords[4][0]+
					ringCoords[5][0])/6;
				ringCenter[1]=(ringCoords[0][1]+ringCoords[1][1]+ringCoords[2][1]+ringCoords[3][1]+ringCoords[4][1]+
						ringCoords[5][1])/6;
				ringCenter[2]=(ringCoords[0][2]+ringCoords[1][2]+ringCoords[2][2]+ringCoords[3][2]+ringCoords[4][2]+
						ringCoords[5][2])/6;
				r= Math.sqrt((coordA[0] - ringCenter[0]) * (coordA[0] - ringCenter[0]) 
						+ (coordA[1] - ringCenter[1]) * (coordA[1] - ringCenter[1]) 
						+ (coordA[2] - ringCenter[2]) * (coordA[2] - ringCenter[2]));
				ringVector1=pp.internuclearVec(ringCoords[1],ringCoords[0]);
				ringVector2=pp.internuclearVec(ringCoords[5],ringCoords[0]);
				ringNormVec=computeCrossProduct(ringVector1,ringVector2);
				dirVec=pp.internuclearVec(ringCenter, coordA);
				cosTheta=cosAngle(ringNormVec,dirVec);			
				
				sumPhe=sumPhe+constant*(1-3*cosTheta*cosTheta)/(r*r*r);
				
			}//if(res.equalsIgnoreCase("PHE") )
			if( res.equalsIgnoreCase("TYR") ){
				Vector<Cartesian> vecAtomRing=pdb.getAtomVec();
				for(int k=0;k<vecAtomRing.size();k++){
	    			Cartesian ccRing=vecAtomRing.elementAt(k);
	    			String atomRing=ccRing.getAtom();
	    			if(atomRing.equalsIgnoreCase("CG"))
	    				ringCoords[0]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CD2"))
	    				ringCoords[1]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CE2"))
	    				ringCoords[2]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CZ"))
	    				ringCoords[3]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CE1"))
	    				ringCoords[4]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CD1")){
	    				isFoundRing=true;	    			
	    				ringCoords[5]=ccRing.getXYZ();
	    			}
				}//for(int k=0;k<vecAtomRing.size();k++)
				if(!isFoundRing)
					continue;
				
				ringCenter[0]=(ringCoords[0][0]+ringCoords[1][0]+ringCoords[2][0]+ringCoords[3][0]+ringCoords[4][0]+
					ringCoords[5][0])/6;
				ringCenter[1]=(ringCoords[0][1]+ringCoords[1][1]+ringCoords[2][1]+ringCoords[3][1]+ringCoords[4][1]+
						ringCoords[5][1])/6;
				ringCenter[2]=(ringCoords[0][2]+ringCoords[1][2]+ringCoords[2][2]+ringCoords[3][2]+ringCoords[4][2]+
						ringCoords[5][2])/6;
				r= Math.sqrt((coordA[0] - ringCenter[0]) * (coordA[0] - ringCenter[0]) 
						+ (coordA[1] - ringCenter[1]) * (coordA[1] - ringCenter[1]) 
						+ (coordA[2] - ringCenter[2]) * (coordA[2] - ringCenter[2]));
				ringVector1=pp.internuclearVec(ringCoords[1],ringCoords[0]);
				ringVector2=pp.internuclearVec(ringCoords[5],ringCoords[0]);
				ringNormVec=computeCrossProduct(ringVector1,ringVector2);
				dirVec=pp.internuclearVec(ringCenter, coordA);
				cosTheta=cosAngle(ringNormVec,dirVec);		
				
				sumTyr=sumTyr+constant*(1-3*cosTheta*cosTheta)/(r*r*r);
				
			}//if(res.equalsIgnoreCase("TYR") )
			if(res.equalsIgnoreCase("HIS") ){
				Vector<Cartesian> vecAtomRing=pdb.getAtomVec();
				for(int k=0;k<vecAtomRing.size();k++){
	    			Cartesian ccRing=vecAtomRing.elementAt(k);
	    			String atomRing=ccRing.getAtom();
	    			if(atomRing.equalsIgnoreCase("CG"))
	    				ringCoords[0]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("ND1"))
	    				ringCoords[1]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CE1"))
	    				ringCoords[2]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("NE2"))
	    				ringCoords[3]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CD2")){
	    				ringCoords[4]=ccRing.getXYZ();
	    				isFoundRing=true;	
	    			}
				}//for(int k=0;k<vecAtomRing.size();k++)
				if(!isFoundRing)
					continue;
				ringCenter[0]=(ringCoords[0][0]+ringCoords[1][0]+ringCoords[2][0]+ringCoords[3][0]+ringCoords[4][0])/5;
				ringCenter[1]=(ringCoords[0][1]+ringCoords[1][1]+ringCoords[2][1]+ringCoords[3][1]+ringCoords[4][1])/5;
				ringCenter[2]=(ringCoords[0][2]+ringCoords[1][2]+ringCoords[2][2]+ringCoords[3][2]+ringCoords[4][2])/5;
				r= Math.sqrt((coordA[0] - ringCenter[0]) * (coordA[0] - ringCenter[0]) 
						+ (coordA[1] - ringCenter[1]) * (coordA[1] - ringCenter[1]) 
						+ (coordA[2] - ringCenter[2]) * (coordA[2] - ringCenter[2]));
				ringVector1=pp.internuclearVec(ringCoords[1],ringCoords[0]);
				ringVector2=pp.internuclearVec(ringCoords[4],ringCoords[0]);
				ringNormVec=computeCrossProduct(ringVector1,ringVector2);
				dirVec=pp.internuclearVec(ringCenter, coordA);
				cosTheta=cosAngle(ringNormVec,dirVec);
								
				sumHis=sumHis+constant*(1-3*cosTheta*cosTheta)/(r*r*r);
			}//if(res.equalsIgnoreCase("HIS") )
			
			if(res.equalsIgnoreCase("TRP") ){
				
				//first ring with 6 atoms
				Vector<Cartesian> vecAtomRing=pdb.getAtomVec();
				for(int k=0;k<vecAtomRing.size();k++){
	    			Cartesian ccRing=vecAtomRing.elementAt(k);
	    			String atomRing=ccRing.getAtom();
	    			if(atomRing.equalsIgnoreCase("CD2"))
	    				ringCoords[0]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CE3"))
	    				ringCoords[1]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CZ3"))
	    				ringCoords[2]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CH2"))
	    				ringCoords[3]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CZ2"))
	    				ringCoords[4]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CE2")){
	    				ringCoords[5]=ccRing.getXYZ();	    	
	    				isFoundRing=true;	
	    			}
				}//for(int k=0;k<vecAtomRing.size();k++)
				if(!isFoundRing)
					continue;
				ringCenter[0]=(ringCoords[0][0]+ringCoords[1][0]+ringCoords[2][0]+ringCoords[3][0]+ringCoords[4][0]+
						ringCoords[5][0])/6;
				ringCenter[1]=(ringCoords[0][1]+ringCoords[1][1]+ringCoords[2][1]+ringCoords[3][1]+ringCoords[4][1]+
						ringCoords[5][1])/6;
				ringCenter[2]=(ringCoords[0][2]+ringCoords[1][2]+ringCoords[2][2]+ringCoords[3][2]+ringCoords[4][2]+
						ringCoords[5][2])/6;
				r= Math.sqrt((coordA[0] - ringCenter[0]) * (coordA[0] - ringCenter[0]) 
						+ (coordA[1] - ringCenter[1]) * (coordA[1] - ringCenter[1]) 
						+ (coordA[2] - ringCenter[2]) * (coordA[2] - ringCenter[2]));	
				ringVector1=pp.internuclearVec(ringCoords[1],ringCoords[0]);
				ringVector2=pp.internuclearVec(ringCoords[5],ringCoords[0]);
				ringNormVec=computeCrossProduct(ringVector1,ringVector2);
				dirVec=pp.internuclearVec(ringCenter, coordA);
				cosTheta=cosAngle(ringNormVec,dirVec);
							
		
				sumTrp6=sumTrp6+constant*(1-3*cosTheta*cosTheta)/(r*r*r);
				
				//ring with 5 atoms
				for(int k=0;k<vecAtomRing.size();k++){
	    			Cartesian ccRing=vecAtomRing.elementAt(k);
	    			String atomRing=ccRing.getAtom();
	    			if(atomRing.equalsIgnoreCase("CG"))
	    				ringCoords[0]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CD2"))
	    				ringCoords[1]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CE2"))
	    				ringCoords[2]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("NE1"))
	    				ringCoords[3]=ccRing.getXYZ();
	    			else if(atomRing.equalsIgnoreCase("CD1")){
	    				ringCoords[4]=ccRing.getXYZ();
	    				isFoundRing=true;	
	    			}
				}//for(int k=0;k<vecAtomRing.size();k++)
				if(!isFoundRing)
					continue;
				ringCenter[0]=(ringCoords[0][0]+ringCoords[1][0]+ringCoords[2][0]+ringCoords[3][0]+ringCoords[4][0])/5;
				ringCenter[1]=(ringCoords[0][1]+ringCoords[1][1]+ringCoords[2][1]+ringCoords[3][1]+ringCoords[4][1])/5;
				ringCenter[2]=(ringCoords[0][2]+ringCoords[1][2]+ringCoords[2][2]+ringCoords[3][2]+ringCoords[4][2])/5;
				r= Math.sqrt((coordA[0] - ringCenter[0]) * (coordA[0] - ringCenter[0]) 
						+ (coordA[1] - ringCenter[1]) * (coordA[1] - ringCenter[1]) 
						+ (coordA[2] - ringCenter[2]) * (coordA[2] - ringCenter[2]));
				ringVector1=pp.internuclearVec(ringCoords[1],ringCoords[0]);
				ringVector2=pp.internuclearVec(ringCoords[4],ringCoords[0]);
				ringNormVec=computeCrossProduct(ringVector1,ringVector2);
				dirVec=pp.internuclearVec(ringCenter, coordA);
				cosTheta=cosAngle(ringNormVec,dirVec);
				//constant=c_trp5;
			
				sumTrp5=sumTrp5+constant*(1-3*cosTheta*cosTheta)/(r*r*r);
			}//if(res.equalsIgnoreCase("TRP") )
			
		}//for(int i=0;i<vecPdb.size();i++)
		
		sumDelta=sumTrp5+sumTrp6+sumHis+sumTyr+sumPhe;
		//tag 0: all, 1:Phe, 2: Tyr, 3:Trp6, 4:Trp5, 5:His
		if(tag==0)
			return sumDelta;	
		else if(tag==1)
			return sumPhe;
		else if(tag==2)
			return sumTyr;
		else if(tag==3)
			return sumTrp6;
		else if(tag==4)
			return sumTrp5;
		else if(tag==5)
			return sumHis;
		return sumDelta;
	}
	
}
