package chemShiftPredict;
/*
HASH Ha Chemical Shift Prediction  Software Version 1.0
Copyright (C) 2009-2012 Bruce Donald Lab, Duke University

HASH is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free
Software Foundation, either version 3 of the License, or (at your option) any
later version.

HASH is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
details.

You should have received a copy of the GNU Lesser General Public License
along with this library; if not, see:
    <http://www.gnu.org/licenses/>.

There are additional restrictions imposed on the use and distribution of this
open-source code, including: (A) this header must be included in any
modification or extension of the code; (B) you are required to cite our
papers in any publications that use this code. The citation for the various
different modules of our software, together with a complete list of
requirements and restrictions are found in the document license.pdf enclosed
with this distribution.

Contact Info:
    Bruce R. Donald
    Duke University
    Department of Computer Science
    Levine Science Research Center (LSRC)
    Durham, NC 27708-0129
    USA
    email: www.cs.duke.edu/brd/

<signature of Bruce Donald>, 01 May, 2011
Bruce R. Donald, Professor of Computer Science and Biochemistry
*/


import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Collections;
import java.util.StringTokenizer;
import java.util.Vector;
import rdcPanda.Cartesian;
import rdcPanda.H1CS;
import rdcPanda.Noe;
import rdcPanda.Pdb;
import rdcPanda.PhiPsi;
import rdcPanda.vdw;

public class shiftx2Database {
	
	/** 
	 * Read the Shiftx training or testing data.
	 *  @param pathCSDatabase path of cs directory
	 *  @param pathPdbDatabase path of PDB directory
	 *  @param isTraining whether it is training data, TRUE for training data, FALSE for testing data
	 *  @return a vector of residueCS
	 */
	public Vector <Vector<residueCS>> readShiftxData(String pathCSDatabase,String pathPdbDatabase,boolean isTraining){		
		Vector<Vector<residueCS>> vecData=new  Vector<Vector<residueCS>>();
		
		ringEffect ring_effect=new ringEffect();
		hBondEffect hbond_effect=new hBondEffect();		
		H1CS h1CS=new H1CS();
		Pdb pp=new Pdb();
		chemicalShiftDatabase csd=new chemicalShiftDatabase();
		
		String userDir = System.getProperty("user.dir");
		
		//read random coil chemical shift table:
	    String pathRandomCoilShift=userDir+"/system/cs_database/random_coil_shifts.txt"; //random coil shift table
		randomCoilShift rcs=new randomCoilShift();
		Vector<randomCoilShift> vecRandomShifts=rcs.randomCoilShiftReader(pathRandomCoilShift);
		
		//read all chemical shift files
    	int counter=1;
    	File myDir = new File(pathCSDatabase);
    	String[] contents = myDir.list();//select all structures
    	if (contents == null) 
    		System.out.println(myDir.getName() + " is not a directory");
    	for (int t=0; t<contents.length;t++)  {  
    	
    		String filename = contents[t]; 
    		String fileNameSaved=filename.substring(0,filename.indexOf("_"));
    		
	    	String strCSFile="";
	    	strCSFile=pathCSDatabase+filename;	 	    	
	    	
	    	String strFoundPdbFile=findMatchedPdbFile(pathPdbDatabase,fileNameSaved);
	    	String strPdbFile=pathPdbDatabase+strFoundPdbFile;
	    		    		    	
	    	//read the training chemical shift data
	    	Vector<H1CS> vecCSTemp= new Vector<H1CS>();
	    	
	    	if(isTraining)
	    		vecCSTemp=h1CS.h1CSReader_BMRB(strCSFile); 	 
	    	else
	    		vecCSTemp=h1CS.h1CSReader_BMRB(strCSFile); 
	    		////vecCSTemp=h1CSReader_BMRB_Modified(strCSFile); 	 //read the testing data
	    	    	
	    	//read the training pdb data	    	
	    	Vector<Pdb> vecPdbOld = pp.readPdb(strPdbFile);
	    	
	    	/////////////////////////////
	    	//test: only use bb
	    	Vector<Pdb> vecBBTemp=pp.OutputBackboneNoCB(vecPdbOld);
	    	vecPdbOld=new Vector<Pdb>();
	    	vecPdbOld.addAll(vecBBTemp);
	    	//pp.print(vecPdbOld);
	    	/////////////////////////////
	    	
	    	Vector<Pdb> vecPdb=pp.nameConvert(vecPdbOld); 
	    	    	
	    	//align the residue number
	    	alignResidueNum(vecPdb,vecCSTemp);
	    	
	    	//update PDB residue names
	    	updatePdbResidueName(vecPdb,vecCSTemp);
	    	
	    	boolean isBMRBNEWNameScheme=false;;// whether using the BMRB_NEW naming scheme for atom names.
	    	for(int x=0;x<vecPdb.size();x++){
	    		Pdb pdb=vecPdb.elementAt(x);
	    		String res=pdb.getResidue();
	    		Vector<Cartesian> vecAtom =pdb.getAtomVec();
	    		for(int y=0;y<vecAtom.size();y++){
	    			Cartesian cc=vecAtom.elementAt(y);
	    			String atom=cc.getAtom();
	    			if( !( res.equalsIgnoreCase("ALA")||res.equalsIgnoreCase("GLY")|| res.equalsIgnoreCase("ILE")
	    					||res.equalsIgnoreCase("PRO")||res.equalsIgnoreCase("THR")||res.equalsIgnoreCase("VAL")) ){
	    				if(atom.equalsIgnoreCase("HB3"))
	    					isBMRBNEWNameScheme=true;
	    				break;
	    			}	    			
	    		}//for(int y=0;y<vecAtom.size();y++)
	    		if(isBMRBNEWNameScheme)
	    			break;
	    	}//for(int x=0;x<vecPdb.size();x++)
	    	if(isBMRBNEWNameScheme){
	    		Vector<Pdb>  vecPdbTemp=pp.nameConvert2PDBNew(vecPdb);
	    		vecPdb=new Vector<Pdb>();
	    		vecPdb.addAll(vecPdbTemp);
	    	}	    	
	    	
	    	//compute the phi/psi angles
	    	Vector<PhiPsi> vecPhiPsi=pp.PhiPsiTotalPdb(vecPdb);	    
	    	   	
	    	//find max res no and min res no	    		    	
	    	int minNoFromCS=9999999;
	    	int maxNoFromCS=-99999;
	    	for(int i=0;i<vecCSTemp.size();i++){
	    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
	    		int resNo=h1cs.getResidueNo();
	    		if(resNo>maxNoFromCS)
	    			maxNoFromCS=resNo;  
	    		if(resNo<minNoFromCS)
	    			minNoFromCS=resNo;
	    	}
	    	int minNoFromPdb=9999999;
	    	int maxNoFromPdb=-99999;
	    	for(int i=0;i<vecPdb.size();i++){
	    		Pdb pdb=(Pdb)vecPdb.elementAt(i);
	    		int resNo=pdb.getResidueNo();
	    		if(resNo>maxNoFromPdb)
	    			maxNoFromPdb=resNo;  
	    		if(resNo<minNoFromPdb)
	    			minNoFromPdb=resNo;
	    	}
	    	int maxNo=Math.min(maxNoFromCS, maxNoFromPdb);
	    	int minNo=Math.max(minNoFromCS, minNoFromPdb);
	    	
	    	//for computing the chemical shift delta from hbond effect:
	    	double[] haCSDelta=new double[maxNo+1];
	    	double[] hnCSDelta=new double[maxNo+1];
	    	for(int i=0;i<=maxNo;i++){
	    		haCSDelta[i]=0.0;
	    		hnCSDelta[i]=0.0;
	    	}
	    	Vector<residueCS> vecOneProteinData=new Vector<residueCS>();
	    	
	    	//we skip the first residue and the last residue
	    	for(int no=minNo+1;no<=maxNo-1;no++){     	
	    		
	    		int ind= Collections.binarySearch(vecPdb, new Pdb(no), new Pdb.PdbComparator());
	     	    if(ind<0)
	     	    	continue;
	     	    Pdb curPdb=vecPdb.elementAt(ind);
	     	    String curRes=curPdb.getResidue();
	     	    
	     	    int ind2= Collections.binarySearch(vecPhiPsi, new PhiPsi(no), new PhiPsi.PPComparator());
	     	    if(ind2<0)
	     	    	continue;
	     	   
	     	    PhiPsi phipsi=vecPhiPsi.elementAt(ind2);
	     	 
	     	    int indPrev= Collections.binarySearch(vecPhiPsi, new PhiPsi(no-1), new PhiPsi.PPComparator());
	     	    double phiPrev=0.0;
	     	    double psiPrev=0.0;
	     	    if( indPrev>-1){
	    		   PhiPsi phipsiPre=vecPhiPsi.elementAt(indPrev);
	    		   phiPrev=chemicalShiftDatabase.convertAngleInPI( phipsiPre.getPhi() );
	    		   psiPrev=chemicalShiftDatabase.convertAngleInPI( phipsiPre.getPsi() );
	     	    }
	    	   
	     	    int indNext= Collections.binarySearch(vecPhiPsi, new PhiPsi(no+1), new PhiPsi.PPComparator());
	     	    double phiNext=0.0;
	     	    double psiNext=0.0;
	     	    if( indNext>-1){
	    		   PhiPsi phipsiNext=vecPhiPsi.elementAt(indNext);
	    		   phiNext=chemicalShiftDatabase.convertAngleInPI( phipsiNext.getPhi() );
	    		   psiNext=chemicalShiftDatabase.convertAngleInPI( phipsiNext.getPsi() );
	     	    }
	     	   
	     	    //check the residue name of the neighboring residues
	     	    String prevResType="";
	     	    ind=Collections.binarySearch(vecPdb, new Pdb(no-1), new Pdb.PdbComparator());
	     	    if(ind>-1){	     	    	
		     	   Pdb prevPdb=vecPdb.elementAt(ind);
		     	   prevResType=prevPdb.getResidue();
	     	    }   
	      	  
	     	   ////////////////////////////////
	     	   //compute the chemical shift delta from hbond and ring.
	     	   float deltaHBond_ha=(float)hbond_effect.computeDeltaFromHBondForHaOld(no, vecPdb);
	     	   float deltaHBond_hn=(float)hbond_effect.computeDeltaFromHBondForHnOld(no, vecPdb);	     	   
	     	   float deltaRing=(float)ring_effect.computeDeltaChemShiftFromRingEffectForHa(no, vecPdb)*1000;	     	
	     	   float deltaRingCa=(float)ring_effect.computeDeltaChemShiftFromRingEffectForCa(no, vecPdb)*1000;	    	 		
	    		
	     	   //find the bb chemical shift of current residue
	    		double csN=-9999.0;
	    		double csCA=-9999.0;
	    		double csHN=-9999.0;
	    		double csHA=-9999.0;
	    		double csCB=-9999.0;
	    		double csCO=-9999.0;
	    		
	    		//find cs of N
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && atom.equalsIgnoreCase("N")){
		    			csN=h1cs.getH1CS();
		    			break;
		    		}
	    		}  		
	    		
	    		//find cs of CA
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && atom.equalsIgnoreCase("CA")){
		    			csCA=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	    	
	    		//find cs of HN
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && (atom.equalsIgnoreCase("HN")|| atom.equalsIgnoreCase("H"))  ){
		    			csHN=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	    		
	    		//find cs of HA
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();		    		
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && atom.equalsIgnoreCase("HA")){
		    			csHA=h1cs.getH1CS();
		    			break;
		    		}	
		    	}//for(int i=0;i<vecCSTemp.size();i++)
	    		
	    		//for GLY, we compute the mean of two HA chemical shifts
	    		if(curRes.equalsIgnoreCase("GLY") || curRes.equalsIgnoreCase("G")){
	    			double sumHaCS=0.0;	  
	    			int counterHaCS=0;
	    			boolean isFoundHaForGly=false;	    			
	    			for(int i=0;i<vecCSTemp.size();i++){
			    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
			    		int resNo=h1cs.getResidueNo();		    		
			    		String atom=h1cs.getAtomName();
			    		if(atom.length()>=2)
			    			atom=atom.substring(0,2);
			    		if(resNo==no && atom.equalsIgnoreCase("HA")){
			    			sumHaCS=sumHaCS+h1cs.getH1CS();
			    			counterHaCS++;
			    			isFoundHaForGly=true;
			    		}	
			    	}//for(int i=0;i<vecCSTemp.size();i++)
	    			
	    			if(isFoundHaForGly && counterHaCS==2){
	    				csHA=sumHaCS/counterHaCS;
	    			}
	    			else
	    				csHA=-9999.0;	    			
	    		}//if(curRes.equalsIgnoreCase("GLY") || curRes.equalsIgnoreCase("G"))
	    		
	    		//find cs of CB
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && atom.equalsIgnoreCase("CB")){
		    			csCB=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	    		
	    		//find cs of CO
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && (atom.equalsIgnoreCase("CO") ||atom.equalsIgnoreCase("C"))){
		    			csCO=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}    		
	  
	    		double csRandCoilHa=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"HA",prevResType);
	    		double csRandCoilCa=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"CA",prevResType);
	    		double csRandCoilCb=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"CB",prevResType);
	    		double csRandCoilCo=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"C",prevResType);
	    		double csRandCoilHn=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"HN",prevResType);
	    		double csRandCoilN=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"N",prevResType);
	    		
	    		csHA=csHA-csRandCoilHa;
	    		csCA=csCA-csRandCoilCa;
	    		csCB=csCB-csRandCoilCb;
	    		csCO=csCO-csRandCoilCo;
	    		csHN=csHN-csRandCoilHn;
	    		csN=csN-csRandCoilN;      		
	    		   		
	    		double phi=chemicalShiftDatabase.convertAngleInPI( phipsi.getPhi());//1.5*Math.cos(3*(phipsi.getPhi() + 2.45*Math.PI/180 ))-3.0*Math.cos(phipsi.getPhi()-2.7*Math.PI/180 );
	    		double psi=chemicalShiftDatabase.convertAngleInPI( phipsi.getPsi() );//-0.4*Math.cos(3* ( phipsi.getPsi() +4.8*Math.PI/180 ) )-1.5*Math.cos(phipsi.getPsi() -0.84*Math.PI/180 )+0.7;
	    		
	    		residueCS resCS=new residueCS(no,curRes,fileNameSaved,(float)csHA,(float)csCA,(float)csCB,(float)csCO,(float)csHN,(float)csN,
	    				(float)deltaHBond_ha,(float)deltaHBond_hn,(float)deltaRing,(float)deltaRingCa,
	    				(float)phi,(float)psi,(float)phiPrev,(float)psiPrev,(float)phiNext,(float)psiNext);
	    		
	    		vecOneProteinData.add(resCS);
	    		///////////////////	    		
	    		//String str_csHA=csToString((float)(csHA));String str_csCA=csToString((float)(csCA));String str_csCB=csToString((float)(csCB));
	    		//String str_csCO=csToString((float)(csCO));String str_csHN=csToString((float)(csHN));String str_csN=csToString((float)(csN));	    						    		
	    		counter++;	    		
	    	}//for(int i=1;i<=maxNo;i++)
	    	
	    	Collections.sort(vecOneProteinData, new residueCS.residueCSComparator());//sort the data points based on residue number
	    	vecData.add(vecOneProteinData);	    	
    	}//for (int t=0; t<contents.length;t++) 
    	
    	return vecData;
	}
	
	
	public static void main (String[] args){
		ringEffect ring_effect=new ringEffect();
		hBondEffect hbond_effect=new hBondEffect();		
		
		chemicalShiftDatabase csd=new chemicalShiftDatabase();	//////////////////////
		
		//String resType="VAL";
		
		H1CS h1CS=new H1CS();
		Pdb pp=new Pdb();
		String userDir = System.getProperty("user.dir");
		
		//two places need to be changed in order to switch between training and testing data sets
		//here is the first place:
		
		//read the chemical shift and PDB data		
		//read the training data
	    String pathCSDatabase=userDir+"/system/cs_database/shifts/";  //chemical shift input file directory	
	   String pathPdbDatabase=userDir+"/system/cs_database/pdb/";  //Pdb input file directory
	  
	    //read the testing data
	  //String pathCSDatabase=userDir+"/system/cs_database/shifts_test/";  //chemical shift input file directory	
	  //  String pathPdbDatabase=userDir+"/system/cs_database/pdb_test/";  //Pdb input file directory	
			
		//read random coil chemical shift table:
	    String pathRandomCoilShift=userDir+"/system/cs_database/random_coil_shifts.txt"; //random coil shift table
		randomCoilShift rcs=new randomCoilShift();
		Vector<randomCoilShift> vecRandomShifts=rcs.randomCoilShiftReader(pathRandomCoilShift);
		
		//read BMRB statistical information:
		//String srcBMRB=userDir+"/system/BMRB_CS.txt";    	
    	//Vector<H1CS> vecBMRB=h1CS.ReadBMRBSatistics(srcBMRB);
    	//Vector<H1CS> vecBMRBNew=new Vector<H1CS>();		
    	//vecBMRBNew=h1CS.CSNameConvertFromBMRBNewToPDBNew(vecBMRB);//assume that the default naming scheme of cs is BMRB
    	   	
    	//read all chemical shift files
    	int counter=1;
    	File myDir = new File(pathCSDatabase);
    	String[] contents = myDir.list();//select all structures
    	if (contents == null) 
    		System.out.println(myDir.getName() + " is not a directory");
    	for (int t=0; t<contents.length;t++)  {  
    	
    		String filename = contents[t]; 
    		String fileNameSaved=filename.substring(0,filename.indexOf("_"));
    		
	    	String strCSFile="";
	    	strCSFile=pathCSDatabase+filename;	 	    	
	    	
	    	String strFoundPdbFile=findMatchedPdbFile(pathPdbDatabase,fileNameSaved);
	    	String strPdbFile=pathPdbDatabase+strFoundPdbFile;
	    	
	    	//two places need to be changed to switch between training and testing 
			//Second place:
	    		    	
	    	//read the training chemical shift data
	    	Vector<H1CS> vecCSTemp=h1CS.h1CSReader_BMRB(strCSFile); 	 
	    	
	    	//read the testing data
	    	///Vector<H1CS> vecCSTemp=h1CSReader_BMRB_Modified(strCSFile); 	 
	    	
	    	//read the training pdb data	    	
	    	Vector<Pdb> vecPdbOld = pp.readPdb(strPdbFile);
	    	Vector<Pdb> vecPdb=pp.nameConvert(vecPdbOld); 
	    	    	
	    	//align the residue number
	    	alignResidueNum(vecPdb,vecCSTemp);
	    	
	    	//update PDB residue names
	    	updatePdbResidueName(vecPdb,vecCSTemp);
	    	
	    	boolean isBMRBNEWNameScheme=false;;// whether using the BMRB_NEW naming scheme for atom names.
	    	for(int x=0;x<vecPdb.size();x++){
	    		Pdb pdb=vecPdb.elementAt(x);
	    		String res=pdb.getResidue();
	    		Vector<Cartesian> vecAtom =pdb.getAtomVec();
	    		for(int y=0;y<vecAtom.size();y++){
	    			Cartesian cc=vecAtom.elementAt(y);
	    			String atom=cc.getAtom();
	    			if( !( res.equalsIgnoreCase("ALA")||res.equalsIgnoreCase("GLY")|| res.equalsIgnoreCase("ILE")
	    					||res.equalsIgnoreCase("PRO")||res.equalsIgnoreCase("THR")||res.equalsIgnoreCase("VAL")) ){
	    				if(atom.equalsIgnoreCase("HB3"))
	    					isBMRBNEWNameScheme=true;
	    				break;
	    			}	    			
	    		}//for(int y=0;y<vecAtom.size();y++)
	    		if(isBMRBNEWNameScheme)
	    			break;
	    	}//for(int x=0;x<vecPdb.size();x++)
	    	if(isBMRBNEWNameScheme){
	    		Vector<Pdb>  vecPdbTemp=pp.nameConvert2PDBNew(vecPdb);
	    		vecPdb=new Vector<Pdb>();
	    		vecPdb.addAll(vecPdbTemp);
	    	}	    	
	    	
	    	//compute the phi/psi angles
	    	Vector<PhiPsi> vecPhiPsi=pp.PhiPsiTotalPdb(vecPdb);	    
	    	   	
	    	//find max res no and min res no	    		    	
	    	int minNoFromCS=9999999;
	    	int maxNoFromCS=-99999;
	    	for(int i=0;i<vecCSTemp.size();i++){
	    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
	    		int resNo=h1cs.getResidueNo();
	    		if(resNo>maxNoFromCS)
	    			maxNoFromCS=resNo;  
	    		if(resNo<minNoFromCS)
	    			minNoFromCS=resNo;
	    	}
	    	int minNoFromPdb=9999999;
	    	int maxNoFromPdb=-99999;
	    	for(int i=0;i<vecPdb.size();i++){
	    		Pdb pdb=(Pdb)vecPdb.elementAt(i);
	    		int resNo=pdb.getResidueNo();
	    		if(resNo>maxNoFromPdb)
	    			maxNoFromPdb=resNo;  
	    		if(resNo<minNoFromPdb)
	    			minNoFromPdb=resNo;
	    	}
	    	int maxNo=Math.min(maxNoFromCS, maxNoFromPdb);
	    	int minNo=Math.max(minNoFromCS, minNoFromPdb);
	    	
	    	//for computing the chemical shift delta from hbond effect:
	    	double[] haCSDelta=new double[maxNo+1];
	    	double[] hnCSDelta=new double[maxNo+1];
	    	for(int i=0;i<=maxNo;i++){
	    		haCSDelta[i]=0.0;
	    		hnCSDelta[i]=0.0;
	    	}
	    	
	    	for(int no=minNo+3;no<=maxNo-2;no++){  		
	    		
	    		//we skip the first residue
	    		if (no==1)
	    			continue;
	    		int ind= Collections.binarySearch(vecPdb, new Pdb(no), new Pdb.PdbComparator());
	     	    if(ind<0)
	     	    	continue;
	     	    Pdb curPdb=vecPdb.elementAt(ind);
	     	    String curRes=curPdb.getResidue();
	     	    
	     	    int ind2= Collections.binarySearch(vecPhiPsi, new PhiPsi(no), new PhiPsi.PPComparator());
	     	    if(ind2<0)
	     	    	continue;
	     	   
	     	    PhiPsi phipsi=vecPhiPsi.elementAt(ind2);
	     	 
	     	    int indPrev= Collections.binarySearch(vecPhiPsi, new PhiPsi(no-1), new PhiPsi.PPComparator());
	     	    double phiPrev=0.0;
	     	    double psiPrev=0.0;
	     	    if( indPrev>-1){
	    		   PhiPsi phipsiPre=vecPhiPsi.elementAt(indPrev);
	    		   phiPrev=chemicalShiftDatabase.convertAngleInPI( phipsiPre.getPhi() );
	    		   psiPrev=chemicalShiftDatabase.convertAngleInPI( phipsiPre.getPsi() );
	     	    }
	    	   
	     	    int indNext= Collections.binarySearch(vecPhiPsi, new PhiPsi(no+1), new PhiPsi.PPComparator());
	     	    double phiNext=0.0;
	     	    double psiNext=0.0;
	     	    if( indNext>-1){
	    		   PhiPsi phipsiNext=vecPhiPsi.elementAt(indNext);
	    		   phiNext=chemicalShiftDatabase.convertAngleInPI( phipsiNext.getPhi() );
	    		   psiNext=chemicalShiftDatabase.convertAngleInPI( phipsiNext.getPsi() );
	     	    }
	     	   
	     	    //check the residue name of the neighboring residues
	     	    String prevResType="";
	     	    ind=Collections.binarySearch(vecPdb, new Pdb(no-1), new Pdb.PdbComparator());
	     	    if(ind>-1){	     	    	
		     	   Pdb prevPdb=vecPdb.elementAt(ind);
		     	   prevResType=prevPdb.getResidue();
	     	    }
	     	   
	     	    String prePreResType="";
	     	    ind=Collections.binarySearch(vecPdb, new Pdb(no-2), new Pdb.PdbComparator());
	     	    if(ind>-1){	     	    	
		     	   Pdb prevPrePdb=vecPdb.elementAt(ind);
		     	   prePreResType=prevPrePdb.getResidue();
	     	    }
	     	   
	     	    String preThreeResType="";
	     	    ind=Collections.binarySearch(vecPdb, new Pdb(no-3), new Pdb.PdbComparator());
	     	    if(ind>-1){	     	    	
		     	   Pdb preThreePdb=vecPdb.elementAt(ind);
		     	  preThreeResType=preThreePdb.getResidue();
	     	    }
	     	   
	     	    String preFourResType="";
	     	    ind=Collections.binarySearch(vecPdb, new Pdb(no-4), new Pdb.PdbComparator());
	     	    if(ind>-1){	     	    	
		     	   Pdb preFourPdb=vecPdb.elementAt(ind);
		     	   preFourResType=preFourPdb.getResidue();
	     	    } 
	     	   
	     	    String nextResType="";
	     	    ind=Collections.binarySearch(vecPdb, new Pdb(no+1), new Pdb.PdbComparator());
	     	    if(ind>-1){	     	    	
		     	   Pdb nextPdb=vecPdb.elementAt(ind);
		     	   nextResType=nextPdb.getResidue();
	     	    }
	     	   
	     	    String nextTwoResType="";
	     	    ind=Collections.binarySearch(vecPdb, new Pdb(no+2), new Pdb.PdbComparator());
	     	    if(ind>-1){	     	    	
		     	   Pdb nextTwoPdb=vecPdb.elementAt(ind);
		     	  nextTwoResType=nextTwoPdb.getResidue();
	     	    }
	     	   
	     	    String nextThreeResType="";
	     	    ind=Collections.binarySearch(vecPdb, new Pdb(no+3), new Pdb.PdbComparator());
	     	    if(ind>-1){	     	    	
		     	 Pdb nextThreePdb=vecPdb.elementAt(ind);
		     	nextThreeResType=nextThreePdb.getResidue();
	     	    }	     	  
	      	  
	     	   ////////////////////////////////
	     	   //compute the chemical shift delta from hbond and ring.
	     	   float deltaHBond_ha=(float)hbond_effect.computeDeltaFromHBondForHaOld(no, vecPdb);
	     	   float deltaHBond_hn=(float)hbond_effect.computeDeltaFromHBondForHnOld(no, vecPdb);	     	   
	     	   float deltaRing=(float)ring_effect.computeDeltaChemShiftFromRingEffectForHa(no, vecPdb)*1000;	     	
	     	   float deltaRingCa=(float)ring_effect.computeDeltaChemShiftFromRingEffectForCa(no, vecPdb)*1000;	    	 		
	    		
	     	   //find the bb chemical shift of current residue
	    		double csN=-9999.0;
	    		double csCA=-9999.0;
	    		double csHN=-9999.0;
	    		double csHA=-9999.0;
	    		double csCB=-9999.0;
	    		double csCO=-9999.0;
	    		
	    		//find cs of N
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && atom.equalsIgnoreCase("N")){
		    			csN=h1cs.getH1CS();
		    			break;
		    		}
	    		}  		
	    		
	    		//find cs of CA
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && atom.equalsIgnoreCase("CA")){
		    			csCA=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	    	
	    		//find cs of HN
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && (atom.equalsIgnoreCase("HN")|| atom.equalsIgnoreCase("H"))  ){
		    			csHN=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	    		
	    		//find cs of HA
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();		    		
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && atom.equalsIgnoreCase("HA")){
		    			csHA=h1cs.getH1CS();
		    			break;
		    		}	
		    	}//for(int i=0;i<vecCSTemp.size();i++)
	    		//for GLY, we compute the mean of two HA chemical shifts
	    		if(curRes.equalsIgnoreCase("GLY") || curRes.equalsIgnoreCase("G")){
	    			double sumHaCS=0.0;	  
	    			int counterHaCS=0;
	    			boolean isFoundHaForGly=false;
	    			for(int i=0;i<vecCSTemp.size();i++){
			    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
			    		int resNo=h1cs.getResidueNo();		    		
			    		String atom=h1cs.getAtomName();
			    		if(atom.length()>=2)
			    			atom=atom.substring(0,2);
			    		if(resNo==no && atom.equalsIgnoreCase("HA")){
			    			sumHaCS=sumHaCS+h1cs.getH1CS();
			    			counterHaCS++;
			    			isFoundHaForGly=true;
			    		}	
			    	}//for(int i=0;i<vecCSTemp.size();i++)
	    			if(isFoundHaForGly){
	    				csHA=sumHaCS/counterHaCS;
	    			}
	    			
	    		}//if(curRes.equalsIgnoreCase("GLY") || curRes.equalsIgnoreCase("G"))
	    		
	    		//find cs of CB
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && atom.equalsIgnoreCase("CB")){
		    			csCB=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	    		
	    		//find cs of CO
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && (atom.equalsIgnoreCase("CO") ||atom.equalsIgnoreCase("C"))){
		    			csCO=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}    		
	  
	    		double csRandCoilHa=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"HA",prevResType);
	    		double csRandCoilCa=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"CA",prevResType);
	    		double csRandCoilCb=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"CB",prevResType);
	    		double csRandCoilCo=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"C",prevResType);
	    		double csRandCoilHn=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"HN",prevResType);
	    		double csRandCoilN=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"N",prevResType);
	    		
	    		csHA=csHA-csRandCoilHa;
	    		csCA=csCA-csRandCoilCa;
	    		csCB=csCB-csRandCoilCb;
	    		csCO=csCO-csRandCoilCo;
	    		csHN=csHN-csRandCoilHn;
	    		csN=csN-csRandCoilN;
	    		
	    		
	    		//////////////////////////
	    		//get the chemical shifts of previous bb atoms
	    		double csPreN=-9999.0;
	    		double csPreCA=-9999.0;
	    		double csPreHN=-9999.0;	    		
	    		double csPreCB=-9999.0;
	    		double csPreCO=-9999.0;
	    		//find cs of N for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no-1 && atom.equalsIgnoreCase("N")){
		    			csPreN=h1cs.getH1CS();
		    			break;
		    		}	    		
		    	}	   		
	    		
	    		//find cs of CA for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no-1 && atom.equalsIgnoreCase("CA")){
		    			csPreCA=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}	    	
	    		//find cs of HN for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no-1 && (atom.equalsIgnoreCase("HN")|| atom.equalsIgnoreCase("H"))  ){
		    			csPreHN=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	      		//find cs of CB for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no-1 && atom.equalsIgnoreCase("CB")){
		    			csPreCB=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}	    		
	    		//find cs of CO for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no-1 && (atom.equalsIgnoreCase("CO") ||atom.equalsIgnoreCase("C"))){
		    			csPreCO=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	    		csRandCoilHa=csd.getRandomCoilChemShift(vecRandomShifts,prevResType,"HA",prePreResType);
	    		csRandCoilCa=csd.getRandomCoilChemShift(vecRandomShifts,prevResType,"CA",prePreResType);
	    		csRandCoilCb=csd.getRandomCoilChemShift(vecRandomShifts,prevResType,"CB",prePreResType);
	    		csRandCoilCo=csd.getRandomCoilChemShift(vecRandomShifts,prevResType,"C",prePreResType);
	    		csRandCoilHn=csd.getRandomCoilChemShift(vecRandomShifts,prevResType,"HN",prePreResType);
	    		csRandCoilN=csd.getRandomCoilChemShift(vecRandomShifts,prevResType,"N",prePreResType);	    		
	    		
	    		csPreCA=csPreCA-csRandCoilCa;
	    		csPreCB=csPreCB-csRandCoilCb;
	    		csPreCO=csPreCO-csRandCoilCo;
	    		csPreHN=csPreHN-csRandCoilHn;
	    		csPreN=csPreN-csRandCoilN;
	    		///////////////////////////////////
	    		
	    		//////////////////////////
	    		//get the chemical shifts of next bb atoms
	    		double csNextN=-9999.0;
	    		double csNextCA=-9999.0;
	    		double csNextHN=-9999.0;	    		
	    		double csNextCB=-9999.0;
	    		double csNextCO=-9999.0;
	    		
	    		//find cs of N for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no+1 && atom.equalsIgnoreCase("N")){
		    			csNextN=h1cs.getH1CS();
		    			break;
		    		}	    	
		    	}	   		
	    		
	    		//find cs of CA for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no+1 && atom.equalsIgnoreCase("CA")){
		    			csNextCA=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}	    	
	    		//find cs of HN for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no+1 && (atom.equalsIgnoreCase("HN")|| atom.equalsIgnoreCase("H"))  ){
		    			csNextHN=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	      		//find cs of CB for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no+1 && atom.equalsIgnoreCase("CB")){
		    			csNextCB=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}	    		
	    		//find cs of CO for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no+1 && (atom.equalsIgnoreCase("CO") ||atom.equalsIgnoreCase("C"))){
		    			csNextCO=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	    		
	    		csRandCoilCa=csd.getRandomCoilChemShift(vecRandomShifts,nextResType,"CA",curRes);
	    		csRandCoilCb=csd.getRandomCoilChemShift(vecRandomShifts,nextResType,"CB",curRes);
	    		csRandCoilCo=csd.getRandomCoilChemShift(vecRandomShifts,nextResType,"C",curRes);
	    		csRandCoilHn=csd.getRandomCoilChemShift(vecRandomShifts,nextResType,"HN",curRes);
	    		csRandCoilN=csd.getRandomCoilChemShift(vecRandomShifts,nextResType,"N",curRes);	    		
	    		
	    		csNextCA=csNextCA-csRandCoilCa;
	    		csNextCB=csNextCB-csRandCoilCb;
	    		csNextCO=csNextCO-csRandCoilCo;
	    		csNextHN=csNextHN-csRandCoilHn;
	    		csNextN=csNextN-csRandCoilN;
	    		///////////////////////////////////
	    		
	    		//////////////////////////
	    		//get the chemical shifts of bb atoms in i-2 residue
	    		double csPreTwoN=-9999.0;
	    		double csPreTwoCA=-9999.0;
	    		double csPreTwoHN=-9999.0;	    		
	    		double csPreTwoCB=-9999.0;
	    		double csPreTwoCO=-9999.0;
	    		//find cs of N for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no-2 && atom.equalsIgnoreCase("N")){
		    			csPreTwoN=h1cs.getH1CS();
		    			break;
		    		}	    		
		    	}	   		
	    		
	    		//find cs of CA for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no-2 && atom.equalsIgnoreCase("CA")){
		    			csPreTwoCA=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}	    	
	    		//find cs of HN for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no-2 && (atom.equalsIgnoreCase("HN")|| atom.equalsIgnoreCase("H"))  ){
		    			csPreTwoHN=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	      		//find cs of CB for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no-2 && atom.equalsIgnoreCase("CB")){
		    			csPreTwoCB=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}	    		
	    		//find cs of CO for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no-2 && (atom.equalsIgnoreCase("CO") ||atom.equalsIgnoreCase("C"))){
		    			csPreTwoCO=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	    		csRandCoilHa=csd.getRandomCoilChemShift(vecRandomShifts,prePreResType,"HA",preThreeResType);
	    		csRandCoilCa=csd.getRandomCoilChemShift(vecRandomShifts,prePreResType,"CA",preThreeResType);
	    		csRandCoilCb=csd.getRandomCoilChemShift(vecRandomShifts,prePreResType,"CB",preThreeResType);
	    		csRandCoilCo=csd.getRandomCoilChemShift(vecRandomShifts,prePreResType,"C",preThreeResType);
	    		csRandCoilHn=csd.getRandomCoilChemShift(vecRandomShifts,prePreResType,"HN",preThreeResType);
	    		csRandCoilN=csd.getRandomCoilChemShift(vecRandomShifts,prePreResType,"N",preThreeResType);	    		
	    		
	    		csPreTwoCA=csPreTwoCA-csRandCoilCa;
	    		csPreTwoCB=csPreTwoCB-csRandCoilCb;
	    		csPreTwoCO=csPreTwoCO-csRandCoilCo;
	    		csPreTwoHN=csPreTwoHN-csRandCoilHn;
	    		csPreTwoN=csPreTwoN-csRandCoilN;
	    		///////////////////////////////////
	    		
	    		//////////////////////////
	    		//get the chemical shifts of bb atoms in residue i+2
	    		double csNextTwoN=-9999.0;
	    		double csNextTwoCA=-9999.0;
	    		double csNextTwoHN=-9999.0;	    		
	    		double csNextTwoCB=-9999.0;
	    		double csNextTwoCO=-9999.0;
	    		
	    		//find cs of N for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no+2 && atom.equalsIgnoreCase("N")){
		    			csNextTwoN=h1cs.getH1CS();
		    			break;
		    		}	    	
		    	}	   		
	    		
	    		//find cs of CA for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no+2 && atom.equalsIgnoreCase("CA")){
		    			csNextTwoCA=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}	    	
	    		//find cs of HN for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no+2 && (atom.equalsIgnoreCase("HN")|| atom.equalsIgnoreCase("H"))  ){
		    			csNextTwoHN=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	      		//find cs of CB for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no+2 && atom.equalsIgnoreCase("CB")){
		    			csNextTwoCB=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}	    		
	    		//find cs of CO for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no+2 && (atom.equalsIgnoreCase("CO") ||atom.equalsIgnoreCase("C"))){
		    			csNextTwoCO=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	    		
	    		csRandCoilCa=csd.getRandomCoilChemShift(vecRandomShifts,nextTwoResType,"CA",nextResType);
	    		csRandCoilCb=csd.getRandomCoilChemShift(vecRandomShifts,nextTwoResType,"CB",nextResType);
	    		csRandCoilCo=csd.getRandomCoilChemShift(vecRandomShifts,nextTwoResType,"C",nextResType);
	    		csRandCoilHn=csd.getRandomCoilChemShift(vecRandomShifts,nextTwoResType,"HN",nextResType);
	    		csRandCoilN=csd.getRandomCoilChemShift(vecRandomShifts,nextTwoResType,"N",nextResType);	    		
	    		
	    		csNextTwoCA=csNextTwoCA-csRandCoilCa;
	    		csNextTwoCB=csNextTwoCB-csRandCoilCb;
	    		csNextTwoCO=csNextTwoCO-csRandCoilCo;
	    		csNextTwoHN=csNextTwoHN-csRandCoilHn;
	    		csNextTwoN=csNextTwoN-csRandCoilN;
	    		///////////////////////////////////
	    		
	    		//////////////////////////
	    		//get the chemical shifts of bb atoms in i-3 residue
	    		double csPreThreeN=-9999.0;
	    		double csPreThreeCA=-9999.0;
	    		double csPreThreeHN=-9999.0;	    		
	    		double csPreThreeCB=-9999.0;
	    		double csPreThreeCO=-9999.0;
	    		//find cs of N for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no-3 && atom.equalsIgnoreCase("N")){
		    			csPreThreeN=h1cs.getH1CS();
		    			break;
		    		}	    		
		    	}	   		
	    		
	    		//find cs of CA for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no-3 && atom.equalsIgnoreCase("CA")){
		    			csPreThreeCA=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}	    	
	    		//find cs of HN for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no-3 && (atom.equalsIgnoreCase("HN")|| atom.equalsIgnoreCase("H"))  ){
		    			csPreThreeHN=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	      		//find cs of CB for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no-3 && atom.equalsIgnoreCase("CB")){
		    			csPreThreeCB=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}	    		
	    		//find cs of CO for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no-3 && (atom.equalsIgnoreCase("CO") ||atom.equalsIgnoreCase("C"))){
		    			csPreThreeCO=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	    		
	    		csRandCoilCa=csd.getRandomCoilChemShift(vecRandomShifts,preThreeResType,"CA",preFourResType);
	    		csRandCoilCb=csd.getRandomCoilChemShift(vecRandomShifts,preThreeResType,"CB",preFourResType);
	    		csRandCoilCo=csd.getRandomCoilChemShift(vecRandomShifts,preThreeResType,"C",preFourResType);
	    		csRandCoilHn=csd.getRandomCoilChemShift(vecRandomShifts,preThreeResType,"HN",preFourResType);
	    		csRandCoilN=csd.getRandomCoilChemShift(vecRandomShifts,preThreeResType,"N",preFourResType);	    		
	    		
	    		csPreThreeCA=csPreThreeCA-csRandCoilCa;
	    		csPreThreeCB=csPreThreeCB-csRandCoilCb;
	    		csPreThreeCO=csPreThreeCO-csRandCoilCo;
	    		csPreThreeHN=csPreThreeHN-csRandCoilHn;
	    		csPreThreeN=csPreThreeN-csRandCoilN;
	    		///////////////////////////////////
	    		
	    		//////////////////////////
	    		//get the chemical shifts of bb atoms in residue i+3
	    		double csNextThreeN=-9999.0;
	    		double csNextThreeCA=-9999.0;
	    		double csNextThreeHN=-9999.0;	    		
	    		double csNextThreeCB=-9999.0;
	    		double csNextThreeCO=-9999.0;
	    		
	    		//find cs of N for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no+3 && atom.equalsIgnoreCase("N")){
		    			csNextThreeN=h1cs.getH1CS();
		    			break;
		    		}	    	
		    	}	   		
	    		
	    		//find cs of CA for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no+3 && atom.equalsIgnoreCase("CA")){
		    			csNextThreeCA=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}	    	
	    		//find cs of HN for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no+3 && (atom.equalsIgnoreCase("HN")|| atom.equalsIgnoreCase("H"))  ){
		    			csNextThreeHN=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	      		//find cs of CB for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no+3 && atom.equalsIgnoreCase("CB")){
		    			csNextThreeCB=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}	    		
	    		//find cs of CO for previous residue
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no+3 && (atom.equalsIgnoreCase("CO") ||atom.equalsIgnoreCase("C"))){
		    			csNextThreeCO=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	    		
	    		csRandCoilCa=csd.getRandomCoilChemShift(vecRandomShifts,nextThreeResType,"CA",nextTwoResType);
	    		csRandCoilCb=csd.getRandomCoilChemShift(vecRandomShifts,nextThreeResType,"CB",nextTwoResType);
	    		csRandCoilCo=csd.getRandomCoilChemShift(vecRandomShifts,nextThreeResType,"C",nextTwoResType);
	    		csRandCoilHn=csd.getRandomCoilChemShift(vecRandomShifts,nextThreeResType,"HN",nextTwoResType);
	    		csRandCoilN=csd.getRandomCoilChemShift(vecRandomShifts,nextThreeResType,"N",nextTwoResType);	    		
	    		
	    		csNextThreeCA=csNextThreeCA-csRandCoilCa;
	    		csNextThreeCB=csNextThreeCB-csRandCoilCb;
	    		csNextThreeCO=csNextThreeCO-csRandCoilCo;
	    		csNextThreeHN=csNextThreeHN-csRandCoilHn;
	    		csNextThreeN=csNextThreeN-csRandCoilN;
	    		///////////////////////////////////
	    		   		
	    		double phi=chemicalShiftDatabase.convertAngleInPI( phipsi.getPhi());//1.5*Math.cos(3*(phipsi.getPhi() + 2.45*Math.PI/180 ))-3.0*Math.cos(phipsi.getPhi()-2.7*Math.PI/180 );
	    		double psi=chemicalShiftDatabase.convertAngleInPI( phipsi.getPsi() );//-0.4*Math.cos(3* ( phipsi.getPsi() +4.8*Math.PI/180 ) )-1.5*Math.cos(phipsi.getPsi() -0.84*Math.PI/180 )+0.7;
	    		
	    		///////////////////	    		
	    		String str_csHA=csToString((float)(csHA));String str_csCA=csToString((float)(csCA));String str_csCB=csToString((float)(csCB));
	    		String str_csCO=csToString((float)(csCO));String str_csHN=csToString((float)(csHN));String str_csN=csToString((float)(csN));
	    		String str_csPreCA=csToString((float)(csPreCA));String str_csPreCB=csToString((float)(csPreCB));String str_csPreCO=csToString((float)(csPreCO));
	    		String str_csPreHN=csToString((float)(csPreHN));String str_csPreN=csToString((float)(csPreN));
	    		String str_csNextCA=csToString((float)(csNextCA));String str_csNextCB=csToString((float)(csNextCB));String str_csNextCO=csToString((float)(csNextCO));
	    		String str_csNextHN=csToString((float)(csNextHN));String str_csNextN=csToString((float)(csNextN));
	    		String str_csPreTwoCA=csToString((float)(csPreTwoCA));String str_csPreTwoCB=csToString((float)(csPreTwoCB));String str_csPreTwoCO=csToString((float)(csPreTwoCO));
	    		String str_csPreTwoHN=csToString((float)(csPreTwoHN));String str_csPreTwoN=csToString((float)(csPreTwoN));
	    		String str_csNextTwoCA=csToString((float)(csNextTwoCA));String str_csNextTwoCB=csToString((float)(csNextTwoCB));String str_csNextTwoCO=csToString((float)(csNextTwoCO));
	    		String str_csNextTwoHN=csToString((float)(csNextTwoHN));String str_csNextTwoN=csToString((float)(csNextTwoN));
	    		
	    		/*System.out.println(counter+ " "+ (float)(csHA-csRandCoilHa)+ " "+(float)(csCA-csRandCoilCa)+" "+(float)(csCB-csRandCoilCb)+" " +(float)(csCO-csRandCoilCo)
	    				+" "+(float)(csHN-csRandCoilHn)+" "+(float)(csN-csRandCoilN)+ " "+(float)deltaHBond_ha+" "+(float)deltaHBond_hn+" "+ (float)deltaRing+" "+ (float)phi
	    				+" "+(float)psi+ " "+ (float)phiPrev+" "+(float)psiPrev +" "+ (float)phiNext+" "+(float)psiNext);
	    		*/
	    		
	    		//System.out.println(counter+ " "+ (float)(csHA)+ " "+(float)(csCA)+" "+(float)(csCB)+" " +(float)(csCO)
	    		//		+" "+(float)(csHN)+" "+(float)(csN)+ " "+(float)deltaHBond_ha+" "+(float)deltaHBond_hn+" "+ (float)deltaRing+" "+deltaRingCa+" "+ (float)phi
	    		//		+" "+(float)psi+ " "+ (float)phiPrev+" "+(float)psiPrev +" "+ (float)phiNext+" "+(float)psiNext+" "+curRes+" "+ no+" "+fileNameSaved);
	    		
	    		System.out.println(counter+ " "+ str_csHA+ " "+str_csCA+" "+str_csCB+" " +str_csCO+" "+str_csHN+" "+str_csN
	    				+" "+str_csPreCA+" "+str_csPreCB+" " +str_csPreCO+" "+str_csPreHN+" "+str_csPreN
	    				+" "+str_csNextCA+" "+str_csNextCB+" " +str_csNextCO+" "+str_csNextHN+" "+str_csNextN
	    				+" "+str_csPreTwoCA+" "+str_csPreTwoCB+" " +str_csPreTwoCO+" "+str_csPreTwoHN+" "+str_csPreTwoN
	    				+" "+str_csNextTwoCA+" "+str_csNextTwoCB+" " +str_csNextTwoCO+" "+str_csNextTwoHN+" "+str_csNextTwoN
	    				//+" "+(float)(csPreThreeCA)+" "+(float)(csPreThreeCB)+" " +(float)(csPreThreeCO)+" "+(float)(csPreThreeHN)+" "+(float)(csPreThreeN)
	    				//+" "+(float)(csNextThreeCA)+" "+(float)(csNextThreeCB)+" " +(float)(csNextThreeCO)+" "+(float)(csNextThreeHN)+" "+(float)(csNextThreeN)	    				
	    				+ " "+(float)deltaHBond_ha+" "+(float)deltaHBond_hn+" "+ (float)deltaRing+" "+deltaRingCa+" "+ (float)phi
	    				+" "+(float)psi+ " "+ (float)phiPrev+" "+(float)psiPrev +" "+ (float)phiNext+" "+(float)psiNext+" "+curRes+" "+ no+" "+fileNameSaved);
	    					    		
	    		counter++;
	    		
	    	}//for(int i=1;i<=maxNo;i++)	    	
    	}//for (int t=0; t<contents.length;t++) 
	}//end of main function
	
	//update the residue name of pdb using the sequence information in chemical shifts
	public static void updatePdbResidueName(Vector<Pdb> vecPdb, Vector<H1CS> vecCS){
		for(int i=0;i<vecPdb.size();i++){
			Pdb pdb=vecPdb.elementAt(i);
			int no=pdb.getResidueNo();
			
			int ind=Collections.binarySearch(vecCS, new H1CS(no), new H1CS.resNoComparator());
			if(ind<0)
				continue;
			H1CS h1csTemp=vecCS.elementAt(ind);
	     	String csResName=h1csTemp.getResidueType();
	     	pdb.setResidue(csResName);
	     	
		}//for(int i=0;i<vecPdb.size();i++)
	}
	//called by the main function. Align the residue number
	public static void alignResidueNum(Vector<Pdb> vecPdb, Vector<H1CS> vecCS){
		
		//find max res no and min res no	    		    	
    	int minNoFromCS=9999999;
    	int maxNoFromCS=-99999;
    	for(int i=0;i<vecCS.size();i++){
    		H1CS h1cs=(H1CS)vecCS.elementAt(i);
    		int resNo=h1cs.getResidueNo();
    		if(resNo>maxNoFromCS)
    			maxNoFromCS=resNo;  
    		if(resNo<minNoFromCS)
    			minNoFromCS=resNo;
    	}
    				
		
		boolean isFoundMathed=false;
		int csNoStart=0;
		int pdbNoStart=0;
		for(int no=minNoFromCS;no<=maxNoFromCS-5;no++){			
			//get the residue type of current resdiue
			String csRes0="",csRes1="",csRes2="",csRes3="",csRes4="";
			for(int i=0;i<vecCS.size();i++){
	    		H1CS h1cs=(H1CS)vecCS.elementAt(i);
	    		int resNo=h1cs.getResidueNo();
	    		if(resNo==no)    			
	    			csRes0=h1cs.getResidueType();       		
	    		if( (no+1)==resNo)
	    			csRes1=h1cs.getResidueType();	    		
	    		if( (no+2)==resNo)  
	    			csRes2=h1cs.getResidueType();
	    		if( (no+3)==resNo)  
	    			csRes3=h1cs.getResidueType();
	    		if( (no+4)==resNo)  
	    			csRes4=h1cs.getResidueType();
	    	}//for(int i=0;i<vecCS.size();i++)    			
			
			for(int i=0;i<vecPdb.size()-5;i++){
				Pdb pdb0=vecPdb.elementAt(i);
				Pdb pdb1=vecPdb.elementAt(i+1);
				Pdb pdb2=vecPdb.elementAt(i+2);
				Pdb pdb3=vecPdb.elementAt(i+3);
				Pdb pdb4=vecPdb.elementAt(i+4);
				
				String pdbRes0=pdb0.getResidue();
				String pdbRes1=pdb1.getResidue();
				String pdbRes2=pdb2.getResidue();
				String pdbRes3=pdb3.getResidue();
				String pdbRes4=pdb4.getResidue();
				if(csRes0.equalsIgnoreCase(pdbRes0) && csRes1.equalsIgnoreCase(pdbRes1) &&
						csRes2.equalsIgnoreCase(pdbRes2) && csRes3.equalsIgnoreCase(pdbRes3)&&
						csRes4.equalsIgnoreCase(pdbRes4)){
					isFoundMathed=true;
					pdbNoStart=pdb0.getResidueNo();
					csNoStart=no;
					break;
				}			
			}//for(int i=0;i<vecPdb.size();i++)
			if(isFoundMathed)
				break;
		}//for(int no=minNoFromCS;no<=maxNoFromCS;no++)
		
		
		if(!isFoundMathed){
			System.out.println("Here match is not found...exist...");
			System.exit(0);
		}
		
		int offset=pdbNoStart-csNoStart;
		for(int i=0;i<vecCS.size();i++){
			H1CS h1cs=vecCS.elementAt(i);
			int resNo=h1cs.getResidueNo();
			h1cs.setResidueNo(resNo+offset);
		}
	}
	
	//called in the main function
	public static String findMatchedPdbFile(String strPdbDir,String csFileName){		
		String foundPdbFile="";		
		
		File myDir = new File(strPdbDir);
    	String[] contents = myDir.list();//select all structures
    	if (contents == null){
    		System.out.println(myDir.getName() + " is not a directory");
    		return foundPdbFile;
    	}
    	for (int t=0; t<contents.length;t++)  {  
        	
    		String filename = contents[t]; 
    		String fileNameSaved=filename.substring(0,filename.indexOf("_"));
    		if(fileNameSaved.equalsIgnoreCase(csFileName)){
    			foundPdbFile=filename;
    			break;
    		}    			
    	}//for (int t=0; t<contents.length;t++)    	
    	
		return foundPdbFile;
	}
	
	private static String csToString(float cs){
		String str_cs="";
		if(cs>-100 && cs< 100)
			str_cs=Float.toString( (float)(cs));
		else
			str_cs="NA";
		
		str_cs=Float.toString( (float)(cs));
		
		return str_cs;
	}
	  /**
	    * read chemical shifts, in BMRB format, but with one more column of residue number (used in SHIFTX2 testing data)
	    * 
	    * @param filename the name of the file
	    * 
	    * @return a vector of h1cs object
	    */
	    public static Vector<H1CS> h1CSReader_BMRB_Modified(String filename)
	    {
	    	Vector inputs = new Vector();
	    	double csHN = 0.0, csN15 = 0.0, csH1 = 0.0, intensity = 0.0;
	    	Vector csVec = new Vector();
	    	StringTokenizer st = new StringTokenizer("");
	    	String str = "";
	    	int index1=0, index2=0;
	    	String order="";
	    	double value=0.0;
	    	int no =0;
	    	String aaType = "", ss ="";
	    	String atomName="";
	    	String resName="";
	    	int resNo=0;
	    	int resNoTemp=0;
	    	int csID=0;
	    	Noe noe=new Noe();
	    	
	    	try
	    	{
	    		BufferedReader in = new BufferedReader(new FileReader(filename));	
	    		ss = in.readLine();
	    		ss=ss.trim();
	    		
	    		stop:
	    			while(true) 
	    			{   	
	    				if(ss.equalsIgnoreCase(""))
	     				{
	     					if ((ss = in.readLine()) == null )
	         				{
	         					break stop;
	         				}
	     					continue;
	     				}
	    				 
	    				st = new StringTokenizer(ss);
	    				if (st.hasMoreTokens())
	    					csID =  Integer.parseInt(st.nextToken());
	    					
	    				if (st.hasMoreTokens())
	    					resNo =  Integer.parseInt(st.nextToken());
	    				
	    				if (st.hasMoreTokens())
	    					resNoTemp =  Integer.parseInt(st.nextToken());
	    				
	    				if (st.hasMoreTokens())
	    					resName = st.nextToken();
	    				if (st.hasMoreTokens())
	    					atomName = st.nextToken();
	    				if (st.hasMoreTokens())
	    					aaType = st.nextToken();
	    				if (st.hasMoreTokens())
	    					csH1  = new Double(st.nextToken()).doubleValue();
	    				
	    				if (csH1>300.0 || csH1<-300.0)
						{
							if ((ss = in.readLine()) == null)
								break stop;     
							else if(ss.equalsIgnoreCase(""))
		    				{
		    					if ((ss = in.readLine()) == null )
		        				{
		        					break stop;
		        				}
		    				}
							continue;
						}
	    				
	    				atomName=noe.UpdateNOEAtomName2(resName, atomName);//changed here
	    				inputs.add(new H1CS(resNo, resName, atomName, csH1));
	    				if ((ss = in.readLine()) == null){
	    					break stop;
	    				}
	    				else if(ss.equalsIgnoreCase(""))
	    				{
	    					if ((ss = in.readLine()) == null )
	        				{
	        					break stop;
	        				}
	    				}
	    			}
	    	}catch (FileNotFoundException e) {
	    		System.out.println("File not found: " + filename);
	    	}catch (IOException e) 
	    	{
	    		System.out.println("IOException: the stack trace is:");
	    		e.printStackTrace();
	    	}
	    	return inputs;
	    }
	  
}
