package chemShiftPredict;
/*
HASH Ha Chemical Shift Prediction  Software Version 1.0
Copyright (C) 2009-2012 Bruce Donald Lab, Duke University

HASH is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free
Software Foundation, either version 3 of the License, or (at your option) any
later version.

HASH is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
details.

You should have received a copy of the GNU Lesser General Public License
along with this library; if not, see:
    <http://www.gnu.org/licenses/>.

There are additional restrictions imposed on the use and distribution of this
open-source code, including: (A) this header must be included in any
modification or extension of the code; (B) you are required to cite our
papers in any publications that use this code. The citation for the various
different modules of our software, together with a complete list of
requirements and restrictions are found in the document license.pdf enclosed
with this distribution.

Contact Info:
    Bruce R. Donald
    Duke University
    Department of Computer Science
    Levine Science Research Center (LSRC)
    Durham, NC 27708-0129
    USA
    email: www.cs.duke.edu/brd/

<signature of Bruce Donald>, 01 May, 2011
Bruce R. Donald, Professor of Computer Science and Biochemistry
*/

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.Collections;
import java.util.Vector;

import Jampack.JampackException;

import rdcPanda.Assign;
import rdcPanda.Cartesian;
import rdcPanda.H1CS;
import rdcPanda.Pdb;
import rdcPanda.PhiPsi;

public class testForNasca {
	
	public static void main (String[] args) throws IOException, InterruptedException{
		long startTime = System.currentTimeMillis();
		
		testForNasca testfornasca=new testForNasca();
		
		shiftx2Database shiftx=new shiftx2Database();		
		haPredictMain haPredMain=new haPredictMain();
		chemicalShiftDatabase csd=new chemicalShiftDatabase();
		
		String userDir = System.getProperty("user.dir");
		
		/////////////////////////////////
		//read the training data from Shiftx database
		String pathCSDatabase=userDir+"/system/cs_database/shiftx/shifts/";  //chemical shift input file directory	
		String pathPdbDatabase=userDir+"/system/cs_database/shiftx/pdb/";    //Pdb input file directory
		//Vector<Vector<residueCS>> vecTrainDataShiftx=shiftx.readShiftxData(pathCSDatabase, pathPdbDatabase,true);
			
		String trainSaveFileName=userDir+"/system/cs_database/shiftx/shiftx_train_data.txt";
	    //haPredMain.writeResdiueCSDataToFile(vecTrainDataShiftx,trainSaveFileName);
		
		Vector<Vector<residueCS>> vecTrainDataShiftx=haPredMain.readResdiueCSDataFromFile(trainSaveFileName);	
	    
	    //write as training data in R.
	    String trainRFileName=userDir+"/system/cs_database/r_files/r_train_data.txt";
	    // haPredMain.writeResdiueCSDataToFileForR(vecTrainDataShiftx,trainRFileName);
		////////////////////////////////
		
	    //////////////////////////////////
	    //read the training data from TALOS
		String pathCSDatabaseTalos=userDir+"/system/cs_database/talos/shifts/";  //chemical shift input file directory	
		String pathPdbDatabaseTalos=userDir+"/system/cs_database/talos/pdb/";    //Pdb input file directory
		//Vector<Vector<residueCS>> vecTrainDataTalos=csd.readTalosData(pathCSDatabaseTalos, pathPdbDatabaseTalos,true);
	    
		String trainSaveFileNameTalos=userDir+"/system/cs_database/talos/talos_train_data.txt";
	   // haPredMain.writeResdiueCSDataToFile(vecTrainDataTalos,trainSaveFileNameTalos);
		
	    Vector<Vector<residueCS>> vecTrainDataTalos=haPredMain.readResdiueCSDataFromFile(trainSaveFileNameTalos);		    
	    //////////////////////////////////
	    
	    
	    
		///////////////////////////
		//read the refDB training data
		refDB refdb=new refDB();
		String pathRefCSDatabase=userDir+"/system/cs_database/refdb/shift_refdb/";  //chemical shift input file directory	
		System.out.println("Reading refDB training data...");
		//Vector<Vector<residueCS>> vecTrainDataRefDB=refdb.readRefDBTrainingData(pathRefCSDatabase);
		
		String savedRefCSDatabase=userDir+"/system/cs_database/test_for_nasca/refdb_data.txt"; 
		//haPredMain.writeResdiueCSDataToFile(vecTrainDataRefDB,savedRefCSDatabase);
		
		Vector<Vector<residueCS>> vecTrainDataRefDB=haPredMain.readResdiueCSDataFromFile(savedRefCSDatabase);
		///////////////////////////
		
			
		////////////////////////////////////////////////////
		//read the testing data (eta. ubq, GB1, FF2, hSRI):
		pathCSDatabase=userDir+"/system/cs_database/test_for_nasca/shifts_test/";   //chemical shift input file directory	
		pathPdbDatabase=userDir+"/system/cs_database/test_for_nasca/pdb_test/";     //Pdb input file directory
		Vector<Vector<residueCS>> vecTestDataShiftx=testfornasca.readTestingData(pathCSDatabase, pathPdbDatabase,false);
		
		String testSaveFileName=userDir+"/system/cs_database/test_for_nasca/nasca_test_data.txt";
		haPredMain.writeResdiueCSDataToFile(vecTestDataShiftx,testSaveFileName);
		
		//Vector<Vector<residueCS>> vecTestDataShiftx=haPredMain.readResdiueCSDataFromFile(testSaveFileName);
		
		
		//write the testing data to R data table
		String testRFileName=userDir+"/system/cs_database/test_for_nasca/r_files/r_test_data_two_neighbors.txt";		
		haPredMain.writeResdiueCSDataToFileForRTwoNeighbors(vecTestDataShiftx,testRFileName);
		
		String testOneNeighborRFileName=userDir+"/system/cs_database/test_for_nasca/r_files/r_test_data_one_neighbor.txt";	
		haPredMain.writeResdiueCSDataToFileForROneNeighbor(vecTestDataShiftx,testOneNeighborRFileName);
		
		String testNoNeighborRFileName=userDir+"/system/cs_database/test_for_nasca/r_files/r_test_data_no_neighbor.txt";	
		haPredMain.writeResdiueCSDataToFileForRNoNeighbor(vecTestDataShiftx,testNoNeighborRFileName);
		
		
		int resWindow=6;//8
		float hErr=0.06f;//0.08f;
		float heavyErr=0.6f;//0.7f;
		boolean isCaUsed=false;
		boolean isCbUsed=false;
		boolean isCoUsed=false;
		boolean isHnUsed=false;
		boolean isNUsed=true;
		
		System.out.println("Residue window = "+ resWindow +", hErr="+hErr+", heavyErr="+heavyErr);
	/*	System.out.print("Types of chemical shifts used: ");
		if(isCaUsed)
			System.out.print("Ca ");
		if(isCbUsed)
			System.out.print("Cb ");
		if(isCoUsed)
			System.out.print("Co ");
		if(isHnUsed)
			System.out.print("Hn ");
		if(isNUsed)
			System.out.print("N ");
		System.out.println("");*/
		
		Vector<Vector<residueCS>> vecTrainData=new Vector<Vector<residueCS>>();
		vecTrainData.addAll(vecTrainDataShiftx);
		vecTrainData.addAll(vecTrainDataTalos);		
		vecTrainData.addAll(vecTrainDataRefDB);
		
		//haPredMain.computeBestMatchedHaCS(vecTestDataShiftx,vecTrainDataShiftx,resWindow,hErr,heavyErr,isCaUsed,isCbUsed,isCoUsed,isHnUsed,isNUsed);
		haPredMain.computeBestMatchedHaCSHDistance(vecTestDataShiftx,vecTrainData,resWindow,hErr,heavyErr);
		//haPredMain.computeBestMatchedHaCSHDistanceAverage(vecTestDataShiftx,vecTrainData,resWindow,hErr,heavyErr);
		
		//////////////////////////////////////
		//execute the R package to perform the prediction based on the structural information
	    String strRBin="H:/workspace/ProteinNMR/system/cs_database/r_files/R-2.11.1/bin/Rscript";
		String rScriptsFile="r_scripts_two_neighbors.R";
		///String rScriptsFile="r_scripts_two_neighbors_no_structure.R";
		
		String rOutFile="RunRResults.txt";		
		testfornasca.runRScripts(strRBin,rScriptsFile,rOutFile);
		
		rScriptsFile="r_scripts_one_neighbor.R";
		///rScriptsFile="r_scripts_one_neighbor_no_structure.R";
		testfornasca.runRScripts(strRBin,rScriptsFile,rOutFile);
		
		rScriptsFile="r_scripts_no_neighbor.R";
		////rScriptsFile="r_scripts_no_neighbor_no_structure.R";
		testfornasca.runRScripts(strRBin,rScriptsFile,rOutFile);
		
		///////////////////////////////////////////////////////
		//read the predicted Ha chemical shifts from R, using both chemical shift and structural information
    	String rResultFileName=userDir+"/system/cs_database/test_for_nasca/r_files/r_predicted_ha.txt";
		
		Vector<Vector<residueCS>> vecRPredictedHaCS=haPredMain.readHaChemicalShiftsFromRFile(rResultFileName);		
		System.out.println("Updating the ha chemical shifts using R predicted results...");
		haPredMain.updateHaChemicalShiftFromRResults(vecTestDataShiftx,vecRPredictedHaCS);
		
	
		String rResultOneNeighborFileName=userDir+"/system/cs_database/test_for_nasca/r_files/r_predicted_ha_one_neighbor.txt";	
		
		Vector<Vector<residueCS>> vecRPredictedHaCSOneNeighbor=haPredMain.readHaChemicalShiftsFromRFile(rResultOneNeighborFileName);		
		System.out.println("Updating the ha chemical shifts using R predicted results, using only structural information...");
		haPredMain.updateHaChemicalShiftFromRResults(vecTestDataShiftx,vecRPredictedHaCSOneNeighbor);
		
		String rResultNoNeighborFileName=userDir+"/system/cs_database/test_for_nasca/r_files/r_predicted_ha_no_neighbor.txt";		
		Vector<Vector<residueCS>> vecRPredictedHaCSNoNeighbor=haPredMain.readHaChemicalShiftsFromRFile(rResultNoNeighborFileName);		
		System.out.println("Updating the ha chemical shifts using R predicted results, using only structural information...");
		haPredMain.updateHaChemicalShiftFromRResults(vecTestDataShiftx,vecRPredictedHaCSNoNeighbor);
		
	/*	String rResultOnlyStructureFileName=userDir+"/system/cs_database/r_files/r_predicted_ha_only_structure.txt";		
		Vector<Vector<residueCS>> vecRPredictedHaCSOnlyStr=haPredMain.readHaChemicalShiftsFromRFile(rResultOnlyStructureFileName);		
		System.out.println("Updating the ha chemical shifts using R predicted results, using only structural information...");
		haPredMain.updateHaChemicalShiftFromRResults(vecTestDataShiftx,vecRPredictedHaCSOnlyStr);
		*/
		
		
		/////////////////////////////////////
		
		haPredMain.haPredictionSummary(vecTestDataShiftx);
		
		long endTime = System.currentTimeMillis() ;
	 	double totalTime = (double) ((endTime - startTime) / 60000.0); //in minutes
	 	System.out.println("Time= "+ totalTime +" minutes");
	}
	
	
	public Vector<residueCS>  predictHaForNasca (Vector<Pdb> vecPdb,Vector<H1CS> vecCS) throws IOException, InterruptedException{
		long startTime = System.currentTimeMillis();
		
		testForNasca testfornasca=new testForNasca();
		
		shiftx2Database shiftx=new shiftx2Database();		
		haPredictMain haPredMain=new haPredictMain();
		chemicalShiftDatabase csd=new chemicalShiftDatabase();
		
		String userDir = System.getProperty("user.dir");
		
		/////////////////////////////////
		//read the training data from Shiftx database
		//String pathCSDatabase=userDir+"/system/cs_database/shiftx/shifts/";  //chemical shift input file directory	
		//String pathPdbDatabase=userDir+"/system/cs_database/shiftx/pdb/";    //Pdb input file directory
		//Vector<Vector<residueCS>> vecTrainDataShiftx=shiftx.readShiftxData(pathCSDatabase, pathPdbDatabase,true);
			
		String trainSaveFileName=userDir+"/system/cs_database/shiftx/shiftx_train_data.txt";
	    //haPredMain.writeResdiueCSDataToFile(vecTrainDataShiftx,trainSaveFileName);
		
		Vector<Vector<residueCS>> vecTrainDataShiftx=haPredMain.readResdiueCSDataFromFile(trainSaveFileName);	
	    
	    //write as training data in R.
	    String trainRFileName=userDir+"/system/cs_database/r_files/r_train_data.txt";
	    // haPredMain.writeResdiueCSDataToFileForR(vecTrainDataShiftx,trainRFileName);
		////////////////////////////////
		
	    //////////////////////////////////
	    //read the training data from TALOS
		String pathCSDatabaseTalos=userDir+"/system/cs_database/talos/shifts/";  //chemical shift input file directory	
		String pathPdbDatabaseTalos=userDir+"/system/cs_database/talos/pdb/";    //Pdb input file directory
		//Vector<Vector<residueCS>> vecTrainDataTalos=csd.readTalosData(pathCSDatabaseTalos, pathPdbDatabaseTalos,true);
	    
		String trainSaveFileNameTalos=userDir+"/system/cs_database/talos/talos_train_data.txt";
	   // haPredMain.writeResdiueCSDataToFile(vecTrainDataTalos,trainSaveFileNameTalos);
		
	    Vector<Vector<residueCS>> vecTrainDataTalos=haPredMain.readResdiueCSDataFromFile(trainSaveFileNameTalos);		    
	    //////////////////////////////////
	    
	    
	    
		///////////////////////////
		//read the refDB training data
		refDB refdb=new refDB();
		String pathRefCSDatabase=userDir+"/system/cs_database/refdb/shift_refdb/";  //chemical shift input file directory	
		System.out.println("Reading refDB training data...");
		//Vector<Vector<residueCS>> vecTrainDataRefDB=refdb.readRefDBTrainingData(pathRefCSDatabase);
		
		String savedRefCSDatabase=userDir+"/system/cs_database/test_for_nasca/refdb_data.txt"; 
		//haPredMain.writeResdiueCSDataToFile(vecTrainDataRefDB,savedRefCSDatabase);
		
		Vector<Vector<residueCS>> vecTrainDataRefDB=haPredMain.readResdiueCSDataFromFile(savedRefCSDatabase);
		///////////////////////////
		
			
		////////////////////////////////////////////////////
		//read the testing data (eta. ubq, GB1, FF2, hSRI):
		//pathCSDatabase=userDir+"/system/cs_database/test_for_nasca/shifts_test/";   //chemical shift input file directory	
		//pathPdbDatabase=userDir+"/system/cs_database/test_for_nasca/pdb_test/";     //Pdb input file directory
		//Vector<Vector<residueCS>> vecTestDataShiftx=testfornasca.readTestingData(pathCSDatabase, pathPdbDatabase,false);
		
		Vector<Vector<residueCS>> vecTestDataShiftx=readTestingDataFromInputPdbNCS(vecPdb,vecCS);
		
		
		String testSaveFileName=userDir+"/system/cs_database/test_for_nasca/nasca_test_data.txt";
		haPredMain.writeResdiueCSDataToFile(vecTestDataShiftx,testSaveFileName);
		
		//Vector<Vector<residueCS>> vecTestDataShiftx=haPredMain.readResdiueCSDataFromFile(testSaveFileName);
		
		
		//write the testing data to R data table
		String testRFileName=userDir+"/system/cs_database/test_for_nasca/r_files/r_test_data_two_neighbors.txt";		
		haPredMain.writeResdiueCSDataToFileForRTwoNeighbors(vecTestDataShiftx,testRFileName);
		
		String testOneNeighborRFileName=userDir+"/system/cs_database/test_for_nasca/r_files/r_test_data_one_neighbor.txt";	
		haPredMain.writeResdiueCSDataToFileForROneNeighbor(vecTestDataShiftx,testOneNeighborRFileName);
		
		String testNoNeighborRFileName=userDir+"/system/cs_database/test_for_nasca/r_files/r_test_data_no_neighbor.txt";	
		haPredMain.writeResdiueCSDataToFileForRNoNeighbor(vecTestDataShiftx,testNoNeighborRFileName);
		
		
		int resWindow=6;//8
		float hErr=0.06f;//0.08f;
		float heavyErr=0.6f;//0.7f;
		boolean isCaUsed=false;
		boolean isCbUsed=false;
		boolean isCoUsed=false;
		boolean isHnUsed=false;
		boolean isNUsed=true;
		
		System.out.println("Residue window = "+ resWindow +", hErr="+hErr+", heavyErr="+heavyErr);
	/*	System.out.print("Types of chemical shifts used: ");
		if(isCaUsed)
			System.out.print("Ca ");
		if(isCbUsed)
			System.out.print("Cb ");
		if(isCoUsed)
			System.out.print("Co ");
		if(isHnUsed)
			System.out.print("Hn ");
		if(isNUsed)
			System.out.print("N ");
		System.out.println("");*/
		
		Vector<Vector<residueCS>> vecTrainData=new Vector<Vector<residueCS>>();
		///vecTrainData.addAll(vecTrainDataShiftx);
		////vecTrainData.addAll(vecTrainDataTalos);		
		///vecTrainData.addAll(vecTrainDataRefDB);
		
		//haPredMain.computeBestMatchedHaCS(vecTestDataShiftx,vecTrainDataShiftx,resWindow,hErr,heavyErr,isCaUsed,isCbUsed,isCoUsed,isHnUsed,isNUsed);
		haPredMain.computeBestMatchedHaCSHDistance(vecTestDataShiftx,vecTrainData,resWindow,hErr,heavyErr);
		//haPredMain.computeBestMatchedHaCSHDistanceAverage(vecTestDataShiftx,vecTrainData,resWindow,hErr,heavyErr);
		
		//////////////////////////////////////
		//execute the R package to perform the prediction based on the structural information
	    String strRBin="H:/workspace/ProteinNMR/system/cs_database/r_files/R-2.11.1/bin/Rscript";
	    //String strRBin="C:/Program Files/R/R-2.15.0/bin/Rscript";
	    
		///String rScriptsFile="r_scripts_two_neighbors.R";
		String rScriptsFile="r_scripts_two_neighbors_no_structure.R";
	    ////String rScriptsFile="r_scripts_two_neighbors_bb.R";
		
		String rOutFile="RunRResults.txt";		
		testfornasca.runRScripts(strRBin,rScriptsFile,rOutFile);
		
		///rScriptsFile="r_scripts_one_neighbor.R";
	    rScriptsFile="r_scripts_one_neighbor_no_structure.R";
		////rScriptsFile="r_scripts_one_neighbor_bb.R";
		testfornasca.runRScripts(strRBin,rScriptsFile,rOutFile);
		
		////rScriptsFile="r_scripts_no_neighbor.R";
		rScriptsFile="r_scripts_no_neighbor_no_structure.R";
		////rScriptsFile="r_scripts_no_neighbor_bb.R";
		testfornasca.runRScripts(strRBin,rScriptsFile,rOutFile);
		
		///////////////////////////////////////////////////////
		//read the predicted Ha chemical shifts from R, using both chemical shift and structural information
    	String rResultFileName=userDir+"/system/cs_database/test_for_nasca/r_files/r_predicted_ha.txt";
		
		Vector<Vector<residueCS>> vecRPredictedHaCS=haPredMain.readHaChemicalShiftsFromRFile(rResultFileName);		
		System.out.println("Updating the ha chemical shifts using R predicted results...");
		haPredMain.updateHaChemicalShiftFromRResults(vecTestDataShiftx,vecRPredictedHaCS);
		
	
		String rResultOneNeighborFileName=userDir+"/system/cs_database/test_for_nasca/r_files/r_predicted_ha_one_neighbor.txt";	
		
		Vector<Vector<residueCS>> vecRPredictedHaCSOneNeighbor=haPredMain.readHaChemicalShiftsFromRFile(rResultOneNeighborFileName);		
		System.out.println("Updating the ha chemical shifts using R predicted results, using only structural information...");
		haPredMain.updateHaChemicalShiftFromRResults(vecTestDataShiftx,vecRPredictedHaCSOneNeighbor);
		
		String rResultNoNeighborFileName=userDir+"/system/cs_database/test_for_nasca/r_files/r_predicted_ha_no_neighbor.txt";		
		Vector<Vector<residueCS>> vecRPredictedHaCSNoNeighbor=haPredMain.readHaChemicalShiftsFromRFile(rResultNoNeighborFileName);		
		System.out.println("Updating the ha chemical shifts using R predicted results, using only structural information...");
		haPredMain.updateHaChemicalShiftFromRResults(vecTestDataShiftx,vecRPredictedHaCSNoNeighbor);
		
	/*	String rResultOnlyStructureFileName=userDir+"/system/cs_database/r_files/r_predicted_ha_only_structure.txt";		
		Vector<Vector<residueCS>> vecRPredictedHaCSOnlyStr=haPredMain.readHaChemicalShiftsFromRFile(rResultOnlyStructureFileName);		
		System.out.println("Updating the ha chemical shifts using R predicted results, using only structural information...");
		haPredMain.updateHaChemicalShiftFromRResults(vecTestDataShiftx,vecRPredictedHaCSOnlyStr);
		*/
		
		
		/////////////////////////////////////
		
		haPredMain.haPredictionSummary(vecTestDataShiftx);
		Vector<residueCS> vecOneData=vecTestDataShiftx.elementAt(0);
		
		long endTime = System.currentTimeMillis() ;
	 	double totalTime = (double) ((endTime - startTime) / 60000.0); //in minutes
	 	System.out.println("Time= "+ totalTime +" minutes");
	 	return vecOneData;
	}
	
	
	/** 
	 * Read the Nasca testing data.
	 *  @param pathCSDatabase path of cs directory
	 *  @param pathPdbDatabase path of PDB directory
	 *  @param isTraining whether it is training data, TRUE for training data, FALSE for testing data
	 *  @return a vector of residueCS
	 */
	public Vector <Vector<residueCS>> readTestingData(String pathCSDatabase,String pathPdbDatabase,boolean isTraining){		
		Vector<Vector<residueCS>> vecData=new  Vector<Vector<residueCS>>();
		
		shiftx2Database shiftx=new shiftx2Database();
		
		ringEffect ring_effect=new ringEffect();
		hBondEffect hbond_effect=new hBondEffect();		
		H1CS h1CS=new H1CS();
		Pdb pp=new Pdb();
		chemicalShiftDatabase csd=new chemicalShiftDatabase();
		
		String userDir = System.getProperty("user.dir");
		
		//read random coil chemical shift table:
	    String pathRandomCoilShift=userDir+"/system/cs_database/random_coil_shifts.txt"; //random coil shift table
		randomCoilShift rcs=new randomCoilShift();
		Vector<randomCoilShift> vecRandomShifts=rcs.randomCoilShiftReader(pathRandomCoilShift);
		
		//read all chemical shift files
    	int counter=1;
    	File myDir = new File(pathCSDatabase);
    	String[] contents = myDir.list();//select all structures
    	if (contents == null) 
    		System.out.println(myDir.getName() + " is not a directory");
    	for (int t=0; t<contents.length;t++)  {  
    	
    		String filename = contents[t]; 
    		String fileNameSaved=filename.substring(0,filename.indexOf("_"));
    		
	    	String strCSFile="";
	    	strCSFile=pathCSDatabase+filename;	 	    	
	    	
	    	String strFoundPdbFile=shiftx2Database.findMatchedPdbFile(pathPdbDatabase,fileNameSaved);
	    	String strPdbFile=pathPdbDatabase+strFoundPdbFile;
	    		    		    	
	    	//read the training chemical shift data
	    	Vector<H1CS> vecCSTemp= new Vector<H1CS>();
	    	
	    	////////////////
	    	Assign asg = new Assign();
	    	String seqFile ="H:/workspace/ProteinNMR/inputFiles/UBQ.seq";
	    	//this is the vector of residues (protein sequence)
	    	Vector<Assign> vecSeq=asg.ReaderSeq(seqFile);    	
	    	vecCSTemp=h1CS.h1CSReader(strCSFile,vecSeq); 
	    	
	    	//if(isTraining)	    	
	    	
	    	//vecCSTemp=h1CS.h1CSReader_BMRB(strCSFile); 	 
	    	//else
	    		//vecCSTemp=h1CSReader_BMRB_Modified(strCSFile); 	 //read the testing data
	    	    	
	    	//read the training pdb data	    	
	    	Vector<Pdb> vecPdbOld = pp.readPdb(strPdbFile);
	    	Vector<Pdb> vecPdb=pp.nameConvert(vecPdbOld); 
	    	    	
	    	//align the residue number
	    	shiftx2Database.alignResidueNum(vecPdb,vecCSTemp);
	    	
	    	//update PDB residue names
	    	shiftx2Database.updatePdbResidueName(vecPdb,vecCSTemp);
	    	
	    	boolean isBMRBNEWNameScheme=false;;// whether using the BMRB_NEW naming scheme for atom names.
	    	for(int x=0;x<vecPdb.size();x++){
	    		Pdb pdb=vecPdb.elementAt(x);
	    		String res=pdb.getResidue();
	    		Vector<Cartesian> vecAtom =pdb.getAtomVec();
	    		for(int y=0;y<vecAtom.size();y++){
	    			Cartesian cc=vecAtom.elementAt(y);
	    			String atom=cc.getAtom();
	    			if( !( res.equalsIgnoreCase("ALA")||res.equalsIgnoreCase("GLY")|| res.equalsIgnoreCase("ILE")
	    					||res.equalsIgnoreCase("PRO")||res.equalsIgnoreCase("THR")||res.equalsIgnoreCase("VAL")) ){
	    				if(atom.equalsIgnoreCase("HB3"))
	    					isBMRBNEWNameScheme=true;
	    				break;
	    			}	    			
	    		}//for(int y=0;y<vecAtom.size();y++)
	    		if(isBMRBNEWNameScheme)
	    			break;
	    	}//for(int x=0;x<vecPdb.size();x++)
	    	if(isBMRBNEWNameScheme){
	    		Vector<Pdb>  vecPdbTemp=pp.nameConvert2PDBNew(vecPdb);
	    		vecPdb=new Vector<Pdb>();
	    		vecPdb.addAll(vecPdbTemp);
	    	}	    	
	    	
	    	//compute the phi/psi angles
	    	Vector<PhiPsi> vecPhiPsi=pp.PhiPsiTotalPdb(vecPdb);	    
	    	   	
	    	//find max res no and min res no	    		    	
	    	int minNoFromCS=9999999;
	    	int maxNoFromCS=-99999;
	    	for(int i=0;i<vecCSTemp.size();i++){
	    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
	    		int resNo=h1cs.getResidueNo();
	    		if(resNo>maxNoFromCS)
	    			maxNoFromCS=resNo;  
	    		if(resNo<minNoFromCS)
	    			minNoFromCS=resNo;
	    	}
	    	int minNoFromPdb=9999999;
	    	int maxNoFromPdb=-99999;
	    	for(int i=0;i<vecPdb.size();i++){
	    		Pdb pdb=(Pdb)vecPdb.elementAt(i);
	    		int resNo=pdb.getResidueNo();
	    		if(resNo>maxNoFromPdb)
	    			maxNoFromPdb=resNo;  
	    		if(resNo<minNoFromPdb)
	    			minNoFromPdb=resNo;
	    	}
	    	int maxNo=Math.min(maxNoFromCS, maxNoFromPdb);
	    	int minNo=Math.max(minNoFromCS, minNoFromPdb);
	    	
	    	//for computing the chemical shift delta from hbond effect:
	    	double[] haCSDelta=new double[maxNo+1];
	    	double[] hnCSDelta=new double[maxNo+1];
	    	for(int i=0;i<=maxNo;i++){
	    		haCSDelta[i]=0.0;
	    		hnCSDelta[i]=0.0;
	    	}
	    	Vector<residueCS> vecOneProteinData=new Vector<residueCS>();
	    	
	    	//we skip the first residue and the last residue
	    	for(int no=minNo+1;no<=maxNo-1;no++){     	
	    		
	    		int ind= Collections.binarySearch(vecPdb, new Pdb(no), new Pdb.PdbComparator());
	     	    if(ind<0)
	     	    	continue;
	     	    Pdb curPdb=vecPdb.elementAt(ind);
	     	    String curRes=curPdb.getResidue();
	     	    
	     	    int ind2= Collections.binarySearch(vecPhiPsi, new PhiPsi(no), new PhiPsi.PPComparator());
	     	    if(ind2<0)
	     	    	continue;
	     	   
	     	    PhiPsi phipsi=vecPhiPsi.elementAt(ind2);
	     	 
	     	    int indPrev= Collections.binarySearch(vecPhiPsi, new PhiPsi(no-1), new PhiPsi.PPComparator());
	     	    double phiPrev=0.0;
	     	    double psiPrev=0.0;
	     	    if( indPrev>-1){
	    		   PhiPsi phipsiPre=vecPhiPsi.elementAt(indPrev);
	    		   phiPrev=chemicalShiftDatabase.convertAngleInPI( phipsiPre.getPhi() );
	    		   psiPrev=chemicalShiftDatabase.convertAngleInPI( phipsiPre.getPsi() );
	     	    }
	    	   
	     	    int indNext= Collections.binarySearch(vecPhiPsi, new PhiPsi(no+1), new PhiPsi.PPComparator());
	     	    double phiNext=0.0;
	     	    double psiNext=0.0;
	     	    if( indNext>-1){
	    		   PhiPsi phipsiNext=vecPhiPsi.elementAt(indNext);
	    		   phiNext=chemicalShiftDatabase.convertAngleInPI( phipsiNext.getPhi() );
	    		   psiNext=chemicalShiftDatabase.convertAngleInPI( phipsiNext.getPsi() );
	     	    }
	     	   
	     	    //check the residue name of the neighboring residues
	     	    String prevResType="";
	     	    ind=Collections.binarySearch(vecPdb, new Pdb(no-1), new Pdb.PdbComparator());
	     	    if(ind>-1){	     	    	
		     	   Pdb prevPdb=vecPdb.elementAt(ind);
		     	   prevResType=prevPdb.getResidue();
	     	    }   
	      	  
	     	   ////////////////////////////////
	     	   //compute the chemical shift delta from hbond and ring.
	     	   float deltaHBond_ha=(float)hbond_effect.computeDeltaFromHBondForHaOld(no, vecPdb);
	     	   float deltaHBond_hn=(float)hbond_effect.computeDeltaFromHBondForHnOld(no, vecPdb);	     	   
	     	   float deltaRing=(float)ring_effect.computeDeltaChemShiftFromRingEffectForHa(no, vecPdb)*1000;	     	
	     	   float deltaRingCa=(float)ring_effect.computeDeltaChemShiftFromRingEffectForCa(no, vecPdb)*1000;	    	 		
	    		
	     	   //find the bb chemical shift of current residue
	    		double csN=-9999.0;
	    		double csCA=-9999.0;
	    		double csHN=-9999.0;
	    		double csHA=-9999.0;
	    		double csCB=-9999.0;
	    		double csCO=-9999.0;
	    		
	    		//find cs of N
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && atom.equalsIgnoreCase("N")){
		    			csN=h1cs.getH1CS();
		    			break;
		    		}
	    		}  		
	    		
	    		//find cs of CA
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && atom.equalsIgnoreCase("CA")){
		    			csCA=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	    	
	    		//find cs of HN
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && (atom.equalsIgnoreCase("HN")|| atom.equalsIgnoreCase("H"))  ){
		    			csHN=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	    		
	    		//find cs of HA
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();		    		
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && atom.equalsIgnoreCase("HA")){
		    			csHA=h1cs.getH1CS();
		    			break;
		    		}	
		    	}//for(int i=0;i<vecCSTemp.size();i++)
	    		
	    		//for GLY, we compute the mean of two HA chemical shifts
	    		if(curRes.equalsIgnoreCase("GLY") || curRes.equalsIgnoreCase("G")){
	    			double sumHaCS=0.0;	  
	    			int counterHaCS=0;
	    			boolean isFoundHaForGly=false;	    			
	    			for(int i=0;i<vecCSTemp.size();i++){
			    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
			    		int resNo=h1cs.getResidueNo();		    		
			    		String atom=h1cs.getAtomName();
			    		if(atom.length()>=2)
			    			atom=atom.substring(0,2);
			    		if(resNo==no && atom.equalsIgnoreCase("HA")){
			    			sumHaCS=sumHaCS+h1cs.getH1CS();
			    			counterHaCS++;
			    			isFoundHaForGly=true;
			    		}	
			    	}//for(int i=0;i<vecCSTemp.size();i++)
	    			
	    			if(isFoundHaForGly && counterHaCS==2){
	    				csHA=sumHaCS/counterHaCS;
	    			}
	    			else
	    				csHA=-9999.0;	    			
	    		}//if(curRes.equalsIgnoreCase("GLY") || curRes.equalsIgnoreCase("G"))
	    		
	    		//find cs of CB
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && atom.equalsIgnoreCase("CB")){
		    			csCB=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}
	    		
	    		//find cs of CO
	    		for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();
		    		String atom=h1cs.getAtomName();
		    		if(resNo==no && (atom.equalsIgnoreCase("CO") ||atom.equalsIgnoreCase("C"))){
		    			csCO=h1cs.getH1CS();
		    			break;
		    		}	    			   		
		    	}    		
	  
	    		double csRandCoilHa=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"HA",prevResType);
	    		double csRandCoilCa=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"CA",prevResType);
	    		double csRandCoilCb=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"CB",prevResType);
	    		double csRandCoilCo=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"C",prevResType);
	    		double csRandCoilHn=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"HN",prevResType);
	    		double csRandCoilN=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"N",prevResType);
	    		
	    		csHA=csHA-csRandCoilHa;
	    		csCA=csCA-csRandCoilCa;
	    		csCB=csCB-csRandCoilCb;
	    		csCO=csCO-csRandCoilCo;
	    		csHN=csHN-csRandCoilHn;
	    		csN=csN-csRandCoilN;      		
	    		   		
	    		double phi=chemicalShiftDatabase.convertAngleInPI( phipsi.getPhi());//1.5*Math.cos(3*(phipsi.getPhi() + 2.45*Math.PI/180 ))-3.0*Math.cos(phipsi.getPhi()-2.7*Math.PI/180 );
	    		double psi=chemicalShiftDatabase.convertAngleInPI( phipsi.getPsi() );//-0.4*Math.cos(3* ( phipsi.getPsi() +4.8*Math.PI/180 ) )-1.5*Math.cos(phipsi.getPsi() -0.84*Math.PI/180 )+0.7;
	    		
	    		residueCS resCS=new residueCS(no,curRes,fileNameSaved,(float)csHA,(float)csCA,(float)csCB,(float)csCO,(float)csHN,(float)csN,
	    				(float)deltaHBond_ha,(float)deltaHBond_hn,(float)deltaRing,(float)deltaRingCa,
	    				(float)phi,(float)psi,(float)phiPrev,(float)psiPrev,(float)phiNext,(float)psiNext);
	    		
	    		vecOneProteinData.add(resCS);
	    		///////////////////	    		
	    		//String str_csHA=csToString((float)(csHA));String str_csCA=csToString((float)(csCA));String str_csCB=csToString((float)(csCB));
	    		//String str_csCO=csToString((float)(csCO));String str_csHN=csToString((float)(csHN));String str_csN=csToString((float)(csN));	    						    		
	    		counter++;	    		
	    	}//for(int i=1;i<=maxNo;i++)
	    	
	    	Collections.sort(vecOneProteinData, new residueCS.residueCSComparator());//sort the data points based on residue number
	    	vecData.add(vecOneProteinData);	    	
    	}//for (int t=0; t<contents.length;t++) 
    	
    	return vecData;
	}
	
	
	/** 
	 * Read the Nasca testing data.
	 *  @param pathCSDatabase path of cs directory
	 *  @param vecCSTemp input cs vector
	 *  @param 
	 *  @return a vector of residueCS
	 */
	public Vector <Vector<residueCS>> readTestingDataFromInputPdbNCS (Vector<Pdb> vecPdb,Vector<H1CS> vecCSTemp){		
		Vector<Vector<residueCS>> vecData=new  Vector<Vector<residueCS>>();
		
		shiftx2Database shiftx=new shiftx2Database();
		
		ringEffect ring_effect=new ringEffect();
		hBondEffect hbond_effect=new hBondEffect();		
		H1CS h1CS=new H1CS();
		Pdb pp=new Pdb();
		chemicalShiftDatabase csd=new chemicalShiftDatabase();
		
		String userDir = System.getProperty("user.dir");
		
		//read random coil chemical shift table:
	    String pathRandomCoilShift=userDir+"/system/cs_database/random_coil_shifts.txt"; //random coil shift table
		randomCoilShift rcs=new randomCoilShift();
		Vector<randomCoilShift> vecRandomShifts=rcs.randomCoilShiftReader(pathRandomCoilShift);
		
		//read all chemical shift files 	
    	
    	////////////////////
    	String fileNameSaved ="test"; 	    	
	    	    	
    	//align the residue number
    	shiftx2Database.alignResidueNum(vecPdb,vecCSTemp);
    	
    	//update PDB residue names
    	shiftx2Database.updatePdbResidueName(vecPdb,vecCSTemp);
    	
    	boolean isBMRBNEWNameScheme=false;;// whether using the BMRB_NEW naming scheme for atom names.
    	for(int x=0;x<vecPdb.size();x++){
    		Pdb pdb=vecPdb.elementAt(x);
    		String res=pdb.getResidue();
    		Vector<Cartesian> vecAtom =pdb.getAtomVec();
    		for(int y=0;y<vecAtom.size();y++){
    			Cartesian cc=vecAtom.elementAt(y);
    			String atom=cc.getAtom();
    			if( !( res.equalsIgnoreCase("ALA")||res.equalsIgnoreCase("GLY")|| res.equalsIgnoreCase("ILE")
    					||res.equalsIgnoreCase("PRO")||res.equalsIgnoreCase("THR")||res.equalsIgnoreCase("VAL")) ){
    				if(atom.equalsIgnoreCase("HB3"))
    					isBMRBNEWNameScheme=true;
    				break;
    			}	    			
    		}//for(int y=0;y<vecAtom.size();y++)
    		if(isBMRBNEWNameScheme)
    			break;
    	}//for(int x=0;x<vecPdb.size();x++)
    	if(isBMRBNEWNameScheme){
    		Vector<Pdb>  vecPdbTemp=pp.nameConvert2PDBNew(vecPdb);
    		vecPdb=new Vector<Pdb>();
    		vecPdb.addAll(vecPdbTemp);
    	}	    	
    	
    	//compute the phi/psi angles
    	Vector<PhiPsi> vecPhiPsi=pp.PhiPsiTotalPdb(vecPdb);	    
    	   	
    	//find max res no and min res no	    		    	
    	int minNoFromCS=9999999;
    	int maxNoFromCS=-99999;
    	for(int i=0;i<vecCSTemp.size();i++){
    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
    		int resNo=h1cs.getResidueNo();
    		if(resNo>maxNoFromCS)
    			maxNoFromCS=resNo;  
    		if(resNo<minNoFromCS)
    			minNoFromCS=resNo;
    	}
    	int minNoFromPdb=9999999;
    	int maxNoFromPdb=-99999;
    	for(int i=0;i<vecPdb.size();i++){
    		Pdb pdb=(Pdb)vecPdb.elementAt(i);
    		int resNo=pdb.getResidueNo();
    		if(resNo>maxNoFromPdb)
    			maxNoFromPdb=resNo;  
    		if(resNo<minNoFromPdb)
    			minNoFromPdb=resNo;
    	}
    	int maxNo=Math.min(maxNoFromCS, maxNoFromPdb);
    	int minNo=Math.max(minNoFromCS, minNoFromPdb);
    	
    	//for computing the chemical shift delta from hbond effect:
    	double[] haCSDelta=new double[maxNo+1];
    	double[] hnCSDelta=new double[maxNo+1];
    	for(int i=0;i<=maxNo;i++){
    		haCSDelta[i]=0.0;
    		hnCSDelta[i]=0.0;
    	}
    	Vector<residueCS> vecOneProteinData=new Vector<residueCS>();
    	
    	//we skip the first residue and the last residue
    	for(int no=minNo+1;no<=maxNo-1;no++){     	
    		
    		int ind= Collections.binarySearch(vecPdb, new Pdb(no), new Pdb.PdbComparator());
     	    if(ind<0)
     	    	continue;
     	    Pdb curPdb=vecPdb.elementAt(ind);
     	    String curRes=curPdb.getResidue();
     	    
     	    int ind2= Collections.binarySearch(vecPhiPsi, new PhiPsi(no), new PhiPsi.PPComparator());
     	    if(ind2<0)
     	    	continue;
     	   
     	    PhiPsi phipsi=vecPhiPsi.elementAt(ind2);
     	 
     	    int indPrev= Collections.binarySearch(vecPhiPsi, new PhiPsi(no-1), new PhiPsi.PPComparator());
     	    double phiPrev=0.0;
     	    double psiPrev=0.0;
     	    if( indPrev>-1){
    		   PhiPsi phipsiPre=vecPhiPsi.elementAt(indPrev);
    		   phiPrev=chemicalShiftDatabase.convertAngleInPI( phipsiPre.getPhi() );
    		   psiPrev=chemicalShiftDatabase.convertAngleInPI( phipsiPre.getPsi() );
     	    }
    	   
     	    int indNext= Collections.binarySearch(vecPhiPsi, new PhiPsi(no+1), new PhiPsi.PPComparator());
     	    double phiNext=0.0;
     	    double psiNext=0.0;
     	    if( indNext>-1){
    		   PhiPsi phipsiNext=vecPhiPsi.elementAt(indNext);
    		   phiNext=chemicalShiftDatabase.convertAngleInPI( phipsiNext.getPhi() );
    		   psiNext=chemicalShiftDatabase.convertAngleInPI( phipsiNext.getPsi() );
     	    }
     	   
     	    //check the residue name of the neighboring residues
     	    String prevResType="";
     	    ind=Collections.binarySearch(vecPdb, new Pdb(no-1), new Pdb.PdbComparator());
     	    if(ind>-1){	     	    	
	     	   Pdb prevPdb=vecPdb.elementAt(ind);
	     	   prevResType=prevPdb.getResidue();
     	    }   
      	  
     	   ////////////////////////////////
     	   //compute the chemical shift delta from hbond and ring.
     	   float deltaHBond_ha=(float)hbond_effect.computeDeltaFromHBondForHaOld(no, vecPdb);
     	   float deltaHBond_hn=(float)hbond_effect.computeDeltaFromHBondForHnOld(no, vecPdb);	     	   
     	   float deltaRing=(float)ring_effect.computeDeltaChemShiftFromRingEffectForHa(no, vecPdb)*1000;	     	
     	   float deltaRingCa=(float)ring_effect.computeDeltaChemShiftFromRingEffectForCa(no, vecPdb)*1000;	    	 		
    		
     	   //find the bb chemical shift of current residue
    		double csN=-9999.0;
    		double csCA=-9999.0;
    		double csHN=-9999.0;
    		double csHA=-9999.0;
    		double csCB=-9999.0;
    		double csCO=-9999.0;
    		
    		//find cs of N
    		for(int i=0;i<vecCSTemp.size();i++){
	    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
	    		int resNo=h1cs.getResidueNo();
	    		String atom=h1cs.getAtomName();
	    		if(resNo==no && atom.equalsIgnoreCase("N")){
	    			csN=h1cs.getH1CS();
	    			break;
	    		}
    		}  		
    		
    		//find cs of CA
    		for(int i=0;i<vecCSTemp.size();i++){
	    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
	    		int resNo=h1cs.getResidueNo();
	    		String atom=h1cs.getAtomName();
	    		if(resNo==no && atom.equalsIgnoreCase("CA")){
	    			csCA=h1cs.getH1CS();
	    			break;
	    		}	    			   		
	    	}
    	
    		//find cs of HN
    		for(int i=0;i<vecCSTemp.size();i++){
	    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
	    		int resNo=h1cs.getResidueNo();
	    		String atom=h1cs.getAtomName();
	    		if(resNo==no && (atom.equalsIgnoreCase("HN")|| atom.equalsIgnoreCase("H"))  ){
	    			csHN=h1cs.getH1CS();
	    			break;
	    		}	    			   		
	    	}
    		
    		//find cs of HA
    		for(int i=0;i<vecCSTemp.size();i++){
	    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
	    		int resNo=h1cs.getResidueNo();		    		
	    		String atom=h1cs.getAtomName();
	    		if(resNo==no && atom.equalsIgnoreCase("HA")){
	    			csHA=h1cs.getH1CS();
	    			break;
	    		}	
	    	}//for(int i=0;i<vecCSTemp.size();i++)
    		
    		//for GLY, we compute the mean of two HA chemical shifts
    		if(curRes.equalsIgnoreCase("GLY") || curRes.equalsIgnoreCase("G")){
    			double sumHaCS=0.0;	  
    			int counterHaCS=0;
    			boolean isFoundHaForGly=false;	    			
    			for(int i=0;i<vecCSTemp.size();i++){
		    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
		    		int resNo=h1cs.getResidueNo();		    		
		    		String atom=h1cs.getAtomName();
		    		if(atom.length()>=2)
		    			atom=atom.substring(0,2);
		    		if(resNo==no && atom.equalsIgnoreCase("HA")){
		    			sumHaCS=sumHaCS+h1cs.getH1CS();
		    			counterHaCS++;
		    			isFoundHaForGly=true;
		    		}	
		    	}//for(int i=0;i<vecCSTemp.size();i++)
    			
    			if(isFoundHaForGly && counterHaCS==2){
    				csHA=sumHaCS/counterHaCS;
    			}
    			else
    				csHA=-9999.0;	    			
    		}//if(curRes.equalsIgnoreCase("GLY") || curRes.equalsIgnoreCase("G"))
    		
    		//find cs of CB
    		for(int i=0;i<vecCSTemp.size();i++){
	    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
	    		int resNo=h1cs.getResidueNo();
	    		String atom=h1cs.getAtomName();
	    		if(resNo==no && atom.equalsIgnoreCase("CB")){
	    			csCB=h1cs.getH1CS();
	    			break;
	    		}	    			   		
	    	}
    		
    		//find cs of CO
    		for(int i=0;i<vecCSTemp.size();i++){
	    		H1CS h1cs=(H1CS)vecCSTemp.elementAt(i);
	    		int resNo=h1cs.getResidueNo();
	    		String atom=h1cs.getAtomName();
	    		if(resNo==no && (atom.equalsIgnoreCase("CO") ||atom.equalsIgnoreCase("C"))){
	    			csCO=h1cs.getH1CS();
	    			break;
	    		}	    			   		
	    	}    		
  
    		double csRandCoilHa=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"HA",prevResType);
    		double csRandCoilCa=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"CA",prevResType);
    		double csRandCoilCb=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"CB",prevResType);
    		double csRandCoilCo=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"C",prevResType);
    		double csRandCoilHn=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"HN",prevResType);
    		double csRandCoilN=csd.getRandomCoilChemShift(vecRandomShifts,curRes,"N",prevResType);
    		
    		csHA=csHA-csRandCoilHa;
    		csCA=csCA-csRandCoilCa;
    		csCB=csCB-csRandCoilCb;
    		csCO=csCO-csRandCoilCo;
    		csHN=csHN-csRandCoilHn;
    		csN=csN-csRandCoilN;      		
    		   		
    		double phi=chemicalShiftDatabase.convertAngleInPI( phipsi.getPhi());//1.5*Math.cos(3*(phipsi.getPhi() + 2.45*Math.PI/180 ))-3.0*Math.cos(phipsi.getPhi()-2.7*Math.PI/180 );
    		double psi=chemicalShiftDatabase.convertAngleInPI( phipsi.getPsi() );//-0.4*Math.cos(3* ( phipsi.getPsi() +4.8*Math.PI/180 ) )-1.5*Math.cos(phipsi.getPsi() -0.84*Math.PI/180 )+0.7;
    		
    		residueCS resCS=new residueCS(no,curRes,fileNameSaved,(float)csHA,(float)csCA,(float)csCB,(float)csCO,(float)csHN,(float)csN,
    				(float)deltaHBond_ha,(float)deltaHBond_hn,(float)deltaRing,(float)deltaRingCa,
    				(float)phi,(float)psi,(float)phiPrev,(float)psiPrev,(float)phiNext,(float)psiNext);
    		
    		vecOneProteinData.add(resCS);
    		///////////////////	    		
    		//String str_csHA=csToString((float)(csHA));String str_csCA=csToString((float)(csCA));String str_csCB=csToString((float)(csCB));
    		//String str_csCO=csToString((float)(csCO));String str_csHN=csToString((float)(csHN));String str_csN=csToString((float)(csN));	    						    		
    		//counter++;	    		
    	}//for(int i=1;i<=maxNo;i++)
    	
    	Collections.sort(vecOneProteinData, new residueCS.residueCSComparator());//sort the data points based on residue number
    	vecData.add(vecOneProteinData);	    	
	
    	
    	return vecData;
	}
	
	
	
	 /**
	 * Run R scripts
	 * 
	 * @param src location of the input file
	 * @param strOut location of the output file
	 * @param strInput  input file name
	 * 
	 * @return void
	 * 
	 * @throws JampackException the jampack exception
	 * @throws InterruptedException the interrupted exception
	 */       
   public void runRScripts(String strRBin, String rScriptsFile,String outLogFile) throws InterruptedException
   {	   
	    System.out.println("Running R scripts...");
   		long startTime = System.currentTimeMillis();
   	
   		String userDir = System.getProperty("user.dir");////
   		String srcRPath=userDir+"/system/cs_database/test_for_nasca/r_files/";
   	
        System.setProperty("user.dir",	srcRPath);
   	
   		String userDirNew = System.getProperty("user.dir");////
   		System.out.println("Current working directory is:"+userDirNew);//for debugging p   	
   	
   		String strRun=strRBin+" "+rScriptsFile;//" r_scripts.R";
   		Runtime runtime = Runtime.getRuntime();
	   	try
	       {    		
	   		//execute the xplor scripts:
	   		File  direc =new File (userDirNew);
	   		Process r = runtime.exec(strRun, null, direc );
	   		InputStream rOut = r.getInputStream();
	   		InputStreamReader isr = new InputStreamReader(rOut);
	   	    BufferedReader br = new BufferedReader(isr);
	   	    String line;
	   	    String outFileName =userDirNew+File.separator+outLogFile;//"RunRResults.txt";
	   	    try
		    	{
		    		PrintWriter out = new PrintWriter(new BufferedWriter(new FileWriter(outFileName)));
		    	    while ((line = br.readLine()) != null)
		    	    {
		    	    	 out.println(line);///to file
		    	         System.out.println(line);//for debugging...
		    	    }
		    	    out.close();
		    	
		        }catch (FileNotFoundException e)
				{
					System.out.println("File not found: " + outFileName);
				}catch (IOException e)
				{
				   System.out.println("IOException: the stack trace is:");
				   e.printStackTrace();
				}
			
	   	    r.waitFor();
	       }
	       catch(IOException e)
	       {
	          System.err.println("Error on exec() method");
	          e.printStackTrace();  
	       }
      
	   	   System.setProperty("user.dir",	userDir);
       
	   	   long endTime = System.currentTimeMillis();
	   	   double totalTime = (double) ((endTime - startTime) / 60000.0); //in minutes
	   	   System.out.println("Finish running R scripts.");
	   	   System.out.println("Time for running R:  "+ totalTime +" minutes");
   	}
   
	

}
