

#read the training chemical shift data, for the previous and next residues
data<-read.table('chemical_shifts_training_neighbor_two_combine_talos_shiftx.txt',sep=' ',)
data<-data[,-1]
colnames(data)<-c("ha","ca","cb","co","hn","n",
    "pre_ca","pre_cb","pre_co","pre_hn","pre_n",
    "next_ca", "next_cb","next_co","next_hn","next_n",
    "preTwo_ca","preTwo_cb","preTwo_co","preTwo_hn","preTwo_n",
    "nextTwo_ca", "nextTwo_cb","nextTwo_co","nextTwo_hn","nextTwo_n",
    "hbond_ha","hbond_hn","ring","ring_ca","cosPhi","cosPsi",
    "cosPhiPrev","cosPsiPrev","cosPhiNext","cosPsiNext","residue")

##process the data for three different cases:
#xFlag:TRUE or FALSE
data_process <- function(data,caFlag,cbFlag,coFlag,hnFlag,nFlag){
  data_temp<-matrix(NaN,length(data[,2]),37+6)
  data_temp[,1:37]<-data.matrix(data[,1:37])
 
  data_temp[,37+1]=as.numeric(data_temp[,1]>-100)*as.numeric(data_temp[,1]<100)
  if(caFlag){  
  	data_temp[,37+2]=as.numeric(data_temp[,2]>-100)*as.numeric(data_temp[,2]<100)
      }  
  else  
	data_temp[,37+2]=as.numeric(TRUE)
 
  if(cbFlag){
  	data_temp[,37+3]=as.numeric(data_temp[,3]>-100)*as.numeric(data_temp[,3]<100)	
	}
  else
	data_temp[,37+3]=as.numeric(TRUE)
 
  if(coFlag){
 	data_temp[,37+4]=as.numeric(data_temp[,4]>-100)*as.numeric(data_temp[,4]<100)	
	}
  else  
	data_temp[,37+4]=as.numeric(TRUE)
  
  if(hnFlag){
	data_temp[,37+5]=as.numeric(data_temp[,5]>-100)*as.numeric(data_temp[,5]<100)
	}
  else  
	data_temp[,37+5]=as.numeric(TRUE)

  if(nFlag){
 	data_temp[,37+6]=as.numeric(data_temp[,6]>-100)*as.numeric(data_temp[,6]<100)
}
  else  
      data_temp[,37+6]=as.numeric(TRUE)

  data_temp[,37+6]=data_temp[,37+1]*data_temp[,37+2]*data_temp[,37+3]*data_temp[,37+4]*data_temp[,37+5]*data_temp[,37+6]

  clean_data<-data_temp[data_temp[,37+6]!= 0,]
  return (clean_data )
}


##################################
#2. parameter estimation 
clean_data<-data_process (data,TRUE,TRUE,FALSE,TRUE,TRUE)
data=clean_data

ha<-data[,1]
ca<-data[,2]
cb<-data[,3]
co<-data[,4]
hn<-data[,5]
n<-data[,6]

pre_ca<-data[,7]
pre_cb<-data[,8]
pre_co<-data[,9]
pre_hn<-data[,10]
pre_n<-data[,11]

next_ca<-data[,12]
next_cb<-data[,13]
next_co<-data[,14]
next_hn<-data[,15]
next_n<-data[,16]

preTwo_ca<-data[,17]
preTwo_cb<-data[,18]
preTwo_co<-data[,19]
preTwo_hn<-data[,20]
preTwo_n<-data[,21]

nextTwo_ca<-data[,22]
nextTwo_cb<-data[,23]
nextTwo_co<-data[,24]
nextTwo_hn<-data[,25]
nextTwo_n<-data[,26]


hbond_ha<-data[,27]
hbond_hn<-data[,28]
ring<-data[,29]
ring_ca<-data[,30]
cosPhi<-data[,31]
cosPsi<-data[,32]
cosPhiPrev<-data[,33]
cosPsiPrev<-data[,34]
cosPhiNext<-data[,35]
cosPsiNext<-data[,36]

library(gam)

dg=16 # degrees of freedom

fit_ha <- gam(ha~s(ca,dg)+s(cb,dg)+s(hn,dg)+s(n,dg)
    +s(hbond_ha,dg)+s(hbond_hn,dg)+s(ring,dg)+s(ring_ca,dg)
    +s(cosPhi,dg)+s(cosPsi,dg)+s(cosPhiPrev,dg)+s(cosPsiPrev,dg)
    +s(cosPhiNext,dg)+s(cosPsiNext,dg),family=gaussian())

summary(fit_ha) # display results

beta=fit_ha$coefficients  # vector of coefficients

sigma_5=sqrt( sum((ha-fit_ha$fitted.value)^2/length(ha)))
sigma_5
corr=cor(ha,fit_ha$fitted.value)
corr
#R2=corr^2
plot(ha,fit_ha$fitted.value, xlim=c(-4,2), ylim=c(-4,2))



##################################
#4. Read the testing data
data_test<-read.table('r_test_data_no_neighbor.txt',sep=' ',)
data_test<-data_test[,-1]
colnames(data_test)<-c("ha","ca","cb","co","hn","n",
    "pre_ca","pre_cb","pre_co","pre_hn","pre_n",
    "next_ca", "next_cb","next_co","next_hn","next_n",
    "preTwo_ca","preTwo_cb","preTwo_co","preTwo_hn","preTwo_n",
    "nextTwo_ca", "nextTwo_cb","nextTwo_co","nextTwo_hn","nextTwo_n",  
    "hbond_ha","hbond_hn","ring","ring_ca","cosPhi","cosPsi",
    "cosPhiPrev","cosPsiPrev","cosPhiNext","cosPsiNext","residue","no","protein")

ha_test<-data_test[,1]

##remove outliers with large ring effect:
ring_temp=data_test[,29]
ca_ring_temp=data_test[,30]

data_temp<-data_test[,1:39]
data_temp<-cbind(data_temp, as.numeric( (abs(ca_ring_temp* beta[9])<1.5)))
data_temp<-cbind(data_temp,as.numeric( (abs(ring_temp* beta[8])<1.5))  )
data_temp<-cbind(data_temp, data_temp[,40]*data_temp[,41])

clean_data<-data_temp[data_temp[,42]!= 0,]
data_test=clean_data[,1:39]

ha_test<-data_test[,1]
out<-predict.gam(fit_ha, newdata=data_test, se.fit = TRUE)



sigma=sqrt(mean((ha_test-out)^2))
sigma
corr=cor(ha_test,out)
corr
#R2=corr^2
plot(ha_test,out)

data_temp<-cbind(data_test,out)
output_data<-cbind(array(data_temp$protein),data_temp$no,data_temp$out)
filename="r_predicted_ha_no_neighbor.txt"

#filename=paste(filename, '.txt', sep='')

write(t(output_data),ncolumns =3, filename )


